puts "BEGIN CMPLAYER"

###### Preferences ###########

set Prefs(sendAhead) .5
set Prefs(cycleTime) 1

set Prefs(mute) 0;
set Prefs(audioOn) 1;
set Prefs(loop) 0;
set Prefs(status) 1;
set Prefs(squeeze) 0;

### Eric14 ###################

# Used to report segment id's to controlling program.
set gRegisterScript {}

# Used to determine that the current script is the expected one.
set gScriptName {}

##############################

set maincm [cmt ""];

set gActionCmd {}
set gActionTime -12345

set gFile {};
set gKiosk 0;

set Prefs(kill) 0;

for {set a 0} {$a < $argc} {incr a} {
    set arg [lindex $argv $a];
	switch -exact -- $arg {
	    "-kill"	{set Prefs(kill) 1}
	    "-kiosk"	{set gKiosk 1}
	    "default"  	{ 
		if {[CMApp_IsURL $arg]} {
		    set gFile $arg;
		    set Tags(title) $arg;
		} elseif {[string index $arg 0] == "/"} {
		    set gFile $arg;
		    set Tags(title) "Untitled"
		} elseif {[string index $arg 0] == "~"} {
		    set gFile $arg;
		    set Tags(title) "Untitled"
		} else {
		    set gFile [pwd]/$arg;
		    set Tags(title) "Untitled"
		}
	    }
	}
}
    

catch {set env(ARCH)} arch;
catch {set env(LIB_DIR)} libDir;
catch {set env(BIN_DIR)} binDir;

set bitDir $libDir/bitmaps

############### SETUP COLORS

source $libDir/scheme.tcl
SetColorScheme gray


if {$gKiosk} {
    SetFontSet 20Point
    set statusFont -*-helvetica-medium-r-normal-*-17-*-*-*-*-*-*-*

    option add *Scale.Width 30 userDefault
    set playWidth 60

    set vcrIX 20
    set vcrIY 15
    set vcrX 3
    set vcrY 3
    
    set ctrlX 5
    set ctrlY 5
    
    set menuX 3
    set labelX 3
    
    set frameY 5
    set frameX 3

    set Prefs(gain) 3;
} else {
    SetFontSet 14Point
    set statusFont -*-helvetica-medium-r-normal-*-12-*-*-*-*-*-*-*

    set vcrIX 6
    set vcrIY 5
    set vcrX 1
    set vcrY 0
    
    set ctrlX 0
    set ctrlY 0
    
    set menuX 0
    set labelX 3
    
    set frameY 3
    set frameX 1

    set Prefs(gain) 3;
}

InitScheme

############### GLOBAL VARIABLES


set Objects(cmt) {};
set Objects(spkr) {};
set Objects(view) {};

set Device(au) {};
set Device(mjpeg) {};
set Device(mpeg) {};
set Device(vq) {};

set cmtObjs(NULL) {};
set localObjs(NULL) {};

set gMinTime  100000
set gMaxTime -100000
set ltsLock 0;

set currentScript {};
set pid -1;
set more 0

############### SCALE / BUTTON COMMANDS and EVENT PROCEDURES

proc setSpeed {s} {
    global lts Prefs;

    #e14 I'm commenting this out because it seems counter
    #    productive to me, but maybe I'm missing something
    #set oldSpeed [lindex [$lts config -speed] 2];
    #if {$oldSpeed == 0} {
    #set oldValue [lindex [$lts config -value] 2];
    #$lts config -speed $s -value [expr $oldValue - $s*$Prefs(sendAhead)];
    #$lts config -speed $s
    #} else {
    #$lts conf -speed $s;
    #}
    $lts conf -speed $s;
    .3.speed set $s;
}

proc setGain {g} {
    global Prefs Objects;

    set Prefs(gain) $g;
    set g [expr double($g)/10.0];
    foreach spkr $Objects(spkr) {
	$spkr conf -gain $g
    }
}

proc mute {} {
    global Prefs Objects maincm

    if {$Prefs(mute) == 1} {
	foreach obj [$maincm getobjs audio-obj] {$obj unready};
	foreach obj $Objects(spkr) {$obj unready};
    } else {
	foreach obj [$maincm getobjs audio-obj] {$obj ready};
	foreach obj $Objects(spkr) {$obj ready};
    }
}

proc updateTimer {} {
    global  lts ltsLock gMinTime gMaxTime bitDir Prefs;
    global  gActionCmd

    set time [lindex [$lts config -value] 2];
    set speed [lindex [$lts config -speed] 2];

    if {($speed > 0) && ($time > $gMaxTime)} {
	if {$Prefs(loop)} {
	    $lts config -value $gMinTime;
	} else {
	    $lts config -value $gMaxTime;
	    .s.play configure -bitmap @$bitDir/play;
	    setSpeed 0;
	}
    }
    if {($speed < 0) && ($time < $gMinTime)} {
	if {$Prefs(loop)} {
	    $lts config -value $gMaxTime;
	} else {
	    $lts config -value $gMinTime;
	    .s.play configure -bitmap @$bitDir/play;
	    setSpeed 0;
	}
    }

    set ltsLock 1;
   
    .s.time set $time;
    .2.time set $time;
    update;

    set ltsLock 0;
    if {$gActionCmd != {}} {
	DoAction
    }
}

proc DoAction {} {
    global  gActionCmd gActionTime lts
    set time [lindex [$lts config -value] 2]

    if {$gActionTime != $time} {
	regsub -all %t $gActionCmd $time out
	eval $out
	set gActionTime $time
    }
}

proc setLts {time} {
    global  lts ltsLock gActionCmd

    if {$ltsLock} { return };
    $lts config -value $time;
    if {$gActionCmd != {}} {
	DoAction
    }
}

proc stepTime {value} {
    global lts

    $lts config -value \
	[expr [lindex [$lts config -value] 2] + $value];
}

proc play {{speed toggle}} {
    global lts bitDir;

    if {$speed == "toggle"} {
	set speed [expr ![lindex [$lts config -speed] 2]]
    } 
	
    if {$speed == 0} {
	.s.play configure -bitmap @$bitDir/play;
    } else {
	.s.play configure -bitmap @$bitDir/stop;
    }
    setSpeed $speed
}

proc more {} {
    global more lts bitDir frameY frameX;

    set speed [lindex [$lts config -speed] 2];

    if $more {
	set more 0;
	pack forget .1 .2 .3 .4;
	if $speed {
	    .s.play configure -bitmap @$bitDir/stop;
	} else {
	    .s.play configure -bitmap @$bitDir/play;
	}
	pack .s -in .ctrl -side top -fill x -pady $frameY -padx $frameX;
    } else {
	set more 1;	
	pack forget .s
	pack .1 .2 .3 .4 -in .ctrl -side top -fill x -pady $frameY -padx $frameX;
    }
}


proc exitPlayer {} {
    global lts maincm lts pid;
    
    $maincm config -local_lost_connection_command {};
    resetPlayer;
    puts "Done."
    exit;
}

proc resetPlayer {} {
    global Objects gScriptName
    global cmtObjs localObjs lts cm maincm lts
    global gMaxTime gMinTime videoWin bitDir Prefs;

    bind $videoWin <Expose> {}
    bind $videoWin <Configure> {}

    set gScriptName {}
    update;

    PutDisp "Destroying remote objects..."

    foreach obj [concat $Objects(spkr) $Objects(view)] {
	puts "Destroying $obj";
	if {[catch {$obj destroy}]} {
	    puts "+++ Problem destroying $obj";
	}
    }

    foreach obj $Objects(cmt) {
	puts "Destroying $obj";
	if {$Prefs(kill)} {
	    $obj config -remote_close_command {exit};
	}
	set d [catch {$obj destroy}];
	if {$d} {
	    puts "+++ Problem destroying $obj";
	}
    }

    PutDisp "CMPLAYER 3.0"

    set Objects(cmt) {};
    set Objects(spkr) {};
    set Objects(view) {};

    unset cmtObjs;
    unset localObjs;

    set cmtObjs(NULL) {};
    set localObjs(NULL) {};

    set gMaxTime 0;
    set gMinTime 10000000;

    .s.play configure -bitmap @$bitDir/play;
    .s.time set 0;
    .2.time set 0;
    .3.speed set 0;

    update;

    global  currentScript  Tags

    set currentScript {}
    set Tags(title) "Untitled"

    Extern_SetActionCmd {}
}


############### Toplevel Frames

wm title . "Cmplayer 3.0"

frame .border  -relief raised -bd 2 -bg turquoise4
frame .border.main -relief sunken -bd 2 -bg turquoise4
label .title -bd 2 -relief raised -text "Cmplayer 3.0" -bg turquoise4 \
    -fg bisque1 -anchor c

canvas .movie -bg black -width 380 -height 270
set videoWin .movie

frame .menu  -borderwidth 2 -relief raised
frame .status -borderwidth 2 -relief raised
frame .ctrl -borderwidth 2 -relief raised
frame .s
frame .k
frame .sep -borderwidth 1 -height 3 -relief sunken;
frame .sep2 -borderwidth 1 -height 3 -relief sunken;

frame .1
frame .2
frame .3
frame .4
frame .5

wm minsize . 0 0

######## Status

label .status.label -text "CMPLAYER 3.0" -font $statusFont;
pack .status.label -fill both -expand yes;


######## About Box

set aboutText {

              Cmplayer 3.0

        for more information mail:
      cmt@bugs-bunny.cs.berkeley.edu

}

toplevel .about
wm withdraw .about
wm minsize .about 0 0
wm title .about "About Cmplayer"

frame .about.frame -relief raised -bd 2
text .about.text -yscrollcommand ".about.scroll set" -relief sunken \
    -font $Scheme(courier_font)   -setgrid 1  -width 0 -height 0
scrollbar .about.scroll -orient vert -relief sunken \
	  -command ".about.text yview";
button .about.close -text Close -command {wm withdraw .about} \
    -pady 3 -padx 3;

pack .about.text -expand yes -fill both -side left -in .about.frame -padx 3 -pady 5
pack .about.scroll -fill both -side left -in .about.frame -padx 3 -pady 5
pack .about.frame -in .about -side top -ipady 20 -ipadx 20 \
    -expand yes -fill both
pack .about.close -side top -pady 5 -padx 5;

.about.text insert 0.0 $aboutText

######## MENU

menubutton .menu.file -menu .menu.file.m -underline 0 -text File
menubutton .menu.options -menu .menu.options.m -underline 0 -text Options
menubutton .menu.help -menu .menu.help.m -underline 0 -text Help

menu .menu.file.m 
menu .menu.options.m 
menu .menu.help.m 

tk_menuBar .menu .menu.file .menu.options .menu.help;

menu .menu.options.m.au
menu .menu.options.m.mjpeg
menu .menu.options.m.mpeg


.menu.file.m add command -label Open... -under 0 -command FileSelectBox;
.menu.file.m add separator;


.menu.file.m add command -label Exit -under 1 -command {exitPlayer}
.menu.options.m add cascade -label "Audio Device" -under 0 \
    -menu .menu.options.m.au
.menu.options.m add cascade -label "Mjpeg Device" -under 1 \
    -menu .menu.options.m.mjpeg
.menu.options.m add cascade -label "Mpeg Device" -under 0 -menu \
    .menu.options.m.mpeg
# .menu.options.m add cascade -label "VQ Device" -under 0 -menu \
    .menu.options.m.vq
.menu.options.m add separator
.menu.options.m add checkbutton -label "Audio On" -variable Prefs(audioOn)
.menu.options.m add checkbutton -label "Movie Resize" \
    -variable Prefs(squeeze)  -command {
	foreach view $Objects(view) {
	    $view config -squeeze $Prefs(squeeze);
	    $view ready;
	    $view refresh;
	    update;
	}
    }
.menu.options.m add checkbutton -label "Movie Loop" -variable Prefs(loop)
.menu.options.m add separator
.menu.options.m add checkbutton -label "Status Bar" -variable Prefs(status) \
    -command {
	if {$Prefs(status)} {
	    pack .status -after .movie -fill both;
	} else {
	    pack forget .status;
	}
    }

.menu.help.m add command -label About... -under 0 -command {
    set x [expr [winfo rootx .] -50]
    set y [expr [winfo rooty .] +70]
    wm geometry .about 37x15+$x+$y
    wm deiconify .about
}


pack .menu.file .menu.options -side left
pack .menu.help -side right
eval pack [pack slaves .menu] -padx $menuX



####### CONTROLS (kiosk)


if {$gKiosk} {
    button .k.exit -text Exit -command {exitPlayer} 
    checkbutton .k.mute -text Mute -variable Prefs(mute) -onvalue 1 -offvalue 0 -relief flat -command mute
    pack .k.mute .k.exit -side left -expand yes -fill y
    eval pack config [pack slaves .k] -pady $ctrlY -padx $ctrlX;
    pack config .k.exit .k.mute -ipady $vcrIY -ipadx $vcrIX;
}

######## CONTROLS (simple)

button .s.play -bitmap @$bitDir/play -command play;
label .s.from -text 0
tscale .s.time -from 0 -to 0 -orient horiz -command {setLts } -resolution .01 -digits 1
label .s.to -text 0
checkbutton .s.mute -text Mute -variable Prefs(mute) -onvalue 1 -offvalue 0 -relief flat -command mute
button .s.more -command more -bitmap @$bitDir/more

pack .s.play -side left -fill both  -ipady $vcrIY -ipadx $vcrIX;
pack .s.from -side left -fill both;
pack .s.time -side left -expand yes -fill both;
pack .s.to -side left -fill both;

if {!$gKiosk} {
    pack .s.mute -side left -fill both;
    pack .s.more -side left -fill both  -ipady $vcrIY -ipadx $vcrIX;
}

bind .s.time <Button-1> {+ set ltsLock 0};
bind .s.time <Button1-Motion> {+ set ltsLock 0};
bind .s.time <ButtonPress-1> {+
    set gSpeed [lindex [$lts config -speed] 2];
    $lts config -speed 0;
}
bind .s.time <ButtonRelease-1> {+ $lts config -speed $gSpeed};

eval pack config [pack slaves .s] -pady $ctrlY -padx $ctrlX;
pack config .s.from .s.to -padx $labelX

########## CONTROLS 1

button .1.rrew -bitmap @$bitDir/rrew -command {setSpeed -3}
button .1.rew -bitmap @$bitDir/rplay -command {setSpeed -1}
button .1.steprew -bitmap @$bitDir/steprew -command {setSpeed 0; stepTime -.05}
button .1.stop -bitmap @$bitDir/stop  -command {setSpeed 0}
button .1.stepfwd -bitmap @$bitDir/stepfwd -command {setSpeed 0; stepTime .05;}
button .1.fwd -bitmap @$bitDir/play -command {setSpeed 1}
button .1.ffwd -bitmap @$bitDir/ffwd -command {setSpeed 3}
button .1.less -command more -bitmap @$bitDir/less

pack .1.rrew -side left -fill both -expand yes
pack .1.rew -side left -fill both -expand yes
pack .1.steprew -side left -fill both -expand yes
pack .1.stop -side left -fill both -expand yes
pack .1.stepfwd -side left -fill both -expand yes
pack .1.fwd -side left -fill both -expand yes
pack .1.ffwd -side left -fill both -expand yes
pack .1.less -side left -fill both

eval pack config [pack slaves .1] \
    -pady $vcrY -padx $vcrX -ipady $vcrIY -ipadx $vcrIX

############ CONTROLS 2

label .2.text -text "Time:" -anchor e -width 7;
label .2.from -text 0 -anchor e -width 2;
tscale .2.time -from 0 -to 0 -orient horiz -command {setLts } -resolution .01 -digits 1
label .2.to -text 0 -anchor w -width 2;

pack .2.text -side left -padx 1
pack .2.from -side left -padx 3
pack .2.time -side left -expand yes -fill both
pack .2.to -side left -padx 3

bind .2.time <Button-1> {+ set ltsLock 0};
bind .2.time <Button1-Motion> {+ set ltsLock 0};
bind .2.time <ButtonPress-1> {+
    set gSpeed [lindex [$lts config -speed] 2];
    $lts config -speed 0;
}
bind .2.time <ButtonRelease-1> {+ $lts config -speed $gSpeed};

eval pack config [pack slaves .2] -pady $ctrlY -padx $ctrlX;
pack config .2.from .2.to -padx $labelX

############ CONTROLS 3

label .3.text -text "Speed:" -anchor e -width 7;
label .3.from -text -5 -anchor e -width 2;
fscale .3.speed -from -5 -to 5 -orient horiz -command {$lts configure -speed } -resolution .1 -digits 2
label .3.to -text 5 -anchor w -width 2;

pack .3.text -side left -padx 1
pack .3.from -side left -padx 3
pack .3.speed -side left -expand yes -fill both
pack .3.to -side left -padx 3

eval pack config [pack slaves .3] -pady $ctrlY -padx $ctrlX;
pack config .3.from .3.to -padx $labelX

############ CONTROLS 4

label .4.text -text "Volume:" -anchor e -width 7;
label .4.from -text 0 -anchor e -width 2;
fscale .4.gain -from 0 -to 10 -digits 3 -resolution .1 -orient horiz -command {setGain }
label .4.to -text 10
checkbutton .4.mute -text Mute -variable Prefs(mute) -onvalue 1 -offvalue 0 -relief flat -command mute

pack .4.text .4.from .4.gain .4.to .4.mute -side left -fill y

if {$gKiosk} {
    pack forget .4.mute;
}

pack configure .4.text -padx 1
pack configure .4.gain -expand yes -fill both
pack configure .4.from -padx 3
pack configure .4.to -padx 3

eval pack config [pack slaves .4]  -pady $ctrlY -padx $ctrlX;
pack config .4.from .4.to -padx $labelX

#############


if {$gKiosk} {
    pack .border -in . -side top -expand yes -fill both;
    pack .border.main -in .border -expand yes -fill both -pady 7 -padx 7;
    pack .title -in .border.main -side top -fill both
    pack .movie -in .border.main -side top -expand yes -fill both
    pack .status .ctrl -in .border.main -side top -fill both
    pack .s -in .ctrl -side top -fill both -pady $frameY -padx $frameX
    pack .sep -in .ctrl -side top -fill both -pady $frameY -padx $frameX;
    pack .4 -in .ctrl -side top -fill both -pady $frameY -padx $frameX;
    pack .sep2 -in .ctrl -side top -fill both -pady $frameY -padx $frameX;
    pack .k -in .ctrl -side top -fill both -pady $frameY -padx $frameX;
} else {
    pack .menu  -side top -fill both
    pack .movie  -side top -expand yes -fill both
    pack .status .ctrl -side top -fill both
    pack .s -in .ctrl -side top -fill both -pady $frameY -padx $frameX
}

update;

##############################################################################


proc SetupScript {script} {
    global Tick currentScript Tags gMinTime gMaxTime

    set tags [lindex $script 0];
    set script [lrange $script 1 end]

    if {[CMApp_GetProp $tags title] != {}} {
	set Tags(title) [CMApp_GetProp $tags title]
    }

    PutDisp "$Tags(title)";

    set i 0;
    foreach track $script {
	foreach clip [lrange $track 1 end] {
	    incr i;
	    CMApp_ListSet {x x ls le} $clip
	    if {$ls < $gMinTime} {set gMinTime $ls};
	    if {$le < $gMinTime} {set gMinTime $le};
	    if {$ls > $gMaxTime} {set gMaxTime $ls};
	    if {$le > $gMaxTime} {set gMaxTime $le};
	}
    }
    set Tick(Size) $i;
    set Tick(Index) -1;
    LoadTicks;
 
    set trackNum 0
    foreach track $script {
	SetupTrack $track $trackNum
	incr trackNum
    }

    PutDisp "$Tags(title)";

}

proc SetupForPlay {{time {}}} {
    global Objects lts bitDir maincm
    global gMinTime gMaxTime Prefs;

    if {$time == {}} {set time $gMinTime}
    $lts config -speed 0 -value $time    

    foreach obj [concat $Objects(spkr) $Objects(view) \
		     [$maincm getobjs audio-obj] [$maincm getobjs video-obj]] {
	#puts "Readying: $obj";
	$obj ready;
    }

    if {$Prefs(mute) == 1} {
	foreach obj [$maincm getobjs audio-obj] {$obj unready};
	foreach obj $Objects(spkr) {$obj unready};
    }
    
    .s.play configure -bitmap @$bitDir/play;
    .s.time set $time
    .2.time set $time
    .3.speed set 0;
    .4.gain set $Prefs(gain);

    setLts [expr -$Prefs(sendAhead)];

    set maxInt [expr round($gMaxTime)];
    set minInt [expr round($gMinTime)];

    set digits [expr [string length $maxInt] + 1];

    set max [TimeValue $maxInt];
    set min [TimeValue $minInt];

    .s.time conf -from $gMinTime -to $gMaxTime -digits $digits;
    .2.time conf -from $gMinTime -to $gMaxTime -digits $digits;
    .s.from conf -text $min;
    .2.from conf -text $min;
    .s.to conf -text $max;
    .2.to conf -text $max;

    incr digits -1;
    .2.to conf -width [string length $max]
    .3.to conf -width [string length $max]
    .2.from conf -width [string length $min]
    .3.from conf -width [string length $min]
    update;
}




proc SetupTrack {track trackNum} {
    set track [lrange $track 1 end];	# Ignore type (audio, video, etc.)

    set clipNum 0
    foreach clip $track {
	puts "--> Track $trackNum";

	Extern_SetupClip $clip $trackNum $clipNum

	LoadTicks;
	update;
	incr clipNum
    }
}

proc SetupAuClip {track dest ss se ls le} {
    global cmtObjs localObjs 
    global Objects;
    global Prefs;
    global arch Device lts maincm lts;
    global childHost childPort;

    if {! $Prefs(audioOn)} {
	return
    }

    set host [lindex $dest 0];
    set port [lindex $dest 1];
    set file [lindex $dest 2];

    if {$host == $childHost && $port == $childPort} {
	puts "local au"
	SetupLocalAu $track $dest $ss $se $ls $le;
	return;
    }

    ### Create CMT connection ###

    if {[catch {set cmtObjs($host,$port)}] == 1} {
	set cm [$maincm create cmt];

	$cm open -host $host $port;
	set rlts [$cm create lts];
	$lts addSlave $rlts;

	set cmtObjs($host,$port) [list $cm $rlts];
	lappend Objects(cmt) $cm;

	puts "Creating: $cm and $rlts";
    } else {
	set cm [lindex $cmtObjs($host,$port) 0];
	set rlts [lindex $cmtObjs($host,$port) 1];
    }	

    ### Create LOCAL Objects ###
   
    if {[catch {set localObjs(au)}] == 1} {
	set pd [$maincm create pktDest];
	set spkr [$maincm create auPlay];

	$pd config -outCmd @$spkr.accept;
	$spkr config -lts $lts -device $Device(au)

	set localObjs(au) $pd;
	lappend Objects(spkr) $spkr;

	puts "Creating: $spkr and $pd";
    } else {
	set pd $localObjs(au);
    }

    ### Create REMOTE Objects ###

    if {[$maincm getobjs $host:$port:$file-file] == {}} {
    	set ps [$cm create pktSrc];
  	set acf [$cm create auFile];
    	$ps config -dest [$pd address];
    	$acf config -outCmd @$ps.accept;
    	$acf config -filename $file -lts $rlts -sendAhead $Prefs(sendAhead) \
		 -cycleTime $Prefs(cycleTime);


	puts "Creating: $acf and $ps";
	$acf addtag audio-obj
	foreach I "$acf $ps" {$I addtag $host:$port:$file}
	$acf addtag $host:$port:$file-file
    } else {
	set acf [$maincm getobjs $host:$port:$file-file]
    }
    
    set seg [$acf addSegment $ss $se -start $ls -end $le;]
    L_RegisterSegment $cm $file $acf $seg $track
}


proc SetupMjpegClip {track dest ss se ls le} {
    global cmtObjs localObjs Objects;
    global Prefs;
    global arch videoWin Device lts maincm lts;
    global childHost childPort;

    set host [lindex $dest 0];
    set port [lindex $dest 1];
    set file [lindex $dest 2];

    if { "$host" == "" || "$port" == "" || "$file" == "" } {
	error "Missing either host, port, or file in address.";
    }
    
    if {$host == $childHost && $port == $childPort} {
	puts "local mjpeg"
	SetupLocalMjpeg $track $dest $ss $se $ls $le;
	return;
    }

    ### Create CMT connection ###

    if {![info exists cmtObjs($host,$port)]} {
	set cm [$maincm create cmt];

	$cm open -host $host $port;
	set rlts [$cm create lts];
	$lts addSlave $rlts;

	set cmtObjs($host,$port) [list $cm $rlts];
	lappend Objects(cmt) $cm;

	puts "Creating: $cm and $rlts";
    } else {
	set cm [lindex $cmtObjs($host,$port) 0];
	set rlts [lindex $cmtObjs($host,$port) 1];
    }	

    ### Create LOCAL Objects ###

    if {[catch {set localObjs(mjpeg)}] == 1} {
	set pd [$maincm create pktDest];
	set view [$maincm create mjpegPlay];

#	if {$arch == "sun4"} {
#	    $pd config -shared 1 -attach 1;
#	}
	$view config -lts $lts -xid [winfo id $videoWin] \
	    -device $Device(mjpeg) -squeeze 0 -xpos -1 -ypos -1;
	$pd config -outCmd @$view.accept;

	bind $videoWin <Expose> "update; $view refresh"
        bind $videoWin <Configure> "+$view config -height %h -width %w; \
                                    $view ready; \
                                    $view refresh"

	set localObjs(mjpeg) [list $view $pd];
	lappend Objects(view) $view;

	puts "Creating: $view and $pd";
    } else {
	set view [lindex $localObjs(mjpeg) 0];
	set pd [lindex $localObjs(mjpeg) 1];
    }

    ### Create REMOTE Objects ###

    if {[$maincm getobjs $host:$port:$file-file] == {}} {
	set ps [$cm create pktSrc];
	set vcf [$cm create mjpegFile];

    	$ps config -dest [$pd address];
    	$vcf config -outCmd @$ps.accept;
    	$vcf config -filename $file -lts $rlts -sendAhead $Prefs(sendAhead) \
	    -cycleTime $Prefs(cycleTime) -maxFrameRate 25;
	puts "Creating: $vcf and $ps";

	$vcf addtag video-obj
	foreach I "$vcf $ps" {$I addtag $host:$port:$file}
	$vcf addtag $host:$port:$file-file
    } else {
	set vcf [$maincm getobjs $host:$port:$file-file]
    }

    set seg [$vcf addSegment $ss $se -start $ls -end $le]
    L_RegisterSegment $cm $file $vcf $seg $track

    set w [expr [lindex [$vcf info] 3] + 60];
    set h [expr [lindex [$vcf info] 4] + 30];

    $videoWin config  -width $w -height $h;
    $view config -width $w -height $h;
    update;

}


proc SetupMpegClip {track dest ss se ls le} {
    global cmtObjs localObjs Objects;
    global Prefs;
    global arch videoWin Device lts maincm lts;
    global childHost childPort;

    set host [lindex $dest 0];
    set port [lindex $dest 1];
    set file [lindex $dest 2];


    if { "$host" == "" || "$port" == "" || "$file" == "" } {
	error "Missing either host, port, or file in address.";
    }
    
    if {$host == $childHost && $port == $childPort} {
	puts "local mpeg"
	SetupLocalMpeg $track $dest $ss $se $ls $le;
	return;
    }

    ### Create CMT connection ###

    if {[catch {set cmtObjs($host,$port)}] == 1} {
	set cm [$maincm create cmt];

	$cm open -host $host $port;
	set rlts [$cm create lts];
	$lts addSlave $rlts;

	set cmtObjs($host,$port) [list $cm $rlts];
	lappend Objects(cmt) $cm;
	puts "Creating: $cm and $rlts";
    } else {
	set cm [lindex $cmtObjs($host,$port) 0];
	set rlts [lindex $cmtObjs($host,$port) 1];
    }	

    ### Create LOCAL Objects ###
    
    if {[catch {set localObjs(mpeg)}] == 1} {
	set ipd [$maincm create pktDest];
	set ppd [$maincm create pktDest];
	set bpd [$maincm create pktDest];
	set view [$maincm create mpegPlay];

	$view config -lts $lts -xid [winfo id $videoWin] \
	    -device $Device(mpeg) -xpos -1 -ypos -1

	$ipd config -outCmd @$view.accept;
	$ppd config -outCmd @$view.accept;
	$bpd config -outCmd @$view.accept;

	set localObjs(mpeg) [list $view $ipd $ppd $bpd];

	bind $videoWin <Expose> "+update; $view refresh"
	bind $videoWin <Configure> "+$view config -height %h -width %w; \
                                    update; $view ready; $view refresh"

	lappend Objects(view) $view;
	puts "Creating: $view and $ipd $ppd $bpd";
    } else {
	set view [lindex $localObjs(mpeg) 0];
	set ipd [lindex $localObjs(mpeg) 1];
	set ppd [lindex $localObjs(mpeg) 2];
	set bpd [lindex $localObjs(mpeg) 3];
    }

    ### Create REMOTE Objects ###

    if {[$maincm getobjs $host:$port:$file-file] == {}} {

	set priority [$cm create mpegPriority];
    	set ips [$cm create pktSrc];
    	set pps [$cm create pktSrc];
    	set bps [$cm create pktSrc];
    	set vcf [$cm create mpegFile];

    	$vcf config -outCmd @$priority.accept;
	$priority config -i_outCmd @$ips.accept -cycleTime $Prefs(cycleTime);
	$priority config -p_outCmd @$pps.accept;
	$priority config -b_outCmd @$bps.accept;
	$priority config -outCmd @$bps.accept;

    	$ips config -dest [$ipd address];
    	$pps config -dest [$ppd address];
    	$bps config -dest [$bpd address];

    	$vcf config -filename $file -lts $rlts -sendAhead $Prefs(sendAhead) \
		-cycleTime 0;

	foreach I "$vcf $priority" {$I addtag video-obj}
	puts "Creating: $vcf and $priority";

	foreach I "$vcf $priority $ips $pps $bps" {$I addtag $host:$port:$file}
	$vcf addtag $host:$port:$file-file
    } else {
	set vcf [$maincm getobjs $host:$port:$file-file]
    }

    set seg [$vcf addSegment $ss $se -start $ls -end $le]
    L_RegisterSegment $cm $file $vcf $seg $track

    set w [expr [lindex [$vcf info] 3] + 60];
    set h [expr [lindex [$vcf info] 4] + 30];

    $videoWin config  -width $w -height $h;
    $view config -width $w -height $h;
    update;
}


# This function initializes the device lists based on the hardware of the
# current host and changes the option menu accordingly.

proc SetupDevices {} {
    global arch;
    global Device;

    set name(af) AudioFile;
    set name(software) Software;
    set name(parallax) Parallax;
    set name(j300) J300;
    set name(sparc) {Sparc Audio};
    set name(vqip) {Infopad VQ};

#    set list(au) spkr
#    set list(mjpeg) view
#    set list(mpeg) view
    
    foreach type {au mpeg mjpeg} {
	
	if {[info command ${type}Play] == ""} {
	    continue;
	}

	set tempObj [${type}Play ""];
	set devList [$tempObj deviceList];

	

	foreach dev $devList {
	    .menu.options.m.$type add radiobutton -label $name($dev) \
		-variable Device($type) -value $dev;
	}
	set Device($type) [lindex [$tempObj config -device] 2];
	$tempObj destroy;
    }
}

proc SetupVqClip {track dest ss se ls le} {
    global cmtObjs localObjs Objects;
    global Prefs;
    global arch videoWin Device lts maincm lts;
    global childHost childPort, env

    catch {set env(PADID)} padid;

    set host [lindex $dest 0];
    set port [lindex $dest 1];
    set file [lindex $dest 2];


    if { "$host" == "" || "$port" == "" || "$file" == "" } {
	error "Missing either host, port, or file in address.";
    }
    
    if {$host == $childHost && $port == $childPort} {
	puts "local vq"
	SetupLocalvq $track $dest $ss $se $ls $le;
	return;
    }

    ### Create CMT connection ###

    if {[catch {set cmtObjs($host,$port)}] == 1} {
	set cm [$maincm create cmt];

	$cm open -host $host $port;
	set rlts [$cm create lts];
	$lts addSlave $rlts;

	set cmtObjs($host,$port) [list $cm $rlts];
	lappend Objects(cmt) $cm;

	puts "Creating: $cm and $rlts";
    } else {
	set cm [lindex $cmtObjs($host,$port) 0];
	set rlts [lindex $cmtObjs($host,$port) 1];
    }	

    ### Create LOCAL Objects ###

    if {[catch {set localObjs(vq)}] == 1} {
	set pd [$maincm create pktDest];
	set cbpd [$maincm create pktDest];
	set view [$maincm create vqPlay];

	$view config -lts $lts -xid $padid -device $Device(vq)
	$pd config -outCmd @$view.accept;
	$cbpd config -outCmd @$view.acceptCodebook;

#	bind $videoWin <Expose> "update; $view refresh"
#        bind $videoWin <Configure> "+$view config -height %h -width %w; \
#                                    $view ready; \
#                                    $view refresh"

	set localObjs(vq) [list $view $pd $cbpd];
	lappend Objects(view) $view;

	puts "Creating: $view, $pd, and $cbpd";
    } else {
	set view [lindex $localObjs(vq) 0];
	set pd [lindex $localObjs(vq) 1];
	set cbpd [lindex $localObjs(vq) 2];
    }

    ### Create REMOTE Objects ###

    if {[$maincm getobjs $host:$port:$file-file] == {}} {
	set ps [$cm create pktSrc];
	set cbps [$cm create pktSrc];
	set vcf [$cm create vqFile];

    	$ps config -dest [$pd address];
	$cbps config -dest [$cbpd address];

    	$vcf config -outCmd @$ps.accept;
	$vcf config -codebookOutCmd @$cbps.accept;

    	$vcf config -filename $file -lts $rlts -sendAhead $Prefs(sendAhead) \
	    -cycleTime $Prefs(cycleTime); 
	# -maxFrameRate 25;
	$vcf addtag video-obj

	puts "Creating: $vcf and $ps";

	foreach I "$vcf $ps $cbps" {$I addtag $host:$port:$file}
	$vcf addtag $host:$port:$file-file
    } else {
	set vcf [$maincm getobjs $host:$port:$file-file]
    }

    set seg [$vcf addSegment $ss $se -start $ls -end $le]
    L_RegisterSegment $cm $file $vcf $seg $track

#     set w [expr [lindex [$vcf info] 3] + 60];
#     set h [expr [lindex [$vcf info] 4] + 30];
#     $videoWin config  -width $w -height $h;
#     $view config -width $w -height $h;

    update;

}

##################### LOCAL OBJECTS ##################

proc SetupLocalAu {track dest ss se ls le} {
    global cmtObjs localObjs Objects;
    global Prefs;
    global arch Device lts maincm lts;
    global childHost childPort;

    set host [lindex $dest 0];
    set port [lindex $dest 1];
    set file [lindex $dest 2];


    
    ### Create PLAY Objects ###
   
    if {[catch {set localObjs(au)}] == 1} {
	set spkr [$maincm create auPlay];

	$spkr config -lts $lts -device $Device(au)
	set localObjs(au) $spkr;
	lappend Objects(spkr) $spkr;
	puts "Creating: $spkr";
    } else {
	set spkr $localObjs(au);
    }

    ### Create FILE Objects ###

    if {[$maincm getobjs $host:$port:$file-file] == {}} {
	set acf [$maincm create auFile];
	
	$acf config -outCmd @$spkr.accept;
	$acf config -filename $file -lts $lts -sendAhead $Prefs(sendAhead) \
	    -cycleTime $Prefs(cycleTime);
	
	puts "Creating: $acf";
	foreach I "$acf" {$I addtag $host:$port:$file}
	$acf addtag $host:$port:$file-file
	$acf addtag audio-obj
    } else {
	set acf [$maincm getobjs $host:$port:$file-file]
    }

    set seg [$acf addSegment $ss $se -start $ls -end $le]
    L_RegisterSegment $maincm $file $acf $seg $track
}

proc SetupLocalMjpeg {track dest ss se ls le} {
    global cmtObjs localObjs Objects;
    global Prefs;
    global arch Device lts maincm lts videoWin;
    global childHost childPort;

    set host [lindex $dest 0];
    set port [lindex $dest 1];
    set file [lindex $dest 2];


    
    ### Create PLAY Objects ###
   
    if {[catch {set localObjs(mjpeg)}] == 1} {
	set view [$maincm create mjpegPlay];

#	if {$arch == "sun4"} {
#	    $pd config -shared 1 -attach 1
#	}

	$view config -lts $lts -xid [winfo id $videoWin] \
	    -device $Device(mjpeg) -squeeze 0 -xpos -1 -ypos -1;

	bind $videoWin <Expose> "update; $view refresh"
        bind $videoWin <Configure> "+$view config -height %h -width %w; \
                                    $view ready; \
                                    $view refresh"

	set localObjs(mjpeg) $view;

	lappend Objects(view) $view;
	puts "Creating: $view";
    } else {
	set view [lindex $localObjs(mjpeg) 0];
    }

    ### Create FILE Objects ###

    if {[$maincm getobjs $host:$port:$file-file] == {}} {
	set vcf [$maincm create mjpegFile];
	
	$vcf config -outCmd @$view.accept;
	$vcf config -filename $file -lts $lts -sendAhead $Prefs(sendAhead) \
	    -cycleTime $Prefs(cycleTime) -maxFrameRate 25;
	
	$vcf addtag video-obj
	puts "Creating: $vcf";

	foreach I "$vcf" {$I addtag $host:$port:$file}
	$vcf addtag $host:$port:$file-file
    } else {
	set vcf [$maincm getobjs $host:$port:$file-file]
    }

    set seg [$vcf addSegment $ss $se -start $ls -end $le]
    L_RegisterSegment $maincm $file $vcf $seg $track
 
    set w [expr [lindex [$vcf info] 3] + 60];
    set h [expr [lindex [$vcf info] 4] + 30];

    $videoWin config  -width $w -height $h;
    $view config -width $w -height $h;
    update;
}

proc SetupLocalMpeg {track dest ss se ls le} {
    global cmtObjs localObjs Objects;
    global Prefs;
    global arch Device lts maincm lts videoWin;
    global childHost childPort;

    set host [lindex $dest 0];
    set port [lindex $dest 1];
    set file [lindex $dest 2];


    
    ### Create PLAY Objects ###
   
    if {[catch {set localObjs(mpeg)}] == 1} {
	set view [$maincm create mpegPlay];

	$view config -lts $lts -xid [winfo id $videoWin] \
	    -device $Device(mpeg) -xpos -1 -ypos -1;

	bind $videoWin <Expose> "update; $view refresh"
        bind $videoWin <Configure> "+$view config -height %h -width %w; \
                                    update; $view ready; $view refresh"

	set localObjs(mpeg) $view;

	lappend Objects(view) $view;
	puts "Creating: $view";
    } else {
	set view [lindex $localObjs(mpeg) 0];
    }

    ### Create FILE Objects ###

    if {[$maincm getobjs $host:$port:$file-file] == {}} {
	set vcf [$maincm create mpegFile];
	
	$vcf config -outCmd @$view.accept;
	$vcf config -filename $file -lts $lts -sendAhead $Prefs(sendAhead) \
	    -cycleTime $Prefs(cycleTime) -maxFrameRate 25;
	
	$vcf addtag video-obj
	puts "Creating: $vcf";

	foreach I "$vcf" {$I addtag $host:$port:$file}
	$vcf addtag $host:$port:$file-file
    } else {
	set vcf [$maincm getobjs $host:$port:$file-file]
    }

    set seg [$vcf addSegment $ss $se -start $ls -end $le]
    L_RegisterSegment $maincm $file $vcf $seg $track
 
    set w [expr [lindex [$vcf info] 3] + 60];
    set h [expr [lindex [$vcf info] 4] + 30];

    $videoWin config  -width $w -height $h;
    $view config -width $w -height $h;
    update;
}

proc SetupLocalVq {track dest ss se ls le} {
    global cmtObjs localObjs Objects;
    global Prefs;
    global arch Device lts maincm lts videoWin;
    global childHost childPort, env;

    set host [lindex $dest 0];
    set port [lindex $dest 1];
    set file [lindex $dest 2];


    catch {set env(PADID)} padid;

    ### Create PLAY Objects ###
   
    if {[catch {set localObjs(vq)}] == 1} {
	set view [$maincm create vqPlay];

	$view config -lts $lts -xid $padid \
	    -device $Device(vq) -squeeze 0 -xpos -1 -ypos -1;

# 	bind $videoWin <Expose> "update; $view refresh"
#         bind $videoWin <Configure> "+$view config -height %h -width %w; $view ready; $view refresh"

	set localObjs(vq) $view;
	
	lappend Objects(view) $view;
	puts "Creating: $view";
    } else {
	set view [lindex $localObjs(vq) 0];
    }

    ### Create FILE Objects ###

    if {[$maincm getobjs $host:$port:$file-file] == {}} {
	set vcf [$maincm create vqFile];
	
	$vcf config -outCmd @$view.accept;
	$vcf config -codebookOutCmd @$view.acceptCodebook;
	$vcf config -filename $file -lts $lts -sendAhead $Prefs(sendAhead) \
	    -cycleTime $Prefs(cycleTime);
	# -maxFrameRate 25;
	
	$vcf addtag video-obj
	puts "Creating: $vcf"

	foreach I "$vcf" {$I addtag $host:$port:$file}
	$vcf addtag $host:$port:$file-file
    } else {
	set vcf [$maincm getobjs $host:$port:$file-file]
    }

    set seg [$vcf addSegment $ss $se -start $ls -end $le]
    L_RegisterSegment $maincm $file $vcf $seg $track
 
#     set w [expr [lindex [$vcf info] 3] + 60];
#     set h [expr [lindex [$vcf info] 4] + 30];
#     $videoWin config  -width $w -height $h;
#     $view config -width $w -height $h;

    update;
}


###########################################################3

proc FileSelectBox {} {
    toplevel .fileSelectBox
    wm title .fileSelectBox "Open File";

    fsbox .fileSelectBox.fsbox [list [list "CMT Scripts\t" *.script,*.cMovie] \
			    [list "Mpeg Movies\t" *.mpg,*.mpeg,*.mpgc] \
			    [list "Sun Audio\t" *.au,*.snd,*.acf] \
			    [list "All Files\t" *] ];
    
    pack .fileSelectBox.fsbox -pady 5 -padx 5 -in .fileSelectBox \
	-expand yes -fill both;
    wm geometry  .fileSelectBox 25x8;
    update;
    
}

proc FsboxExit {} {
    pack forget .fsbox;
    destroy .fileSelectBox;
    update;
}

proc FsboxOpenFile {file} {
    global currentScript;
    global Tags;

    FsboxExit;
    resetPlayer;

    set currentScript $file;
    set Tags(title) "Untitled"

    PutDisp "Retrieving script"

    if { [catch {CMApp_ReadURL $file} script] == 0 } {
	set script [CMApp_ResolveMovie $script];
	SetupScript $script;
	SetupForPlay
    } else {
	tkerror "Can't play $file.\nNot a valid filename or URL";
    }
}

#-------------------------------------------------------------

proc LoadTicks {} {
    global Tick;

    for {set i 0} {$i < $Tick(Size)} {incr i} {
	if {$i <= $Tick(Index)} {
	    lappend string +;
	} else {
	    lappend string -;
	}
    }
    set string [join $string {}];
    PutDisp "Initializing remote objects: $string";
    incr Tick(Index);
}





##########################################################################


proc TimeValue {t} {
    set abs [expr abs($t)];
    set hours [expr int($abs / 3600)];
    set minutes [expr (int($abs / 60)) % 60]
    set seconds [expr $abs - $hours*3600 - $minutes*60];

    set retval {};

    if {$hours > 0} {
	lappend retval $hours:
	if {$minutes < 10} {set minutes 0$minutes};
	lappend retval $minutes:
	if {$seconds < 10} {set seconds 0$seconds};
	lappend retval $seconds
    } elseif {$minutes > 0} {
	lappend retval $minutes:
	if {$seconds < 10} {set seconds 0$seconds};
	lappend retval $seconds
    } else {	
	lappend retval $seconds;
    }

    return [join $retval {}];
}

#-------------------------------------------------------------
# Setup Connection to Child Process 

proc ProcessCommandLine {} {
    global  gTempPort currentScript binDir libDir lts;
    global  Prefs pid gFile childPort
    set lts [lts ""];

    set childPort {}

    every 1000 updateTimer;
    set gTempPort [dp_MakeRPCServer];
    set pid [exec $binDir/cmwish -f $libDir/edit_child.tcl $gTempPort &];
    puts "Child process: $pid"

    if {$childPort == {}} {
	PutDisp "Connecting to server"
	tkwait variable childPort
	PutDisp "Connection Done"
    }
    
    if {$gFile != {}} {
	
	PutDisp "Retrieving script"

	if { [catch {CMApp_ReadURL $gFile} script] == 0 } {
	    set script [CMApp_ResolveMovie $script];
	    SetupScript $script;
	    SetupForPlay
	} else {
	    tkerror "Can't play $gFile.\nNot a valid filename or URL";
	}
    }
}

#-------------------------------------------------------------

proc MakeConnection {port} {
    global maincm lts gTempPort rpcFile childPort childHost

    set childHost [exec hostname]
    $maincm destroy;
    set maincm [cmt ""];
    $maincm open -host $childHost $port;
    CMApp_SetLocalHost $childHost $port;
    puts "Made connection: port $port";
    $maincm config -remote_lost_connection_command {exit};
    $maincm config -remote_close_command {exit};
    $maincm config -local_lost_connection_command {exit};
    $lts destroy;
    set lts [$maincm create lts];
    update;

    # do this set last, because the connection script waits on it!!
    set childPort $port;
}

#-------------------------------------------------------------

proc PutDisp {string {append 0}} {
    if {$append} {
	set oldString [lindex [.status.label conf -text] 4];
	set string [concat $oldString $string];
    }

    .status.label conf -text $string;
    update;
}

#-------------------------------------------------------------
# Returns the current script name

proc Extern_ScriptName {} {
    global  gScriptName
    return $gScriptName
}

#-------------------------------------------------------------
# Called via rpc - Added for editor interface, eric14
# Makes the given script the cmplayers current script.  Anytime the
#     play time is changed action is invoked (with all %t's replaced
#     by the new time).  Whenever a file segment is created (one per
#     clip in the script) register will be called with the parameters
#     {trackNum clipNum obj segID}

proc Extern_NewScript {script name action register} {
    global   gRegisterScript gScriptName
    resetPlayer;
    set gScriptName $name

    Extern_SetActionCmd $action
    set gRegisterScript $register

    SetupScript $script
    SetupForPlay
 puts "@@@Extern_NewScript done"
}

#-------------------------------------------------------------
# Called via rpc - Added for editor interface, eric14

proc Extern_NewFile {file {action {}}} {
    global currentScript;

    resetPlayer;
    Extern_SetActionCmd $action

    set currentScript $file;

    PutDisp "Retrieving script"

    if { [catch {CMApp_ReadURL $file} script] == 0 } {
	set script [CMApp_ResolveMovie $script];
	SetupScript $script;
	SetupForPlay
    } else {
	tkerror "Can't play $file.\nNot a valid filename or URL";
    }
}


#-------------------------------------------------------------
# Called via rpc - Added for editor interface, eric14
# Stores cmd and calls it periodically when playing or whenever the
#     slider is moved.  Used to update the current selection in the
#     editor.  Any %t's appearing in cmd are replaced by the lts's
#     current time.

proc Extern_SetActionCmd {cmd} {
    global  gActionCmd
    
    set gActionCmd $cmd
    set gActionTime -12345
}

#-------------------------------------------------------------
# Called when each clipfile segment is created.  Used to give the
#     editor info needed to modify the editlist.  Index is a list
#     containing the track and clip number of the given segment.

proc L_RegisterSegment {cmt file obj segID index} {
    global  gRegisterScript

    if {$gRegisterScript != {}} {
	eval "$gRegisterScript $cmt $file $obj $index $segID"
    }
}

#-------------------------------------------------------------
# Called to prepare a clip for playing

proc Extern_SetupClip {clip trackNum clipNum} {
    CMApp_ListSet {ss se ls le file type tags dest} $clip
    set idx [list $trackNum $clipNum]

    puts "Extern_SetupClip: $trackNum $clipNum $clip"
    switch -glob $type {
	*/au {SetupAuClip $idx $dest $ss $se $ls $le}
	*/mjpeg      { SetupMjpegClip $idx $dest $ss $se $ls $le; }
	*/mpeg       { SetupMpegClip $idx $dest $ss $se $ls $le; }
	*/vq       { SetupVqClip $idx $dest $ss $se $ls $le; }
	default    { error "Unsupported type: $type"; }
    }
}

#-------------------------------------------------------------
# Called to prepare a clip for playing

proc Extern_RemoveClip {cmt obj host port file segID} {
    PutDisp "Reconfiguring Script - Delete"
    puts "Extern_RemoveClip $cmt $obj $file $segID"
    $obj removeSegment $segID

    if {[llength $obj segmentList] == 0} {
	foreach I [$maincm getobjs $host:$port:$file] {
	    puts "Destroying $I"
	
	    if {[catch {$I destroy}]} {
		puts "+++ Problem destroying $I"
	    }
	}
    }
}

#-------------------------------------------------------------
# Called to prepare a clip for playing

proc Extern_ConfigClip {cmt obj ls le ss se segID} {
    PutDisp "Reconfiguring Script - Shift"
    puts "Extern_ConfigClip $cmt $obj $ls $le $ss $se $segID"
    $obj configureSegment $segID $ss $se -start $ls -end $le
}

#-------------------------------------------------------------
# Called to prepare a clip for playing

proc Extern_Restart {start end time} {
    global  gMinTime gMaxTime lts

    puts "\# Restart $start $end $time"
    set gMinTime $start
    set gMaxTime $end
    SetupForPlay $time
    play 1
}

#-------------------------------------------------------------

##### Load source libraries #####

source $libDir/fsbox.tcl;

SetupDevices;
update;

##### Load user defaults from ~/.cmplayerrc #####

if {[file exists ~/.cmplayerrc]} {
    if {[catch {source ~/.cmplayerrc} out]} {
	puts "\# ERROR in ~/.cmplayerrc:\n\#\t$out"
    }
}


ProcessCommandLine
set gSpeed [lindex [$lts config -speed] 2]

# call this last!!!
# Indicates that the player is ready to recieve external commands.

CMApp_SlaveReady
