proc slistbox {path} {
    frame $path;
    set p $path;

    frame $p.top;
    frame $p.bot;

    listbox $p.lbox -relief sunken -setgrid 1 \
	-yscrollcommand "$p.vert set" -xscrollcommand "$p.horiz set";
    scrollbar $p.vert -orient vertical -relief sunken \
	-command "$p.lbox yview";
    scrollbar $p.horiz -orient horizontal -relief sunken \
	-command "$p.lbox xview";
    frame $p.nop -width 19 -height 19;

    pack $p.vert $p.lbox -in $p.top -fill both -side right;
    pack configure $p.lbox -expand yes;
    pack $p.horiz $p.nop -in $p.bot -fill both -side left;
    pack configure $p.horiz -expand yes;

    pack $p.bot $p.top -in $p -fill both -side bottom;
    pack configure $p.top -expand yes;
    return $path;
}


proc fsbox { path {flist {}} } {
    global fspath currentDir filter filterList; 

    # defaults
    
    set fspath $path;
    set currentDir [pwd];
    set filterList $flist;

    if {"$flist" == {}} {
	set filterList [list "All Files\t" *];
    }
    
    set filter [lindex $filterList 0];
    
    # toplevel frames
    
    frame $fspath;
    frame $fspath.boxes;
    frame $fspath.options;
    frame $fspath.sep -borderwidth 1 -height 2 -relief sunken;
    frame $fspath.buttons;
    
   ##### Boxes frame #####
    
    set f $fspath.boxes.files;
    set d $fspath.boxes.dirs;
    
    frame $f;
    frame $d;
    
    label $f.l -text Files: -anchor w;
    entry $f.e -relief sunken -width 1;
    slistbox $f.list;
    
    label $d.l -text Directories: -anchor w;
    label $d.e -text "$currentDir" -width 1 -anchor w;
    slistbox $d.list;
    
    pack $f.l $f.e $f.list -side top -pady 1 -fill both;
    pack configure $f.list -expand yes;
    
    pack $d.l $d.e $d.list -side top -pady 1 -fill both;
    pack configure $d.list -expand yes;
    
    pack $f $d -in $fspath.boxes -side left -expand yes -fill both -padx 10;
    
   ##### Options frame #####
    
    set o $fspath.options;
    
    menubutton $o.filter -relief groove -pady 3 -padx 3 -width 1\
	-menu $o.filter.m -text "    [Fdesc]" -anchor w;
    menu $o.filter.m;

#   label $o.hidden -pady 3 -padx 3 -width 1;
   
    pack $o.filter -expand yes -fill both -side left \
 	-in $o -padx 10;
  
    foreach fpair $filterList {
	set fdesc [lindex $fpair 0];
	set fspec [lindex $fpair 1];
	$o.filter.m add radiobutton -label "$fdesc   $fspec" \
	    -variable filter -value $fpair \
	    -command { 
		$fspath.options.filter configure -text "    [Fdesc]";
		fillBoxes $currentDir [Fspec]; 
	    }
    }
    
   ##### Buttons frame #####
    
    set b $fspath.buttons;
    
    frame $b.ok -borderwidth 1 -relief sunken;
    button $b.ok.b -text OK -pady 5 -padx 8 -command {
	set selFile [$fspath.boxes.files.e get];
	cd $currentDir;
	if {[file isfile $selFile]} {
	    FsboxOpenFile $selFile;
	} else {
	    $fspath.boxes.files.e delete 0 end;
	    $fspath.boxes.files.e insert 0 "Filename Not Found";
	    update;
	    after 500
	    $fspath.boxes.files.e delete 0 end;
	    $fspath.boxes.files.e insert 0 $selFile;
	}
    }            
    
    button $b.cancel -text Cancel -pady 5 -padx 8 -command FsboxExit;
    
    pack $b.ok.b -in $b.ok -pady 5 -padx 5;
    pack $b.ok $b.cancel -expand yes -in $b -side left;
    
    # Pack widgets
    
    pack $b $fspath.sep $o $fspath.boxes -side bottom -fill both \
	-pady 5 -padx 5 -in $fspath;
    pack configure $fspath.boxes -expand yes;
    
    # Make neccesary bindings
    
    focus $f.e;
    
    bind $f.e <Return> {keyEnter};
    bind $f.e <Tab> {keyTab};
    
    bind $f.list.lbox <1> {
	set selFile [%W get [%W index @%x,%y]]
	$fspath.boxes.files.e delete 0 end;
	$fspath.boxes.files.e insert 0 $selFile;
    };

    bind $f.list.lbox <B1-Motion> {
	set selFile [%W get [%W index @%x,%y]]
	$fspath.boxes.files.e delete 0 end;
	$fspath.boxes.files.e insert 0 $selFile;
    };
	
    bind $f.list.lbox <Double-1> {FsboxOpenFile $selFile; break};

    bind $d.list.lbox <Double-1> {
	set selDir [%W get [%W index @%x,%y]]
	changeDir $selDir;
	$fspath.boxes.files.e delete 0 end;
	fillBoxes $currentDir [Fspec];
    };
    
    fillBoxes $currentDir [Fspec];
}


# Takes a directory and a filter-spec, and returns a sorted list of the
# files that match the filter-spec in that directory.

proc GetFiles {dir fspec} {
    cd $dir;
    set fileList {};
    set fspec [split $fspec \;\,\:\ ];
    foreach filter $fspec {
	set fileList [concat $fileList [glob -nocomplain $filter]];
    }
    set fileList [lsort $fileList];
    return $fileList;
}


proc fillBoxes {dir fspec} {
    global fspath;
    
    set fileList {};
    set dirList {};
    
    set files [GetFiles $dir $fspec];
    set dirs [GetFiles $dir *];

    foreach f $files {
	if {[file isfile $f] && [file readable $f]} {
	    lappend fileList [file tail $f];
	}
    }
        
    foreach d $dirs {
	if {[file isdirectory $d] && [file readable $d]} {
	    lappend dirList [file tail $d]/;
	}
    }
    
    set dirList [concat ./ ../ $dirList];
    
    set flbox $fspath.boxes.files.list.lbox;
    set dlbox $fspath.boxes.dirs.list.lbox;
    
    $flbox delete 0 end;
    $dlbox delete 0 end;
    eval $flbox insert 0 $fileList;
    eval $dlbox insert 0 $dirList;
}

proc keyEnter {} {
    global filter currentDir openFileCommand fspath;
    
    set entry $fspath.boxes.files.e;
    set file [$entry get];
    
    # Check to see if we have a full pathname in $file
    
    if {[file dirname $file] != "."} {
	set path [file dirname $file];
	set fspec [file tail $file];
    } else {
	set path $currentDir;
	set fspec $file;
    }
    
    if {"$fspec" == ""} {
	set fspec .;
    }
    
    set size [llength [GetFiles $path $fspec]];

    if {[string first * $fspec] != -1 ||
	[string first \; $fspec] != -1 ||
	[string first , $fspec] != -1 ||
	[string first \  $fspec] != -1} {
	set filtering 1;
    } else {
	set filtering 0;
    }
	    
    $entry delete 0 end;

    if {$size == 0 && !$filtering} {
	$entry delete 0 end;
	$entry insert 0 "Filename Not Found.";
	update;
	after 500
	$entry delete 0 end;
	$entry insert 0 $file;
	fillBoxes $currentDir [Fspec];
	return;
    } elseif {$size > 1 || $filtering} {
	changeDir $path;
	newFilter $fspec;
	fillBoxes $currentDir [Fspec];
	return;
    } else {
	if {[file isdirectory $fspec]} {
	    changeDir $fspec;
	    fillBoxes $currentDir [Fspec];
	    return;
	} else {
	    FsboxOpenFile $fspec;
	    return;
	}
    }
}

proc keyTab {} {
    global currentDir fspath;
    
    set fentry [$fspath.boxes.files.e get];
    
    # used in case we try to access ~user when user does not exist.

    set noUser [catch {file dirname $fentry} path];
    if {$noUser} {
	return;
    }
    
    if {$path != "."} {
	set path [file dirname $fentry];
	set file [file tail $fentry];
    } else {
	set path $currentDir;
	set file $fentry;
    }
    
    # used in case they give a path that doesn't exist

    set noPath [catch {cd $path}]
    if {$noPath} {
	return;
    }

    set f [glob -nocomplain $file*];
    set size [llength $f];
    
    if {$size == 0} {
	return;
    } elseif {$size == 1} {
	$fspath.boxes.files.e delete 0 end;
	if {[file dirname $fentry] == "."} {
	    $fspath.boxes.files.e insert 0 $f;
	} else {
	    if {$path == "/"} {
		$fspath.boxes.files.e insert 0 /$f;
	    } else {
		$fspath.boxes.files.e insert 0 $path/$f;
	    }		
	}
	fillBoxes $path $file*;
	return;
    } else {
	fillBoxes $path $file*;
	return;
    }
}

proc newFilter {f} {
    global filter fspath;
    set filter [list "User Defined" $f];
    $fspath.options.filter configure -text "    [Fdesc]";
}

proc Fspec {} {
    global filter;
    return [lindex $filter 1];
}

proc Fdesc {} {
    global filter;
    return "[lindex $filter 0]   [lindex $filter 1]";
}

proc changeDir {newDir} {
    global currentDir fspath;

    cd $newDir;
    set currentDir [pwd];
    $fspath.boxes.dirs.e conf -text "$currentDir";
}


