###### Global Preferences and Config Options ###########

set Prefs(sendAhead) 1;		
set Prefs(cycleTime) .3;		       

set Prefs(mute) 0;			# If true, cmplayer is silent
set Prefs(file) {};			# File or URL of movie to play
set Prefs(gain) .3;			# Volume level

##############################


### Read File from commandline

set arg [lindex $argv 0];

if {$arg == ""} {
    puts stderr "No filename or URL specified on commandline"
    puts stderr "Usage: cmplay <filename | URL>"
    exit;
} else {
    set Prefs(file) $arg;
    set Title $arg;
}


###### GLOBAL VARIABLES

set minTime 999999
set maxTime -1

set objectList {}


###### GLOBAL OBJECTS

set lts [lts ""]
set speaker [auPlay ""]
set mpegScreen [mpegPlay ""]
set jpegScreen [mjpegPlay ""]

$speaker config -lts $lts -gain $Prefs(gain)



###### BUTTON COMMANDS

proc setSpeed {s} {
    global lts Prefs;

    set oldSpeed [lindex [$lts config -speed] 2];

    if {$oldSpeed == 0} {
	set oldValue [lindex [$lts config -value] 2];
	$lts config -speed $s -value [expr $oldValue - $s*$Prefs(sendAhead)];
    } else {
	$lts conf -speed $s;
    }
}


proc mute {} {
    global Prefs speaker;
    if {$Prefs(mute)} {
	$speaker unready;
    } else {
	$speaker ready;
    }
}


proc setTime {time} {
    global lts;

    $lts config -value $time;
}


##### SCRIPT INTITIALIZATION


proc InitializePlayer {} {
    global Prefs

    CMApp_SetLocalHost localhost 0;

    set script [CMApp_ReadURL $Prefs(file)];
    set script [CMApp_ResolveMovie $script];
    
    SetupScript $script;
    SetupForPlay;
}


proc SetupScript {script} {
    global Title

    set tags [lindex $script 0];
    set script [lrange $script 1 end]
					
    # Find title of movie
    foreach pair $tags {
	switch [lindex $pair 0] {
	    title { set Title [lindex $pair 1] }
	    default {}
	}
    }

    foreach track $script {
	SetupTrack $track;
    }
}


proc SetupTrack {track} {
    set track [lrange $track 1 end];	# Ignore type (audio, video, etc.)

    foreach clip $track {

	# Find host and type, start/
	CMApp_ListSet {ss se ls le file type tags dest} $clip

	switch -glob $type {
	    */au 	{ SetupAuClip $dest $ss $se $ls $le; }
	    */mjpeg	{ SetupMjpegClip $dest $ss $se $ls $le; }
	    */mpeg	{ SetupMpegClip $dest $ss $se $ls $le; }
	    default	{ error "Unsupported type: $type"; }
	}
    }
}


proc SetupForPlay {} {
    global lts speaker jpegScreen mpegScreen Title
    global minTime maxTime Prefs objectList

    $lts config -speed 0 -value 0;    
    
    # Ready audio objects and handle errors.

    foreach object $objectList {
	$object ready;
    }

    $jpegScreen ready;
    $mpegScreen ready;
    $speaker ready;

    .status.title config -text $Title

    setTime [expr -$Prefs(sendAhead)];
}


proc SetupAuClip {dest ss se ls le} {
    global Prefs arch Device lts speaker;
    global minTime maxTime objectList

    set host [lindex $dest 0];
    set port [lindex $dest 1];
    set file [lindex $dest 2];

    if {"$host" == "localhost" && $port == 0} {
	set local 1
    } else {
	set local 0
    }

    if {$ls < $minTime} {set minTime $ls};
    if {$le < $minTime} {set minTime $le};
    if {$ls > $maxTime} {set maxTime $ls};
    if {$le > $maxTime} {set maxTime $le};

    
    ### Create CMT connection ###

    if $local {
  	set acf [auFile ""];
	$acf config                       \
	    -outCmd @$speaker.accept      \
	    -filename $file               \
	    -lts $lts                     \
	    -sendAhead $Prefs(sendAhead)  \
	    -cycleTime $Prefs(cycleTime);
    } else {
	set cm [cmt ""];
	$cm open -host $host $port;
	
	set rlts [$cm create lts];
	$lts addSlave $rlts;

	set pd [pktDest ""];
	$pd config -outCmd @$speaker.accept;
		
    	set ps [$cm create pktSrc];
  	set acf [$cm create auFile];
    	$ps config -dest [$pd address];
	$acf config                       \
	    -outCmd @$ps.accept           \
	    -filename $file               \
	    -lts $rlts                    \
	    -sendAhead $Prefs(sendAhead)  \
	    -cycleTime $Prefs(cycleTime);
    }
    $acf addSegment $ss $se -start $ls -end $le;
    lappend objectList $acf;
}


proc SetupMjpegClip {dest ss se ls le} {
    global Prefs arch Device lts jpegScreen
    global minTime maxTime objectList

    set host [lindex $dest 0];
    set port [lindex $dest 1];
    set file [lindex $dest 2];

    if {"$host" == "localhost" && $port == 0} {
	set local 1
    } else {
	set local 0
    }

    if {$ls < $minTime} {set minTime $ls};
    if {$le < $minTime} {set minTime $le};
    if {$ls > $maxTime} {set maxTime $ls};
    if {$le > $maxTime} {set maxTime $le};

    
    ### Create CMT connection ###

    if $local {
  	set vcf [mjpegFile ""];
	$vcf config                       \
	    -outCmd @$jpegScreen.accept   \
	    -filename $file               \
	    -lts $lts                     \
	    -sendAhead $Prefs(sendAhead)  \
	    -cycleTime $Prefs(cycleTime);

    } else {
	set cm [cmt ""];
	$cm open -host $host $port;
	
	set rlts [$cm create lts];
	$lts addSlave $rlts;

	set pd [pktDest ""];
	$pd config -outCmd @$jpegScreen.accept;
		
    	set ps [$cm create pktSrc];
  	set vcf [$cm create mjpegFile];
    	$ps config -dest [$pd address];
	$vcf config                       \
	    -outCmd @$ps.accept           \
	    -filename $file               \
	    -lts $rlts                    \
	    -sendAhead $Prefs(sendAhead)  \
	    -cycleTime $Prefs(cycleTime);
    }
    $vcf addSegment $ss $se -start $ls -end $le;

    set w [expr [lindex [$vcf info] 3] + 60];
    set h [expr [lindex [$vcf info] 4] + 30];

    .movie config  -width $w -height $h;
    $jpegScreen config  -width $w -height $h;

    lappend objectList $vcf;
    update idletasks;
}


proc SetupMpegClip {dest ss se ls le} {
    global Prefs arch Device lts mpegScreen
    global minTime maxTime objectList

    set host [lindex $dest 0];
    set port [lindex $dest 1];
    set file [lindex $dest 2];

    if {"$host" == "localhost" && $port == 0} {
	set local 1
    } else {
	set local 0
    }

    if {$ls < $minTime} {set minTime $ls};
    if {$le < $minTime} {set minTime $le};
    if {$ls > $maxTime} {set maxTime $ls};
    if {$le > $maxTime} {set maxTime $le};

    
    ### Create CMT connection ###

    if $local {
  	set vcf [mpegFile ""];
	$vcf config                       \
	    -outCmd @$mpegScreen.accept   \
	    -filename $file               \
	    -lts $lts                     \
	    -sendAhead $Prefs(sendAhead)  \
	    -cycleTime $Prefs(cycleTime);

    } else {
	set cm [cmt ""];
	$cm open -host $host $port;
	
	set rlts [$cm create lts];
	$lts addSlave $rlts;

	set pd [pktDest ""];
	$pd config -outCmd @$mpegScreen.accept;
		
    	set ps [$cm create pktSrc];
  	set vcf [$cm create mpegFile];
    	$ps config -dest [$pd address];
	$vcf config                       \
	    -outCmd @$ps.accept           \
	    -filename $file               \
	    -lts $rlts                    \
	    -sendAhead $Prefs(sendAhead)  \
	    -cycleTime $Prefs(cycleTime);
    }
    $vcf addSegment $ss $se -start $ls -end $le;

    set w [expr [lindex [$vcf info] 3] + 60];
    set h [expr [lindex [$vcf info] 4] + 30];

    .movie config  -width $w -height $h;
    $mpegScreen config  -width $w -height $h;

    lappend objectList $vcf;
    update idletasks;
}



##### USER INTERFACE

frame .movie -relief raised -bd 2
frame .status -relief raised -bd 2 
frame .control 

button .control.rewind -text "Rewind" -command {setSpeed 0; setTime $minTime;}
button .control.reverse -text "Reverse" -command {setSpeed -1}
button .control.stop -text "Stop" -command {setSpeed 0}
button .control.forward -text "Forward" -command {setSpeed 1}
checkbutton .control.mute -text "Mute" -variable Prefs(mute) -command {mute}
button .control.quit -text "QUIT" -command {exit}
foreach b [list rewind reverse stop forward mute quit] {
    pack .control.$b -in .control -expand yes -fill both -side left
}

label .status.title -text ""
pack .status.title -in .status

pack .movie -in . -expand yes -fill both -side top
pack .status -in . -fill both
pack .control -in . -fill both

wm minsize . 0 0



##### CONFIGURE VIDEO WINDOWS

update idletasks

$jpegScreen config -lts $lts -xid [winfo id .movie] \
    -squeeze 0 -xpos -1 -ypos -1;

$mpegScreen config -lts $lts -xid [winfo id .movie] \
    -squeeze 0 -xpos -1 -ypos -1;


##### START THE PLAYER

InitializePlayer;








