/*
 * COPYRIGHT NOTICE 
 * 
 * Copyright (c) Digital Equipment Corporation, 1993, 1994.
 * All Rights reserved. Unpublished rights reserved under the 
 * copyright laws of the United States. Copyright is claimed in 
 * the computer program and user interface thereof. 
 * 
 * The software contained on this media is proprietary to and 
 * embodies the confidential technology of Digital Equipment 
 * Corporation. Possession, use, duplication or dissemination of 
 * the software and media is authorized only pursuant to a valid
 * written license from Digital Equipment Corporation. 
 * 
 * The name of Digital Equipment Corporation may not be used to endorse or 
 * promote products derived from this software without specific prior 
 * written permission. All other rights reserved.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, IMPLIED WARRANTIES OF
 * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * Digital assumes no responsibility AT ALL for the use or reliability of 
 * this software.
 * 
 * 
 * +---------------------------------------------------------------------+
 * | USE, DUPLICATION OR DISCLOSURE BY THE U.S. GOVERNMENT IS SUBJECT TO | 
 * | RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH (c)                       |
 * | DFARS 252.227-7013, OR IN FAR 52.227-19 OR IN 52.227-14 ALT III,    |
 * | AS APPLICABLE                                                       |
 * +---------------------------------------------------------------------+
 */
/*
 * HISTORY
 */
/*
**++
** FACILITY:  Workstation Multimedia  (WMM)  v1.0 
** 
** FILE NAME:   
** MODULE NAME: 
**
** MODULE DESCRIPTION: 
** 
** DESIGN OVERVIEW: 
** 
**--
*/
/*
*  Header file for comunication with AVI installable compressors/decompressors
*
*  Copyright (c) 1990-1993, Microsoft Corp.  All rights reserved. 
*
* Installable compressors should be listed in SYSTEM.INI as
* follows:
*
* [Instalable Compressors]
*      VIDC.MSSQ = mssqcomp.drv
*      VIDC.XXXX = foodrv.drv
*
* [Drivers]
*      VIDC.MSSQ = mssqcomp.drv
*      VIDC.XXXX = foodrv.drv
*
* That is, an identifying FOURCC should be the key, and the value
* should be the driver filename
*
*/

#ifndef _INC_COMPMAN
#define _INC_COMPMAN

#ifdef __cplusplus
extern "C" {            /* Assume C declarations for C++ */
#endif  /* __cplusplus */

/************************************************************************

    messages and structures.

************************************************************************/

#include <mme/compddk.h>            /* include this file for the messages. */

/************************************************************************

    data types.

************************************************************************/

typedef HANDLE HIC;     /* Handle to a Insatlable Compressor */

/************************************************************************

    ICM function declarations     
	
************************************************************************/

BOOL    ICInfo(DWORD fccType, DWORD fccHandler, ICINFO FAR * lpicinfo);
BOOL    ICInstall(DWORD fccType, DWORD fccHandler, LPARAM lParam, LPSTR szDesc, UINT wFlags);
BOOL    ICRemove(DWORD fccType, DWORD fccHandler, UINT wFlags);

HIC     ICOpen(DWORD fccType, DWORD fccHandler, UINT wMode);
	/* ICCallbackProc is a DEC extension */
MMRESULT ICCallbackProc(HIC hic, void (*CallbackProc)(), DWORD dwInstance);
MMRESULT ICClose(HANDLE hic);
MMRESULT ICSendMessage(HIC hic, UINT msg, LPARAM dw1, LPARAM dw2);
MMRESULT ICGetInfo(HIC hic, ICINFO FAR *picinfo, DWORD cb);

/* Values for wFlags of ICInstall() */
#define ICINSTALL_FUNCTION      0x0001  /* lParam is a DriverProc (function ptr) */
#define ICINSTALL_DRIVER        0x0002  /* lParam is a driver name (string) */
#define ICINSTALL_HDRV          0x0004  /* lParam is a HDRVR (driver handle) */

/************************************************************************

    query macros
	
************************************************************************/
#define ICMF_CONFIGURE_QUERY     0x00000001
#define ICMF_ABOUT_QUERY         0x00000001

#define ICQueryAbout(hic) \
    (ICSendMessage(hic, ICM_ABOUT, (LPARAM) -1, ICMF_ABOUT_QUERY) == ICERR_OK)

#define ICAbout(hic, hwnd) \
    ICSendMessage(hic, ICM_ABOUT, (LPARAM)(hwnd), (LPARAM)0)

#define ICQueryConfigure(hic) \
    (ICSendMessage(hic, ICM_CONFIGURE, (LPARAM) -1, (LPARAM)ICMF_CONFIGURE_QUERY) == ICERR_OK)

#define ICConfigure(hic, hwnd) \
    ICSendMessage(hic, ICM_CONFIGURE, (LPARAM)(hwnd), (LPARAM)0)

/************************************************************************

    get/set state macros
	
************************************************************************/

#define ICGetState(hic, pv, cb) \
    ICSendMessage(hic, ICM_GETSTATE, (LPARAM)(LPVOID)(pv), (LPARAM)(cb))

#define ICSetState(hic, pv, cb) \
    ICSendMessage(hic, ICM_SETSTATE, (LPARAM)(LPVOID)(pv), (LPARAM)(cb))

#define ICGetStateSize(hic) \
    ICGetState(hic, NULL, (LPARAM)0)

/************************************************************************

    get value macros
	
************************************************************************/
extern DWORD * _ICInternalScratchDWORD;

#define ICGetDefaultQuality(hic) \
    (ICSendMessage(hic, ICM_GETDEFAULTQUALITY, (LPARAM)(LPVOID)_ICInternalScratchDWORD, sizeof(DWORD)), *_ICInternalScratchDWORD)

#define ICGetDefaultKeyFrameRate(hic) \
    (ICSendMessage(hic, ICM_GETDEFAULTKEYFRAMERATE, (LPARAM)(LPVOID)_ICInternalScratchDWORD, sizeof(DWORD)), *_ICInternalScratchDWORD)

#define ICGetSaturation(hic) \
    (ICSendMessage(hic, ICM_GETSATURATION, \
	(LPARAM)(LPVOID)_ICInternalScratchDWORD, sizeof(DWORD)), \
    *_ICInternalScratchDWORD)

#define ICGetContrast(hic) \
    (ICSendMessage(hic, ICM_GETCONTRAST, \
	(LPARAM)(LPVOID)_ICInternalScratchDWORD, sizeof(DWORD)), \
    *_ICInternalScratchDWORD)

#define ICGetMirror(hic) \
    (ICSendMessage(hic, ICM_GETMIRROR, \
	(LPARAM)(LPVOID)_ICInternalScratchDWORD, sizeof(DWORD)), \
    (BOOL)(*_ICInternalScratchDWORD))

#define ICGetBrightness(hic) \
    (ICSendMessage(hic, ICM_GETBRIGHTNESS, \
	(LPARAM)(LPVOID)_ICInternalScratchDWORD, sizeof(DWORD)), \
    *_ICInternalScratchDWORD)

#define ICGetSharpening(hic) \
    (ICSendMessage(hic, ICM_GETSHARPENING, \
	(LPARAM)(LPVOID)_ICInternalScratchDWORD, sizeof(DWORD)), \
    *_ICInternalScratchDWORD)

/************************************************************************

    set value macros
	
************************************************************************/
#define ICSetSaturation(hic, value) \
    (ICSendMessage(hic, ICM_SETSATURATION, (DWORD)value, sizeof(DWORD)))

#define ICSetContrast(hic, value) \
    (ICSendMessage(hic, ICM_SETCONTRAST, (DWORD)value, sizeof(DWORD)))

#define ICSetMirror(hic, value) \
    (ICSendMessage(hic, ICM_SETMIRROR, (BOOL)value, sizeof(BOOL)))

#define ICSetBrightness(hic, value) \
    (ICSendMessage(hic, ICM_SETBRIGHTNESS, (DWORD)value, sizeof(DWORD)))

#define ICSetSharpening(hic, value) \
    (ICSendMessage(hic, ICM_SETSHARPENING, (DWORD)value, sizeof(DWORD)))

/************************************************************************

    draw window macro
	
************************************************************************/
#define ICDrawWindow(hic, prc) \
    ICSendMessage(hic, ICM_DRAW_WINDOW, (LPARAM)(LPVOID)(prc), sizeof(RECT))

	

/************************************************************************

    compression functions

************************************************************************/
/*
 *  ICCompress()
 *
 *  compress a single frame
 *
 */
DWORD ICCompress(
    HIC                 hic,
    DWORD               dwFlags,        /* flags */
    LPBITMAPINFOHEADER  lpbiOutput,     /* output format */
    LPVOID              lpData,         /* output data */
    LPBITMAPINFOHEADER  lpbiInput,      /* format of frame to compress */
    LPVOID              lpBits,         /* frame data to compress */
    LPDWORD             lpckid,         /* ckid for data in AVI file */
    LPDWORD             lpdwFlags,      /* flags in the AVI index. */
    LONG                lFrameNum,      /* frame number of seq. */
    DWORD               dwFrameSize,    /* reqested size in bytes. (if non zero) */
    DWORD               dwQuality,      /* quality within one frame */
    LPBITMAPINFOHEADER  lpbiPrev,       /* format of previous frame */
    LPVOID              lpPrev);        /* previous frame */

/*
 *  ICCompressBegin()
 *
 *  start compression from a source format (lpbiInput) to a dest
 *  format (lpbiOuput) is supported.
 *
 */
#define ICCompressBegin(hic, lpbiInput, lpbiOutput) \
    ICSendMessage(hic, ICM_COMPRESS_BEGIN, (LPARAM)(LPVOID)(lpbiInput), (LPARAM)(LPVOID)(lpbiOutput))

/*
 *  ICCompressQuery()
 *
 *  determines if compression from a source format (lpbiInput) to a dest
 *  format (lpbiOuput) is supported.
 *
 */
#define ICCompressQuery(hic, lpbiInput, lpbiOutput) \
    ICSendMessage(hic, ICM_COMPRESS_QUERY, (LPARAM)(LPVOID)(lpbiInput), (LPARAM)(LPVOID)(lpbiOutput))

/*
 *  ICCompressGetFormat()
 *
 *  get the ouput format, (format of compressed data)
 *  if lpbiOuput is NULL return the size in bytes needed for format.
 *
 */
#define ICCompressGetFormat(hic, lpbiInput, lpbiOutput) \
    ICSendMessage(hic, ICM_COMPRESS_GET_FORMAT, (LPARAM)(LPVOID)(lpbiInput), (LPARAM)(LPVOID)(lpbiOutput))

#define ICCompressGetFormatSize(hic, lpbi) \
    ICCompressGetFormat(hic, lpbi, NULL)

/*
 *  ICCompressSize()
 *
 *  return the maximal size of a compressed frame
 *
 */
#define ICCompressGetSize(hic, lpbiInput, lpbiOutput) \
    ICSendMessage(hic, ICM_COMPRESS_GET_SIZE, (LPARAM)(LPVOID)(lpbiInput), (LPARAM)(LPVOID)(lpbiOutput))

#define ICCompressEnd(hic) \
    ICSendMessage(hic, ICM_COMPRESS_END, (LPARAM)0, (LPARAM)0)

MMRESULT ICCompressPrepareHeader(
    HIC			hic,
    LPBITMAPINFOHEADER  lpbiOutput,     /* output format */
    LPVOID              lpData,         /* output data */
    LPBITMAPINFOHEADER  lpbiInput,      /* format of frame to compress */
    LPVOID              lpBits          /* frame data to compress */
);
MMRESULT ICCompressUnprepareHeader(
    HIC			hic,
    LPBITMAPINFOHEADER  lpbiOutput,     /* output format */
    LPVOID              lpData,         /* output data */
    LPBITMAPINFOHEADER  lpbiInput,      /* format of frame to compress */
    LPVOID              lpBits          /* frame data to compress */
);
/************************************************************************

    decompression functions

************************************************************************/

/*
 *  ICDecompress()
 *
 *  decompress a single frame
 *
 */
#define ICDECOMPRESS_HURRYUP    0x80000000L     /* don't draw just buffer (hurry up!) */

DWORD ICDecompress(
    HIC                 hic,
    DWORD               dwFlags,    /* flags (from AVI index...) */
    LPBITMAPINFOHEADER  lpbiFormat, /* BITMAPINFO of compressed data */
                                    /* biSizeImage has the chunk size */
    LPVOID              lpData,     /* data */
    LPBITMAPINFOHEADER  lpbi,       /* DIB to decompress to */
    LPVOID              lpBits);

/*
 *  ICDecompressBegin()
 *
 *  start compression from a source format (lpbiInput) to a dest
 *  format (lpbiOuput) is supported.
 *
 */
#define ICDecompressBegin(hic, lpbiInput, lpbiOutput) \
    ICSendMessage(hic, ICM_DECOMPRESS_BEGIN, (LPARAM)(LPVOID)(lpbiInput), (LPARAM)(LPVOID)(lpbiOutput))

/*
 *  ICDecompressQuery()
 *
 *  determines if compression from a source format (lpbiInput) to a dest
 *  format (lpbiOuput) is supported.
 *
 */
#define ICDecompressQuery(hic, lpbiInput, lpbiOutput) \
    ICSendMessage(hic, ICM_DECOMPRESS_QUERY, (LPARAM)(LPVOID)(lpbiInput), (LPARAM)(LPVOID)(lpbiOutput))

/*
 *  ICDecompressGetFormat()
 *
 *  get the ouput format, (format of un-compressed data)
 *  if lpbiOuput is NULL return the size in bytes needed for format.
 *
 */
#define ICDecompressGetFormat(hic, lpbiInput, lpbiOutput) \
    ((LONG) ICSendMessage(hic, ICM_DECOMPRESS_GET_FORMAT, (DWORD)(LPVOID)(lpbiInput), (DWORD)(LPVOID)(lpbiOutput)))

#define ICDecompressGetFormatSize(hic, lpbi) \
    ICDecompressGetFormat(hic, lpbi, NULL)

/*
 *  ICDecompressGetPalette()
 *
 *  get the ouput palette
 *
 */
#define ICDecompressGetPalette(hic, lpbiInput, lpbiOutput) \
    ICSendMessage(hic, ICM_DECOMPRESS_GET_PALETTE, (LPARAM)(LPVOID)(lpbiInput), (LPARAM)(LPVOID)(lpbiOutput))

#define ICDecompressEnd(hic) \
    ICSendMessage(hic, ICM_DECOMPRESS_END, (LPARAM)0, (LPARAM)0)

MMRESULT ICDecompressPrepareHeader(
    HIC			hic,
    LPBITMAPINFOHEADER  lpbiFormat, /* BITMAPINFO of compressed data */
                                    /* biSizeImage has the chunk size */
    LPVOID              lpData,     /* data */
    LPBITMAPINFOHEADER  lpbi,       /* DIB to decompress to */
    LPVOID              lpBits);
MMRESULT ICDecompressUnprepareHeader(
    HIC			hic,
    LPBITMAPINFOHEADER  lpbiFormat, /* BITMAPINFO of compressed data */
                                    /* biSizeImage has the chunk size */
    LPVOID              lpData,     /* data */
    LPBITMAPINFOHEADER  lpbi,       /* DIB to decompress to */
    LPVOID              lpBits);
/************************************************************************

    drawing functions

************************************************************************/

/*
 *  ICDrawBegin()
 *
 *  start decompressing data with format (lpbiInput) directly to the screen
 *
 *  return zero if the decompressor supports drawing.
 *
 */

#define ICDRAW_QUERY        0x00000001L   /* test for support */
#define ICDRAW_FULLSCREEN   0x00000002L   /* draw to full screen */
#define ICDRAW_HDC          0x00000004L   /* draw to a HDC/HWND */

DWORD ICDrawBegin(
        HIC                 hic,
        DWORD               dwFlags,        /* flags */
        HPALETTE            hpal,           /* palette to draw with */
        HWND                hwnd,           /* window to draw to */
        HDC                 hdc,            /* HDC to draw to */
        int                 xDst,           /* destination rectangle */
        int                 yDst,
        int                 dxDst,
        int                 dyDst,
        LPBITMAPINFOHEADER  lpbi,           /* format of frame to draw */
        int                 xSrc,           /* source rectangle */
        int                 ySrc,
        int                 dxSrc,
        int                 dySrc,
        DWORD               dwRate,         /* frames/second = (dwRate/dwScale) */
        DWORD               dwScale);

/*
 *  ICDraw()
 *
 *  decompress data directly to the screen
 *
 */

#define ICDRAW_HURRYUP      0x80000000L   /* don't draw just buffer (hurry up!) */
#define ICDRAW_UPDATE       0x40000000L   /* don't draw just update screen */

DWORD ICDraw(
        HIC                 hic,
        DWORD               dwFlags,        /* flags */
        LPVOID		    lpFormat,       /* format of frame to decompress */
        LPVOID              lpData,         /* frame data to decompress */
        DWORD               cbData,         /* size of data */
        LONG                lTime);         /* time to draw this frame */

/*
 *  ICDrawQuery()
 *
 *  determines if the compressor is willing to render the specified format.
 *
 */
#define ICDrawQuery(hic, lpbiInput) \
    ICSendMessage(hic, ICM_DRAW_QUERY, (LPARAM)(LPVOID)(lpbiInput), (LPARAM)0L)

#define ICGetBuffersWanted(hic, lpdwBuffers) \
    ICSendMessage(hic, ICM_GETBUFFERSWANTED, (LPARAM)(LPVOID)(lpdwBuffers), (LPARAM)0)

#define ICDrawEnd(hic) \
    ICSendMessage(hic, ICM_DRAW_END, (LPARAM)0, (LPARAM)0)

#define ICDrawStart(hic) \
    ICSendMessage(hic, ICM_DRAW_START, (LPARAM)0, (LPARAM)0)

#define ICDrawStop(hic) \
    ICSendMessage(hic, ICM_DRAW_STOP, (LPARAM)0, (LPARAM)0)

#define ICDrawGetTime(hic, lplTime) \
    ICSendMessage(hic, ICM_DRAW_GETTIME, (LPARAM)(LPVOID)(lplTime), (LPARAM)0)

#define ICDrawSetTime(hic, lTime) \
    ICSendMessage(hic, ICM_DRAW_SETTIME, (LPARAM)lTime, (LPARAM)0)

#define ICDrawRealize(hic, hdc, fBackground) \
    ICSendMessage(hic, ICM_DRAW_REALIZE, (LPARAM)(UINT)(HDC)(hdc), (LPARAM)(BOOL)(fBackground))

#define ICDrawFlush(hic) \
    ICSendMessage(hic, ICM_DRAW_FLUSH, (LPARAM)0, (LPARAM)0)

#define ICDrawRenderBuffer(hic) \
    ICSendMessage(hic, ICM_DRAW_RENDERBUFFER, (LPARAM)0, (LPARAM)0)


/************************************************************************

helper routines for DrawDib and MCIAVI...

************************************************************************/

#define ICDecompressOpen(fccType, fccHandler, lpbiIn, lpbiOut) \
    ICLocate(fccType, fccHandler, lpbiIn, lpbiOut, ICMODE_DECOMPRESS)

#define ICDrawOpen(fccType, fccHandler, lpbiIn) \
    ICLocate(fccType, fccHandler, lpbiIn, NULL, ICMODE_DRAW)

HIC  ICLocate(DWORD fccType, DWORD fccHandler, LPBITMAPINFOHEADER lpbiIn, LPBITMAPINFOHEADER lpbiOut, WORD wFlags);
HIC  ICGetDisplayFormat(HIC hic, LPBITMAPINFOHEADER lpbiIn, LPBITMAPINFOHEADER lpbiOut, int BitDepth, int dx, int dy);



#ifdef __cplusplus
}                       /* End of extern "C" { */
#endif	/* __cplusplus */

#endif /* _INC_COMPMAN */
