/*
 * COPYRIGHT NOTICE 
 * 
 * Copyright (c) Digital Equipment Corporation, 1993, 1994.
 * All Rights reserved. Unpublished rights reserved under the 
 * copyright laws of the United States. Copyright is claimed in 
 * the computer program and user interface thereof. 
 * 
 * The software contained on this media is proprietary to and 
 * embodies the confidential technology of Digital Equipment 
 * Corporation. Possession, use, duplication or dissemination of 
 * the software and media is authorized only pursuant to a valid
 * written license from Digital Equipment Corporation. 
 * 
 * The name of Digital Equipment Corporation may not be used to endorse or 
 * promote products derived from this software without specific prior 
 * written permission. All other rights reserved.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, IMPLIED WARRANTIES OF
 * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * Digital assumes no responsibility AT ALL for the use or reliability of 
 * this software.
 * 
 * 
 * +---------------------------------------------------------------------+
 * | USE, DUPLICATION OR DISCLOSURE BY THE U.S. GOVERNMENT IS SUBJECT TO | 
 * | RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH (c)                       |
 * | DFARS 252.227-7013, OR IN FAR 52.227-19 OR IN 52.227-14 ALT III,    |
 * | AS APPLICABLE                                                       |
 * +---------------------------------------------------------------------+
 * 
 * 
 * 
 * 
 * 
 * 
 */
/*
 * HISTORY
 */
/*
**++
** FACILITY:	Digital Equipment Corporation
**		Multimedia Services for Alpha AXP
**		Development Kit Sample Code
**
** FILE NAME:	mmc_buffers.h
**
** PURPOSE:	Handle audio or video buffers for a multimedia application.
**
** NOTES:	none.
**--
*/
#ifndef _mmc_buffers.h_
#define _mmc_buffers.h_

typedef enum
{
    Empty,
    Filling,
    Full,
    Playing,
    NumberOfStates /* used only to establish the dimensions of arrays */
} mmcBufferStatus_e;

typedef struct mmcBuffer_s
{
    mmcBufferStatus_e status;
    char * lpData;
    int nbytes; /* number of bytes available to be processed */
} mmcBuffer_t, *mmcBuffer_p;


#define mmcBufferNone -1 /* index value indicating no buffer */

typedef struct mmcBufferList_s
{
    int n;          /* number of buffers */
    int nbytes;     /* number of bytes allocated for each buffer */
    mmcBuffer_p b;  /* "n" element array of pointers to start of each buffer */
    int first_arrived[NumberOfStates]; /* Index of first buffer that entered 
					*  this state. To use this variable, 
					*  set it to mmcBufferNone and then 
					*  wait for it to change.
					* If the application never sets this 
					*  variable, then it will be set
					*  once and ignored thereafter.
					*/
    int arrival_count[NumberOfStates]; /* incremented every time a buffer 
					* enters this state. 
					*/
    int total_arrival_count[NumberOfStates]; /* initialized to zero and 
					      * available for caller's use. 
					      */
} mmcBufferList_t, *mmcBufferList_p;

/* The ".n" field is used as a flag to indicate whether of not the rest of the
 * structure has been initialized:
 *	0 = uninitialized
 *	non-zero = initialized
 * Therefore, no other fields need to be initialized if the ".n" field is 0.
 */
#define mmcBufferListInitialValue { 0 }

MMRESULT mmcBuffersCreate (mmcBufferList_p buffer_list,
			   int nbuf,
			   int nbytes,
			   int npadbytes);

void mmcBuffersFree (mmcBufferList_p buffer_list);

/* Return index of first buffer matching requested status */
/* if none found, return mmcNoBuffer */
int mmcBufferFind (const mmcBufferList_p buffer_list,
		   mmcBufferStatus_e status);

void mmcBufferSetStatus (mmcBufferList_p buffer_list,
			 int i,
			 mmcBufferStatus_e status,
			 int nbytes);

#endif /* mmc_buffers.h */

