/*
 * COPYRIGHT NOTICE 
 * 
 * Copyright (c) Digital Equipment Corporation, 1993, 1994.
 * All Rights reserved. Unpublished rights reserved under the 
 * copyright laws of the United States. Copyright is claimed in 
 * the computer program and user interface thereof. 
 * 
 * The software contained on this media is proprietary to and 
 * embodies the confidential technology of Digital Equipment 
 * Corporation. Possession, use, duplication or dissemination of 
 * the software and media is authorized only pursuant to a valid
 * written license from Digital Equipment Corporation. 
 * 
 * The name of Digital Equipment Corporation may not be used to endorse or 
 * promote products derived from this software without specific prior 
 * written permission. All other rights reserved.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, IMPLIED WARRANTIES OF
 * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * Digital assumes no responsibility AT ALL for the use or reliability of 
 * this software.
 * 
 * 
 * +---------------------------------------------------------------------+
 * | USE, DUPLICATION OR DISCLOSURE BY THE U.S. GOVERNMENT IS SUBJECT TO | 
 * | RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH (c)                       |
 * | DFARS 252.227-7013, OR IN FAR 52.227-19 OR IN 52.227-14 ALT III,    |
 * | AS APPLICABLE                                                       |
 * +---------------------------------------------------------------------+
 * 
 * 
 * 
 * 
 * 
 * 
 */
/*
 * HISTORY
 */
/*
**++
** FACILITY:	Digital Equipment Corporation
**		Multimedia Services for Alpha AXP
**		Development Kit Sample Code
**
** FILE NAME:	mmc_cmdlin.h
**
** PURPOSE:	Handle command-line parsing.
**
** NOTES:	This module is purposely as simple as possible because
**		command-line parsing has nothing to do with multimedia.
**		These routines are intended to minimize the reader's
**		distraction from the real multimedia code.
**--
*/
#ifndef _mmc_cmdlin_h_
#define _mmc_cmdlin_h_

#define MAXARGS		512
#define MAX_STRING	128

typedef enum
{
    apChar,
    apShort,
    apInt,
    apLong,
    apMask,
    apChoice,
    apPointer
} apValueType_e;

typedef enum
{
    apSuccess,
    apBadValue, 
    apMissingValue, 
    apUnknownOption,
    apAmbiguousOption,
    apMissingOption
} apParseStatus_e;

typedef enum
{
    apOptionRequired, /* Option is required */
    apOptionNotRequired  /* Option is not required */
} apOptionPresence_e;

typedef enum
{
    apOptionNoArg,	/* Value is specified in OptionDescRec.value        */
    apOptionIsArg,      /* Value is the option string itself            */
    apOptionStickyArg,  /* Value is characters immediately following option */
    apOptionSepArg,     /* Value is next argument in argv           */
    apOptionSkipArg,    /* Ignore this option and the next argument in argv */
    apOptionSkipLine,   /* Ignore this option and the rest of argv      */
    apOptionSkipNArgs  /* Ignore this option and the next
			   OptionDescRes.value arguments in argv */
} apOptionKind_e;

typedef struct apEnumDesc_s
{
    char * option;
    int caseSensitive;
    void * value;
} apEnumDesc_t, *apEnumDesc_p;

typedef struct apChoiceDesc_s
{
    char * name;
    int retValue;
    char * desc;
} apChoiceDesc_t, *apChoiceDesc_p;

typedef struct apOptionDesc_s
{
    char            * option;   /* Option abbreviation in argv      */
    apOptionKind_e  argKind;    /* Which style of option it is      */
    apValueType_e   type;       /* type of storage in value         */
    apOptionPresence_e presence;  /* *Not* an option                  */
    const void       * defValue; /* Value to provide if apOptionNoArg or */
                            /* Pointer to choice array is apChoice */
    void        * value;    /* return value             */
    char        * optionArg; /* description of option               */
    char        * description; /* description of option             */
} apOptionDesc_t, *apOptionDesc_p;

#define mmcNumberOf(array) ((unsigned int)(sizeof(array) / sizeof(array[0])))
#define isNumeric(c) (isdigit(c[0]) || ((c[0] == '-') && (isdigit(c[1]))) || ((c[0] == '+') && (isdigit(c[1])))) ? 1 : 0

const char * mmcExtractProgramName (const char * argv0);

const char * mmcGetChoiceString (const apChoiceDesc_t choiceTable[], 
				 const int choiceValue);

void mmcUsage (apOptionDesc_p options,
	       int num_options, 
	       char ** argv,
	       int argc,
	       apParseStatus_e parseStatus);

apParseStatus_e mmcParseArguments (apOptionDesc_p options,
				int num_options, 
				int * argc,
				char ** argv);

int mmcEnumValue (char * find,
		  apEnumDesc_p list,
		  unsigned int count);

char * mmcEnumString (void  * value,
		      apEnumDesc_p list,
		      unsigned int count);

/***** MACROS TO DEFINE COMMAND-LINE ARGUMENTS *****/

#define apOptIntSep_(name, ptr, usage_string) \
{ name, apOptionSepArg, apInt, apOptionNotRequired, \
      NULL, ptr, "#", usage_string }

#define apOptMaskSep_(name, ptr, usage_string) \
{ name, apOptionSepArg, apMask, apOptionNotRequired, \
      NULL, ptr, "#", usage_string }

#define apOptIntSticky_(name, ptr, usage_string) \
{ name, apOptionStickyArg, apInt, apOptionNotRequired, \
      NULL, ptr, "#", usage_string }

#define apOptFlag_(name, ptr, usage_string) \
{ name, apOptionNoArg, apInt, apOptionNotRequired, \
      "1", (void *)ptr, NULL, usage_string }

#define apOptString_(name, ptr, usage_desc, usage_string) \
{ name, apOptionSepArg, apPointer, apOptionNotRequired, \
      NULL, ptr, usage_desc, usage_string }

#define apOptChoice_(name, ptr, usage_desc, usage_string, choice_desc) \
{ name, apOptionSepArg, apChoice, apOptionNotRequired, \
      choice_desc, ptr, usage_desc, usage_string }

#define apReqString_(name, ptr, usage_desc, usage_string) \
{ name, apOptionNoArg, apPointer, apOptionNotRequired, \
      NULL, ptr, usage_desc, usage_string }

#define apReqOption_(name, ptr, usage_desc, usage_string) \
{ name, apOptionSepArg, apPointer, apOptionRequired, \
      NULL, ptr, usage_desc, usage_string }

#define apHelp_(ptr) \
{"-help", apOptionNoArg, apInt, apOptionNotRequired, "1", (void *)ptr, NULL, \
     "display this help information and exit" }

#endif /* _mmc_cmdlin_h_ */
