/*
 * COPYRIGHT NOTICE 
 * 
 * Copyright (c) Digital Equipment Corporation, 1993, 1994.
 * All Rights reserved. Unpublished rights reserved under the 
 * copyright laws of the United States. Copyright is claimed in 
 * the computer program and user interface thereof. 
 * 
 * The software contained on this media is proprietary to and 
 * embodies the confidential technology of Digital Equipment 
 * Corporation. Possession, use, duplication or dissemination of 
 * the software and media is authorized only pursuant to a valid
 * written license from Digital Equipment Corporation. 
 * 
 * The name of Digital Equipment Corporation may not be used to endorse or 
 * promote products derived from this software without specific prior 
 * written permission. All other rights reserved.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, IMPLIED WARRANTIES OF
 * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * Digital assumes no responsibility AT ALL for the use or reliability of 
 * this software.
 * 
 * 
 * +---------------------------------------------------------------------+
 * | USE, DUPLICATION OR DISCLOSURE BY THE U.S. GOVERNMENT IS SUBJECT TO | 
 * | RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH (c)                       |
 * | DFARS 252.227-7013, OR IN FAR 52.227-19 OR IN 52.227-14 ALT III,    |
 * | AS APPLICABLE                                                       |
 * +---------------------------------------------------------------------+
 * 
 * 
 * 
 * 
 * 
 * 
 */
/*
 * HISTORY
 */
/*
**++
** FACILITY:	Digital Equipment Corporation
**		Multimedia Services for Alpha AXP
**		Development Kit Sample Code
**
** FILE NAME:	mmc_wave_file.h
**
** PURPOSE:	Handle reading or writing a WAVE file.
**
** NOTES:
**	A WAVE file consists of one RIFF chunk of type 'WAVE'.
**	This 'WAVE' chunk contains two RIFF sub-chunks:
**	1. a RIFF chunk of type 'fmt ' which contains the data format, and
**	2. a RIFF chunk of type 'data' which contains all of the audio
**	   sample data.
**--
*/
#ifndef _mmc_wave_file_h_
#define _mmc_wave_file_h_

typedef struct mmcWaveFileState_s
{
    HMMIO hmmio;
    FILE * fp;
    MMCKINFO chunks[2]; /* a wave file's chunks only nest 2 levels deep. */

} mmcWaveFileState_t, *mmcWaveFileState_p;

/* The ".hmmio" field is used as a flag to indicate whether of not the chunks
 * the structure has been initialized:
 *	NULL = uninitialized
 *	non-NULL = initialized
 * Therefore, the chunks[2] need not be initialized if the ".hmmio" field
 * is NULL.
 */
#define mmcWaveFileStateInitialValue { NULL, NULL }

/* Wave Out file routines */

int mmcWaveOutFileOpen (char * file_name,
			const LPPCMWAVEFORMAT format_p,
			mmcWaveFileState_p f);

int mmcWaveOutFileClose (mmcWaveFileState_p f);

int mmcWaveOutFileWrite (mmcWaveFileState_p f,
			 HPSTR data,
			 int n);

/* Wave In file routines */

int mmcWaveInFileClose (mmcWaveFileState_p f);

int mmcWaveInFileOpen (char * file_name,
		       const LPPCMWAVEFORMAT format_p,
		       mmcWaveFileState_p f);

#endif /* _mmc_wave_file_h_ */
