/*
 * COPYRIGHT NOTICE 
 * 
 * Copyright (c) Digital Equipment Corporation, 1993, 1994.
 * All Rights reserved. Unpublished rights reserved under the 
 * copyright laws of the United States. Copyright is claimed in 
 * the computer program and user interface thereof. 
 * 
 * The software contained on this media is proprietary to and 
 * embodies the confidential technology of Digital Equipment 
 * Corporation. Possession, use, duplication or dissemination of 
 * the software and media is authorized only pursuant to a valid
 * written license from Digital Equipment Corporation. 
 * 
 * The name of Digital Equipment Corporation may not be used to endorse or 
 * promote products derived from this software without specific prior 
 * written permission. All other rights reserved.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, IMPLIED WARRANTIES OF
 * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * Digital assumes no responsibility AT ALL for the use or reliability of 
 * this software.
 * 
 * 
 * +---------------------------------------------------------------------+
 * | USE, DUPLICATION OR DISCLOSURE BY THE U.S. GOVERNMENT IS SUBJECT TO | 
 * | RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH (c)                       |
 * | DFARS 252.227-7013, OR IN FAR 52.227-19 OR IN 52.227-14 ALT III,    |
 * | AS APPLICABLE                                                       |
 * +---------------------------------------------------------------------+
 * 
 * 
 * 
 * 
 * 
 * 
 */
/*
 * HISTORY
 */
/*
**++
** FACILITY:	Digital Equipment Corporation
**		Multimedia Services for Alpha AXP
**		Development Kit Sample Code
**
** FILE NAME:	mmc_wave_format.h
**
** PURPOSE:	Check and fix consistency of fields in WAVEFORMAT structure.
**
** NOTES:
**		The WAVE audio specification is somewhat vague on the meanings
**		of the various fields in the WAVEFORMAT structure.
**
**		Two of these fields (nBlockAlign and wAvgBytesPerSec) appear
**		to be redundant.
**
**		Another of them (nBitsPerSample) is only used for one format
**		of data (WAVE_FORMAT_PCM)
**
**		So if these are written incorrectly (or even assigned a
**		different meaning), many WAVE drivers might not notice.
**
**		However, the two WAVE drivers shipped with DEC Multimedia
**		services check every field of this structure, so it must be
**		totally internally consistent before it can be passed to
**		waveInOpen or waveOutOpen.
**
**		mmcWaveFormatFix fixes up incorrect fields in this structure,
**		and then reports back what it fixed.
**--
*/
#ifndef _mmc_wave_format_h_
#define _mmc_wave_format_h_

/* Do nothing */
#define WAVE_FORMAT_FIX_NONE       	0x00

/* If Mu-Law and bits-per-sample != 8, fix it */
#define WAVE_FORMAT_FIX_MULAW_BPS	0x01

/* If Mu-Law and samples-per-second != 8000, fix it */
#define WAVE_FORMAT_FIX_MULAW_RATE	0x02

/* Calculate nBlockAlign from bits-per-sample and num-channels.
 * If incorrect, fix it. */
#define WAVE_FORMAT_FIX_BLOCK_ALIGN	0x04

/* Calculate nAvgBytesPerSec from nBlockAlign and samples-per-second.
 * If incorrect, fix it. */
#define WAVE_FORMAT_FIX_AVG_BPS    	0x08

/* Do all of the above */
#define WAVE_FORMAT_FIX_ALL		0x0f

/* returns a bit flag indicating what it fixed */
int mmcWaveFormatFix(LPPCMWAVEFORMAT formatp,
		      int things_to_fix);

#endif /* _mmc_wave_format_h_ */
