/*This line lets emacs recognize this as -*- C -*- Code
*-------------------------------------------------------------------------
* 
*   File:	XilDefines.h
*   Project:	XIL
*   Created:	92/05/09
*   RespEngr:	Paul Ramsey & Chuck Mosher - Sun NC Development Center
*   Revision:	1.25
*   Last Mod:	13:15:55, 10 Jun 1993
*  
*   Description:
*                 
* 	Definition of basic XIL defines and typedefs for both C and C++ 
*	programs.
*         
*         
*-----------------------------------------------------------------------*/

#ifndef XIL_DEFINES_H
#define XIL_DEFINES_H

typedef unsigned char Xil_unsigned8;
typedef char Xil_signed8;
typedef unsigned short Xil_unsigned16;
typedef short Xil_signed16;
typedef unsigned int Xil_unsigned32;
typedef int Xil_signed32;

typedef int Xil_boolean;

#define NULL 0
#define XIL_SUCCESS 0
#define XIL_FAILURE 2

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef MAXBYTE
#define MAXBYTE		(Xil_unsigned8)255
#endif

#ifndef MINBYTE
#define MINBYTE		(Xil_unsigned8)0
#endif

#ifndef MAXSHORT
#define MAXSHORT	(Xil_signed16)32767
#endif

#ifndef MINSHORT
#define MINSHORT	(Xil_signed16)-32768
#endif

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#ifdef ERROR_BUILD
extern unsigned long  xil_failure[];
extern char*          xil_print_failures;

void XilAssertFailure(int failure, int onoff);
void XilPrintConditionalFailure(int x);

#define XIL_SIMULATE_FAILURE(x,code) \
   if (--xil_failure[x]==0) { \
      XilPrintConditionalFailure(x); \
      code; \
   }
#else
#define XIL_SIMULATE_FAILURE(x,code)
#endif

#define XIL_DATATYPES	4
#define XIL_OP_CACHE_INIT {-1, -1, -1, -1}

typedef enum __XilDataType {
    XIL_BIT, XIL_BYTE, XIL_SHORT, XIL_FLOAT
} XilDataType;

/* this indicates what the alignment is on bit images */
#define XIL_BIT_ALIGNMENT 8

typedef enum __XilEdgeCondition {
    XIL_EDGE_NO_WRITE,
    XIL_EDGE_ZERO_FILL,
    XIL_EDGE_EXTEND
} XilEdgeCondition;

typedef enum __XilFlipType {
    XIL_FLIP_X_AXIS,
    XIL_FLIP_Y_AXIS,
    XIL_FLIP_MAIN_DIAGONAL,
    XIL_FLIP_ANTIDIAGONAL,
    XIL_FLIP_90,
    XIL_FLIP_180,
    XIL_FLIP_270
} XilFlipType;

typedef enum {
   XIL_IMAGE,
   XIL_IMAGE_TYPE,
   XIL_LOOKUP,
   XIL_CIS,
   XIL_DITHER_MASK,
   XIL_KERNEL,
   XIL_SEL,
   XIL_ROI,
   XIL_ROI_LIST,
   XIL_HISTOGRAM,
   XIL_COLORSPACE
} XilObjectType;

typedef void (*XIL_FUNCPTR_DONE_WITH_DATA)(void*);

char *XilUseEnv(char filename[]);
int RealTypeToPipeName(char*,int,char*);

typedef struct {
	Xil_unsigned16 majorVersion;
	Xil_unsigned16 minorVersion;
} XilVersionRec, *XilVersionPtr;

XilVersionPtr XilGetVersion();

#define INCL_STDLIB
#include "fmv_internal.h"

#define XIL_IS_NAN(x) ((x)!=(x))

/*
 *  Defines for XilCis
 */
#define EACH_FRAME       -1
#define EACH_FRAME_GROUP -2
#define BUFFER_FULL      -3
#define ALL_FRAMES       -4

#endif
