/*
 *  @(#)fmv_internal.h	1.42 93/02/19 Sun Microsystems North Carolina Development Center
 *
 * fmv_internal.h
 *
 * Macros and prototypes missing from include files.
 * Over time, this file should shrink as the compiler "vendor"
 *  cleans up its' include files.
 *
 *  Need to be protected using #ifdefs so they are only seen
 *   if they could possibly be used (in case they need typedefs
 *   or structures not yet defined).
 *
 * This file provides fixes for various include file problems under
 *  various compilers.  It also provides a cleaner way to get the
 *  right system include files.
 *
 *  INCL_STDLIB                 most section 2, section 3 prototypes
 *  INCL_MALLOC                 malloc, etc.
 *  INCL_MEMORY                 memcpy, etc.
 *  INCL_STRING                 string routines
 *  INCL_TIME                   everything to do with time
 *  INCL_SOCKET                 and sockets
 *  INCL_VARARGS                varargs support
 *  INCL_SIGNAL                 everything to do with signals
 *  INCL_AUDIO                  audio misc.
 *  INCL_DGA                    DGA include files - ONLY ON SPARC
 *  INCL_MATH                   math functions
 *  INCL_IPC                    all IPC stuff: shm, msgq, sem, etc.
 */

#if ! defined(_fmv_internal_h)      /* protect top part of fmv_internal.h */
#define _fmv_internal_h

/*
 * Salsa types
 */

typedef unsigned char byte;
typedef unsigned long word;

/*
 *  Machine/CPU Information
 */
#if #cpu(sparc)
/*
 *  The SPARC is a BIGENDIAN CPU
 */
#define BIGENDIAN
#undef  LITTLEENDIAN

/*
 *  The SPARC Solaris has libdga for its frame buffers
 */
#define HAS_LIBDGA

#elif #cpu(i386)

/*
 *  The Intel x86 CPUs are LITTLEENDIAN
 */
#undef  BIGENDIAN
#define LITTLEENDIAN

/*
 *  The Intel Solaris does not have libdga
 */
#undef HAS_LIBDGA

#else
/*
 *  Xil doesn't know about this CPU type
 */
#error "UNKNOWN CPU TYPE"
#endif

#ifdef __cplusplus
# define BEGINCPROTO extern "C" {
# define ENDCPROTO  }
#else
# define BEGINCPROTO
# define ENDCPROTO 
#endif

/*
 * Placeholders for multi-threading locking
 */
#define FMV_MTLOCK(x)
#define FMV_MTUNLOCK(x)

/*
 * CAST provides commentary for follow-up cleanup
 * eg:  my_grungy_routine(CAST(double *) int_ptr);
 */
#define CAST(a) (a)

#endif /* _fmv_internal_h */

#ifdef INCL_STDLIB              /* INCL_STDLIB is shorthand for all these */
#define INCL_STDIO
#define INCL_MEMORY
#define INCL_STRING
#define INCL_MALLOC
#endif

/***********************************************************************
 *
 * The rest of the file can be evaluated multiple times
 *
 ***********************************************************************/


#if defined(INCL_STDIO) && ! defined(__INCL_STDIO)
#define __INCL_STDIO
#include <stdio.h>
#include <fcntl.h>
#endif

#if defined(INCL_MEMORY) && ! defined(__INCL_MEMORY)
#define __INCL_MEMORY
#include <memory.h>
#endif

#if defined(INCL_STRING) && ! defined(__INCL_STRING)
#define __INCL_STRING
#include <string.h>
#endif

#if defined(INCL_MALLOC) && ! defined(__INCL_MALLOC)
#define __INCL_MALLOC
#include <malloc.h>
#endif

#if defined(INCL_STDLIB) && ! defined(__INCL_STDLIB)
#define __INCL_STDLIB
#include <sys/types.h>
#include <sys/ioccom.h>
#include <unistd.h>
#include <stdlib.h>
#endif

#if defined(INCL_VARARGS) && ! defined(__INCL_VARARGS)
#define __INCL_VARARGS
#include <stdarg.h>
#endif

#if defined(INCL_SIGNAL) && ! defined(__INCL_SIGNAL)
#define __INCL_SIGNAL
#include <signal.h>
#endif

#if defined(INCL_MATH) && ! defined(__INCL_MATH)
#define __INCL_MATH
#include <math.h>
#endif

#if defined(INCL_TIME) && ! defined(__INCL_TIME)
#define __INCL_TIME
#include <sys/time.h>
#endif

#if defined(INCL_SOCKET) && ! defined(__INCL_SOCKET)
#define __INCL_SOCKET
#include <sys/socket.h>
#endif

#if defined(INCL_IPC) && ! defined(__INCL_IPC)
#define __INCL_IPC
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>
#include <sys/msg.h>
#endif

#ifdef HAS_LIBDGA
#if defined(INCL_DGA) && ! defined(__INCL_DGA)
#define __INCL_DGA
#ifdef svr4
#include "dga-kludge-svr4.h"
#include <sys/fbio.h>
#else
#include "dga-kludge.h"
#include <sun/fbio.h>
#endif svr4

BEGINCPROTO

/* cmap_grab.c */
Dga_cmap dga_cm_grab(int fd, Dga_token filesuffix);
void     dga_cm_ungrab(Dga_cmap cginfo, int cflag);
void     dga_cm_put(Dga_cmap cginfo, int index, int count,
		    u_char *red, u_char *green, u_char *blue);


/* dga_Xrequests.c */
Dga_token XDgaGrabWindow(Display *dpy, Window win);
int XDgaUnGrabWindow(Display *dpy, Window win);
Dga_token XDgaGrabColormap(Display *dpy, Colormap cmap);
int XDgaUnGrabColormap(Display *dpy, Colormap cmap);


/* win_grab.c */
Dga_window dga_win_grab(int devfd, Dga_token token);
void dga_win_ungrab(Dga_window clientp, int cflag);
int dga_win_devfd(Dga_window win);
char* dga_win_fbname(Dga_window win);

short *dga_win_clipinfo(Dga_window win);
int dga_win_clipchg(Dga_window win);
int dga_win_empty(Dga_window win);
void dga_win_bbox(Dga_window win, int *xp, int *yp, int *widthp, int *heightp);

ENDCPROTO
#endif INCL_DGA
#endif HAS_LIBDGA


#if defined(INCL_AUDIO) && ! defined(__INCL_AUDIO)
#define __INCL_AUDIO
#include <multimedia/audio_device.h>
#include <multimedia/libaudio.h>
  
BEGINCPROTO

/*
 *  Routines from libaudio.a
 */

/* device_ctl.c */
int audio_getinfo(int fd, Audio_info* ip);
int audio_setinfo(int fd, Audio_info* ip);
int audio__setplayhdr(int fd, Audio_hdr* hdrp, unsigned which);
int audio__setval(int fd, unsigned* valp, unsigned which);
int audio__setgain(int fd, double* valp, unsigned which);
int audio__setpause(int fd, unsigned which);
int audio__flush(int fd, unsigned which);
int audio_drain(int fd, int sig);
int audio_play_eof(int fd);

/* filehdr.c */
int audio_write_filehdr(int fd, Audio_hdr* hdrp, char* infop, unsigned ilen);
int audio_rewrite_filesize(int fd, unsigned size);
int audio_read_filehdr(int fd, Audio_hdr* hdrp, char* infop, unsigned ilen);
int audio_isaudiofile(char* name);
int audio_decode_filehdr(unsigned char* buf, Audio_hdr* hdrp, unsigned* ilen);

/* hdr_misc.c */
double    audio_bytes_to_secs(Audio_hdr *hp, unsigned cnt);
unsigned  audio_secs_to_bytes(Audio_hdr *hp, double sec);
double    audio_str_to_secs(char* str);
char*     audio_secs_to_str(double sec, char* str, int precision);
int       audio_cmp_hdr(Audio_hdr* h1, Audio_hdr* h2);

/* gen_ulaw2linear.c */
unsigned char audio_linear2ulaw(int pcm_val);
short    audio_ulaw2linear(int u_val);
ENDCPROTO
#endif

/*
 * The following are missing from one or more of 
 *  the various stdlib.h files we use for different 
 *  compilers.  In the event they are present in
 *  stdlib.h this won't hurt (as long as it is correct!)
 */

BEGINCPROTO

long    tell(int fd);

#ifndef SVR4
int getpagesize (void);
#endif

ENDCPROTO

#if 0
#if defined(rewinddir)  /* <dirent.h> */
BEGINCPROTO
int getdents (int fd, struct dirent *buf, int nbytes);
ENDCPROTO
#endif
#endif

#if defined(FD_ZERO) && ! defined(svr4)  /* redefine to use memset instead of bzero */
#undef FD_ZERO
#define	FD_ZERO(p)	memset((p), 0, sizeof (*(p)))
#endif

#ifdef DST_USA
#ifdef svr4
BEGINCPROTO
/* sys/time.h is missing the gettimeofday prototype */
int gettimeofday (struct timeval *tp);
ENDCPROTO
#endif
#endif

#if 0
#if defined(FD_SET) && defined(SOCK_STREAM)     /* types.h, socket.h */
BEGINCPROTO
int accept  (int sock, struct sockaddr *name, int *addrlen);
int socket  (int domain, int type, int protocol);
int setsockopt (int s, int level, int optname, const char *optval, int optlen);
int bind    (int s, struct sockaddr *name, int namelen);
int listen  (int s, int backlog);
ENDCPROTO
#endif
#endif

#if 0
#ifdef UIO_USERSPACE        /* uio.h */
BEGINCPROTO
int writev (int fd, struct iovec *iov, int iovcnt);
int readv  (int fd, struct iovec *iov, int iovcnt);
ENDCPROTO
#endif
#endif

