/*This line lets emacs recognize this as -*- C -*- Code
*------------------------------------------------------------------------
*
*  File:	xil.h
*  Project:	XIL
*  Created:	92/14/12
*  RespEngr:	Chuck Mosher - Sun NC Development Center
*  Revision:	1.19
*  Last Mod:	15:24:06, 20 Oct 1993
*
*  Description:
*		
*	C-side of defines and includes for XIL c-bindings
*	
*-----------------------------------------------------------------------*/

#ifndef XIL_C_H
#define XIL_C_H

/* 
 *  Include definition of XilVersionNumber.
 */
#include <xil/XilVersionNumber.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <xil/XilDefines.h>
#include <xil/XilCellDefines.h>
#include <xil/XilJpegDefines.h>
#include <xil/XilJpegLLDefines.h>
#include <xil/XilMpeg1Defines.h>
#include <xil/XilH261Defines.h>
#include <xil/XilColorDefines.h>
#include <xil/XilErrorDefines.h>
#include <xil/XilPhotoCD.h>

/* this enables typechecking, even though the ptr inside is garbage */
typedef struct __XilObject {
  caddr_t ptr;
} *XilObject;

typedef struct __XilImage {
  caddr_t ptr;
} *XilImage;

typedef struct __XilImageType {
  caddr_t ptr;
} *XilImageType;

typedef struct __XilRoi {
  caddr_t ptr;
} *XilRoi;

typedef struct __XilSystemState {
  caddr_t ptr;
} *XilSystemState;

typedef struct __XilSel {
  caddr_t ptr;
} *XilSel;

typedef struct __XilKernel {
  caddr_t ptr;
} *XilKernel;

typedef struct __XilLookup {
  caddr_t ptr;
} *XilLookup;

typedef struct __XilError {
  caddr_t ptr;
} *XilError;

typedef Xil_boolean (*XilErrorFunc)(XilError);

typedef struct __XilCis {
  caddr_t ptr;
} *XilCis;

typedef struct __XilDitherMask {
  caddr_t ptr;
} *XilDitherMask;

typedef struct __XilHistogram {
  caddr_t ptr;
} *XilHistogram;

typedef struct __XilColorspace {
  caddr_t ptr;
} *XilColorspace;


/* C Definition of XIL's in-memory defines and typedefs */

typedef struct __XilMemoryStorageBit {
   Xil_unsigned8* data;            /* pointer to the first byte of the image */
   unsigned int scanline_stride;   /* the number of bytes between scanlines */
   unsigned long band_stride;      /* the number of bytes between bands */
   unsigned char offset;           /* the number of bits to the first pixel */
} XilMemoryStorageBit;

typedef struct __XilMemoryStorageByte {
   Xil_unsigned8* data;            /* pointer to the first byte of the image */
   unsigned long scanline_stride;  /* the number of bytes between scanlines */
   unsigned int pixel_stride;      /* the number of bytes between pixels */
} XilMemoryStorageByte;

typedef struct __XilMemoryStorageShort {
   Xil_signed16* data;             /* pointer to the first word of the image */
   unsigned long scanline_stride;  /* the number of words between scanlines */
   unsigned int pixel_stride;      /* the number of words between pixels */
} XilMemoryStorageShort;

typedef struct __XilMemoryStorageFloat {
   float* data;                    /* pointer to the first float in the image */
   unsigned long scanline_stride;  /* the number of floats between scanlines */
   unsigned int pixel_stride;      /* the number of floats between pixels */
} XilMemoryStorageFloat;

typedef union __XilMemoryStorage {
   XilMemoryStorageBit   bit;
   XilMemoryStorageByte  byte;
   XilMemoryStorageShort shrt;
   XilMemoryStorageFloat flt;
} XilMemoryStorage;

/* include C-prototypes of XIL functions */
#include <xil/c_bindings.h>

#ifdef __cplusplus
}
#endif

#endif
