#!/usr/cluster/bin/cmx -f

##### Preferences

set maxUsers 8
set logFile /tmp/vfs.log

##### Globals
set hostname [exec /bin/hostname]
set startTime [systime]

#### Logging code

if [catch {open $logFile w} vfsLogFile] {
    puts stderr "Warning: couldn't open $logFile: $vfsLogFile"
    puts stderr "         logging to stderr"
    set vfsLogFile stderr
}

proc vfslog {args} {
    global vfsLogFile hostname startTime
    set t [expr [systime]-$startTime]
    puts $vfsLogFile "$hostname [format %1.2f $t]: $args"
    flush $vfsLogFile
}

vfslog Started vfs on [exec date]

#### Admission control

set numUsers 0
proc Vfs_Admit {} {
    global maxUsers numUsers rpcFile
    if {$numUsers >= $maxUsers} {
	  error "Sorry, too many users are currently logged in (only $maxUsers are allowed).  Please try again later."
    }
    incr numUsers
    dp_atclose $rpcFile append {incr numUsers -1}
    return 0
}

proc Vfs_LogConnection {inetAddr} {
    vfslog Accepting connection from $inetAddr
}

#### Security code
proc Vfs_Check {cmd args} {
    global rpcFile
    case $cmd in {
	CID_Init { Vfs_Admit; return }
	CMP_ExamineFile return
	CMTP_Translate return
	CMAppSetCleanUp return
	CMAppClearCleanup return
	CMApp_ReadMovie return
	dp_CloseRPCFile return
	objectTag return
	cmt* return
	eval {return -code continue}
	catch {return -code continue}
	if {return -code continue}
	mpegSegment return
	mjpegSegment return
	auSegment return
	mpegPriority return
	mjpegPriority return
	pktSrc return
	lts return
    }
    vfslog Attempt to execute illegal command "$cmd $args" from $rpcFile
    return -code break
}

dp_MakeRPCServer 1490 Vfs_LogConnection Vfs_Check
auto_load_all
# log open [open vfs-log w] g
# source trace.tcl
