.TH auPlay l

.SH NAME
\fBauPlay\fP - Create a new audio play object.

.SH SYNOPSIS
\fBauPlay\fP \fIparentName\fP \fR?options\fR?

.SH DESCRIPTION
The auPlay command create a new object which can be used to play
audio data the consists of a sequence of discrete samples (au format,
which is not the same as Sparc .au format).

An \fBauPlay\fP object cannot play sound until at least the \fIlts\fP
and \fIdevice\fP slots are set.  Once they are both set and the object
is ready, incoming data is played at the appropriate time using the
appropriate device.  A device is part of the \fBauPlay\fP object
responsible for actually sending audio data to a physical device and
playing it.  Devices currently available are a Sparc audio device
(\fI"sparc"\fP), an AudioFile device (\fI"af"\fP), a Windows-compatible
device (\fI"winaudio"\fP), and a Linux audio device (\fI"linux_audio"\fP). 
At compile
time, one of those devices is designated as the default and can be
referred to with the name \fI"default"\fP.

\fBauPlay\fP objects can be in one of two states: \fIready\fP or
\fIunready\fP. When first created the object is in the \fIunready\fP
state. The object does nothing in the \fIunready\fP state and all
buffers passed to it will be dropped. The object can be put into the
\fIready\fP state by calling the object's \fIready\fP command.  The
required slots (lts and device) must be set before calling the
\fIready\fP command.  Any call to the object's configure command that
sets a slot value automatically puts the object in the \fIunready\fP
state and the \fIready\fP command must be given again to put the
object in action. If the configure command was used to query a slot's
value, the \fIready\fP/\fIunready\fP state is unchanged.

.SH OPTIONS

.SS -gain
The output gain at which sound should be played, specified as a
floating point number between 0 and 1.  The object does not actually
use this value, but passes it to the device.
.SS -sampleRate
An integer specifying the number of samples per second.  This option
is also used only by the device.
.SS -channels
An integer specifying the number of channels of ouput.  Used only by
the device.
.SS -output
Destination for the sound.  Valid values are the strings
\fI"speaker"\fP and \fI"headphone"\fP.  Used only by the device.
.SS -sendAhead
How many seconds in advance of scheduled playback time to send audio data to
the device. This affects the amount of data buffered by the object. 
.SS -device
Should be set to the decoding device type. If \fI"default"\fP, the
compiled default device will be used. Current device types are:
\fI"sparc"\fP and \fI"af"\fP. This slot must be set for the object to
work. Attempting to issue the "ready" command to the object when this
slot has not yet been set will generate an error.
.SS -lts
Should be set to the logical time system object (lts) to be used for
synchronization.  This slot must be set for the resource object to
work. Attempting to issue the "ready" command to the object when this
slot has not yet been set will generate an error.

.SH WIDGET COMMANDS
The auPlay command creates a new command \fIobjectName\fP.
\fIObjectName\fP is generating by concatenating the value of
\fIparent\fP and a unique string generated internally.  The empty
string is a valid value for \fIparent\fP.  This new command,
\fIobjectName\fP, is used to invoke various operations on the new
object.

For more information on object names and why they are generated
by the cm process and not set by the user, please read the object
management overview for the cm toolkit.

The command \fIobjectName\fP has the following general form:

.RS
\fIobjectName\fP \fBoption\fP \fR?\fIarg arg arg...\fP\fR?
.RE

\fBOption\fP and the \fIargs\fP determine the exact behavior of the
command.  The following commands are available:

\fIobjectName\fP \fBconfigure\fP \fR?\fIoptions\fP\fR?
.RS
Query or modify the configuration options described above. If the object
was in the ready state (see the ready command below), the configure command
puts the object into the unready state, requiring the the ready command be
given again for data to start flowing.
.RE

\fIobjectName\fP \fBdestroy\fP
.RS 
Destroys the object. 
.RE

\fIobjectName\fP \fBready\fP
.RS
This command readies the object. The required slots (lts and device)
must be set.
.RE

\fIobjectName\fP \fBunready\fP
.RS
This command unreadies the object stopping data from being played.
.RE

\fIobjectName\fP \fBaccept\fP \fIscatter-buffer-list\fP
.RS
This command accepts \fIscatter-buffer-list\fP as au data to be
played.  A standard scatter buffer list accepting C interface to this
command is registered as \fIobjectName\fP.accept.
.RE

\fIobjectName\fP \fBdeviceList\fP
.RS
This command returns a list of devices that are legal for the configuration
slot -device.
.RE

\fIobjectName\fP \fBstats\fP
.RS
This command is not yet implemented but will eventually return a list
of statistics about the objects performance.
.RE

.SH KEYWORDS
cm, audio, audiofile, play, cm toolkit

.SH SEE ALSO
AF(1), audio(5), lts(l)

.SH AUTHORS
Andrew Swan (aswan@CS.Berkeley.EDU)
Jonathan Logan (logan@acm.org)

