.TH auRecord l

.SH NAME
\fBauRecord\fP - Create an audio recording object.

.SH SYNOPSIS
\fBauRecord\fP \fIparentName\fP \fR?options\fR?

.SH DESCRIPTION
The auRecord command create a new object which can be used to record
audio data that consists of a sequence of discrete samples (au format).

An \fBauRecord\fP object cannot record sound until the \fIlts\fP
and \fIdevice\fP slots are set. Audio data will not be recorded unless
the lts object's -speed slot is set to 1.0. Recording begins immediately
once the object is readied and the speed is set appropriately.
A device is the part of the \fBauRecord\fP object
responsible for recording audio from an input source 
(e.g. microphone or line-in). 
The only input device currently available is Sparc audio device
(\fI"sparc"\fP).  

\fBauRecord\fP objects can be in one of two states: \fIready\fP or
\fIunready\fP. When first created the object is in the \fIunready\fP
state.  The object can be put into the \fIready\fP state by calling the 
object's \fIready\fP command.  The
required slots (lts and device) must be set before calling the
\fIready\fP command. 

.SH OPTIONS

.SS -gain
The output gain at which sound should be recorded, specified as a
floating point number between 0 and 1.  The object does not actually
use this value, but passes it to the device.

.SS -sampleRate
An integer specifying the number of samples per second.  This option
is also used only by the device.

.SS -channels
An integer specifying the number of channels to be recorded.  Used only by
the device.

.SS -input
Input source for the sound.  Valid values are the strings
\fI"microphone"\fP and \fI"line_in"\fP.  Used only by the device.

.SS -delayTime
How many seconds after the time the sound started to be recorded to 
schedule playback. Note that this should be longer than the cycleTime.

.SS -cycleTime
How frequently to send the accumulated audio data. Determines buffer
requirements.

.SS -device
Should be set to the recording device type. If \fI"default"\fP, the
compiled default device will be used. Current device types are:
\fI"sparc"\fP. This slot must be set for the object to
work. Attempting to issue the "ready" command to the object when this
slot has not yet been set will generate an error.

.SS -format
Indicates the format of audio to be recorded. Valid choices are dependent
on the device type. With sparc audio devices, valid choices are "mu-law" and
"linear16". This slot is only used by the device and may not affect anything 
if the device does not support the format chosen.

.SS -lts
Should be set to the logical time system object (lts) to be used for
synchronization.  This slot must be set for the resource object to
work. Attempting to issue the "ready" command to the object when this
slot has not yet been set will generate an error. Currently, to record, the lts
-speed slot must be set to 1.0.

.SS -outCmd
Indicates the command to call when sending data. If the command begins with
a "@" symbol, an appropriate registered C command is looked up and used, otherwise
the command is evaluated as a Tcl command with a Tcl string representation of a
scatter buffer list appended as the final argument.

.SH WIDGET COMMANDS
The auRecord command creates a new command \fIobjectName\fP.
\fIObjectName\fP is generating by concatenating the value of
\fIparent\fP and a unique string generated internally.  The empty
string is a valid value for \fIparent\fP.  This new command,
\fIobjectName\fP, is used to invoke various operations on the new
object.

For more information on object names and why they are generated
by the cm process and not set by the user, please read the object
management overview for the cm toolkit.

The command \fIobjectName\fP has the following general form:

.RS
\fIobjectName\fP \fBoption\fP \fR?\fIarg arg arg...\fP\fR?
.RE

\fBOption\fP and the \fIargs\fP determine the exact behavior of the
command.  The following commands are available:

\fIobjectName\fP \fBconfigure\fP \fR?\fIoptions\fP\fR?
.RS
Query or modify the configuration options described above. If the object
was in the ready state (see the ready command below), the configure command
puts the object into the unready state, requiring the the ready command be
given again for data to start flowing.
.RE

\fIobjectName\fP \fBdestroy\fP
.RS 
Destroys the object. 
.RE

\fIobjectName\fP \fBready\fP
.RS
This command readies the object. The required slots (lts and device)
must be set. In order to record, the lts -speed must be 1.0.
.RE

\fIobjectName\fP \fBunready\fP
.RS
This command unreadies the object stopping data from being recorded.
.RE

\fIobjectName\fP \fBstats\fP
.RS
This command is not yet implemented but will eventually return a list
of statistics concerning the object's performance.
.RE

\fIobjectName\fP \fBdeviceList\fP
.RS
This command returns a list of valid device types.
.RE

.SH KEYWORDS
cm, audio, audiofile, play, cm toolkit

.SH SEE ALSO
audio(5), lts(l), auPlay(l)

.SH AUTHORS
Jonathan Logan (logan@acm.org)

