.TH auSegment New "November 7, 1995"
.SH NAME
.B auSegment
- Create a new au file segment object.
.SH SYNOPSIS
auSegment
.I parentName
.RI ? option ?
.SH OPTIONS
.SS -lts
Indicates which lts object to use for timing information. This option
must be set at some time before the segment object will do anything.
.SS -sendAhead
Indicates how many seconds in advance to send data. By default, this 
value is set at 0.2 seconds.
.SS -bandwidth
Indicates maximum number of bytes per second to send. If set to 0, no
maximum is enforced. For this option to work correctly, cycleTime must
not be non-zero. See cycleTime option below. In reality, this option isn't
really implemented and does not affect anything.
.SS -cycleTime
Indicates how many seconds worth of data to send at a time. If set to 0,
the object will send data one frame at a time and schedule itself to be
called again when the next frame needs to be sent.
.SS -filename
Indicates which file to use as a au file. This object can understand 
Sparc .au files as well as .acf audio chunk files (an augmented format we invented
that includes preparsed information). This option can be set only once.
.SS -outCmd
Indicates the command to call when sending data. This command will be appended
with a scatter buffer list and evaluated. A C interface to a scatter buffer list
accepting command can be specified using the @ syntax discussed in the application
programmer's guide.
.SS -clipStart 
Used to specify where in the clip the segment begins. If set to "?", this value is
computed from the other timing parameters as:
.br
clipStart = clipEnd - (logicalEnd - logicalStart)
.SS -clipEnd
Used to specify where in the clip the segment ends. If set to "end", indicates the
end of the clip. If set to "?", this value is computed from the other timing
parameters as:
.br
clipEnd = clipStart + (logicalEnd - logicalStart)
.SS -logicalStart
Used to specify when in logical time the segment should be sent. This value must
be less than the value for logicalEnd. If set to "?", this value is computed
from the other timing parameters as:
.br
logicalStart = logicalEnd - (clipEnd - clipStart)
.SS -logicalEnd
Used to specify when in logical time the segment should end. This value must
be greater than the value for logicalStart If set to "?", this value is 
computed from the other timing parameters as:
.br
logicalEnd = logicalStart + (clipEnd - clipStart)
.SH DESCRIPTION
The auSegment command creates a new au segment object which can be used
to read au data from a file and send the data to another object. Data is sent
using the standard scatter buffer list method and can be sent to any object that
can accept scatter buffer lists. The data is intended to be eventually received
by an auPlay object which would presumably play the data through speakers.
The au segment object expects the data to be stored on disk in the format of
an au chunk file. If this format is not true of the file specified, the object 
attempts to read the file as a SPARC audio file.
.SH WIDGET COMMANDS
The auSegment command creates a new command 
.IR objectName .
.I ObjectName
is generated by concatenating the value of
.I parent
and a unique string which is generated on the fly.
The empty string is a valid value for
.IR parent .
This new command, 
.IR objectName ,
is used to invoke various operations on the new object.

For more information on object names and why they are generated
by the cm process and not set by the user, please read the object
management overview for the cm toolkit.

The command
.I objectName
has the following general form:

.RS
.I
objectName option ?arg arg arg...?
.RE

.I Option
and the
.I args
determine the exact behavior of the command.
The following commands are possible:

.I objectName
configure 
.I option ?value option value ...?
.RS
Query or modify the configuration options described above. If the object
was in the ready state (see the ready command below), the configure command
puts the object into the unready state, requiring the the ready command be
given again for data to start flowing.
.RE

.I objectName
destroy
.RS 
Destroys the object. 
.RE

.I objectName
info
.I ?startTime ?endTime??
.RS
Returns a list of information about the interval specified by startTime and
endTime. If neither value is given, information is returned about the interval
defined by the attribute -clipStart and -clipEnd. If only startTime is given,
information is returned about the specific frame at the time given. All times
are taken as clip times (i.e. where 0.0 equals the clip beginning). The information
returned consists of a list of five items: {numFrames clipLength format samplingRate 
channelCount}. The numFrames number
indicates how many frames are in the interval; clipLength is the length in 
seconds of the file; format is the format of the data; samplingRate is the number
of samples per second; and channel count indicates the number of channels defined
in the file.
.RE

.I objectName
ready
.RS
This command readies the object. The lts is checked to see if data needs to be
sent. If so, data is sent and the object queues itself to send data according to
cycleTime. Before ready can be invoked, the -lts option
must be specified and -clipStart and -clipEnd must specify an inteval with length
greater than 0 (i.e., clipStart cannot equal clipEnd). The same is true for
-logicalStart and -logicalEnd.
.RE

.I objectName
unready
.RS
This command unreadies the object stopping data from being sent.
.RE

.SH KEYWORDS
cm, au, file, segment, acf, auSegment, .au, .acf, SPARC audio, cm toolkit
.SH AUTHORS
Ketan Patel (kpatel@cs.berkeley.edu)

