.TH buffer l

.SH NAME
buffer \- Tcl interface to CMT Buffer Manager

.SH SYNOPSIS
\fBbuffer\fP create \fR?options\fR?
.br
\fBbuffer\fP delete \fIbufferid\fP
.br
\fBbuffer\fP config \fIbufferid\fP \fR?options\fR?
.br
\fBbuffer\fP addtag \fIbufferid tag\fP
.br
\fBbuffer\fP dtag \fIbufferid tag\fP
.br
\fBbuffer\fP gettags \fIbufferid\fP
.br
\fBbuffer\fP find \fR?options\fR?
.br
\fBbuffer\fP findOrCreate \fR?options\fR?

.SH DESCRIPTION
The \fBbuffer\fP command provides a Tcl level interface to the CMT
Buffer Manager.  The various options are different buffer
management commands that can be run.
.PP

.SH OPTIONS

.SS buffer create \fR?options\fR?
Creates a new buffer configured with the given options
and returns the \fIbufferid\fP required by other buffer
manager commands.  Valid options are:

.RS
-size
.RS
The size of the buffer, in bytes
.RE

-shared
.RS
Whether the buffer should be in shared memory (if the
value is true) or the application heap (if the value is
false).  Default is false (application heap).
.RE

-xattach
.RS
Whether XShmAttach should be called for this buffer.
This option is only relevant for buffers in shared memory.
Default value is false.
.RE

-state
.RS
The state of the buffer (any character string).  Default
is the empty string.
.RE

-callback
.RS
The callback that should be executed to free the buffer when
the reference count reaches 0.  (See also \fBattach\fP and
\fBdetach\fP)
.RE

.RE

.SS buffer delete \fIbufferid\fP
Takes the \fIbufferid\fP of an existing
buffer and destroys the buffer, freeing all memory associated
with it.  Note that applications will not do this very often.
Rather, they should put a buffer in the state free, so that
another object can re-use the buffer later.
.PP

.SS buffer config \fIbufferid\fP \fR?options\fR?
Without any other arguments, the name, default value,
and current value of all configuration slots on the
buffer \fIbufferid\fP are printed.
With a single argument that specifies a configuration slot,
the default and current values of that slot are printed.
If more than one argument is present, the argument should be
pairs of slots and values.  Each slot is re-configured to
the given value.  Valid slots are those listed for the
\fBcreate\fP option.
.br
Most objects will not need to explicitly configure existing
buffers.  The \fBattach\fP and \fBdetach\fP facility should
be used for managing object state.  Other parameters should
not be modified since buffers may be shared between multiple
objects and other objects probably will not expect buffer
parameters to change unexpectedly.
.PP

.SS buffer addtag \fIbufferid tag\fB
Adds \fItag\fP to the list of tags kept for the buffer
\fIbufferid\fP.
.PP

.SS buffer dtag \fIbufferid tag\fP
If \fItag\fP is in the list of tags for \fIbufferid\fP, it
is removed from the list.  If it is not in the list, no
error is generated.
.PP

.SS buffer gettags \fIbufferid\fP
Returns a list of all the tags that have been added to
\fIbufferid\fP.
.PP

.SS buffer find \fR?options\fR?
Searches all the existing buffers for a buffer or buffers
matching the given parameters.  If the -unique option is
specified, a single bufferid is returned.  Otherwise, a
list of all matching bufferid's is returned.  Valid
parameters are:

.RS
-minsize
.RS
The minimum size of a buffer that should match, in bytes
.RE

-maxsize
.RS
The maximum size of a buffer that should match, in bytes
.RE

-shared
.RS
Whether buffers that match should be in shared memory (if
the value is true) or the application heap (if the value
is false).
.RE

-xattach
.RS
Whether buffers that match should be attached to the X server.
.RE

-state
.RS
Match only buffers in the given state.
.RE

-tag
.RS
Match only buffers that contain this tag
.RE

.RE


.SS buffer findOrCreate \fR?options\fR?
Operates in a similar fashion to the find command, but if
a matching buffer is not found, an appropriate buffer is
created, with the state set to free.  Valid options are the
same as those for the find command.

.SS buffer attach \fIbufferid\fP
Increments the reference count on the buffer identified by
\fIbufferid\fP.  If a
particular buffer is going to be passed between objects,
the first action that the object that creates the buffer
takes should be to attach to it.

.SS buffer detach \fIbufferid\fP
Decrements the reference count on the buffer identified by
\fIbufferid\fP.  When the count reaches 0, the buffer is
freed.  It is freed by calling the callback routine that set
through the configure command, or by setting the state to
free if a callback has not been set.

.PP

.SH KEYWORDS
cm, cmt, buffer

.SH AUTHOR
Andrew Swan (aswan@CS.Berkeley.EDU)
