.TH bufferCapture l

.SH NAME
bufferCapture \- Tcl interface to CMT Buffer Capture object

.SH SYNOPSIS
bufferCapture \fIparent\fB
.br
<obj> configure \fI[-outCmd [newCmd]] [-verbose [level]] [-filename [fileName]]\fP
.br
<obj> destroy
.br
<obj> accept \fIscatterBufferList\fP
.br
<obj> open
.br
<obj> close

.SH DESCRIPTION
The Tcl bufferCapture command creates a CMT buffer capture object and
returns the name of the new object.  BufferCapture objects are useful for
debugging CMT applications.  When inserted into a CMT graph, they print
certain information about every buffer that goes by before passing the
buffers on to the outCmd and possibly capture the contents of the buffers
into a specified file.  The degree of detail can be controlled using the
verbose flag. 

The name of the new object is internally generated, using the string
\fIparent\fP as the leading characters.  The returned name is the
name of a new Tcl command that can be used to send messages to the
object.  BufferCapture objects understand the following messages:

.SS \fIobject\fP \fBdestroy\fP
Destroys the buffer capture \fIobject\fP.
.PP

.SS \fIobject\fP \fBaccept\fP
Used to pass a scatter buffer list to the buffer capture \fIobject\fP.
A C interface to this function is registered as <objectName>.accept.
.PP

.SS \fIobject\fP \fBconfigure\fP \fR?-options\fR?
Sets configuration parameters for \fIobject\fP.  Valid options are:

.RS
\fB-outCmd \fIcmd\fR
.RS
Where to send data as it arrives.  \fICmd\fR should be a string
conforming to the usual CMT semantics for passing data
between objects.  That is, \fIcmd\fR should be either the name of a
Tcl procedure to be called every time data is available, or
a string of the form @\fIobject\fP.accept where \fIobject\fP
is some CMT object that will accept the data.
.RE

\fB-verbose \fIlevel\fR
.RS
Specifies the degree of detail with which buffers in the
scatter-buffer lists passed to the objects accept command
are printed.
If \fIlevel\fP is less than 1, no information is printed.
If \fIlevel\fP is 1 or greater, the buffer name of
every buffer in the scatter-buffer list and current time 
are printed.
If \fIlevel\fP is 2 or greater, the type and size of each chunk of
every buffer in the scatter-buffer list is printed.
If \fIlevel\fP is 3 or greater, detailed information about the headers
contained in each chunk of every buffer in the scatter-buffer
list  is printed.
The default value for \fIlevel\fP is 2.
.RE

\fB-lts \fIltsName\fR
.RS
Specifies the name of the LTS to use when printing time values.
If \fIltsName\fR is the empty string, time values are printed
using the value on the system time clock.  Otherwise, the current
time of the lts is used.  In combination with verbose level of 3 or
more, this option can be useful in determining if buffer are arriving
early or late.
.RE

\fB-filename \fIfileName\fR
.RS
Specifies the name of the file to write buffer data into. Writing into
the file does not commence until the open command is given (see below)
and is suspended when the close command is given. 
.RE
.RE

.SS \fIobject\fP \fBopen\fP
This command instructs the object to open the file specified by the configuration
option -filename in order to write the actual contents of the data buffers. Content
is written based on the stream type of the data. Currently only audio data is
fully supported although the object will capture data from any buffers generically
typed as CM_DATA_SEG. Since most streams (including mpeg and mjpeg) use this
generic type for non-header data, this object can still be useful. 
.PP

.SS \fIobject\fP \fBclose\fP
This command suspends output to the file.
.PP

.SH DATA PASSING
The buffer capture object obeys the CMT semantics for passing
data between objects.  To summarize, you can set the output
function of another object to be the string
@\fIobject.accept\fP where \fIobject\fP is the name of a
buffer capture object.  Then data produced by the other
object will automatically be handed to the buffer capture and
on to the object specified by the \fIoutCmd\fP. 


.SH SEE ALSO
cmwish(1), cmx(1)

.SH AUTHOR
Brian Smith (bsmith@cs.cornell.edu)
