.TH mjpegFile New "August Jan. 7, 1995"
.SH NAME
.B mjpegFile
- Create a new mjpeg file object.
.SH SYNOPSIS
mjpegFile
.I parentName 
.RI ? options ?
.SH OPTIONS
.SS -lts
Indicates which lts object to use for timing information. This option
must be set at some time before the file object will do anything.
.SS -sendAhead
Indicates how many seconds in advance to send data. By default, this 
value is set at 0.2 seconds.
.SS -bandwidth
Indicates maximum number of bytes per second to send. If set to 0, no
maximum is enforced. For this option to work correctly, cycleTime must
not be non-zero. See cycleTime option below.
.SS -cycleTime
Indicates how many seconds worth of data to send at a time. If set to 0,
the object will send data one frame at a time and schedule itself to be
called again when the next frame needs to be sent.
.SS -fileName
Indicates which file to use as a mjpeg file. If this file is not in
the correct mjpeg chunk file format, an attempt to read the file as a Parallax
XVideo .vid file and as a .avi file is made. If both these attempts fail, an
error is returned and no file is opened.
.SS -outCmd
Indicates the command to call when sending data. This command will be appended
with a scatter buffer list and evaluated. A C interface to a scatter buffer list
accepting command can be specified using the @ syntax discussed in the application
programmer's guide.
.SS -maxFrameRate
Used to specify a maximum sending frame rate. If set to 0, no maximum frame rate
is enforced. This option only works if cycleTime is non-zero (see cycleTime above).
.SH DESCRIPTION
The mjpegFile command creates a new mjpeg file object which can be used
to read mjpeg data from a file and send the data to another object. Data is sent
using the standard scatter buffer list method and can be sent to any object that
can accept scatter buffer lists. The data is intended to be eventually received
by a mjpegPlay object which would presumably display the frames in a window.
The mjpeg file object expects the data to be stored on disk in the format of
a mjpeg chunk file. If this format is not true of the file specified, the object 
attempts to read the file as a parallax j_movie, .vid, or J300 .avi file and parse the information normally
found in a chunk file on the fly. 
.SH WIDGET COMMANDS
The mjpegFile command creates a new command 
.IR objectName .
.I ObjectName
is generated by concatenating the value of
.I parent
and a unique string which is generated on the fly.
The empty string is a valid value for
.IR parent .
This new command, 
.IR objectName ,
is used to invoke various operations on the new object.

For more information on object names and why they are generated
by the cm process and not set by the user, please read the object
management overview for the cm toolkit.

The command
.I objectName
has the following general form:

.RS
.I
objectName option ?arg arg arg...?
.RE

.I Option
and the
.I args
determine the exact behavior of the command.
The following commands are possible:

.I objectName
configure 
.I option ?value option value ...?
.RS
Query or modify the configuration options described above. If the object
was in the ready state (see the ready command below), the configure command
puts the object into the unready state, requiring the the ready command be
given again for data to start flowing.
.RE

.I objectName
destroy
.RS 
Destroys the object. 
.RE

.I objectName
appendSegment
.I <clip_start> <clip_end> ?-start <logical_start>? ?-end <logical_end>?
.RS
Specifies what part of the file should be played and when. The values
<clip_start> and <clip_end> specify in what part of the file is to be
played. For example, the values 0.0 and 10.0 would indicate that the first
10 seconds of the file are to be played. The option -start and -end
parameters indicate when in terms of logical time (i.e. in terms of the 
lts set with the -lts configuration option) the segment should be played.
If neither -start or -end is indicated, the segment is assumed to be played
at logical time 0.0. If -start is indicated alone, the segment is set to
start at that time. If -end is indicated alone, the segment is set to play in
such a way as to end at that time. If both values are indicated, the segment
is set to start and end at the times specified with the appropriate stretching
or squeezing necesary to make the segment fit. This command returns a segment
id in the form of an integer that can be used to refer to the segment in other
command. If a segment is specified to play at a logical time during which a different
segment has already been set to play, an error is returned.
.RE

.I objectName
deleteSegment
.I <segment_id>
.RS
Removes the segment identified by <segment_id> from the list of segments
to be played.
.RE

.I objectName
clearSegments
.RS
Clears the list of segments to be played. Equivalent to deleting each 
segment individually with the deleteSegment command.
.RE

.I objectName
segmentInfo
.I <segment_id>
.RS
Returns a list of information about the segment <segment_id>. This list is in
the form:
.RS
{clip_start clip_end log_start log_end num_frames max_size}
.RE
where clip_start and clip_end specify which part of the file is being played,
log_start and log_end specify when the segment is being played according to logical
time, num_frames is the number of frames in the segment, and max_size is the largest
frame size in this segment in bytes.
.RE

.I objectName
info
.RS
Returns a list of information about the file in general. This list is
in the form:
.RS
{num_frames clip_length max_size width height}
.RE
where num_frames is the total number of frames in the file, clip_length is
the length of the file in seconds, max_size is the largest frame size in the 
file, width is the width of each frame, and height is the height of each frame.
If width or height is set to 0, this indicates that the value is undetermined (i.e.
variable or not known).
.RE

.I objectName
ready
.RS
This command readies the object. The lts is checked to see if data needs to be
sent. If so, data is sent and the object queues itself to send data according to
cycleTime and the segments specified. Before ready can be invoked, the -lts option
must be specified and at least one segment specified. If either of these are not
true, the ready command will effectively do nothing.
.RE

.I objectName
unready
.RS
This command unreadies the object stopping data from being sent.
.RE

.SH KEYWORDS
cm, mjpeg, file, mjpgc, mjpegFile, cm toolkit
.SH AUTHORS
Ketan Patel (kpatel@cs.berkeley.edu)

