.TH mpegPlay New "Feb. 10, 1995"
.SH NAME
.B mpegPlay
- Create a new mpeg play object.
.SH SYNOPSIS
mpegPlay
.I parentName 
.RI ? options ?
.SH OPTIONS
.SS -width
If set, should specify the width of the window where data is supposed to be
displayed. If not set (i.e. set to 0), the object assumes that the window is 
as big as the mpeg frames. This value is used when the image is being resized
to fit the display window (see the -sqeeze option).
.SS -height
If set, should specify the height of the window where data is being displayed.
If not set, the object assumes that the window is as big as the mpeg frames.
This value is used for centering the image in the display window. This value
is used when the image is being resized to fit the display window (see the -squeeze
option).
.SS -xpos
Specifies how many pixels to the left of the left side of the display window to
offset the image. If set to -1 and the squeeze option is not set, the object attempts
to center the image horizontally.
.SS -ypos
Specifies how many pixels below the top edge of the display window to offset the
image. If set to -1 and the squeeze option is not set, the object attempts
to center the image vertically.
.SS -squeeze
If non-zero, indicates to the resource to resize the image to the size of the
display window if possible. Not all devices will be able to support this.
.SS -xid
Should be set to the X window id of the display window. If the display window is
a Tk widget, this value can be generated with the "winfo id" command. This slot
must be set for the resource object to work. Attempting to issue the "ready" command
to the object when this slot has not yet been set will generate an error.
.SS -device
Should be set to the decoding device type. If "default", the compiled default
device will be used. Current device types are: "software". This slot
must be set for the resource object to work. Attempting to issue the "ready" command
to the object when this slot has not yet been set will generate an error. 
.SS -lts
Should be set to the logical time system object (lts) to be used for synchronization.
This slot
must be set for the resource object to work. Attempting to issue the "ready" command
to the object when this slot has not yet been set will generate an error. 
.SS -maxFrameRate
Indicates the maximum frame rate to display. If set to 0, no maximum is imposed.
Not currently implemented.

.SH DESCRIPTION
The mpegPlay command create a new mpeg play object which can be used to 
display mpeg frames in a window. This object requires that its lts, xid, and
device slots be filled. The object display frames in the window specified by
xid. Frame start and end times sent with the mpeg data are
compared to the current lts time. If the frame end time has already passed, the
frame is dropped (i.e. not displayed). If the frame start time has yet to arrive,
the frame is scheduled for future display. If the current time is in the range of
start to end time, the frame is displayed immediately. Whether a frame is decoded
in time for its display time is variable with the device used.
.br
MpegPlay objects can be in one of two states: ready or unready. When first
created the object is in the unready state. The object does nothing in the
unready state and all buffers passed to it will be dropped. The object can be put
into the ready state by calling the object's ready command. All of the required slots
(i.e. lts, xid, and device) must be set before calling the ready command. 
Any call to the
object's configure command that set a slot value
automatically puts the object in the unready state and
the ready command must be given again to put the object in action. If the configure
command was used to query a slot's value, the ready/unready state is unchanged.
.sp
Each mpegPlay object registers the following event types with the cmBind mechanism:
.RS
<FRAME_DROPPED>
.RS
Occurs whenever a frame that has been received is thrown away without being decoded.
.RE
<FRAME_RECEIVED>
.RS
Occurs whenever a frame is received.
.RE
<FRAME_PLAYED>
.RS
Occurs whenever a frame is displayed.
.RE
<FRAME_DECODE_BEGIN>
.RS
Occurs whenever a frame is first chosen for decoding.
.RE
<FRAME_DECODE_DONE>
.RS
Occurs whenever a frame completed decoding.
.RE
.RE
.sp
In addition the following substitution parameters are defined for scripts associated 
with any of these events:
.RS
%s -- logical start time
.br
%e -- logical end time
.br
%l -- length of frame data
.br
%w -- decoded frame width
.br
%h -- decode frame height
.br
%t -- type of frame (1 == I, 2 == P, 3 == B)
.br
%p -- index number of past reference frame
.br
%n -- index number of future reference frame
.br
%i -- index number of this frame
.RE
.SH WIDGET COMMANDS
The mpegPlay command creates a new command 
.IR objectName .
.I ObjectName
is generated by concatenating the value of
.I parent
and a unique string which is generated on the fly.
The empty string is a valid value for
.IR parent .
This new command, 
.IR objectName ,
is used to invoke various operations on the new object.

For more information on object names and why they are generated
by the cm process and not set by the user, please read the object
management overview for the cm toolkit.

The command
.I objectName
has the following general form:

.RS
.I
objectName option ?arg arg arg...?
.RE

.I Option
and the
.I args
determine the exact behavior of the command.
The following commands are possible:

.I objectName
configure 
.I option ?value option value ...?
.RS
Query or modify the configuration options described above. If the object
was in the ready state (see the ready command below), the configure command
puts the object into the unready state, requiring the the ready command be
given again for data to start flowing.
.RE

.I objectName
destroy
.RS 
Destroys the object. 
.RE

.I objectName
ready
.RS
This command readies the object. The required slots (i.e. lts, xid, and device)
must be validly set.
.RE

.I objectName
unready
.RS
This command unreadies the object stopping data from being decoded or displayed.
.RE

.I objectName
accept <scatter_buffer_list>
.RS
This command accepts <scatter_buffer_list> as mpeg data to be displayed. 
A standard
scatter buffer list accepting C interface to this command is registered as
<objectName>.accept.
.RE

.I objectName
stats
.RS
Returns a list of statistics about the device's performance. Exact
format of this list has not been worked out yet. Currently does nothing.
.RE

.I objectName
refresh
.RS
Causes the last frame to be displayed to be displayed again. This command
is useful to deal with expose events that occur in the display window.
.RE

.I objectName
deviceList
.RS
Returns a list of devices -- valid strings for the -device configuration
option.
.RE


.SH KEYWORDS
cm, mpeg, mjpegPlay, play, cm toolkit
.SH AUTHORS
Ketan Patel (kpatel@cs.berkeley.edu)

