.TH pktSrc l

.SH NAME
pktSrc \- Packet Source Object

.SH SYNOPSIS
pktSrc \fIprefix\fB \fR?options\fR?

.SH DESCRIPTION
The pktSrc command creates a new packet source object, used
to transmit data using the cyclic-UDP protocl.  For a detailed
description of the protocol, which is useful in understanding
the various configuration slots and how they affect the behavior
of the packet objects, see [Smith 94].
.PP
The name of the new object (which is created by appending a
unique suffix to \fIprefix\fP) is returned.  This name is
also the name of a Tcl command that can be used with the
following options.
.PP

.SH OPTIONS

.SS \fIobject\fP \fBdestroy\fP
Destroys the packet source named \fIobject\fP.
.PP

.SS \fIobject\fP \fBconfigure\fB \fR?-options\fR?
Sets configuration parameters for the packet source named
\fIobject\fP.  Valid options are:

.RS
-state
.RS
The current state of \fIobject\fP.  It may be either idle,
sending, eoc, or partition, depending on what the object is
currently doing.
.RE

-cycleId
.RS
The number of the current cycle.
.RE

-pktSize
.RS
The size of the packets that the data is broken in to for
sending, in bytes.
.RE

-burstSize
.RS
Ideal size of each burst, in bytes.  The packet source will
try to send this much data at each burst, sleeping between
bursts.
.RE

-maxBurstSize
.RS
The maximum amount of data to send in one burst, in bytes.
The packet source will always sleep after sending this much
data in one burst.
.RE

-targetDelayMsec
.RS
Target end-to-end delay between the packet source and packet
dest, in milliseconds.
.RE

-xpctdDrpRate
.RS
The expected percentage of UDP packets that will be dropped
on a scale of 0-255 where 0 = 0% and 255 = 100%.
.RE

-maxBW
.RS
The maximum bandwidth required, in bytes per second.
.RE

-minBW
.RS
The minimum bandwidth required, in bytes per second.
.RE

-skewMsec
.RS
Estimate of the skew between the clocks on the sending and
receiving machines, in milliseconds.
.RE

-ltBW
.RS
Long term estimate of the bandwidth available between sender
and receiever, in bytes per second.
.RE

-epochDuration
.RS
Duration of each epoch, in milliseconds.
.RE

-estBW
.RS
Estimated bandwidth available bewteen sender and receiver,
in bytes per second.
.RE

-dest
.RS
Where to find the corresponding packet dest object to
send the data to.  This option, unlike the others, takes
two arguments, a hostname and a port number.
.RE

-priorityGroup
.RS
Specifies the priority group this object is part of.  This
is usually one of tk, audio, or video.  By default, an object
is part of the tk priority group.
.RE

.RE

.SS \fIobject\fP \fBnewCycle\fP
Starts a new cycle.  Any data from the previous cycle that
did not get through is discarded (the buffers are detached).
Any buffers added with the addBuffers command become part
of the new cycle which begins when startCycle is called.
.PP

.SS \fIobject\fP \fBaddBuffers\fP \fIscatter-buffer-list\fP
Adds a list of scatter buffers to the active list of buffers
to be sent as part of the current cycle.
.PP

.SS \fIobject\fP \fBstartCycle\fP
Starts sending the data that is part of the current cycle.
If \fIobject\fP is already sending data, the command is
ignored.
.PP

.SH DATA PASSING
The packet source object obeys the CMT semantics for passing
data between objects.  To summarize, you can set the output
function of another object to be the string
@\fIobject\fP.accept where \fIobject\fP is the name of a
packet source object.  Then data produced by the other
object will automatically be handed to the packet source and
send to the packet dest.  Essentially, the new cycle, add
buffers, and start cycle commands are all called
immediately.

.SH KEYWORDS
packet, network

.SH SEE ALSO
pktSrc(l), buffer(l), cmt(l), cmwish(1), cmx(1)

.SH REFERENCES
[Smith94] Brian C. Smith, "Implementation Techniques for Continuous Media
Systems and Applications," Ph. D. Dissertation, University of California,
Berkeley, CA., Sept 1994

.SH AUTHORS
Andrew Swan (aswan@CS.Berkeley.EDU)
.br
Brian Smith (bsmith@cs.cornell.edu)
