.TH rtpDest l

.SH NAME
\fBrtpDest\fP \- RTP Dest Object

.SH SYNOPSIS
\fBrtpDest\fP \fIprefix\fP \fR?options\fR?

.SH DESCRIPTION
The \fBrtpDest\fP command creates a new RTP dest object, used
to receive data using the Real-Time Transport Protocl.  For a
detailed description of the protocol, which is useful in
understanding the various configuration slots and how they affect
the behavior of the RTP objects, see [Schulzrinne 95].
.PP
The name of the new object (which is created by appending a
unique suffix to \fIprefix\fP) is returned.  This name is
also the name of a Tcl command that can be used with the
following options.
.PP

.SH OPTIONS

.SS \fIobject\fP \fBdestroy\fP
Destroys the packet dest named \fIobject\fP.
.PP

.SS \fIobject\fP \fBmembers\fP
Returns a list containing the SDES CNAMEs of all other users who
have sent a valid control or data packet to this session.
.PP

.SS \fIobject\fP \fBconfigure\fP \fR?-options\fR?
Sets configuration parameters for \fIobject\fP.  Valid options are:

.RS
\fB-outCmd\fP
.RS
Where to send data as it arrives.  This should be a string
conforming to the usual CMT semantics for passing data
between objects.  That is, it should be either the name of a
Tcl procedure to be called every time data is available, or
a string of the form @\fIobject\fP.accept where \fIobject\fP
is some CMT object that will accept the data.
.RE

\fB-address\fP
.RS
The IP address being listened to.  Only multicast addresses are
currently supported, and they must be written in dot notation.
.RE

\fB-port\fP
.RS
The port number on which data packets are sent and received.  It
should be an even number; control packets are implicity sent and
received on the next port number.
.RE

\fB-cname\fP
.RS
A string to be used as the CNAME in SDES packets.  Default is
username@hostname.  Can only be set after \fBaddress\fP and
\fBport\fP are set.
.RE

\fB-name\fP
.RS
A string to be used as the NAME in SDES packets.  Default is
taken from user's finger info.  Can only be set after \fBaddress\fP
and \fBport\fP are set.
.RE

\fB-pg\fP
.RS
Specifies the priority group this object is part of.  This
is usually one of tk, audio, or video.  By default, an object
is part of the tk priority group.  Can only be set after \fBaddress\fP
and \fBport\fP are set.
.RE

\fB-ttl\fP
.RS
Multicast Time-to-Live (TTL) in every outgoing control packet.
Default is 2.  Can only be set after \fBaddress\fP and \fBport\fP
are set.

.RE

.SH OUTPUT
An \fIrtpDest\fP object can send received packets to other CMT objects
using the regular CMT buffer passing semantics.  However, since a
single \fIrtpDest\fP object can receive data from multiple senders,
multiple outCmd configuration slots are needed.  The syntax is similar
to the usual \fB-outCmd\fP configuration syntax, with the addition of a
a parenthesized SDES CNAME.  For example, if \fIobjName\fP is an
\fBrtpDest\fP object, the following command will cause all packets from
\fIuser@host.domain.edu\fP to be sent to the CMT out command
\fI@obj2.accpet\fP.

.br
\fBobjName\fP configure -outCmd(\fIuser@host.domain.edu\fP) \fI@obj2.accept\fP

.SH CM BINDINGS
An \fIrtpDest\fP object provides several events that can be bound
via cmbind(l).  In the callbacks for these events, several escape
sequences are available:

.RS
\fB%c\fP
.RS
SDES CNAME of the user generating the event.
.RE

\fB%n\fP
.RS
SDES NAME of the user generating the event, or the CNAME if no
NAME has been seen.
.RE

\fB%e\fP
.RS
Encoding of packets (the PT field of the RTP header) send by this
user.  Only relevant for \fB<NEW_SENDER>\fP and \fB<ENCODING_CHANGED>\fP.
.RE

.RE

The events are:

.RS
\fB<NEW_SENDER>\fP
.RS
Generated whenever some session participant sends their first data
packet to the session.  This could be a user just joining the group,
or a who has already sent control packets (and thus generated a
\fI<NEW_LISTENER>\fP event) beginning to send data.
.RE

\fB<NEW_LISTENER>\fP
.RS
A new user has joined the session, but has only sent control
packets.
.RE

\fB<NAME_CHANGED>\fP
.RS
A user's SDES NAME has changed.
.RE

\fB<MEMBER_LEFT>\fP
.RS
The given user has quit the session.
.RE

\fB<ENCODING_CHANGED>\fP
.RS
The value of the PT field of the RTP header has changed.
.RE

.RE

.SH SEE ALSO
rtpSrc(l), cmbind(l), buffer(l), cmt(l), cmwish(1), cmx(1)

.SH REFERENCES
[Schulzrinne 95] Henning Schulzrinne, et. al., "RTP: A Transport
Protocol for Real-Time Applications," Internet Engineering Task Force
Internet-Draft, March 1995

.SH AUTHORS
Andrew Swan (aswan@CS.Berkeley.EDU)
