.TH rtpSrc l

.SH NAME
rtpSrc \- RTP Source Object

.SH SYNOPSIS
rtpSrc \fIprefix\fB \fR?options\fR?

.SH DESCRIPTION
The rtpSrc command creates a new RTP source object, used to
transmit data using the Real-Time Transport Protocol.  For
a detailed description of the protocol, which is useful in
understanding the various configuration slots and how they
affect the behavior of the packet objects, see [Schulzrinne 95].
.PP
The name of the new object (which is created by appending a
unique suffix to \fIprefix\fP) is returned.  This name is
also the name of a Tcl command that can be used with the
following options.
.PP

.SH OPTIONS

.SS \fIobject\fP \fBdestroy\fP
Destroys the packet source named \fIobject\fP.
.PP

.SS \fIobject\fP \fBconfigure\fB \fR?-options\fR?
Sets configuration parameters for the packet source named
\fIobject\fP.  Valid options are:

.RS
\fB-address\fP
.RS
The IP address packets should be sent to.  Only multicast
addresses are currently supported, and they must be written in
dot notation.
.RE

\fB-port\fP
.RS
The port number on which data packets should be sent.  It should
be an even number; control packets are implicity sent and received
on the next highest port number.
.RE

\fB-cname\fP
.RS
A string to be used as the CNAME in SDES packets.  Default is
username@hostname.  Can only be set after \fBaddress\fP and
\fBport\fP are set.
.RE

\fB-name\fP
.RS
A string to be used as the NAME in SDES packets.  Default is
taken from user's finger info.  Can only be set after \fBaddress\fP
and \fBport\fP are set.
.RE

\fB-pg\fP
.RS
Specifies the priority group this object is part of.  This
is usually one of tk, audio, or video.  By default, an object
is part of the tk priority group.  Can only be set after \fBaddress\fP
and \fBport\fP are set.
.RE

\fB-ttl\fP
.RS
Multicast Time-to-Live (TTL) in every outgoing data and control
packet. Default is 2.  Can only be set after \fBaddress\fP and
\fBport\fP are set.

.RE

.SH INPUT
An \fBrtpSrc\fP object register an atom @\fIobjName\fP.accept so
that it can be the target of the out command of another object.
It takes the buffers it receives and sends them out on the
appropriate multicast address.

.SH SEE ALSO
rtpDest(l), buffer(l), cmt(l), cmwish(1), cmx(1)

.SH REFERENCES
[Schulzrinne 95] Henning Schulzrinne, et. al., "RTP: A Transport
Protocol for Real-Time Applications," Internet Engineering Task Force
Internet-Draft, March 1995

.SH AUTHORS
Andrew Swan (aswan@CS.Berkeley.EDU)
