/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */



/*
 * $Id: cmAuFileInterface.h,v 1.3 1996/02/06 22:40:01 kpatel Exp $
 *
 * Defines the common interface for reading au audio files
 * used by the auSegment object.
 */

#include "cmCookies.h"
#include "cmAuFile.h"
#include "cmChunk.h"

/* 
 * The following structure is kept per sparc audio file being accessed
 * through a segment object.
 */

typedef struct AuDataFile {
    char *filename;                    /* Name of file. */
    tChunkFileState  *cfState;         /* Chunk file state pointer. */
    SparcAuHeader *hdr;                /* Sparc Audio Header structure. */
    MediaIndepHeader *mediaHdr;        /* CM Media Independent Header. */
    int sampleSize;                    /* Samples per second. */
    int dataRate;                      /* Bytes per second. */
    double clipLength;                 /* Length of clip in seconds. */
    int dataStart;                     /* Offset to samples from beginning of file. */
    int numFrames;                     /* Number of frames. Often just 1. */
    int fd;                            /* File descriptor for data. */
    RawSparcAuHeader *rawHdr;          /* Raw header used for raw audio data. */ 
    int alignSize;                     /* Byte alignment size. */
    int raw_flag;                      /* Indicates that file is a raw audio file. */
    double *FTT;                       /* Frame time table. */
    int *FOT;                          /* Frame offset table. */
    int file_refCount;                 /* Number of segment objects that use
					  this file interface structure. */
    int open_refCount;                 /* Number of segment objects that think
					  the file is currently cached open. */
} AuDataFile;

/*
 * The following structure is kept per segment object.
 */ 

typedef struct AuFileInterface {
    int is_open;                 /* Indicates that this segment objects
				    believes that file is already open. */
    AuDataFile *data_file;       /* Pointer to underlying file structure
				    defined above. */
} AuFileInterface;


/* Declarations for Audio File Interface Routines. */

AuFileInterface * Au_FIP_Open(char *fname); 
void Au_FIP_Close(AuFileInterface *fip);
void Au_FIP_FDClose(AuFileInterface *fip);
void Au_FIP_FDOpen(AuFileInterface *fip);
int Au_FIP_NumFrames(AuFileInterface *fip, double start, double end);
int Au_FIP_ChannelCount(AuFileInterface *fip);
int Au_FIP_DataFormat(AuFileInterface *fip);
double Au_FIP_ClipLength(AuFileInterface *fip);
int Au_FIP_SamplingRate(AuFileInterface *fip);
Buffer *Au_FIP_ReadAndResample(Tcl_Interp *interp, AuFileInterface *fip,
			       int frameNum, double start, double end, 
			       double delta, double speed, 
			       int *dataLengthPtr);




