/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


#ifndef _SpeakerRsrc_
#define _SpeakerRsrc_

#include <stdio.h>
#include <tcl.h>
#include <tk.h>
#include "cmCookies.h"
#include "cmBufferlist.h"

struct SpeakerRsrc;

/*
 * This structure is used for queued data.
 */
typedef struct CBInfo {
    struct SpeakerRsrc *objPtr;	/* The SpeakerResource Object */
    SparcAuBufferHeader hdr;	/* The frame header */
    char *data;			/* The frame data */
    int len;			/* Number of bytes of data */
    int at_token;		/* Index into resource at_token table */
    double start;		/* Logical start time */
    double end;			/* Logical end time */
    Buffer *buffer;		/* Pointer to buffer where data came
    				 * from (needed for Detach) */
} CBInfo;

typedef struct SpeakerDevice {
  ClientData deviceData;
  int (*initProc)(struct SpeakerRsrc *objPtr);
  int (*reinitProc)(struct SpeakerRsrc *objPtr);
  int (*shutdownProc)(struct SpeakerRsrc *objPtr);
  int (*unreadyProc)(struct SpeakerRsrc *objPtr);
  int (*ltsProc)(struct SpeakerRsrc *objPtr);
  int (*playProc)(CBInfo *bcPtr);
  char *(*errorProc)(struct SpeakerRsrc *objPtr, int errNum);
} SpeakerDevice;

typedef struct SpeakerRsrc {
    Tcl_Interp *interp;		/* Tcl interpreter that created object. */
    char *name;			/* Tcl string name of this object. */

    SpeakerDevice *devicePtr;	/* Device control structure. */
    char *devtype;		/* String for device type (e.g. "sparc") */
    int inited;			/* Have we called the device initProc? */
    int ready;
    
    int sampleRate;
    int channels;
    int format;        /* SUN_ULAW or SUN_LIN_16, currently */
    double gain;		/* What's the current gain (0..1).  Must
				   call reinitProc if changed */
    int headphone;		/* Where are we sending the output.  Must
				   call reinitProc if changed */

    ClientData lts;		/* LTS for object */
    int ltsTriggerNum;
    char *ltsName;

	double sendAhead;     /* How much ahead of the audio stream we try to
							 stay (seconds) - default 0.2 */

    int *at_tokens;             /* Tokens on the At Queue */
    int num_at_tokens;		/* Size of at token list. */

    char *acceptHandle;		/* Atom registered for accept procedure */
} SpeakerRsrc;

/*
 * Devices currently supported
 */
extern SpeakerDevice *GetSparcDevice(SpeakerRsrc *objPtr);
extern SpeakerDevice *GetLinuxAuDevice(SpeakerRsrc *objPtr);
extern SpeakerDevice *GetIrixAuDevice(SpeakerRsrc *objPtr);
extern SpeakerDevice *GetAFDevice(SpeakerRsrc *objPtr);
extern SpeakerDevice *GetWinAudioDevice(SpeakerRsrc *objPtr);

#ifdef AF_DEFAULT
#  define GetDefaultSpeakerDevice	GetAFDevice
#  define DEFAULT_AU_PLAY_DEV_TYPE "af"
#elif SPARC_AU_DEFAULT
#  define GetDefaultSpeakerDevice	GetSparcDevice
#  define DEFAULT_AU_PLAY_DEV_TYPE "sparc"
#elif LINUX_AU_DEFAULT
#  define GetDefaultSpeakerDevice	GetLinuxAuDevice
#  define DEFAULT_AU_PLAY_DEV_TYPE "linux"
#elif WIN_AUDIO_DEFAULT
#  define GetDefaultSpeakerDevice	GetWinAudioDevice
#  define DEFAULT_AU_PLAY_DEV_TYPE "winaudio"
#else
#  define GetDefaultSpeakerDevice(x)	NULL
#  define DEFAULT_AU_PLAY_DEV_TYPE ""
#endif

#endif /* _SpeakerRsrc_ */
