/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


#ifndef _AudioInRsrc_
#define _AudioInRsrc_

#include <stdio.h>
#include <tcl.h>
#include <tk.h>
#include "cmCookies.h"
#include "cmBufferlist.h"

struct AudioInRsrc;

/*
 * This structure is used for queued data.
 */
typedef struct AuInBufInfo {
    struct AudioInRsrc *objPtr;	/* The audio input Resource Object */
    SparcAuBufferHeader hdr;	/* The frame header */
    Buffer *bufPtr;		        /* Pointer to buffer with data */
    int len;			        /* Length of buffer */
	int dlen;                   /* length of actual audio data */
	int offset;                 /* where in buffer actual audio data starts */
    double start;		        /* Logical start time */
    double end;			        /* Logical end time */
} AuInBufInfo;

/* device dependent functions */
typedef struct AudioInDevice {
  ClientData deviceData;
  int (*initProc)(struct AudioInRsrc *objPtr);
  int (*reinitProc)(struct AudioInRsrc *objPtr);
  int (*shutdownProc)(struct AudioInRsrc *objPtr);
  int (*unreadyProc)(struct AudioInRsrc *objPtr);
  int (*ltsProc)(struct AudioInRsrc *objPtr);
  int (*recordProc)(AuInBufInfo *bcPtr);
  char *(*errorProc)(struct AudioInRsrc *objPtr, int errNum);
} AudioInDevice;

typedef struct AudioInRsrc {
    Tcl_Interp *interp;		/* Tcl interpreter that created object. */
    char *name;			/* Tcl string name of this object. */

    AudioInDevice *devicePtr;	/* Device control structure. */
    char *devtype;		/* String for device type (e.g. "sparc") */
    int inited;			/* Have we called the device initProc? */
    int ready;
    
	int sampleSize;   /* size of each audio sample, in bytes (typ. 1-2)*/
	
    int sampleRate;
    int channels;
    int format;
    double gain;		/* What's the current gain (0..1).  Must
						   call reinitProc if changed */
    int inputPort;		/* Where are we recording from?  Must
						   call reinitProc if changed */
#define CM_LINE_IN        (1)
#define MICROPHONE     (2) 

    ClientData lts;		/* LTS for object */
    int ltsTriggerNum;
    char *ltsName;

	InFunc outCCmd;      /* C function called to pass along data */
	ClientData outCD;    /* Object data is sent to */
	char *outCmd;        /* Tcl command used for sending out data */
	
	SB_List *sbl;         /* bufferlist for data */

	double cycleTime;     /* how often data (a frame) is 
							   transmitted (in seconds) */ 
	double delayTime;    /* added to current logical time to timestamp frame */

    int    atToken;           /* Token on the At Queue */
    double atWhen;		/* Size of at token list. */

} AudioInRsrc;

/*
 * Devices currently supported
 */
extern AudioInDevice *GetSparcAuInDevice(AudioInRsrc *objPtr);

#ifdef SPARC_AU_DEFAULT
#  define GetDefaultAudioInDevice	GetSparcAuInDevice
#  define DEFAULT_AU_RECORD_DEV_TYPE "sparc"
#else
#  define GetDefaultAudioInDevice(x)	NULL
#  define DEFAULT_AU_RECORD_DEV_TYPE ""
#endif

#endif /* _AudioInRsrc_ */


