/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */



#ifndef _BUFFER_H_
#define _BUFFER_H_

#define	BUFFER_DONT_MODIFY	(-2)
#define	BUFFER_DONT_CARE	(-1)

#ifndef NO_X11
#ifdef HAVE_SHARED_MEMORY
#include <X11/extensions/XShm.h>
#endif
#endif

/*
 * One of these structs is kept for each buffer
 */
typedef struct {
    int id;			/* Current ID of this buffer */
    char **tags;		/* Tags attached to this buffer */
    int tagSize;		/* Number of tag slots allocated */
    int tagCount;		/* Number of tag slots used */
    char *state;		/* Current state of the buffer */
    int size;			/* Total size of buffer */
    int shared;			/* Is it in shared memory? */
    int key;			/* shared memory key */
    int xAttach;		/* Is it attached to an X server? */
    int shmId;			/* Id of the shared memory segment */
    int refCount;		/* Reference Count */
    char *callback;		/* Callback to release */
    char *data;			/* The actual data */
#ifndef NO_X11
    Display *dpy;		/* Pointer to X display if any */
# ifdef HAVE_SHARED_MEMORY
    XShmSegmentInfo shminfo;	/* Shared memory id, etc... */
# endif
#endif
} Buffer;

typedef int (*bufCallback)(Buffer *buffer);

/*
 * Struct used when reconfiguring buffers
 */
typedef struct {
    int size;
    int shared;
    int xAttach;
    char *state;
    char *callback;
} ConfigureInfo;

/*
 * Struct used for find queries
 */

typedef struct {
    int minsize;		/* Minimum size */
    int maxsize;		/* Maximum size */
    int unique;			/* Generate on unique answer? */
    char *state;		/* In which state */
    int shared;			/* In shared memory? */
    int xAttach;		/* Attached to an X server? */
    char *tag;			/* Tag to match */
} FindQuery;

/*
 * Prototypes:
 */
int Tcm_BufferInit	(Tcl_Interp *interp);
int CreateBuffer	(Tcl_Interp *interp, ConfigureInfo *info);
Buffer *MakeExistingBuffer(Tcl_Interp *interp, ConfigureInfo *info,
			   char *mem, int size);
Buffer *MakeExistingShmBuffer(Tcl_Interp *interp, ConfigureInfo *info,
			      char *addr, int shmid, int size);
void DestroyBuffer	(Tcl_Interp *interp, Buffer *buffer);
int ConfigureBuffer	(Tcl_Interp *interp, Buffer *buffer,
			 ConfigureInfo *info);
int *FindBuffer		(Tcl_Interp *interp, FindQuery *query);
int FindOrCreateBuffer	(Tcl_Interp *interp, FindQuery *query);
int AddTag		(Tcl_Interp *interp, Buffer *buffer, char *tag);
void DeleteTag		(Buffer *bufPtr, char *tag);
Buffer *GetBufferFromID	(int id);
int GetBuffer		(Tcl_Interp *interp, char *string, Buffer **bufPtr);
int AttachBuffer	(Tcl_Interp *interp, Buffer *buffer, char *debugStr);
int DetachBuffer       	(Tcl_Interp *interp, Buffer *buffer, char *debugStr);

#endif /* _BUFFER_H_ */
