/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


#ifndef CM_BUFFERLIST_H

#include <tcl.h>
#include <tk.h>
#include "cmBuffer.h"

typedef struct {
        int count;
        Buffer **buffers;
        int *offsets;
        int *lens;
} ScatterBuffer;

typedef struct {
        int count;
        ScatterBuffer **sbs;
} SB_List;

typedef int (*InFunc)(void *objPtr, SB_List *data);
typedef int (*OutFunc)(void *objPtr, SB_List **data);

extern Buffer *MakeBuffer(Tcl_Interp *interp, int size, char *debugStr);
extern Buffer *MakeSharedBuffer(Tcl_Interp *interp, int size, char *debugStr);
extern SB_List *MakeSBList();
extern ScatterBuffer *MakeScatterBuffer();
extern SB_List *CopySBList(SB_List *sb_list);
extern ScatterBuffer *CopyScatterBuffer(ScatterBuffer *sb);
extern void FreeScatterBuffer(ScatterBuffer *sb);
extern void FreeSBList(SB_List *sb_list);
extern char *MakeSBListString(SB_List *sb_list);
extern int AttachScatterBuffer(Tcl_Interp *interp, ScatterBuffer *sb,
			      char *debugStr);
extern int DetachScatterBuffer(Tcl_Interp *interp, ScatterBuffer *sb,
			      char *debugStr);
extern int AttachToAllBuffers(Tcl_Interp *interp, SB_List *sb_list,
			      char *debugStr);
extern int DetachToAllBuffers(Tcl_Interp *interp, SB_List *sb_list,
			      char *debugStr);
extern void MergeSBLists(SB_List *mainSB, SB_List *subSB);
extern char *GetFromSB(ScatterBuffer *sb, int type, Buffer **bufPtr, int *sizePtr);

extern int ParseSBList(Tcl_Interp *interp, char *str, SB_List *sbl);

#define CM_BUFFERLIST_H

#endif
