/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


/*----------------------------------------------------------------
 * File:    chunk.h
 * {c2doc AUTHOR Eric14@cs.berkeley.edu}
 *
 * {c2doc DESCRIPTION}
 *    Provides a simple IFF (an existing chunkfile standard)
 *    like chunk-file abstraction.  Chunk files consist of a short
 *    header and a sequence of chunks.  
 *
 *    Each chunk consists of a twelve byte header, an optional
 *    variable length string and a data section.  A chunk header
 *    contains the string and data size and the type ID of the chunk.
 *    The type ID is a simple 32bit unsigned int (but it is usually
 *    viewed as a sequence of four characters.  The data section is
 *    simply binary data encapsilated within the chunk.  The string
 *    can be used to store chunk naming information (or anything
 *    else).  The hope is that it will prove usefull when building Tcl
 *    interfaces to chunk files.
 */
 

#if !defined(_CHUNK_H_)
#define _CHUNK_H_

typedef int   int32;

typedef struct
{
    int32   	type;
    int32   	chunkSize;
    int32   	stringSize;
} tChunkHeader;

typedef struct
{
    int   	fd;
    int32    	readHeaderStart;
    int32	writeHeaderStart;
    int32   	chunkSize;
    int32   	stringSize;
    int		readIndex;
    char  	writeFlag;
    char	createFlag;
} tChunkFileState, *tChunk;


#define kChunk_EndOfFile	-1414
#define kChunk_Error		-1415
#define kChunk_ErrIndexRange	-1416


tChunkFileState*  Chunk_OpenFile(char* name, char* options);
int    Chunk_CloseFile(tChunkFileState* cf);
int    Chunk_WriteHeader(tChunkFileState* cf, int32 type, char* tagStr);
int    Chunk_ReadHeader(tChunkFileState* cf, tChunkHeader* hdr);
char*  Chunk_ReadString(tChunkFileState* cf, char* outBuf);
void   Chunk_Type2Str(char* outStr, int32 type);
int32  Chunk_Str2Type(char* str);
int    Chunk_FindIndex(tChunkFileState* cf, int idx, tChunkHeader* hdr);
int    Chunk_FindType (tChunkFileState* cf, int32 type, tChunkHeader* hdr);
int    Chunk_ReadRewind (tChunkFileState* cf);


#define C_TYPE(a,b,c,d)   	 ((a<<24)|(b<<16)|(c<<8)|d)

#endif /* !defined(_CHUNKS_H_) */












