/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


/*
 * tcmConfig.h --
 *
 *	Declarations for Tcm_ConfigSpec's.
 *
 *
 */

#ifndef _TCM_CONFIG_H
#define _TCM_CONFIG_H

#ifndef _TCL
#include <tcl.h>
#endif
#ifndef _TK
#include <tk.h>
#endif
#ifndef _XLIB_H
#include <X11/Xlib.h>
#endif

/*
 * Structure used to specify information for Tcm_ConfigureObject.  Each
 * structure gives complete information for one option, including
 * how the option is specified on the command line.
 */

typedef struct Tcm_ConfigSpec {
    int type;			/* Type of option, such as TCM_CONFIG_COLOR;
				 * see definitions below.  Last option in
				 * table must have type TCM_CONFIG_END. */
    char *argvName;		/* Switch used to specify option in argv.
				 * NULL means this spec is part of a group. */
    char *defValue;		/* Default value for option if not
				 * specified in command line. */
    int offset;			/* Where in object record to store value;
				 * use Tk_Offset macro to generate values
				 * for this. */
    int specFlags;		/* Any combination of the values defined
				 * in tk.h. */
    Tk_CustomOption *customPtr;	/* If type is TK_CONFIG_CUSTOM then this is
				 * a pointer to info about how to parse and
				 * print the option.  Otherwise it is
				 * irrelevant. */
} Tcm_ConfigSpec;

/*
 * Type values for Tcm_ConfigSpec structures.  See the user
 * documentation for details.
 */

#define TCM_CONFIG_BOOLEAN	1
#define TCM_CONFIG_INT		2
#define TCM_CONFIG_DOUBLE	3
#define TCM_CONFIG_STRING	4
#define TCM_CONFIG_UID		5
#define TCM_CONFIG_COLOR	6
#define TCM_CONFIG_FONT		7
#define TCM_CONFIG_BITMAP	8
#define TCM_CONFIG_BORDER	9
#define TCM_CONFIG_RELIEF	10
#define TCM_CONFIG_CURSOR	11
#define TCM_CONFIG_ACTIVE_CURSOR	12
#define TCM_CONFIG_JUSTIFY	13
#define TCM_CONFIG_ANCHOR	14
#define TCM_CONFIG_SYNONYM	15
#define TCM_CONFIG_CAP_STYLE	16
#define TCM_CONFIG_JOIN_STYLE	17
#define TCM_CONFIG_PIXELS	18
#define TCM_CONFIG_MM		19
#define TCM_CONFIG_CUSTOM	20
#define TCM_CONFIG_END		21

/*
 * Macro to use to fill in "offset" fields of Tcm_ConfigSpec structures.
 * Computes number of bytes from beginning of structure to a
 * given field.
 */

#define Tcm_Offset(type, field) ((int) ((char *) &((type *) 0)->field))

/*          WOW ----------------^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 */

/*
 * Possible values for flags argument to Tcm_ConfigureObject:
 * Possibly ignored in Tcm_ConfigureObject, by the way.
 */

#define TCM_CONFIG_ARGV_ONLY	1

/*
 * Possible flag values for Tcm_ConfigSpec structures.  Any bits at
 * or above TCM_CONFIG_USER_BIT may be used by clients for selecting
 * certain entries.  Before changing any values here, coordinate with
 * tcmConfig.c (internal-use-only flags are defined there).
 */

#define TCM_CONFIG_COLOR_ONLY		1
#define TCM_CONFIG_MONO_ONLY		2
#define TCM_CONFIG_NULL_OK		4
#define TCM_CONFIG_DONT_SET_DEFAULT	8
#define TCM_CONFIG_OPTION_SPECIFIED	0x10
#define TCM_CONFIG_USER_BIT		0x100

extern int		Tcm_ConfigureInfo (Tcl_Interp *interp,
			    Tcm_ConfigSpec *specs,
			    char *objRec, char *argvName, int flags);
extern int		Tcm_ConfigureObject (Tcl_Interp *interp,
			    Tcm_ConfigSpec *specs,
			    int argc, char **argv, char *objRec, int flags);
#endif /* _TCM_CONFIG_H */
