/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


/* This file contains the two functions necessary for the
   Clientdata custom configuration option used for various
   things like LTS's, etc. It assumes only one thing: that the
   name of the Clientdata that was registered using tcmAtom
   follow directly after the Clientdata slot in the widget record.
*/

#include "tk.h"


int 
ClientData_ParseProc (ClientData clientData,
		      Tcl_Interp *interp,
		      Tk_Window *tkwin,
		      char *value,
		      char *widgRec,
		      int offset)
{
  ClientData new_data;

  if ((strlen(value) == 0) || (strcmp(value, "NULL") == 0)) {
    (ClientData) (widgRec+offset) = NULL;
    if ((char *) (widgRec+offset+sizeof(ClientData)) != NULL)
      ckfree((char *) (widgRec+offset+sizeof(ClientData)));
    (char *) (widgRec+offset+sizeof(ClientData)) = (char *) ckalloc(1);
    *((char *) widgRec+offset+sizeof(ClientData)) = '\0';
    return TCL_OK;
  }
   
  new_data = Tutil_FindAtom(interp, value);

  if (new_data == NULL) {
    Tcl_AppendResult(interp, "No clientdata associated with name: ", value, (char *) NULL);
    return TCL_ERROR;
  }

  (ClientData) (widgRec+offset) = new_data;
  if ((char *) (widgRec+offset+sizeof(ClientData)) != NULL) 
    ckfree((char *) (widgRec+offset+sizeof(ClientData)));
  (char *) (widgRec+offset+sizeof(ClientData)) = (char *) malloc(strlen(value)+1);
  strcpy((char *) (widgRec+offset+sizeof(ClientData)), value);
  
  return TCL_OK;
}

char *
ClientData_PrintProc(
		     ClientData clientData,
		     Tk_Window tkwin,
		     char *widgRec,
		     int offset,
		     Tcl_FreeProc **freeProcPtr)
{
  char *ret_val;

  return (char *) (widgRec+offset+sizeof(ClientData));
}


  
		      
