/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


/*
 * $Id: cmCosmoDevice.h,v 1.3 1996/02/06 22:40:44 kpatel Exp $
 *
 * Defines a device to work with the CM MjpegPlay object to
 * decode a motion JPEG stream with the Cosmo card.
 *
 */

#define GRAPHICS               10
#define VIDEO_1_FIELD          11
#define VIDEO_2_FIELD          12

#define PLAY_STEP 0
#define PLAY_CONT 1

#include <strings.h>
#include <unistd.h>	/* geteuid(), getuid(), etc. */

/*
 * for schedctl
 */
#include <limits.h>
#include <sys/types.h>
#include <sys/prctl.h>
#include <sys/schedctl.h>


/*
 * X include files
 *
 */
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/keysym.h>

/*
 * dmdev include files
 */
#include <dmedia/cl.h>
#include <dmedia/cl_cosmo.h>
#include <dmedia/dm_image.h>
#include <vl/vl.h>

#include <vl/dev_ev1.h>

#include <X11/Xlib.h>

typedef struct {
    int isTrack;  			/*is there a video ('image') track?*/
    int height;
    int width;
    DMinterlacing interlacing;
    DMorientation orientation;
    int display;
    int numberFrames;
    double frameRate;
} _Image;

typedef struct {
    CLcompressorHdl Hdl;
    int OriginalFormat;
    int engine;
    int singleFrame;
} _Codec;

typedef struct {
    VLServer svr;
    VLPath path;		/* video data path from Cosmo */
    VLNode src;			/* from Cosmo */
    VLNode drn;			/* screen out */
    VLNode voutdrn;		/* video out */
    VLNode dataTiming;		/* timing portion of data path */
    VLNode devNode;
    int    dataActive;		/* vlBeginTransfer on data path? */
    int    dataFrozen;		/* has VL_FREEZE been called? */

    VLNode timingSrc;		/* Timing path */
    VLNode timingDrn;
    VLPath timingPath;
    int    timingActive;	/* is the timing path currently active? */
} _Video;


extern void clClose(Tcl_Interp*);
