/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */



#ifndef _H261PLAY_H_
#define _H261PLAY_H_

#include <tcl.h>
#include "cmBufferlist.h"

/*
 * Masks for elements in h.261 options
 */

/*
 * early alpha versions of vic 2.7 (up to a31 so far...) reverse
 * the MBAP and GOBN fields in the h.261/rtp header.
 */
#define BROKEN_VIC
#ifdef BROKEN_VIC

#define H261_SBIT(x)	(((x) >> 29) & 0x7)
#define H261_EBIT(x)	(((x) >> 26) & 0x7)
#define H261_INTRA(x)	(((x) >> 25) & 0x1)
#define H261_MV(x)	(((x) >> 24) & 0x1)
#define H261_MBAP(x)	(((x) >> 19) & 0x1f)
#define H261_GOBN(x)	(((x) >> 15) & 0xf)
#define H261_QUANT(x)	(((x) >> 10) & 0x1f)
#define H261_HMVD(x)	(((x) >> 5) & 0x1f)
#define H261_VMVD(x)	((x) & 0x1f)

#else

#define H261_SBIT(x)	((x) >> 29)
#define H261_EBIT(x)	(((x) >> 26) & 0x7)
#define H261_INTRA(x)	(((x) >> 25) & 0x1)
#define H261_MV(x)	(((x) >> 24) & 0x1)
#define H261_GOBN(x)	(((x) >> 20) & 0xf)
#define H261_MBAP(x)	(((x) >> 15) & 0x1f)
#define H261_QUANT(x)	(((x) >> 10) & 0x1f)
#define H261_HMVD(x)	(((x) >> 5) & 0x1f)
#define H261_VMVD(x)	((x) & 0x1f)

#endif BROKEN_VIC

struct H261Device;

typedef struct H261Packet {
    double start;
    double end;
    ClientData deviceData;
    
    Buffer *buf;
    unsigned char *data;
    int len;
    int options;		/* Options as in draft-ietf-avt-h261 */
    int atToken;
    
    struct H261Packet *prev;
    struct H261Packet *next;
} H261Packet;

typedef struct {
    Tcl_Interp *interp;		/* Tcl Interpreter where cmd is registered */
    char *name;			/* Name of the object (and Tcl command) */
    
    struct H261Device *devicePtr;
    char *devtype;
    int inited;			/* Has the device been initialized? */
    int ready;			/* Has the object itself been readied? */

    int indecode;		/* Currently decoding? */
    H261Packet *decodePkt;	/* Packet to be decoded */
    H261Packet *displayPkt;	/* Packet to be displayed */
    
    char *ltsName;		/* Info about LTS */
    ClientData lts; 
    int ltsTriggerNum;

    H261Packet *qFirst;		/* Queue of packets to decode */
    H261Packet *qLast;
    int queueLen;
    int enqueued;		/* Number of packets in the queue */
    int *atTokens;
    int numAtTokens;		/* At events we own */
    
    char *acceptHandle;		/* Token for accept routine */
    int xid;			/* X id for window to play to */
} H261Play;

typedef struct H261Device {
    ClientData deviceData;
    int (*initProc)(H261Play *objPtr);
    int (*reinitProc)(H261Play *objPtr);
    int (*shutdownProc)(H261Play *objPtr);
    int (*decodeProc)(H261Play *objPtr, H261Packet *pkt);
    int (*displayProc)(H261Play *objPtr, H261Packet *pkt);
    int (*refreshProc)(H261Play *objPtr, H261Packet *pkt);
    int (*freePktProc)(H261Play *objPtr, H261Packet *pkt);
    int (*configProc)(H261Play *objPtr, char *option, char *value);
    char *(*errorProc)(H261Play *objPtr, int errNum);
} H261Device;

/*
 * Devices currently supported
 */
extern H261Device *GetSwH261Device(H261Play *objPtr);

#define	GetDefaultH261Device	GetSwH261Device

#endif _H261PLAY_H_
