/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


#ifdef _WINDOWS
#    define WIN32_LEAN_AND_MEAN
#    include <windows.h>
#    include <winsock.h>
#    undef WIN32_LEAN_AND_MEAN
#endif /* _WINDOWS */

#ifdef NO_STDLIB_H
#  include "compat/stdlib.h"
#else
#  include <stdlib.h>
#endif

#include <sys/time.h>

#include <tcl.h>
#include <tk.h>
#include "cmEvery.h"
#include "tcmEvent.h"
#include "cmAtom.h"
#include "cmAuUtil.h"

#define CM_VERSION "3.0"
#define CM_PATCH_LEVEL "3.0a5"

extern int Tcm_LogCmd				(ClientData notUsed,
						 Tcl_Interp *interp,
						 int argc, char **argv);
extern int Tcm_LtsCmd				(ClientData notUsed,
						 Tcl_Interp *interp,
						 int argc, char **argv);
extern int Tcm_BufferCmd			(ClientData clientData,
						 Tcl_Interp *interp,
						 int argc, char **argv);
extern int Tcm_ObjectTagCmd			(ClientData clientData,
						 Tcl_Interp *interp,
						 int argc, char **argv);

extern int Tcm_TraceInit			(Tcl_Interp *interp);
extern int InitCMBindModule                     (Tcl_Interp *interp);
extern int UdpSourceModuleInit			(Tcl_Interp *interp);
extern int UdpDestModuleInit			(Tcl_Interp *interp);
extern int PacketSourceModuleInit		(Tcl_Interp *interp);
extern int PacketDestModuleInit			(Tcl_Interp *interp);
extern int SparcAudioClipFileModuleInit		(Tcl_Interp *interp);
extern int AuFileSegmentModuleInit              (Tcl_Interp *interp);
extern int MJPEG_CFModuleInit 			(Tcl_Interp *interp);
extern int MJPEGSegmentModuleInit               (Tcl_Interp *interp);
extern int MJPEGResourceModuleInit 		(Tcl_Interp *interp);
extern int YUVResourceModuleInit 		(Tcl_Interp *interp);
extern int SpeakerRsrcModuleInit 		(Tcl_Interp *interp);
extern int BufferPrinterModuleInit  		(Tcl_Interp *interp);
extern int MJPEG_PriorityModuleInit 		(Tcl_Interp *interp);
extern int MPEG_CFModuleInit                    (Tcl_Interp *interp);
extern int MPEG_DemuxModuleInit                    (Tcl_Interp *interp);
extern int MPEG_SysFileModuleInit               (Tcl_Interp *interp);
extern int MPEGSegmentModuleInit                (Tcl_Interp *interp);
extern int MPEG_PriorityModuleInit		(Tcl_Interp *interp);
extern int MPEGPlayModuleInit                   (Tcl_Interp *interp);
extern int vq_CFModuleInit			(Tcl_Interp *interp);
extern int VQResourceModuleInit			(Tcl_Interp *interp);
extern int AuPriorityModuleInit                 (Tcl_Interp *interp);
extern int MJPEGCaptureModuleInit               (Tcl_Interp *interp);
extern int H261PlayModuleInit			(Tcl_Interp *interp);
extern int YUVFilterModuleInit                  (Tcl_Interp *interp);

#ifdef USE_RTP
extern int RTPDestModuleInit			(Tcl_Interp *interp);
extern int RTPSourceModuleInit			(Tcl_Interp *interp);
extern int RTPJpegFragModuleInit		(Tcl_Interp *interp);
extern int RTPJpegDefragModuleInit		(Tcl_Interp *interp);
extern int RTPH261DefragModuleInit		(Tcl_Interp *interp);
#endif

/*--------------------------------------------------
 *
 * CM_Init --
 *
 *      Initializes all modules in cm libraries.
 * 
 *--------------------------------------------------*/

#ifndef CMT_ROOT
#   define CMT_ROOT "/usr/local/lib/cmt"
#endif


int
CM_Init(Tcl_Interp *interp, char* startCmd)
{
#ifndef _WINDOWS
    struct timeval time;
    struct timezone tz;
#endif
    
    static char initCmd[] = 
	"if [file exists $cmp_library/init.tcl] {\
	    source $cmp_library/init.tcl\n\
	} else {\n\
	    set msg \"can't find $cmp_library/init.tcl; perhaps you \"\n\
	    append msg \"need to set your CMT_ROOT \"\n\
	    append msg \"environment variable?\"\n\
	    error $msg\n\
	}";
    char *libDir;
    Tcl_DString dString;

    if (Tcl_GetVar(interp, "cmp_library", TCL_GLOBAL_ONLY) == NULL) {
	/*
	 * A library directory hasn't already been set, so figure out
	 * which one to use.
	 */

	libDir = getenv("CMT_ROOT");
#ifdef _WINDOWS
	if (libDir == NULL) {
	    extern char *Tknt_GetRegistryString(char *, char *);
	    libDir = Tknt_GetRegistryString("SOFTWARE\\" APPLICATION_NAME, "CMT_ROOT");
	}
#endif
	if (libDir == NULL) {
	    libDir = CMT_ROOT;
	}
	Tcl_SetVar(interp, "cmt_root", libDir, TCL_GLOBAL_ONLY);
	Tcl_DStringInit(&dString);
	Tcl_DStringAppend(&dString, libDir, -1);
	Tcl_DStringAppend(&dString, "/cmp", -1);
	Tcl_DStringAppend(&dString, CM_VERSION, -1);
	Tcl_SetVar(interp, "cmp_library", Tcl_DStringValue(&dString),
		   TCL_GLOBAL_ONLY);
	Tcl_DStringFree(&dString);

	Tcl_DStringInit(&dString);
	Tcl_DStringAppend(&dString, libDir, -1);
	Tcl_DStringAppend(&dString, "/cmapp", -1);
	Tcl_DStringAppend(&dString, CM_VERSION, -1);
	Tcl_SetVar(interp, "cmapp_library", Tcl_DStringValue(&dString),
		   TCL_GLOBAL_ONLY);
	Tcl_DStringFree(&dString);
    }
    Tcl_SetVar(interp, "cm_version", CM_VERSION, TCL_GLOBAL_ONLY);
    Tcl_SetVar(interp, "cm_patchLevel", CM_PATCH_LEVEL, TCL_GLOBAL_ONLY);

    if (Tcl_Eval(interp, initCmd) != TCL_OK) {
	fprintf(stderr,"## Error finding CM_Init Scripts\n");
	fprintf(stderr,"##  Perhaps your CMT_ROOT variable\n");
	fprintf(stderr,"##  is not properly set?\n%s\n", interp->result);
	return TCL_ERROR;
    }

    Tcm_CreatePriorityGroup ("audio");
    Tcm_CreatePriorityGroup ("video");
    Tcm_CreatePriorityGroup ("background");

    Tcl_GlobalEval (interp, "priority move audio after tk");
    Tcl_GlobalEval (interp, "priority move video after audio");
    Tcl_GlobalEval (interp, "priority move background after video");

    Tutil_AtomInit(interp);
    Tcm_AuUtilInit(interp);
    Tcm_TraceInit(interp);


    Tcl_CreateCommand(interp, "log", Tcm_LogCmd,
		      (ClientData) NULL, (void (*) ()) NULL);
    Tcl_CreateCommand(interp, "lts", Tcm_LtsCmd,
		      (ClientData) NULL, (void (*) ()) NULL);
    Tcl_CreateCommand(interp, "buffer", Tcm_BufferCmd,
		      (ClientData) NULL, (void (*) ()) NULL);
    Tcl_CreateCommand(interp, "objectTag", Tcm_ObjectTagCmd,
		      (ClientData) NULL, (void (*) ()) NULL);
    Tcl_CreateCommand(interp, "every", Tcm_EveryCmd,
		      (ClientData) NULL, (void (*) ()) NULL);

#ifndef _WINDOWS
#ifdef USE_RTP
    /*
     * Seed the random number generator
     * (used by RTP object)
     */
    gettimeofday(&time, &tz);
    srand48(time.tv_sec);
#endif
#endif
    
    /*
     * Initialize CM modules
     */
    SparcAudioClipFileModuleInit(interp);
    AuFileSegmentModuleInit(interp);
    MJPEG_CFModuleInit(interp);
    MPEGSegmentModuleInit(interp);
    MJPEGSegmentModuleInit(interp);
    YUVFilterModuleInit(interp);
    MPEG_CFModuleInit(interp);
    MPEG_PriorityModuleInit(interp);

    MPEG_SysFileModuleInit(interp);
    MPEG_DemuxModuleInit(interp);

#if HAVE_INFOPAD
    VQResourceModuleInit(interp);
#endif

#ifndef _WINDOWS
    vq_CFModuleInit(interp);
#endif

    BufferPrinterModuleInit(interp);
    UdpSourceModuleInit(interp);
    PacketSourceModuleInit(interp);
    MJPEG_PriorityModuleInit(interp);
    UdpDestModuleInit(interp);
    PacketDestModuleInit(interp);
    
#ifdef USE_RTP
    RTPDestModuleInit(interp);
    RTPSourceModuleInit(interp);
    RTPJpegFragModuleInit(interp);
    RTPJpegDefragModuleInit(interp);
    RTPH261DefragModuleInit(interp);
#endif

    InitCMBindModule(interp);

    BufferCaptureModuleInit(interp);
    AuPriorityModuleInit(interp);
    AuReassemModuleInit(interp);

#ifndef NO_X11
    AudioInRsrcModuleInit(interp);
    MJPEGResourceModuleInit(interp);
    YUVResourceModuleInit(interp);
    SpeakerRsrcModuleInit(interp);
    MPEGPlayModuleInit(interp);
    MJPEGCaptureModuleInit(interp);
    H261PlayModuleInit(interp);
#endif

    if (Otcl_Init(interp) != TCL_OK) {
	fprintf(stderr, "Otcl_Init failed: %s\n", interp->result);
	return TCL_ERROR;
    }
    if (Tcl_Eval(interp, "source $cmapp_library/customize.otcl")) {
	fprintf(stderr,"## Could not find library file: customize.otcl\n");
	fprintf(stderr,"##  Perhaps your CMT_ROOT variable\n");
	fprintf(stderr,"##  is not properly set?\n%s\n", interp->result);
	return TCL_ERROR;
    }
    if (Tcl_Eval(interp, startCmd) != TCL_OK) {
	fprintf(stderr,"## Start command failed: %s\n", startCmd);
	fprintf(stderr,"##  Perhaps your CMT_ROOT variable\n");
	fprintf(stderr,"##  is not properly set?\n%s\n", interp->result);
	return TCL_ERROR;
    }

    return TCL_OK;
}
