/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


/*
 * $Id: cmJ300Device.c,v 1.16 1996/02/06 22:41:18 kpatel Exp $
 *
 * Implements the interface to the J300 device for the mjpegPlay
 * object, used to do hardware assisted jpeg decoding (using
 * the mme server to talk to the j300, not the jv2driver)
 *
 */

#ifdef _WINDOWS
#    define WIN32_LEAN_AND_MEAN
#    include <windows.h>
#    include <winsock.h>
#    undef WIN32_LEAN_AND_MEAN
#endif /* _WINDOWS */

#include <stdio.h>
#ifdef NO_STDLIB_H
#    include "compat/stdlib.h"
#else
#    include <stdlib.h>
#endif
#include <tcl.h>
#include <tk.h>
#include "cmMJPEGPlay.h"


#if J300

#include "cmJ300Device.h"
#include "cmUtil.h"

#ifdef PARALLAX_GRABBED_INPUT
#define DEMO
#endif

#ifdef DEMO
#  define JFIF_HEADER_SIZE 195
#else
#  define JFIF_HEADER_SIZE 187
#endif


/* defns */
MJPEGDevice *GetMJPEGDevice		(MJPEGResource *objPtr);
static int J300ActualDecode		(MJPEGFrame *frameptr);
int J300Decode				(MJPEGFrame *frameptr);
int J300Display				(MJPEGFrame *frameptr);
int J300FreeFrame			(MJPEGFrame *frameptr);
int J300Refresh				(MJPEGResource *objPtr);
int J300Init				(MJPEGResource *objPtr);
int J300Reinit				(MJPEGResource *objPtr);
int J300Shutdown			(MJPEGResource *objPtr);
MJPEGDevice *  GetJ300Device		(MJPEGResource *objPtr);
char * J300Error			(MJPEGResource *objPtr, int errNum);
static int SetupIOHeaders		(J300ClientData *jdevice);
static int GetDecompressDevice		(J300ClientData *jdevice);
static int AllocateJ300Buffers		(J300ClientData *jdevice,int flag);
static int FreeJ300Buffers		(J300ClientData *jdevice,int flag);
int DecodeVidFrame			(MJPEGFrame *framePtr);
int DecodeJfifNoHuffmanFrame		(MJPEGFrame *framePtr);
static void CreateJfif			(char *buf, int width,
					 int height, int qfactor);
void mymmcMapImage 			(Display *dpy, Visual *visual, GC gc, Window window,
	          			int Lines, int PixelsPerLine, int depth, XShmSegmentInfo *shminfo,
	          			char * address, int noxshm, int xoff, int yoff);

#define _PixelWidth(bits) (bits == 8 ? 1 : bits == 24 ? 4 : 2)
#define SOI sizeof(int)
#define _ComputePaddedWidth(width,bits)	\
	(SOI*((_PixelWidth(bits)*width+SOI-1)/SOI))

#define ComputeBufSize(width, height,bits) \
	(_ComputePaddedWidth(width,bits) * height)


/*
 *--------------------------------------------------------------
 *
 * J300Decode --
 *
 *	Decode interface procedure for J300 device.  
 *      To be compatible with the parallax object, this call does not
 *	do the actual decoding. It initializes the decompression
 *	device if it has not already been inited. (as frame
 *	width and height can only be got from the frame).
 *      The Display function decodes and displays the frame.
 *
 * Results:
 *	0 for success
 *
 * Side effects:
 *	None.
 *
 * Parameters:
 *	objPtr - pointer to the resource object data structure
 *
 *--------------------------------------------------------------
 */
int
J300Decode ( MJPEGFrame *framePtr)
{
    MJPEGResource *objPtr;
    J300ClientData *jdev;
    int newW,newH,newQ;
    MMRESULT status;
    int ret;

    objPtr = framePtr->objPtr;
    jdev = (J300ClientData *) objPtr->devicePtr->deviceData;

    /*
     * if it is the first frame,can allocate the buffers and 
     * init the decompression device
     */
    if (jdev->inited != 1) {
	return 0;
    }

    if(jdev->decompressDevInited != 1) {
	jdev->inputW =  framePtr->frameHeader.width;
	jdev->inputH =  framePtr->frameHeader.height;
	if((!jdev->squeeze) || (jdev->w == 0) || (jdev->h == 0)) {
		/*
		 * output frame size is same as input
		 */
		jdev->outputW =  framePtr->frameHeader.width;
		jdev->outputH =  framePtr->frameHeader.height;
    		jdev->lpbiOut->biSizeImage = ComputeBufSize(
			jdev->lpbiOut->biWidth,
			jdev->lpbiOut->biHeight,jdev->screenDepth);
	}
	if( (ret = AllocateJ300Buffers(jdev,INPUT_AND_OUTPUT)) != 0)
		return ret;
	if( (ret=OpenDecompressionDevice(jdev)) != 0)
		return ret;
	jdev->decompressDevInited = 1;
    }
    return 0;
}


/*
 *--------------------------------------------------------------
 *
 * J300ActualDecode --
 *
 *	This function actually decodes the given frame.
 *
 * Results:
 *	0 for success
 *
 * Side effects:
 *	None.
 *
 * Parameters:
 *	objPtr - pointer to the resource object data structure
 *
 *--------------------------------------------------------------
 */
static
int
J300ActualDecode ( MJPEGFrame *framePtr)
{
    MJPEGResource *objPtr;
    J300ClientData *jdev;
    int newW,newH,newQ;
    MMRESULT status;
    int ret;

    objPtr = framePtr->objPtr;
    jdev = (J300ClientData *) objPtr->devicePtr->deviceData;
    if(jdev->inited != 1)
	    return 0;

   /*
    * chk if the frame W or H has changed. If yes, will need to
    * close and reopen the decompression device. 
    * However, do not need to reallocate the input buffer
    * if the squeeze flag is not set, then the output frame size
    * is set to the input frame size. Will need to reallocate the
    * output buffer.
    */
    newW = framePtr->frameHeader.width;
    newH = framePtr->frameHeader.height;

    if( (jdev->inputW != newW) || (jdev->inputH != newH)) {
	CloseDecompressionDevice(jdev);	

	jdev->inputW = newW;
	jdev->inputH = newH;

    	jdev->lpbiIn->biWidth = jdev->inputW;
    	jdev->lpbiIn->biHeight = jdev->inputH;

	if((!jdev->squeeze )  || (jdev->w == 0) || (jdev->h == 0)){
		jdev->outputW =  framePtr->frameHeader.width;
		jdev->outputH =  framePtr->frameHeader.height;
		jdev->lpbiOut->biWidth =  framePtr->frameHeader.width;
		jdev->lpbiOut->biHeight =  framePtr->frameHeader.height;
    		jdev->lpbiOut->biSizeImage = ComputeBufSize(
			jdev->lpbiOut->biWidth,
			jdev->lpbiOut->biHeight,jdev->screenDepth);
		FreeJ300Buffers(jdev,ONLY_OUTPUT);
		if( (ret = AllocateJ300Buffers(jdev,ONLY_OUTPUT)) != 0)
			return ret;
	}
	if( (ret = OpenDecompressionDevice(jdev)) != 0)
		return ret;

	/*
	 * set flag to recreate the jfif header if it is needed
	 */
	 jdev->jfifInited = 0;
    }

     /*
      * Decode for correct sub format.  Produces a sub-image that can be 
      * displayed using mmcMapImage() call below.
      */
    switch(framePtr->frameHeader.subformat) {
      case VID_FRAME_CODE:
	ret = DecodeVidFrame( framePtr);
	break;
      case JFIF_FRAME_CODE:
	break;
      case JFIF_NO_HUFFMAN_FRAME_CODE:
	ret = DecodeJfifNoHuffmanFrame( framePtr );
	break;
      case RAW_JPEG_NO_HEADERS_CODE:
	ret = DecodeRawFrame( framePtr);
	break;
    }
    return ret;
}





/*
 *--------------------------------------------------------------
 *
 * DecodeVidFrame --
 *
 *	Call the hardware to decode a frame in VID_FRAME_CODE format.
 *
 * Results:
 *	0 for success, error code otherwise
 *
 * Side effects:
 *
 * Parameters:
 *	MJPEGFrame *framePtr	Frame to be decoded.
 *
 *--------------------------------------------------------------
 */
int
DecodeVidFrame( MJPEGFrame *framePtr)
{
    MJPEGResource *objPtr;
    J300ClientData *jdev;
    int newQ;
    MMRESULT status;

    objPtr = framePtr->objPtr;
    jdev = (J300ClientData *) objPtr->devicePtr->deviceData;
    if (jdev->inited != 1) {
	return 0;
    }
    newQ = framePtr->frameHeader.qfactor;

    /*
     */
    if ((jdev->jfifInited == 0)|| (jdev->qfactor != newQ) ) {
	jdev->qfactor = newQ;
	CreateJfif(jdev->jfifHeader,jdev->inputW,jdev->inputH,newQ);	
	jdev->jfifInited = 1;

    }

    /*
     * Copy data into buffer leaving 607 bytes for the jfif_header
     * and excluding the 20 bytes parallax header
     */
    memcpy (jdev->lpInput[0], jdev->jfifHeader, JFIF_HEADER_SIZE);
    memcpy (&(jdev->lpInput[0][JFIF_HEADER_SIZE]), framePtr->data+20,
	   framePtr->dataLength-20);
    jdev->lpbiIn->biSizeImage = JFIF_HEADER_SIZE + framePtr->dataLength - 20;
    status = ICDecompress (jdev->hic, (int )NULL, jdev->lpbiIn, jdev->lpInput[0],
			   jdev->lpbiOut, jdev->lpOutput[0]);


    if (status != ICERR_OK) {
	mmcErrorDisplay( "ICDecompress failed, status = %d", status);
	return J300_ICDECOMPRESS_FAILED;
    }
    jdev->decompressed = 1;
    return 0;
}


/*
 *--------------------------------------------------------------
 *
 * DecodeRawFrame --
 *
 *	Call the hardware to decode a frame in RAW_JPEG_NO_HEADERS_CODE format.
 *
 * Results:
 *	0 for success, error code otherwise
 *
 * Side effects:
 *
 * Parameters:
 *	MJPEGFrame *framePtr	Frame to be decoded.
 *
 *--------------------------------------------------------------
 */
int
DecodeRawFrame( MJPEGFrame *framePtr)
{
    MJPEGResource *objPtr;
    J300ClientData *jdev;
    int newQ;
    MMRESULT status;

    objPtr = framePtr->objPtr;
    jdev = (J300ClientData *) objPtr->devicePtr->deviceData;
    if (jdev->inited != 1) {
	return 0;
    }
    newQ = framePtr->frameHeader.qfactor;

    /*
     */
    if ((jdev->jfifInited == 0)|| (jdev->qfactor != newQ) ) {
	jdev->qfactor = newQ;
	CreateJfif(jdev->jfifHeader,jdev->inputW,jdev->inputH,newQ);	
	jdev->jfifInited = 1;

    }

    /*
     * Copy data into buffer leaving 607 bytes for the jfif_header.
     */
    memcpy (jdev->lpInput[0], jdev->jfifHeader, JFIF_HEADER_SIZE);
    memcpy (&(jdev->lpInput[0][JFIF_HEADER_SIZE]), framePtr->data,
	   framePtr->dataLength);
    jdev->lpbiIn->biSizeImage = JFIF_HEADER_SIZE + framePtr->dataLength;
    status = ICDecompress (jdev->hic, (int )NULL, jdev->lpbiIn, jdev->lpInput[0],
			   jdev->lpbiOut, jdev->lpOutput[0]);


    if (status != ICERR_OK) {
	mmcErrorDisplay( "ICDecompress failed, status = %d", status);
	return J300_ICDECOMPRESS_FAILED;
    }
    jdev->decompressed = 1;
    return 0;
}


/*
 *--------------------------------------------------------------
 *
 * DecodeJfifNoHuffmanFrame --
 *
 *	Decode a frame with a format tag of JFIF_NO_HUFFMAN_FRAME_CODE
 *
 * Results:
 *	0 for success, error code otherwise
 *
 * Side effects:
 *
 * Parameters:
 *	MJPEGFrame *framePtr	Frame to be decoded.
 *
 *--------------------------------------------------------------
 */
int
DecodeJfifNoHuffmanFrame( MJPEGFrame *framePtr)
{
    MJPEGResource *objPtr;
    J300ClientData *jdev;
    MMRESULT status;

    objPtr = framePtr->objPtr;
    jdev = (J300ClientData *) objPtr->devicePtr->deviceData;
    if(jdev->inited != 1)
	    return 0;

    /*
     * Since this is the native format, directly copy frame data into lpInput
     * data contains JFIF_HEADER_SIZE byte header + video jpeg data
     */
    memcpy (&(jdev->lpInput[0][0]), framePtr->data, framePtr->dataLength);
    jdev->lpbiIn->biSizeImage = framePtr->dataLength;
    status = ICDecompress (jdev->hic, (int)NULL, jdev->lpbiIn, jdev->lpInput[0],
			  jdev->lpbiOut, jdev->lpOutput[0]);

    if (status != ICERR_OK) {
	mmcErrorDisplay( "ICDecompress failed, status = %d", status);
	return J300_ICDECOMPRESS_FAILED;
    }
    jdev->decompressed = 1;
    return 0;
}


/*
 *--------------------------------------------------------------
 *
 * J300Display --
 *
 *	Display a video frame in the window associated with this
 *	device
 *
 * Results:
 *	0 if successful, non-zero error code otherwise
 *
 * Side effects:
 *	Window is redrawn
 *
 * Parameters:
 *	MJPEGFrame *framePtr	Frame to be displayed
 *
 *--------------------------------------------------------------
 */
int
J300Display (MJPEGFrame *framePtr)
{
    MJPEGResource *objPtr;
    J300ClientData *jdev;
    int depth;
    int xoff, yoff;
    int numPixels,i;
    char tmp;

    objPtr = framePtr->objPtr;
    jdev = objPtr->devicePtr->deviceData;
    if( jdev->inited != 1)
	return 0;
     
    /*
     * Decode for correct sub format.  Produces a sub-image that can be displayed
     * using mmcMapImage() call below.
     */
    J300ActualDecode(framePtr);

    if (jdev->decompressed != 1) {
	return 0;
    }

	/*
	 * Calculate xoff and yoff for positioning image in window
	 */
    	xoff = yoff = 0;
	if(jdev->squeeze != 1) {
		xoff = objPtr->xpos;
		yoff = objPtr->ypos;
		if(xoff == -1) {
			if(jdev->w > framePtr->frameHeader.width)
				xoff = (jdev->w - framePtr->frameHeader.width
				-1) / 2;
			else
				xoff = 0;
		}
		if(yoff == -1) {
			if(jdev->h > framePtr->frameHeader.height)
				yoff = (jdev->h - framePtr->frameHeader.height
				-1) / 2;
			else
				yoff = 0;
		}
	}

    /*
     * Call the hardware routine to throw the image up on
     * the screen
     */
    if (jdev->desiredDepth != 8) {
	depth = 24;
	if(jdev->swapRB) {

		/*
		 * swap the R and the B bytes in software for now
		 * TO BE DONE -- tell the J300 to produce output in the form FRGB instead of FBGR
		 */
		numPixels = jdev->lpbiOut->biHeight * jdev->lpbiOut->biWidth;
		for( i=0; i < numPixels; i++ )
		{
			tmp = *(char *)(jdev->lpOutput[0] + (i*4) + 0);
			*(char *)(jdev->lpOutput[0] + (i*4) + 0) = *(char *)(jdev->lpOutput[0] + (i*4) + 2);
			*(char *)(jdev->lpOutput[0] + 2 + (i*4)) = tmp;
		}

	}
    } else {
	depth = 8;
    }
    mymmcMapImage (jdev->dpy, jdev->xvisual, jdev->gc, jdev->win,
		 jdev->lpbiOut->biHeight, jdev->lpbiOut->biWidth, depth,
		 &jdev->ShmInfo, jdev->lpOutput[0],jdev->noShm,xoff,yoff);

    return 0;
}



/*
 *--------------------------------------------------------------
 *
 * mymmcMapImage --
 *
 *	mmcMapImage modified to accept destx and dest y parameters.
 *
 * Results:
 *	0 if successful, non-zero error code otherwise
 *
 * Side effects:
 *	Window is redrawn
 *
 * Parameters:
 *
 *--------------------------------------------------------------
 */
void mymmcMapImage (Display *dpy,
	          Visual *visual,
	          GC gc,
	          Window window,
	          int Lines,
	          int PixelsPerLine,
		  int depth,
	          XShmSegmentInfo *shminfo,
	          char * address,
                  int noxshm,
		  int xoff,
		  int yoff)
{
    XImage  *ximage;
#define DIV_ROUND_UP_(n,d) (((n) + (d) - 1) / (d))
    int    BytesPerLine = DIV_ROUND_UP_(PixelsPerLine, 4) * 4 * 
	(depth == 24 ? 4 : 1);

    /*
     * Set up input data structure for the image
     */
    ximage =
	XCreateImage(dpy,		/* X display identifier    */
		     visual,		/* Visual type             */
		     depth,		/* Depth of image          */
		     ZPixmap,		/* X Image format          */
		     0,			/* Initial offset in data  */
		     address, 		/* Address of data         */
		     PixelsPerLine, 	/* Width in pels           */
		     Lines,		/* Number of scanlines     */
		     XBitmapPad(dpy), 	/* Bitmap pad?             */
		     BytesPerLine); 	/* Bytes per scanline      */

    if ( !noxshm )
    {
       ximage->obdata = (char *)shminfo;

       /*
        * Display image
        */
       XShmPutImage(dpy, 
		    window, 
		    gc,		/* XDefaultGC(display,0),  */
		    ximage,                       
		    0,                           
		    0,                           
		    xoff,                               
		    yoff,                               
		    PixelsPerLine,
		    Lines,
		    False);
        ximage->obdata = NULL;
        ximage->data = NULL;
    }
    else
    {
       /*
        * Display image
        */
       XPutImage(dpy, 
	         window, 
	         gc,		/* XDefaultGC(display,0),  */
	         ximage,                       
	         0,                           
	         0,                           
		 xoff,                               
		 yoff,                               
	         PixelsPerLine,
	         Lines);
    }

    XFlush (dpy);
    XDestroyImage (ximage);
    XSync (dpy, False);
}				/* end of mapImage() */



/*
 *--------------------------------------------------------------
 *
 * J300FreeFrame --
 *
 *	Called to free any memory we've allocated to decode the
 *	frame passed as a parameter.
 *
 * Results:
 *	0 for success
 *
 * Side effects:
 *	None.
 *
 * Parameters:
 *	None.
 *
 *--------------------------------------------------------------
 */
int
J300FreeFrame (MJPEGFrame *framePtr)
{
    return 0;
}


/*
 *--------------------------------------------------------------
 *
 * J300Refresh --
 *
 *	Called when the window associated with this device needs
 *	to be redrawn.
 *
 * Results:
 *	0 for success.
 *
 * Side effects:
 *	Window is redrawn
 *
 * Parameters:
 *	MJPEGResource *objPtr	Pointer to the resource object data structure
 *
 *--------------------------------------------------------------
 */
int
J300Refresh ( MJPEGResource *objPtr)
{
    J300ClientData *jdev;
    int depth;
    int xoff,yoff;

    jdev = objPtr->devicePtr->deviceData;
    if(jdev->inited != 1)
	return 0;


    /*
     * If no decompressed buffer exists, just ignore.  Otherwise,
     * Call the hardware routine to throw the image up on the screen.
     */
    if(jdev->decompressed != 1) {
	return 0;
    }

    	xoff = yoff = 0;
	if(jdev->squeeze != 1) {
		xoff = objPtr->xpos;
		yoff = objPtr->ypos;
		if(xoff == -1) {
			if(jdev->w > jdev->outputW)
				xoff = (jdev->w - jdev->outputW
				-1) / 2;
			else
				xoff = 0;
		}
		if(yoff == -1) {
			if(jdev->h > jdev->outputH)
				yoff = (jdev->h - jdev->outputH
				-1) / 2;
			else
				yoff = 0;
		}
	}

    /*
     * Call the hardware routine to throw the image up on
     * the screen
     */
    if (jdev->desiredDepth != 8) {
	depth = 24;
    } else {
	depth = 8;
    }
    mymmcMapImage (jdev->dpy, jdev->xvisual, jdev->gc, jdev->win,
		 jdev->lpbiOut->biHeight, jdev->lpbiOut->biWidth, depth,
		 &jdev->ShmInfo, jdev->lpOutput[0],jdev->noShm,xoff,yoff);

    return 0;
}

/*
 *--------------------------------------------------------------
 *
 * J300Init --
 *
 *	Called to initialize the J300 device.
 *
 * Results:
 *	0 if successful, non-zero error code otherwise
 *
 * Side effects:
 *	None.
 *
 * Parameters:
 *	MJPEGResource *objPtr	Pointer to the resource object data structure
 *
 *--------------------------------------------------------------
 */
int
J300Init (MJPEGResource *objPtr)
{
    J300ClientData *jdevice;
    int screen;
    Colormap colormap;
    RGBQUAD oldrgb[MAX_COLORS], newrgb[MAX_COLORS];
    int cmapStrategy;
    int numColors;
    int i;
    MMRESULT status;
    int outPaletteSize;
    int ret;

    XWindowAttributes win_attrs;
    XVisualInfo visual_template;
    XVisualInfo *visual_list;
    int num_visuals;

    jdevice = objPtr->devicePtr->deviceData;

    /*
     * if grab (not decompress) mode return
     */
    if (jdevice->mode == J300_COMPRESS_MODE) {
	return 0;
    }

    outPaletteSize = sizeof(RGBQUAD) * MAX_COLORS;
    /* 
     * If the xid is 0, we're only using the J300 for decompression,
     * not display.
     */
    if (objPtr->xid == 0) {
	/* 
         * THIS OPTION IS NOT CURRENTLY SUPPORTED
	 */
	jdevice->display = J300_NODISPLAY;
	jdevice->desiredDepth = 8;
	jdevice->screenDepth = 8;
    } else {

	/*
	 * Set the device parameters: display data, tell it which
	 * window, the width and height, to block for decompression,
	 * 8 pixels display
	 */
	jdevice->display = J300_DISPLAY;
	jdevice->xid = objPtr->xid;
	jdevice->win = objPtr->xid;
	jdevice->w = objPtr->width;
	jdevice->h = objPtr->height;
	jdevice->squeeze = objPtr->squeeze;
	jdevice->blocking = TRUE;
	jdevice->desiredDepth = 8;
	cmapStrategy = DEFAULT_CMAP;

	/*
	 * chk for valid parameters 
	 * If squeeze is set, then the size of the decompressed
	 * bitmap is the same as the window size. Else it is the same
	 * as the input frame size.
	 */
	if((jdevice->squeeze)&& (jdevice->w != 0) && (jdevice->h != 0)) {
		 jdevice->outputW = jdevice->w;
		 jdevice->outputH = jdevice->h;
	} else {
		/* 
		 * set outputw and outputh to some default value for the J300
		 * device initialization to be happy
		 */
		 jdevice->outputW = 320;
		 jdevice->outputH = 240;
	}

	/*
	 * set input w and h to default value -- the actualt value
	 * will be got from each frame itself
	 */
	jdevice->inputW = 320;
	jdevice->inputH = 240;


	/* 
	 * since initialising the decompression device needs the
	 * actual width and height of frame, we will only do it on
	 * getting an actual frame
	 */
	jdevice->decompressDevInited = 0;

	/*
	 * Initialize all the stuff for X windows.
	 */
	jdevice->dpy = (Display *) Tcm_Display();
	screen = XDefaultScreen (jdevice->dpy);

	/* 
	 * get visual Info 
	 */
	XGetWindowAttributes(jdevice->dpy, jdevice->win, &win_attrs);
	jdevice->xvisual = win_attrs.visual;
	jdevice->desiredDepth = win_attrs.depth;
	jdevice->screenDepth = win_attrs.depth;
	jdevice->swapRB = 0;

	/* 
	 * chk whether the window is FRGB or FBGR
	 * J300 creates 24 bit display using FBGR
	 */
	visual_template.depth = win_attrs.depth;
	visual_list = XGetVisualInfo(jdevice->dpy, VisualDepthMask, &visual_template, &num_visuals);

	for ( i=0; i<num_visuals; i++) {
	    if (visual_list[i].visual == win_attrs.visual) {
	        /* Found the right one, break. */
		if(visual_list[i].blue_mask == 0x000000ff)	/*==> FRGB */
			jdevice->swapRB = 1;
		break;
	    }
	}



	/*
	 * create gc
	 */
	jdevice->gc = XCreateGC (jdevice->dpy, jdevice->win, 0L, NULL);
	XSetForeground (jdevice->dpy, jdevice->gc, BlackPixel(jdevice->dpy, 0));
	XSetBackground (jdevice->dpy, jdevice->gc, WhitePixel(jdevice->dpy, 0));
	XSetLineAttributes (jdevice->dpy, jdevice->gc, 0, LineOnOffDash,
			    CapButt, JoinMiter);


	/*
	 * assign colormap to window
	 */
	mmcInitColormap (jdevice->dpy, jdevice->xvisual, screen, &colormap,
			oldrgb, cmapStrategy, 0);
	if (cmapStrategy != DEFAULT_CMAP) {
	    XSetWindowColormap(jdevice->dpy,jdevice->win, colormap);
	}


	if( (ret = SetupIOHeaders (jdevice)) != 0)
		return ret;
	if( (ret = GetDecompressDevice (jdevice)) != 0)
		return ret;

	/* 
	 *Set up the colors to note what's available, what's already taken
	 */

	if  (cmapStrategy == DEFAULT_CMAP) {
	    /*
	     * get rgb and free color cells so other applications can use
	     * these free cells in the default colormap
	     */
	    mmcGetRgbFromColormap (jdevice->dpy, colormap, &numColors, oldrgb, 1);
	} else {
	    mmcGetRgbFromColormap (jdevice->dpy, colormap, &numColors, oldrgb, 0);
	}
	memcpy (jdevice->lpbiPalette, oldrgb, outPaletteSize);


	/*
	 * Get palette for the output format of the decompressor. The
	 * palette is the range of colors that can be used for display
	 */
	status = ICDecompressGetPalette (jdevice->hic, jdevice->lpbiIn,
					 jdevice->lpbiOut);
	if (status <= ICERR_OK) {
	    mmcErrorDisplay ( "ICDecompressGetPalette failed %d",status);
	    return J300_ICDECOMPRESSGETPALETTE_FAILED;
	}

	/*
	 * Use decompressor's colormap for display
	 */
	memcpy (newrgb, jdevice->lpbiPalette, outPaletteSize);
	if  (cmapStrategy == DEFAULT_CMAP) {
	    mmcSetColorsInColormap (jdevice->dpy, colormap, newrgb, oldrgb,
				    cmapStrategy,1);
	} else {
	    mmcSetColorsInColormap (jdevice->dpy, colormap, newrgb, oldrgb,
				    cmapStrategy,0);
	}

	/*
	 * allocate output buffers
	 * will need > 1 buffer if we implement the J300 streaming interface
	 */
	jdevice->numBuffers = 1;

	/* 
	 * should do the actual allocation only after getting the
	 * first frame so we know the o/p width and height
	 */

	/*
	 * set jfif flag to 0
	 * we create the jfif header only if we need it (subformat
	 * VID_FRAME_CODE)
	 */
	jdevice->jfifInited = 0;

    }
    jdevice->inited = 1;
    jdevice->decompressed=0;

    return 0;
}

/*
 *--------------------------------------------------------------
 *
 * SetupIOHeaders --
 *
 *	This allocates memory for the headers needed to communicate
 *	with the J300 device. (refer note below).
 *
 * Results:
 *	None
 *
 * Side effects:
 *	    
 *
 * Parameters:
 *	J300ClientData *jdevice		Device to set up headers for.
 *
 *--------------------------------------------------------------
 */
static int
SetupIOHeaders (J300ClientData *jdevice)
{
    /* J300 NOTE */
    /* Applications need to use the mme memory allocation functions whenever
     * the memory is passed to the Multimedia services. mmeAllocMem is used
     * to allocate data structures, mmeAllocBuffer or
     * mmeAllocBufferAndGetShminfo to allocate data buffers
     * All pointers passed between a client app and the mmeserver are passed
     * as shared memory pointers.
     */


    EXBMINFOHEADER *exbi;	/* extended bit map header */
    JPEGINFOHEADER *jpegbi;
    BITMAPINFOHEADER *lpbiIn, *lpbiOut; /* bitmap information header 
					 * for info on BITMAPINFOHEADER
					 * refer Chpt 7, Image Formats
					 * of Multimedia Services for
					 * DEC/OSF - programmer's Guide
					 */
    RGBQUAD *lpbiPalette;	/*for bitmap color table */
    int inHeaderSize, outHeaderSize, outPaletteSize;
    void *memory;


    inHeaderSize = sizeof(EXBMINFOHEADER) + sizeof(JPEGINFOHEADER); 
    outHeaderSize = sizeof(BITMAPINFOHEADER);
    outPaletteSize = sizeof(RGBQUAD) * MAX_COLORS;

    /*
     * To reduce the number of shared memory segments, allocate  enough 
     * memory to contain several structures at once.
     */
    memory = (void *)mmcAllocMem(inHeaderSize+outHeaderSize+outPaletteSize);
    if (memory == NULL) {
	mmcErrorDisplay( "Cannot allocate memory for headers");
	mmcErrorDisplay( "Is the mmeserver process running?");
	return J300_MEMORYALLOCATION_FAILED;
    }
    lpbiIn = (BITMAPINFOHEADER *)memory;
    lpbiOut = (BITMAPINFOHEADER *)((unsigned long)memory + inHeaderSize);
    lpbiPalette = (RGBQUAD *)((unsigned long)lpbiOut + outHeaderSize);

    /*
     * Set up output bitmap header for Ximage format
     */
    lpbiOut->biSize = outHeaderSize;
    lpbiOut->biWidth = jdevice->outputW;
    lpbiOut->biHeight = jdevice->outputH;

    lpbiOut->biPlanes = 1;
    lpbiOut->biBitCount = jdevice->screenDepth;
    lpbiOut->biCompression = BICOMP_DECXIMAGEDIB;
    lpbiOut->biSizeImage = ComputeBufSize(lpbiOut->biWidth,
			  lpbiOut->biHeight,jdevice->screenDepth);
    lpbiOut->biXPelsPerMeter = 0;
    lpbiOut->biYPelsPerMeter = 0;


    /*
     * 8 or 24-bit color ie bits per pixel
     */
    if (lpbiOut->biBitCount == 8) {
        lpbiOut->biClrUsed = MAX_COLORS;
    } else {	/* 24 bits */
        lpbiOut->biClrUsed = 0;
    }
    lpbiOut->biClrImportant = 0;
    

    /*
     * Set up input bitmap header for jpeg  format
     */
    lpbiIn->biSize = inHeaderSize;


    /* 
     * note -- this should be w,h for frame -- should be got from frame 
     */
    lpbiIn->biWidth = jdevice->inputW;
    lpbiIn->biHeight = jdevice->inputH;

    lpbiIn->biPlanes = 1;
    lpbiIn->biBitCount = 24 ; /* for jpeg */ /*8;*/
    lpbiIn->biCompression = MJPG_DIB; /* DIB - device independent bitmap */
    exbi  = (EXBMINFOHEADER *) lpbiIn;
    exbi->biExtDataOffset = sizeof(EXBMINFOHEADER);
    jpegbi  = (JPEGINFOHEADER *) ( (unsigned long)exbi + 
				exbi->biExtDataOffset);
    jpegbi->JPEGSize = sizeof(JPEGINFOHEADER);
    jpegbi->JPEGProcess=JPEG_PROCESS_BASELINE;
    jpegbi->JPEGColorSpaceID = JPEG_YCbCr;

    /* bits per sample per component for above color space*/
    jpegbi->JPEGBitsPerSample=8;
    jpegbi->JPEGHSubSampling=JPEG_SUBSAMPLING_2;
    jpegbi->JPEGVSubSampling=JPEG_SUBSAMPLING_1;

    /*
     * this value needs be valid only for ICDecompress
     */
    lpbiIn->biSizeImage = 0;  /*ComputeBufSize(lpbiIn->biWidth,
			  lpbiIn->biHeight,jdevice->screenDepth);*/
    lpbiIn->biXPelsPerMeter = 0;
    lpbiIn->biYPelsPerMeter = 0;

    /*
     * 8 or 24-bit color
     */
    if (lpbiIn->biBitCount == 8) {
        lpbiIn->biClrUsed = MAX_COLORS;
    } else {
        lpbiIn->biClrUsed = 0;
        lpbiIn->biClrImportant = 0;
    }

    jdevice->lpbiIn = lpbiIn;
    jdevice->lpbiOut = lpbiOut;
    jdevice->lpbiPalette = lpbiPalette;
    return 0;
}



/*
 *--------------------------------------------------------------
 *
 * GetDecompressDevice --
 *
 *	Lokks for a device that can handle the given parameters.
 *	If it cant find a hardware device, it will try to return
 *	a software one.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	   
 *
 * Parameters:
 *	J300ClientData *jdev	device to initialize
 *
 *--------------------------------------------------------------
 */
static int
GetDecompressDevice ( J300ClientData *jdev)
{
    HIC hic;
    ICINFO *icinfo;
    DWORD fccHandler;
    int i;
    int count=0;

    if ( !( icinfo = (ICINFO *)mmcAllocMem(sizeof(ICINFO)) ) ) {
	mmcErrorDisplay("System out of memory");
	return J300_MEMORYALLOCATION_FAILED;
    }

    /*
     * Open each decompressor and query if it can handle the input
     * and output formats 
     */
    for (i = 1; ICInfo(ICTYPE_VIDEO, i, icinfo); i++) {
	hic = ICOpen(icinfo->fccType, icinfo->fccHandler, ICMODE_QUERY);
	if (hic) {
	    if (ICDecompressQuery(hic, jdev->lpbiIn, jdev->lpbiOut) == ICERR_OK) {
		mmcMsgDisplay("Device %d is the %s.", i, icinfo->szDescription);
		count++;
		break;	/*HACK*/
	    }
	    ICClose(hic);
	}
    }
    ICClose(hic);    /*hack*/

    if (count == 0) {
	printf("No decompression device \n");
	return J300_NODECOMPRESS_DEVICE;
    }
    fccHandler= icinfo->fccHandler;
    jdev->hic = ICLocate (ICTYPE_VIDEO, fccHandler, jdev->lpbiIn, jdev->lpbiOut,
			  ICMODE_DECOMPRESS);
    if (jdev->hic == 0) {
	 mmcErrorDisplay ("Cannot open video decompressor");
	 return J300_CANNOT_OPEN_DECOMPRESSOR;
    }
    return 0;
}


/*
 *--------------------------------------------------------------
 *
 * OpenDecompressionDevice --
 *
 *	Opens the decompression device. The J300 needs to close
 *	and reopen the device if any of the parameters (such as
 *	i/p frame w,h o/p w,h) changes.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	   
 *
 * Parameters:
 *	J300ClientData *jdev	device to initialize
 *
 *--------------------------------------------------------------
 */
int
OpenDecompressionDevice(J300ClientData *jdev)
{
int status;

    status = ICDecompressBegin(jdev->hic, jdev->lpbiIn, jdev->lpbiOut);
    if (status != ICERR_OK) {
	mmcErrorDisplay( "ICDecompressBegin error %d", status);
	 return J300_ICDECOMPRESSBEGIN_FAILED;
    }


    jdev->inImageSize = PAGE_PAD(jdev->lpbiIn->biSizeImage);
    jdev->outImageSize = PAGE_PAD(jdev->lpbiOut->biSizeImage);
    jdev->lpbiIn->biSizeImage = jdev->inImageSize * jdev->numBuffers;
    jdev->lpbiOut->biSizeImage = jdev->outImageSize * jdev->numBuffers;

    status = ICDecompressPrepareHeader (jdev->hic, jdev->lpbiIn,
		jdev->lpInput[0], jdev->lpbiOut, jdev->lpOutput[0]);
    if (status != ICERR_OK) {
	mmcErrorDisplay( "Cannot prepare header");
	return J300_ICPREPAREHEADER_FAILED;
    }

    jdev->lpbiOut->biSizeImage = jdev->outImageSize;
    jdev->lpbiIn->biSizeImage = jdev->inImageSize;
    return 0;
}


/*
 *--------------------------------------------------------------
 *
 * CloseDecompressionDevice --
 *
 *	Closes the decompression device. The J300 needs to close
 *	and reopen the device if any of the parameters (such as
 *	i/p frame w,h o/p w,h) changes.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	   
 *
 * Parameters:
 *	J300ClientData *jdev	device to initialize
 *
 *--------------------------------------------------------------
 */
int CloseDecompressionDevice(J300ClientData *jdev)
{
int status;

    /* 
     * Unprepare all buffers used.
     * Use inImageSize here since lpbiIn->biSizeImage has
     * been changing all over the place based on the input
     * image sizes 
     */
    jdev->lpbiIn->biSizeImage = jdev->inImageSize * jdev->numBuffers;
    jdev->lpbiOut->biSizeImage = jdev->outImageSize * jdev->numBuffers;
    status = ICDecompressUnprepareHeader (jdev->hic, jdev->lpbiIn, jdev->lpInput[0],
					  jdev->lpbiOut, jdev->lpOutput[0]);
    if (status != ICERR_OK) {
	mmcErrorDisplay( "Cannot unprepare header, error = %d", status);
	return J300_ICUNPREPAREHEADER_FAILED;
    }
    
    /*
     * Stop decompression
     */
    status = ICDecompressEnd(jdev->hic);
    if (status != ICERR_OK) {
	mmcErrorDisplay( "ICDecompressEnd error %d", status);
	return J300_ICDECOMPRESSEND_FAILED;
    }
    return 0;
}


/*
 *--------------------------------------------------------------
 *
 * FreeJ300Buffers --
 *
 *      Frees all the buffers allocated through the mmeAlloc calls
 *
 * Results:
 *	None.
 *
 * Side effects:
 *         
 *
 * Parameters:
 *	J300ClientData *jdev	device to initialize
 *
 *--------------------------------------------------------------
 */
int
FreeJ300Buffers(J300ClientData *jdev, int flag)
{
    if(flag == ONLY_INPUT) {
    	free(jdev->lpInput);
	return 0;
    }
    /*
     * Detach shared memory
     */
    if ((jdev->display == J300_DISPLAY) && (jdev->noShm == False)) {
	XShmDetach(jdev->dpy, &jdev->ShmInfo);
    }
    mmeFreeBuffer(jdev->ShmInfo.shmaddr);
    
    /*
     * Free remaining memory
     */
    free(jdev->lpOutput);
    if(flag == ONLY_OUTPUT) {
	return 0;
    }

    free(jdev->lpInput);

    mmcFreeAll();
    
    return 0;
}


/*
 *--------------------------------------------------------------
 *
 * AllocateJ300Buffers --
 *
 *      This allocates the input and output buffers for the J300
 *	device
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	   
 *
 * Parameters:
 *	J300ClientData *jdev	device to initialize
 *
 *--------------------------------------------------------------
 */
static int
AllocateJ300Buffers (J300ClientData *jdev, int flag)
{
    int size;
    MMRESULT status;
    int inImageSize, outImageSize;
    int verbose =0;
    int bytesPerImage;

    /*
     * first reinit headers as the h & w may have changed
     */
    jdev->lpbiOut->biWidth = jdev->outputW;
    jdev->lpbiOut->biHeight = jdev->outputH;
    jdev->lpbiOut->biSizeImage = ComputeBufSize(jdev->lpbiOut->biWidth,
				  jdev->lpbiOut->biHeight,jdev->screenDepth);
    jdev->lpbiIn->biWidth = jdev->inputW;
    jdev->lpbiIn->biHeight = jdev->inputH;


    /*
     * Build array of output buffers
     */
    jdev->lpOutput = (char **)malloc(sizeof(char *) * jdev->numBuffers);
    if (!jdev->lpOutput) {
	mmcErrorDisplay( "1:Cannot allocate memory for output buffers");
	return J300_MEMORYALLOCATION_FAILED;
    }


    size = PAGE_PAD(jdev->lpbiOut->biSizeImage) * jdev->numBuffers;
    jdev->lpOutput[0] = mmeAllocBufferAndGetShminfo (size, 
			      (MMShmSegmentInfo *)&jdev->ShmInfo);
    if (!jdev->lpOutput[0]) {
	mmcErrorDisplay( "2:Cannot allocate memory for output buffers");
	return J300_MEMORYALLOCATION_FAILED;
    }

    
    if (jdev->display == J300_DISPLAY) {
        jdev->noShm = mmcAttach(jdev->dpy, &jdev->ShmInfo);
    }
    
    
    if(flag == ONLY_OUTPUT)
	return 0;

    /*
     * allocate input buffers
     */

    jdev->lpInput = (char **)malloc(sizeof(char *) * jdev->numBuffers);
    if (!jdev->lpInput) {
	mmcErrorDisplay( "Cannot allocate memory for input buffers");
	return J300_MEMORYALLOCATION_FAILED;
    }
    if (jdev->lpbiIn->biSizeImage == 0) {
	jdev->lpbiIn->biSizeImage = JPEG_SIZE * jdev->numBuffers;
    }
    bytesPerImage = PAGE_PAD(jdev->lpbiIn->biSizeImage);
    jdev->lpInput[0] = (void *)mmcAllocBuffer(bytesPerImage * jdev->numBuffers);
    if (!jdev->lpInput[0]) {
	mmcErrorDisplay( "Cannot allocate input buffer");
	return J300_MEMORYALLOCATION_FAILED;
    }
    return 0;
    
}


/*
 *-------------------------------------------------------------- *
 * J300Reinit --
 *
 *	Called when the resource object is configured.
 *	Checks to see whether the parameters it cares about (squeeze
 *	flag, window width or height) have changed.
 *
 * Results:
 *	0 for success
 *
 * Side effects:
 *	None
 *
 * Parameters:
 *	MJPEGResource *objPtr	pointer to the resource object data structure
 *
 *--------------------------------------------------------------
 */
int
J300Reinit (MJPEGResource *objPtr)
{
J300ClientData *jdev;
int oldOutputW, oldOutputH;
int ret;

    jdev = objPtr->devicePtr->deviceData;

    /*
     * if grab (not decompress) mode return
     */
    if (jdev->mode == J300_COMPRESS_MODE) {
	return 0;
    }
    if( (jdev->squeeze != objPtr->squeeze) || (jdev->w != objPtr->width)
	|| (jdev->h != objPtr->height)) {
	/*
	 * store old values
	 */
	oldOutputW = jdev->outputW;
	oldOutputH = jdev->outputH;

	jdev->squeeze = objPtr->squeeze;
	jdev->w = objPtr->width;
	jdev->h = objPtr->height;

	if((jdev->squeeze) && (jdev->w != 0) && (jdev->h != 0)) {
		jdev->outputW = jdev->w;
		jdev->outputH = jdev->h;
	} else {
		/*
		 * take the output w and h from the input w and h
		 */
		jdev->outputW = jdev->inputW;
		jdev->outputH = jdev->inputH;
	}
	/*
	 * if the new output frame size is different, close the
	 * decompression device, free the output buffer, allocate a new output
	 * buffer and reopen the device
	 * However, if the device has not yet been initialised, dont
	 * bother-- it will be done whenever the first frame comes in.
	 */
	 if(jdev->decompressDevInited != 1)
		return 0;
	 if( (oldOutputW != jdev->outputW) || (oldOutputH != jdev->outputH)) {
	        if( (ret =CloseDecompressionDevice(jdev)) != 0)
			return ret;	
		jdev->lpbiOut->biWidth = jdev->outputW;
		jdev->lpbiOut->biHeight = jdev->outputH;
		jdev->lpbiOut->biSizeImage = ComputeBufSize(jdev->lpbiOut->biWidth,
			jdev->lpbiOut->biHeight,jdev->screenDepth);
		FreeJ300Buffers(jdev,ONLY_OUTPUT);
		if( (ret = AllocateJ300Buffers(jdev,ONLY_OUTPUT)) != 0)
			return ret;
		if( (ret = OpenDecompressionDevice(jdev)) != 0)
			return ret;
	 }
    }
    return 0;
}


/*
 *--------------------------------------------------------------
 *
 * J300Shutdown --
 *
 *	Called to deallocate and shut down the device.
 *
 * Results:
 *	0 for success.
 *
 * Side effects:
 *	The device must not be used again.
 *
 * Parameters:
 *	MJPEGResource *objPtr	pointer to the resource object data structure
 *
 *--------------------------------------------------------------
 */
int
J300Shutdown (MJPEGResource *objPtr)
{
    J300ClientData *jdev;
    MMRESULT status;

    jdev = (J300ClientData *) objPtr->devicePtr->deviceData;
    if ((!jdev->inited)&& (jdev != NULL)) {
	ckfree( (char *) jdev);
	objPtr->devicePtr->deviceData = NULL;
	return 0;
    }

    CloseDecompressionDevice(jdev);
    FreeJ300Buffers(jdev,INPUT_AND_OUTPUT);
    
    ckfree( (char *) jdev);
    objPtr->devicePtr->deviceData = NULL;
    return 0;
}


/*
 *--------------------------------------------------------------
 *
 * J300Error --
 *
 *	Called by the resource to translate an error code returned
 *	by one of the routines above into a human readable error
 *	message.
 *
 * Results:
 *	Pointer to error message
 *
 * Side effects:
 *	None.
 *
 * Parameters:
 *	MJPEGResource *objPtr	pointer to the resource object data structure
 *	int errNum		Error code from above.
 *
 *--------------------------------------------------------------
 */
char *
J300Error (MJPEGResource *objPtr, int errNum)
{
	switch(errNum) {
		case J300_ICDECOMPRESS_FAILED :
			return "J300: could not decompress frame";
		break;
		case J300_ICDECOMPRESSGETPALETTE_FAILED :
			return "J300: could not get palette";
		break;
		case J300_MEMORYALLOCATION_FAILED :
			return "J300: could not allocate memory";
		break;
		case J300_CANNOT_OPEN_DECOMPRESSOR :
			return "J300: could not open decompressor ";
		break;
		case J300_NODECOMPRESS_DEVICE :
			return "J300: could not find decompressor ";
		break;
		case J300_ICDECOMPRESSBEGIN_FAILED :
			return "J300: could not begin decompression ";
		break;
		case J300_ICPREPAREHEADER_FAILED :
			return "J300: could not prepare decompression header";
		break;
		case J300_ICUNPREPAREHEADER_FAILED :
			return "J300: could not unprepare decompression header";
		break;
		case J300_ICDECOMPRESSEND_FAILED :
			return "J300: could not end decompression ";
		break;
	}
	return "some random J300 error";
}


#ifndef DEMO

/*
 *--------------------------------------------------------------
 *
 * CreateJfif --
 *
 *	Create a JFIF header, which is missing in some video
 *	formats (esp, the parallax format)
 *
 * Results:
 *	None
 *
 * Side effects:
 *	None
 *
 * Parameters:
 *	char *buf	(out)	Buffer with resulting header
 *	int width	(in)	Width of image
 *	int height	(in)	Height of image
 *	int qfactor	(in)	Quality factor for JPEG compression
 *
 *--------------------------------------------------------------
 */
static void
CreateJfif (char *buf, int width, int height, int qfactor)
{
	
    int i;
    int temp = 0;

    /*
     * The next array represents the default JFIF header for
     * quality = 100 and size = 640x480. The values are
     * adjusted as the J_Movie header is read.  
     *
     * The buffer contains the jfif header without the huffman table
     */

     char inbuffer[187] = {
	 0xFF, 0xD8, 0xFF, 0xE0, 0x00, 0x10, 0x4A, 0x46,
	 0x49, 0x46, 0x00, 0x01, 0x01, 0x00, 0x00, 0x01,
	 0x00, 0x01, 0x00, 0x00, 0xFF, 0xC0, 0x00, 0x11,
	 0x08, 0x01, 0xE0, 0x02, 0x80, 0x03, 0x01, 0x21,
	 0x00, 0x02, 0x11, 0x01, 0x03, 0x11, 0x01, 0xFF,
	 0xDB, 0x00, 0x84, 0x00, 0x10, 0x0B, 0x0C, 0x0E,
	 0x0C, 0x0A, 0x10, 0x0E, 0x0D, 0x0E, 0x12, 0x11,
	 0x10, 0x13, 0x18, 0x28, 0x1A, 0x18, 0x16, 0x16,
	 0x18, 0x31, 0x23, 0x25, 0x1D, 0x28, 0x3A, 0x33,
	 0x3D, 0x3C, 0x39, 0x33, 0x38, 0x37, 0x40, 0x48,
	 0x5C, 0x4E, 0x40, 0x44, 0x57, 0x45, 0x37, 0x38,
	 0x50, 0x6D, 0x51, 0x57, 0x5F, 0x62, 0x67, 0x68,
	 0x67, 0x3E, 0x4D, 0x71, 0x79, 0x70, 0x64, 0x78,
	 0x5C, 0x65, 0x67, 0x63, 0x01, 0x11, 0x12, 0x12,
	 0x18, 0x15, 0x18, 0x2F, 0x1A, 0x1A, 0x2F, 0x63,
	 0x42, 0x38, 0x42, 0x63, 0x63, 0x63, 0x63, 0x63,
	 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63,
	 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63,
	 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63,
	 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63,
	 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63,
	 0x63, 0x63, 0x63, 0x63, 0x63, 
	 0xFF, 0xDA, 0x00, 0x0C, 0x03, 0x01, 0x00,
	 0x02, 0x11, 0x03, 0x11, 0x00, 0x3F, 0x00
    };

    /*
     * set image width in JFIF header
     */
    inbuffer[27] = (char)(0xFF & (width >> 8));
    inbuffer[28] = (char)(0xFF & width);

    /*
     * set image height in JFIF header
     */
    inbuffer[25] = (char)(0xFF & (height >> 8));
    inbuffer[26] = (char)(0xFF & height);


    /*
     * Adjust the quality factor.
     *
     * The default quality factor = 100, therefore, if
     * our quality factor does not equal 100 we must
     * scale the quantization matrices in the JFIF header
     * Note values are clipped to a max of 255
     */
    if (qfactor != 100) {
        for (i=44; i<108; i++) {
            temp = (inbuffer[i]*qfactor)/100;
	    inbuffer[i] = (char)((temp<255) ? temp : 255);
	}
	for (i=109; i<173; i++){
	    temp = (inbuffer[i]*qfactor)/100;
	    inbuffer[i] = (char)((temp<255) ? temp : 255);
	}    
    }

    /*
     * copy into given buffer
     */
    memcpy (buf, inbuffer, 187);
}

#else	/* DEMO */

/*
 *--------------------------------------------------------------
 *
 * CreateJfif --
 *
 *	Create a JFIF header, which is missing in some video
 *	formats (esp, the parallax format).  This routine has
 *	the width, height, and qfactor wired in at 640, 480, and
 *	100, respectively.This was done to get decent quality display
 *      for the vid files which are grabbed using the parallax card
 *	-- need a good mapping between parallax and j300 qfactors
 *
 * Results:
 *	None
 *
 * Side effects:
 *	None
 *
 * Parameters:
 *	char *buf	(out)	Buffer with resulting header
 *	int width	(in)	Width of image
 *	int height	(in)	Height of image
 *	int qfactor	(in)	Quality factor for JPEG compression
 *
 *--------------------------------------------------------------
 */
static void
CreateJfif (char *buf, int width, int height, int qfactor)
{
    /*
     * The next array represents the default JFIF header for
     * quality = 100 and size = 640x480. 
     *
     * The buffer contains the jfif header without the huffman table
     */
     char inbuffer[195] = {
	 0xFF, 0xD8, 0xFF, 0xE0, 0x00, 0x0e, 0x41, 0x56,
	 0x49, 0x31, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	 0x00, 0x00, 0xff, 0xDD, 0x00, 0x04, 0x00, 0x00,
	 0xFF, 0xDB, 0x00, 0x43, 0x00, 0x21, 0x16, 0x18,
	 0x1D, 0x18, 0x14, 0x21, 0x1D, 0x1B, 0x1D, 0x25,
	 0x23, 0x21, 0x27, 0x31, 0x53, 0x36, 0x31, 0x2D,
	 0x2D, 0x31, 0x65, 0x48, 0x4c, 0x3c, 0x53, 0x78,
	 0x6a, 0x7e, 0x7c, 0x76, 0x6a, 0x74, 0x72, 0x85,
	 0x95, 0xbf, 0xa2, 0x85, 0x8d, 0xb4, 0x8f, 0x72,
	 0x74, 0xa6, 0xe2, 0xa8, 0xb4, 0xc5, 0xcb, 0xd6,
	 0xd8, 0xd6, 0x80, 0xa0, 0xeb, 0xfb, 0xe8, 0xd0,
	 0xf9, 0xbf, 0xd2, 0xd6, 0xcd, 0xff, 0xdb, 0x00,
	 0x43, 0x01, 0x23, 0x25, 0x25, 0x31, 0x2b, 0x31,
	 0x61, 0x36, 0x36, 0x61, 0xcd, 0x89, 0x74, 0x89,
	 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd,
	 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd,
	 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd,
	 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd,
	 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd,
	 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd,
	 0xcd, 0xcd, 0xff, 0xc0, 0x00, 0x11, 0x08, 0x00,
	 0xf0, 0x01, 0x40, 0x03, 0x01, 0x21, 0x00, 0x02,
	 0x11, 0x01, 0x03, 0x11, 0x01, 0xff, 0xda, 0x00,
	 0x0c, 0x03, 0x01, 0x00, 0x02, 0x11, 0x03, 0x11,
	 0x00, 0x3f, 0x00
    };

    /*
     * copy into given buffer
     */
    memcpy(buf,inbuffer,JFIF_HEADER_SIZE);
}
#endif /* DEMO */


/*
 *--------------------------------------------------------------
 *
 * GetJ300Device --
 *
 *	This fills the data structure with pointers to the corresponding
 *	functions.
 *
 * Results:
 *	a pointer to a MJPEGDevice data structure
 *
 * Side effects:
 *	None.
 *
 * Parameters:
 *	MJPEGResource *objPtr	pointer to the resource object data structure
 *
 *--------------------------------------------------------------
 */
MJPEGDevice *
GetJ300Device (MJPEGResource *objPtr)
{
    MJPEGDevice *new_dev;
    J300ClientData *tmp;

    new_dev = (MJPEGDevice *) ckalloc(sizeof(MJPEGDevice));

    new_dev->asyncDecodeFlag = 0;
    new_dev->decodeProc = J300Decode;
    new_dev->refreshProc = J300Refresh;
    new_dev->displayProc = J300Display;
    new_dev->initProc = J300Init;
    new_dev->reinitProc = J300Reinit;
    new_dev->shutdownProc = J300Shutdown;
    new_dev->freeFrameProc = J300FreeFrame;
    new_dev->errorProc = J300Error;
    new_dev->deviceData = (J300ClientData *) ckalloc(sizeof(J300ClientData));
    new_dev->configProc = NULL;
    new_dev->nextFrameProc = NULL;

    /*
     * for now , we assume this call => get decompress device
     */
    tmp = (J300ClientData *) new_dev->deviceData;
    tmp->mode = J300_DECOMPRESS_MODE;
    tmp->inited = 0;

    /*
     * also need to update the global structure for the device
     */
    return new_dev;
}



#else /* J300 */

/*
 *--------------------------------------------------------------
 *
 * GetJ300Device --
 *
 *	Stub routine for machines without the J300 board
 *
 * Results:
 *	NULL
 *
 * Side effects:
 *	None.
 *
 * Parameters:
 *	MJPEGResource *objPtr	pointer to the resource object data structure
 *
 *--------------------------------------------------------------
 */
MJPEGDevice *
GetJ300Device (MJPEGResource *objPtr)
{
    return NULL;
}

#endif  /* J300 */
