/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


/*
 * $Id: cmJ300Device.h,v 1.9 1996/02/06 22:41:20 kpatel Exp $
 *
 * Defines the interface to the J300 device for the mjpegPlay
 * object, used to do hardware assisted jpeg decoding (using
 * the mme server to talk to the j300, not the jv2driver)
 *
 */

#include <stdio.h>
#ifdef NO_STDLIB_H
#include "compat/stdlib.h"
#else
#include <stdlib.h>
#endif
#include <errno.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <sys/shm.h>
#include <sys/types.h>
#include <sys/mman.h>
#include "mme_api.h"
#include "avifmt.h"
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/extensions/XShm.h>
#include "common.h"
#include "mmc_cmdlin.h"
#include "mmc_memory.h"
#include "cmCookies.h"
#include "cmBuffer.h"

#define J300_DECOMPRESS_MODE	1
#define J300_COMPRESS_MODE	0
#define J300_DISPLAY	1
#define J300_NODISPLAY	0
#define ONLY_INPUT	0
#define ONLY_OUTPUT	1
#define INPUT_AND_OUTPUT	2

#define JPEG_SIZE  (PAGE_PAD(256*1024))


typedef struct {
	int mode;		/* 1=> grab,  0=> display */
	int blocking;		/* 1=> yes, 0=> no */
	int numBuffers;		/* 1 for now, more if we put in the J300
				 * streaming interface
				 */
	char ** lpOutput;
	char ** lpInput;
	XShmSegmentInfo ShmInfo;
	BOOL            noShm;
	int display;		/* 1=>true, 0=> no display */
	HIC hic			/* handle to decompressor/compressor */	;
	BITMAPINFOHEADER *lpbiIn, *lpbiOut; /*lpbiIn - contains format of
						input data */
	RGBQUAD *lpbiPalette;
	Visual *xvisual;
	int inImageSize,outImageSize;
	int xid;		/* x id of display window*/
	int inited;
	int decompressed;	/* frame has been decompressed*/
	int swapRB;
	/*
	int x;
	int y;
	*/
	Display *dpy;
	Window win;
	GC gc;
	int desiredDepth;
	int screenDepth;
	int w;			/* w of display window */
	int h;			/* h of display window */
	int qfactor;
	int subformat; 		/*subformats defined in cookies.h */
	char jfifHeader[607];	/* required only for subformat VID_FRAME_CODE */
	int jfifInited;		/* 1=> jfif header has been created */
	int squeeze;	/* 1=> squeeze o/p frame to outputW, outputH */
	int outputW;	/* desired width of decompressed frame*/
	int outputH;	/* desired height of decompressed frame*/
	int inputW;	/* width of input frame */
	int inputH;	/* height of input frame */
	int decompressDevInited;

	/* what about JFIF_NO_HUFFMAN_FRAME_CODE ? -- use the same table for all
	displays or maintain a different table for each media stream ? */
}J300ClientData;


/* This structure is currebtly not being used */
/* will be reqd if we have only one device object for the system */
typedef struct {
	int refcount;
	int numGrabObjects;
	int numDisplayObjects;
	int grabFrameRate;
	int cumDisplayFrameRate;
} J300Device;


/* error numbers */
#define J300_ICDECOMPRESS_FAILED 		10
#define J300_ICDECOMPRESSGETPALETTE_FAILED 	2
#define J300_MEMORYALLOCATION_FAILED 		3
#define J300_CANNOT_OPEN_DECOMPRESSOR		4
#define J300_ICDECOMPRESSBEGIN_FAILED 		5
#define J300_ICPREPAREHEADER_FAILED 		6
#define J300_ICUNPREPAREHEADER_FAILED 		7
#define J300_ICDECOMPRESSEND_FAILED 		8
#define J300_NODECOMPRESS_DEVICE		9
