/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


#ifndef _MJPEGCapture_
#define _MJPEGCapture_

#include <stdio.h>
#include <tcl.h>
#include <tk.h>
#include "cmCookies.h"
#include "cmBufferlist.h"

/*
 * This file contains the data structures used by the MJPEG Capture object.
 * It should be included by any capture devices.
 */

typedef struct MJPEGFrame {
    MJPEGBufferHeader frameHeader;
    double logStart;
    double logEnd;
    ScatterBuffer *sb;
    Buffer *dataBuffer;
    char *data;
    int dataLength;
    ClientData deviceData;
    struct MJPEGCapture *objPtr;
    struct MJPEGFrame *next;
    struct MJPEGFrame *prev;
    int readyId;
} MJPEGFrame;

typedef struct MJPEGCaptureDevice {
    ClientData deviceData;
    int asyncEncodeFlag;
    int (*captureProc)	(MJPEGFrame *framePtr);
    int (*encodeProc)	(MJPEGFrame *framePtr);
    int (*refreshProc)  (struct MJPEGCapture *objPtr);
    int (*initProc)	(struct MJPEGCapture *objPtr);
    int (*reinitProc)	(struct MJPEGCapture *objPtr);
    int (*shutdownProc)	(struct MJPEGCapture *objPtr);
    int (*freeFrameProc)(MJPEGFrame *framePtr);
    int (*configProc)	(struct MJPEGCapture *objPtr, char *option, char *value);
    char *(*errorProc)	(struct MJPEGCapture *objPtr, int errNum);
} MJPEGCaptureDevice;

typedef struct MJPEGCapture {
    Tcl_Interp *interp;		/* Tcl interpreter that created object. */
    char *name;			/* Tcl string name of this object. */
    int width;			/* Width of window. */
    int height;			/* Height of window. */
    int xpos;			/* X offset of image in window. If  -1,
				 * device should center. */
    int ypos;			/* Y offset of image in window. If -1, device
				 * should center. */
    int squeeze;		/* Squeeze flag.  */
    int qfactor;		/* Q Factor for JPEG compression */ 

    ClientData lts;		/* LTS for object */
    int ltsTriggerNum;
    char *ltsName;

    double maxFrameRate;	/* max frame rate, if 0, means don't limit*/

    MJPEGCaptureDevice
	*devicePtr;		/* Device control structure. */
    char *devtype;		/* Name of device */

    double fps;			/* Frames per second to capture */ 

    double statStartTime;	/* Used for statistics and feedback control. */
    int captured;		/* Number of frames captured. */
    int bytes;			/* Number of bytes captured. */

    int ready;			/* Ready flag. */

    int inited;			/* Flag that indicates device init proc. has
				 * been called. */

    int inAsynchEncode;		/* Flag that indicates device currenly
				 * decoding. */

    MJPEGFrame *currFrame;	/* Currently available encoded frame */
    MJPEGFrame *encodeFrame;	/* Frame being encoded */

    int atToken;		/* At token for next scheduled trigger. */

    int bindToken;              /* CM Bind id. */
    MJPEGFrame *bindFramePtr;   /* Pointer to frame that subst. routines should use. */

    char *triggerHandle;	/* The trigger atom name */

    char *outCmd;               /* Command to send frames to. */
    InFunc outCCmd;             /* C pointer to out command. */
    ClientData outCD;           /* Client data for C pointer to out command. */

    double delay;               /* Amount of delay to add to start and end times. */

} MJPEGCapture;

#ifdef PARALLAX
extern MJPEGCaptureDevice *GetParallaxCaptureDevice(MJPEGCapture *objPtr);
#endif

#ifdef PARALLAX_DEFAULT
#  define GetDefaultJpegCaptureDevice	GetParallaxCaptureDevice
#  define DEFAULT_JPEG_CAPTURE_DEV_TYPE "parallax"
#else
#  define GetDefaultJpegCaptureDevice(x)	NULL
#  define DEFAULT_JPEG_CAPTURE_DEV_TYPE ""
#endif

#endif	/* _MJPEGCapture_ */
