/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


/*
 * $Id: cmMJPEGPlay.h,v 1.14 1996/02/06 22:41:45 kpatel Exp $
 *
 * Defines data structures, etc.. used by the "mjpegPlay" object
 * which manages a device that decodes and displays MJPEG
 * compressed video
 *
 */

#ifndef _MJPEGResource_
#define _MJPEGResource_

#include <stdio.h>
#include <tcl.h>
#include <tk.h>
#include "cmCookies.h"
#include "cmBufferlist.h"

typedef struct MJPEGFrame {
    MJPEGBufferHeader frameHeader;
    double logStart;
    double logEnd;
    ScatterBuffer *sb;
    Buffer *dataBuffer;
    char *data;
    int dataLength;
    ClientData deviceData;
    int at_token;
    struct MJPEGResource *objPtr;
    struct MJPEGFrame *next;
    struct MJPEGFrame *prev;
} MJPEGFrame;

typedef struct MJPEGDevice {
    ClientData deviceData;
    int asyncDecodeFlag;
    int (*decodeProc)(MJPEGFrame *framePtr);
    int (*refreshProc)(struct MJPEGResource *objPtr);
    int (*displayProc)(MJPEGFrame *framePtr);
    int (*initProc)(struct MJPEGResource *objPtr);
    int (*reinitProc)(struct MJPEGResource *objPtr);
    int (*shutdownProc)(struct MJPEGResource *objPtr);
    int (*freeFrameProc)(MJPEGFrame *framePtr);
    int (*nextFrameProc)(struct MJPEGResource *objPtr, MJPEGFrame *frameList, 
			 MJPEGFrame **chosenFrame);
    int (*configProc)(struct MJPEGResource *objPtr, char *option, char *value);
    char *(*errorProc)(struct MJPEGResource *objPtr, int errNum);
} MJPEGDevice;

typedef struct MJPEGResource {
    Tcl_Interp *interp;		/* Tcl interpreter that created object. */
    char *name;			/* Tcl string name of this object. */
    int width;			/* Width of window. */
    int height;			/* Height of window. */
    int xpos;			/* X offset of image in window. If  -1,
				 * device should center. */
    int ypos;			/* Y offset of image in window. If -1, device
				 * should center. */
    int squeeze;		/* Squeeze flag.  */
    int xid;			/* Xid of window. */
    ClientData lts;		/* LTS for object */
    int ltsTriggerNum;
    char *ltsName;
    int maxFrameRate;		/* max frame rate, if 0, means don' */
    MJPEGDevice *devicePtr;	/* Device control structure. */
    char *devtype;		/* Name of device */
    double statStartTime;	/* Used for statistics and feedback control. */
    int missed;			/* Number of frames missed on arrival. */
    int late;			/* Number of frames late after decode. */
    int played;			/* Number of frames queued for play. */
    int bytes;			/* Number of bytes sent to decode. */
    int ready;			/* Ready flag. */
    int inited;			/* Flag that indicates device init proc. has
				 * been called. */
    int inDecode;		/* Flag that indicates device currenly
				 * decoding. */
    MJPEGFrame *queue;          /* Queue of frames to be decoded and played */
    char *cntrlEveryHandle;	/* Every handle for feedback control. */
    int destroyFlag;
    int *at_tokens;             /* At tokens. */
    int num_at_tokens;          /* Size of at token list. */
    char *acceptHandle;
    int bindToken;              /* CM Bind id. */
    MJPEGFrame *bindFramePtr;   /* Pointer to frame that subst. routines should use. */
} MJPEGResource;

extern int MJPEG_RSRC_DecodeDone(MJPEGFrame *framePtr);
extern MJPEGDevice *GetParallaxDevice(MJPEGResource *objPtr);
extern MJPEGDevice *GetJ300Device(MJPEGResource *objPtr);
extern MJPEGDevice *GetSoftwareJpegDevice(MJPEGResource *objPtr);
extern MJPEGDevice *GetSunVideoJpegDevice(MJPEGResource *objPtr);
extern MJPEGDevice *GetCosmoDevice(MJPEGResource *objPtr);

#ifdef PARALLAX_DEFAULT
#  define GetDefaultJpegDevice	GetParallaxDevice
#  define DEFAULT_JPEG_PLAY_DEV_TYPE "parallax"
#elif J300_DEFAULT
#  define GetDefaultJpegDevice	GetJ300Device
#  define DEFAULT_JPEG_PLAY_DEV_TYPE "j300"
#elif COSMO
#  define GetDefaultJpegDevice	GetCosmoDevice
#  define DEFAULT_JPEG_PLAY_DEV_TYPE "cosmo"
#elif JPEG_SW_DEFAULT
#  define GetDefaultJpegDevice	GetSoftwareJpegDevice
#  define DEFAULT_JPEG_PLAY_DEV_TYPE "software"
#elif SUN_VIDEO
#  define GetDefaultJpegDevice	GetSunVideoJpegDevice
#  define DEFAULT_JPEG_PLAY_DEV_TYPE "sunvideo"
#else
#  define GetDefaultJpegDevice(x)	NULL
#  define DEFAULT_JPEG_PLAY_DEV_TYPE ""
#endif

#endif	/* _MJPEGResource_ */
