/*
 * COPYRIGHT NOTICE 
 * 
 * Copyright (c) Digital Equipment Corporation, 1993, 1994.
 * All Rights reserved. Unpublished rights reserved under the 
 * copyright laws of the United States. Copyright is claimed in 
 * the computer program and user interface thereof. 
 * 
 * The software contained on this media is proprietary to and 
 * embodies the confidential technology of Digital Equipment 
 * Corporation. Possession, use, duplication or dissemination of 
 * the software and media is authorized only pursuant to a valid
 * written license from Digital Equipment Corporation. 
 * 
 * The name of Digital Equipment Corporation may not be used to endorse or 
 * promote products derived from this software without specific prior 
 * written permission. All other rights reserved.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, IMPLIED WARRANTIES OF
 * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. 
 * Digital assumes no responsibility AT ALL for the use or reliability of 
 * this software.
 * 
 * 
 * +---------------------------------------------------------------------+
 * | USE, DUPLICATION OR DISCLOSURE BY THE U.S. GOVERNMENT IS SUBJECT TO | 
 * | RESTRICTIONS AS SET FORTH IN SUBPARAGRAPH (c)                       |
 * | DFARS 252.227-7013, OR IN FAR 52.227-19 OR IN 52.227-14 ALT III,    |
 * | AS APPLICABLE                                                       |
 * +---------------------------------------------------------------------+
 * 
 * 
 * 
 * 
 * 
 * 
 */
/*
 * HISTORY
 */
/*
**++
** FACILITY:	Digital Equipment Corporation
**		Multimedia Services for Alpha AXP
**		Development Kit Sample Code
**
** FILE NAME:	mmc_messages.c
**
** PURPOSE:	Handle message display routines.
**
** NOTES:	Public interface definition in mmc_messages.h.
**--
*/

#include <mme/mme_api.h>
#include <stdio.h>
#include <stdarg.h>
#include "mmc_messages.h"

/* Global Storage */
char *ProgName=NULL;
int LineFeedOn = TRUE;

/*
** ++
** FUNCTION:  mmcErrorDisplay
**
** PURPOSE:  Display error message with multiple args
**
** INPUT:
**	fmt = formatted string
**      variable arguments to be used in formatted string
**
** OUTPUT:   none.
**
** RETURN VALUE:  none.
**
** SIDE EFFECTS: 
**        Error message displayed to stderr
**
** NOTES:  none.
**--
*/
void mmcErrorDisplay (char *fmt, ...)
{
     va_list args;

     va_start(args, fmt);
     if (ProgName != NULL)
       fprintf(stderr, "%s: <error> ",ProgName);
     else
       fprintf(stderr, "<error> ");
     vfprintf(stderr, fmt, args);
     if (LineFeedOn)
       fprintf(stderr,"\n");
     va_end(args);
}


/*
** ++
** FUNCTION:  mmcStatusDisplay
**
** PURPOSE:  Display status message with multiple args
**
** INPUT:
**	fmt = formatted string
**      variable arguments to be used in formatted string
**
** OUTPUT:   none.
**
** RETURN VALUE:  none.
**
** SIDE EFFECTS: 
**        Status message displayed to stdout
**
** NOTES:  none.
**--
*/
void mmcStatusDisplay (char *fmt, ...)
{
     va_list args;

     va_start(args, fmt);
     if (ProgName != NULL)
       fprintf(stdout, "%s: ",ProgName);
     vfprintf(stdout, fmt, args);
     if (LineFeedOn)
       fprintf(stdout,"\n");
     va_end(args);
}


/*
** ++
** FUNCTION:  mmcMsgDisplay
**
** PURPOSE:  Display message with multiple args
**
** INPUT:
**	fmt = formatted string
**      variable arguments to be used in formatted string
**
** OUTPUT:   none.
**
** RETURN VALUE:  none.
**
** SIDE EFFECTS: 
**        Status message displayed to stdout
**
** NOTES:  none.
**--
*/
void mmcMsgDisplay (char *fmt, ...)
{
     va_list args;

     va_start(args, fmt);
     if (ProgName != NULL)
       fprintf(stdout, "%s: ",ProgName);
     vfprintf(stdout, fmt, args);
     if (LineFeedOn)
       fprintf(stdout,"\n");
     va_end(args);
}


/*
** ++
** FUNCTION:  mmcVerboseDisplay
**
** PURPOSE:  Display verbose message with multiple args
**
** INPUT:
**      verbose = 1->to be displayed, 0->no display
**	fmt = formatted string
**      variable arguments to be used in formatted string
**
** OUTPUT:   none.
**
** RETURN VALUE:  none.
**
** SIDE EFFECTS: 
**        Verbose message displayed to stdout
**
** NOTES:  none.
**--
*/
void mmcVerboseDisplay (int verbose, char *fmt, ...)
{
    va_list args;
    
    if (verbose)
    {
	va_start(args, fmt);
	if (ProgName != NULL)
	  fprintf(stdout, "%s: ",ProgName);
	vfprintf(stdout, fmt, args);
	if (LineFeedOn)
	  fprintf(stdout,"\n");
	va_end(args);
    }
    printf(stdout, "verbose: ");
}


/*
** ++
** FUNCTION:  mmeMessageInitProgName
**
** PURPOSE:  Store program name to be display with each of the display messages
**
** INPUT:
**      progName = name of program
**
** OUTPUT:   none.
**
** RETURN VALUE:  none.
**
** SIDE EFFECTS: none.
**
** NOTES:  none.
**--
*/
void mmcMessageInitProgName (char *progName)
{
    if (ProgName != NULL)
      free (ProgName);

    ProgName = (char *)malloc(strlen(progName)+1);
    strcpy(ProgName, progName);
}


/*
** ++
** FUNCTION:  mmeMessageInitLineFeed
**
** PURPOSE:  Store program name to be display with each of the display messages
**
** INPUT:
**      progName = name of program
**
** OUTPUT:   none.
**
** RETURN VALUE:  none.
**
** SIDE EFFECTS: none.
**
** NOTES:  none.
**--
*/
void mmcMessageInitLineFeed (int onFlag)
{
    LineFeedOn = onFlag;
}




