/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


/*
 * $Id: cmMPEGFileInterface.h,v 1.4 1996/02/06 22:41:55 kpatel Exp $
 *
 * Defines data structures used by the MPEG file segment
 * object and the MPEG file interface
 *
 */

#include "cmCookies.h"
#include "cmBufferlist.h"
#include "cmChunk.h"

#define MPEG_I_FRAME 1
#define MPEG_P_FRAME 2
#define MPEG_B_FRAME 3
#define MPEG_SYS_HDR_START_CODE 0x000001b3
#define MPEG_END_SEQ_CODE       0x000001b7
#define MPEG_PIC_START_CODE     0x00000100

typedef struct RefEntry {
  int type;
  int ref1;
  int ref2;
  int headerRef;   /* -1 = default matrices. */
} RefEntry;

typedef struct HdrEntry {
    int intra_matrix_flag;
    int intra_matrix[64];
    int non_intra_matrix_flag;
    int non_intra_matrix[64];
} HdrEntry;

typedef struct MPEGDataFile {
    char *filename;
    tChunkFileState *cfState;
    int raw_flag;
    int fd;
    int *FOT;
    double *FTT;
    RefEntry *refTable;
    int numFrames;
    double clipLength;
    int dataStart;
    int numHeaders;
    HdrEntry *headerTable;
    MPEGHeader *cfInfo;
    int file_refCount;
    int open_refCount;
} MPEGDataFile;

typedef struct DynamicBitArray {
    int offset;
    char *bit_array;
    int size;
} DynamicBitArray;

typedef struct MPEGFileInterface {
    int is_open;
    DynamicBitArray *bit_array;
    MPEGDataFile *data_file;
} MPEGFileInterface;


int MPEG_FIP_AspectRatioCode (MPEGFileInterface *fip);
int MPEG_FIP_Height (MPEGFileInterface *fip);
int MPEG_FIP_Width (MPEGFileInterface *fip);
int MPEG_FIP_FrameRateCode (MPEGFileInterface *fip);
int MPEG_FIP_VBVBufferSize (MPEGFileInterface *fip);
int MPEG_FIP_BitRate (MPEGFileInterface *fip);
int MPEG_FIP_ConstrainedParamFlag (MPEGFileInterface *fip);
double MPEG_FIP_ClipLength (MPEGFileInterface *fip);
void MPEG_FIP_FDClose (MPEGFileInterface *fip);
void MPEG_FIP_FDOpen (MPEGFileInterface *fip);
int MPEG_FIP_ReadFrameFromFile (MPEGFileInterface *fip, int frameNum, char *buffer);
int MPEG_FIP_FrameListFromTime (MPEGFileInterface *fip, double start, double end, int reverseFlag, int **framelistPtr, double *etimePtr);
Buffer *MPEG_FIP_CreateIntraMatrixBuffer (Tcl_Interp *interp, MPEGFileInterface *fip, int frameNum, int *buflenPtr);
Buffer *MPEG_FIP_CreateNonIntraMatrixBuffer (Tcl_Interp *interp, MPEGFileInterface *fip, int frameNum, int *buflenPtr);
int MPEG_FIP_GetAllFrameInfo (MPEGFileInterface *fip, int frameNum, int *sizePtr, double *startPtr, double *endPtr, int *ref1Ptr, int *ref2Ptr, int *typePtr);
int MPEG_FIP_TimeFromFrame (MPEGFileInterface *fip, int frameNum, double *startPtr, double *endPtr);
void MPEG_FIP_Close (MPEGFileInterface *fip);
MPEGFileInterface *MPEG_FIP_Open (char *filename);
void MPEG_FIP_Info (MPEGFileInterface *fip, double start, double end, int *numFramesPtr, int *maxSizePtr, int *numIPtr, int *numPPtr, int *numBPtr);

void MPEG_FIP_BitArraySet(DynamicBitArray *barray, int pos);
int MPEG_FIP_BitArrayGet(DynamicBitArray *barray, int pos);
void MPEG_FIP_BitArrayClear(DynamicBitArray *barray);
