/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


/*
 * $Id: cmMPEGPlay.h,v 1.9 1996/02/06 22:42:02 kpatel Exp $
 *
 * Defines the "mpegPlay" object, which uses an mpeg decoding
 * device to decode and display mpeg compressed video.
 *
 */

#ifndef _MPEGPlay_
#define _MPEGPlay_

#include <stdio.h>
#include <tcl.h>
#include <tk.h>
#include "cmCookies.h"
#include "cmBufferlist.h"

typedef struct MPEGFrame {
    MPEGBufferHeader frameHeader;
    double logStart;
    double logEnd;
    ScatterBuffer *sb;
    Buffer *dataBuffer;
    char *data;
    int dataLength;
    ClientData deviceData;
    int at_token;
    struct MPEGPlay *objPtr;
    struct MPEGFrame *ref1;
    struct MPEGFrame *ref2;
    int *intra_matrix;
    int *non_intra_matrix;
    int refCount;
    int onRefQueue;
    int freeFlag;
    struct MPEGFrame *next;
    struct MPEGFrame *prev;
} MPEGFrame;

typedef struct MPEGDevice {
    ClientData deviceData;
    int asyncDecodeFlag;
    int (*decodeProc)(MPEGFrame *framePtr);
    int (*refreshProc)(struct MPEGPlay *objPtr);
    int (*displayProc)(MPEGFrame *framePtr);
    int (*initProc)(struct MPEGPlay *objPtr);
    int (*reinitProc)(struct MPEGPlay *objPtr);
    int (*shutdownProc)(struct MPEGPlay *objPtr);
    int (*freeFrameProc)(MPEGFrame *framePtr);
    int (*configProc)(struct MPEGPlay *objPtr, char *option, char *value);
    int (*nextFrameProc)(struct MPEGPlay *objPtr, MPEGFrame *frameList, 
			 MPEGFrame **chosenFrame);
    char *(*errorProc)(struct MPEGPlay *objPtr, int errNum);
} MPEGDevice;

typedef struct MPEGPlay {
    Tcl_Interp *interp;		/* Tcl interpreter that created object. */
    char *name;			/* Tcl string name of this object. */
    int width;			/* Width of window. */
    int height;			/* Height of window. */
    int xpos;			/* X offset of image in window. If  -1,
				 * device should center. */
    int ypos;			/* Y offset of image in window. If -1, device
				 * should center. */
    int squeeze;		/* Squeeze flag.  */
    int xid;			/* Xid of window. */
    ClientData lts;		/* LTS for object */
    int ltsTriggerNum;
    char *ltsName;
    int maxFrameRate;		/* max frame rate, if 0, means don' */
    char *ctrlCallback;		/* Frame rate control callback. */
    double ctrlCycleTime;	/* Control cycle time. */
    MPEGDevice *devicePtr;	/* Device control structure. */
    char *devtype;		/* Name of device */
    double statStartTime;	/* Used for statistics and feedback control. */
    int missed;			/* Number of frames missed on arrival. */
    int late;			/* Number of frames late after decode. */
    int played;			/* Number of frames queued for play. */
    int bytes;			/* Number of bytes sent to decode. */
    int ready;			/* Ready flag. */
    int inited;			/* Flag that indicates device init proc. has
				 * been called. */
    MPEGFrame *inDecodeFrame;   /* Pointer to frame currently being decoded.
				   Serves as a flag to when we are decoding. */
    MPEGFrame *queue;		/* Queue of frames ready to be decoded and played */
    MPEGFrame *waitQueue;       /* Q fo frames waiting for ref frames to be decoded. */
    MPEGFrame *refQueue;        /* Queue of frames already decoded that may be ref's */
    char *cntrlEveryHandle;	/* Every handle for feedback control. */
    int *at_tokens;             /* At tokens. */
    int num_at_tokens;          /* Size of at token list. */
    char *acceptHandle;
    int bindToken;              /* CM Bind id. */
    MPEGFrame *bindFramePtr;    /* Pointer to frame that subst. routines should use. */
} MPEGPlay;

extern int MPEG_RSRC_DecodeDone(MPEGFrame *framePtr);
extern MPEGDevice *GetSoftwareMpegDevice(MPEGPlay *objPtr);
extern MPEGDevice *GetOptiVideoMpegDevice(MPEGPlay *objPtr);

#ifdef MPEG_SW_DEFAULT
#  define GetDefaultMpegDevice	GetSoftwareMpegDevice
#  define DEFAULT_MPEG_PLAY_DEV_TYPE "software"
#elif MPEG_OPTIVIDEO_DEFAULT
#  define GetDefaultMpegDevice  GetOptiVideoMpegDevice
#  define DEFAULT_MPEG_PLAY_DEV_TYPE "optivideo"
#else
#  define GetDefaultMpegDevice(x)	NULL
#  define DEFAULT_MPEG_PLAY_DEV_TYPE ""
#endif

#endif	/* _MPEGPlay_ */

