/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */



/*
 * Make sure we have ntohs/ntohl
 */
#include <netinet/in.h>

/*
 * This structure defines the header on every RTP packet
 *
 *  0                     1                   2                   3
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |V=2|P|X|   CC  |M|     PT      |       sequence number         |
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |                           timestamp                           |
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |          synchronization source (SSRC) identifier             |
 * +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
 * |           contributing source (CSRC) identifies               |
 * |                           ....                                |
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * 
 */

typedef struct RTPHeader {
    unsigned short flags;
    unsigned short seqno;
    unsigned int timestamp;
    unsigned int ssrc;
    unsigned int csrc[0];
} RTPHeader;

#define rtpver(rh) (ntohs((rh).flags) >> 14)
#define rtppad(rh) ((ntohs((rh).flags) & 0x2000) >> 13)
#define rtpext(rh) ((ntohs((rh).flags) & 0x1000) >> 12)
#define rtpcc(rh) ((ntohs((rh).flags) & 0x0f00) >> 8)
#define rtpmark(rh) ((ntohs((rh).flags) & 0x80) >> 7)
#define rtppt(rh) (ntohs((rh).flags) & 0x7f)

typedef struct RTCPHeader {
    unsigned char flags;
    unsigned char pt;
    unsigned short len;
    unsigned int ssrc;
} RTCPHeader;

#define rtcpver(rh) ((rh).flags >> 6)
#define rtcppad(rh) (((rh).flags & 0x40) >> 5)
#define rtcpsc(rh) ((rh).flags & 0x1f)

typedef struct RTCPRecv {
    unsigned int ssrc;
    unsigned int lost;
    unsigned int seqno;
    unsigned int jitter;
    unsigned int lsr;
    unsigned int dlsr;
} RTCPRecv;

#define rtcprrfraclost(rr) ((ntohl(rr).lost & 0xf000) >> 24)
#define rtcprrnumlost(rr) (ntohl(rr).lost & 0xfff)

typedef struct RTCPSend {
    unsigned int ntphi;
    unsigned int ntplo;
    unsigned int rtptime;
    unsigned int pkts;
    unsigned int bytes;
} RTCPSend;

typedef struct RTPSRPkt {
    RTCPHeader hdr;
    RTCPSend sr;
    RTCPRecv rr[0];
} RTPSRPkt;

typedef struct RTPRRPkt {
    RTCPHeader hdr;
    RTCPRecv rr[0];
} RTPRRPkt;

typedef enum {
    SDES_END	= 0,
    SDES_CNAME	= 1,
    SDES_NAME	= 2,
    SDES_EMAIL	= 3,
    SDES_PHONE	= 4,
    SDES_LOC	= 5,
    SDES_TOOL	= 6,
    SDES_NOTE	= 7,
    SDES_PRIV	= 8
} SDESType;

typedef struct {
    unsigned char type;
    unsigned char len;
    char data[0];
} RTCPSDES;

typedef struct {
    int ssrc;
    RTCPSDES sdes[0];
} RTCPSDESEntry;


typedef enum {
    RTCP_SR	= 200,
    RTCP_RR	= 201,
    RTCP_SDES	= 202,
    RTCP_BYE	= 203,
    RTCP_APP	= 204,

    OLD_RTCP_SR	= 0,
    OLD_RTCP_RR	= 1,
    OLD_RTCP_SDES	= 2,
    OLD_RTCP_BYE	= 3,
    OLD_RTCP_APP	= 4
} RTCPType;

#define RTCP_SESSION_BW_FRACTION (0.05)
#define RTCP_SENDER_BW_FRACTION (0.25)
#define RTCP_RECV_BW_FRACTION (1.0 - RTCP_SENDER_BW_FRACTION)
#define RTCP_MIN_INTERVAL (5.)
