/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */



/* Loads a clipfile from disk and intializes data structures. */

int loadSparcClip( SparcAudioClipFile *objPtr )
{
  tChunkFileState *cf;
  tChunkHeader ch;
  SparcAudioHeader *sparcHdr;
  MediaIndepHeader *mediaHdr;

  cf = objPtr->cf = Chunk_OpenFile( objPtr->filename, "r");

  /*
   *  Check to see if this is a valid chunk file.  If not, process raw
   *  audio file.  Otherwise, load media indep. and sparcAu headers.
   */

  if( cf == NULL ) {	
    Tcl_AppendResult( objPtr->interp, "File is not a valid clipfile.", 
		     (char *) NULL );
    return TCL_ERROR;
  }    
  
  if( Chunk_FindType(cf, CM_MEDIA_INDEP_CF, &ch) == 0 ) {
    if( ch.chunkSize != sizeof(MediaIndepHeader) ) {
      Tcl_AppendResult( objPtr->interp, "Problem reading header.", 
		       (char *) NULL );
      return TCL_ERROR;
    }
    else {
      mediaHdr = (MediaIndepHeader *) ckalloc( sizeof(MediaIndepHeader) );
      read( cf->fd, (char *)mediaHdr, ch.chunkSize);
    }
  }
  else {
    Tcl_AppendResult( objPtr->interp, "Can't find media header in clipfile.", 
		     (char *) NULL );
    return TCL_ERROR;		/* Couldn't find media indep. header. */
  }

  Chunk_ReadRewind( cf ); 

  if( Chunk_FindType(cf, CM_SPARC_AU_CF, &ch) == 0 ) {
    if( ch.chunkSize != sizeof(SparcAudioHeader) ) {
      Tcl_AppendResult( objPtr->interp, "Problem reading header.", 
		       (char *) NULL );
      return TCL_ERROR;
    }
    else {
      sparcHdr = (SparcAudioHeader *) ckalloc( sizeof(SparcAudioHeader) );
      read( cf->fd, (char *)sparcHdr, ch.chunkSize);
    }
  }
  else {
    Tcl_AppendResult( objPtr->interp, "Can't find sparc header in clipfile.", 
		     (char *) NULL );
    return TCL_ERROR;		/* Couldn't find audio header */
  }

  objPtr->hdr = sparcHdr;
  objPtr->mediaHdr = mediaHdr;
  objPtr->fd = cf->fd;
  return 0;
}

