/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


/*
 * cmUtil.c --
 *
 *	This file provides miscellaneous utility routines.
 *
 */

#ifdef NO_STRING_H
#   include "compat/string.h"
#else
#   include <string.h>
#endif

#include <sys/types.h>
#include <sys/time.h>
#include <tcl.h>

/*
 *--------------------------------------------------------------
 *
 * Tcm_GetLocalAddress --
 *
 *	Returns the address of the local host as a string
 *
 * Results:
 *	Pointer to string the has local Internet address in dot notation.
 *
 * Side effects:
 *	Interpreter result is destroyed, and the Tcl procedure
 *	CM_GetLocalAddress is defined.
 *
 * Parameters:
 *	interp - pointer to Tcl interpreter
 *	port   - 
 *
 *--------------------------------------------------------------
 */
char *
Tcm_GetLocalAddress (Tcl_Interp *interp)
{
    static int inited = 0;
    static char cache[256];

    if (!inited) {
	inited++;
	
	if (Tcl_GlobalEval (interp, "CM_GetLocalAddress") == TCL_OK) {
	    strcpy (cache, interp->result);
	}
    }
    return cache;
}

/*
 *----------------------------------------------------------------------
 *
 * Tcm_ReadSysClock --
 *
 *	Return the value of the system clock as a double.
 *
 * Results:
 *	Value of the system clock.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */
double Tcm_ReadSysClock()
{
    struct timeval tv;
    (void) gettimeofday(&tv, NULL);
    return (tv.tv_sec + tv.tv_usec / 1000000.0);
}
