/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


#ifndef _YUVResource_
#define _YUVResource_

#include <stdio.h>
#include <tcl.h>
#include <tk.h>
#include "cmCookies.h"
#include "cmBufferlist.h"

typedef struct YUVFrame {
    YUVBufferHeader frameHeader;
    double logStart;
    double logEnd;
    ScatterBuffer *sb;
    Buffer *dataBuffer;
    char *data;
    int dataLength;
    int at_token;
    struct YUVResource *objPtr;
    struct YUVFrame *next;
    struct YUVFrame *prev;
    ClientData deviceData;
} YUVFrame;

typedef struct YUVDevice {
    ClientData deviceData;
    int (*refreshProc)(struct YUVResource *objPtr);
    int (*ditherProc)(YUVFrame *framePtr);
    int (*displayProc)(YUVFrame *framePtr);
    int (*initProc)(struct YUVResource *objPtr);
    int (*reinitProc)(struct YUVResource *objPtr);
    int (*shutdownProc)(struct YUVResource *objPtr);
    int (*freeFrameProc)(YUVFrame *framePtr);
    int (*configProc)(struct YUVResource *objPtr, char *option, char *value);
    char *(*errorProc)(struct YUVResource *objPtr, int errNum);
    int (*nextFrameProc)(struct YUVResource *objPtr, YUVFrame *frameList, 
			 YUVFrame **chosenFrame);
} YUVDevice;

typedef struct YUVResource {
    Tcl_Interp *interp;		/* Tcl interpreter that created object. */
    char *name;			/* Tcl string name of this object. */
    int width;			/* Width of window. */
    int height;			/* Height of window. */
    int xpos;			/* X offset of image in window. If  -1,
				 * device should center. */
    int ypos;			/* Y offset of image in window. If -1, device
				 * should center. */
    int squeeze;		/* Squeeze flag.  */
    int xid;			/* Xid of window. */
    ClientData lts;		/* LTS for object */
    int ltsTriggerNum;
    char *ltsName;
    int maxFrameRate;		/* max frame rate, if 0, means don't */
    char *ctrlCallback;		/* Frame rate control callback. */
    double ctrlCycleTime;	/* Control cycle time. */
    YUVDevice *devicePtr;	/* Device control structure. */
    char *devtype;		/* Name of device */
    double statStartTime;	/* Used for statistics and feedback control. */

    YUVFrame *queue;		/* Incoming YUV frames */
    int indecode;

    double ditherTime;
    int missed;			/* Number of frames missed on arrival. */
    int late;			/* Number of frames late after decode. */
    int played;			/* Number of frames queued for play. */
    int bytes;			/* Number of bytes sent to decode. */
    int ready;			/* Ready flag. */
    int inited;			/* Flag that indicates device init proc. has
				 * been called. */
    char *cntrlEveryHandle;	/* Every handle for feedback control. */
    int destroyFlag;
    int *at_tokens;             /* At tokens. */
    int num_at_tokens;          /* Size of at token list. */
    char *acceptHandle;
    int bindToken;              /* CM Bind id. */
    YUVFrame *bindFramePtr;	/* Pointer to frame that subst. routines should use. */
} YUVResource;

extern YUVDevice *GetSoftwareYUVDevice(YUVResource *objPtr);

#define GetDefaultYUVDevice	GetSoftwareYUVDevice
#define DEFAULT_YUV_PLAY_DEV_TYPE "software"

#endif	/* _YUVResource_ */
