/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


/*--------------------------------------------------------------*\
 *	darray.c
 * author:	Eric14@cs.berkeley.edu
 * date:	10-22-94
 *
 * purpose:  See <darray.h>
 *
\*--------------------------------------------------------------*/

#include "vwGeneral.h"
#include "darray.h"

/*--------------------------------------------------------------*/

void
DA_Init (tDArray* da)
{
    da->cnt = 0;
    da->max = 0;
    da->vals = NULL;
}

/*--------------------------------------------------------------*/

static int
sDA_Resize (tDArray* da, int size)
{
    if (!da->vals) {
	da->vals = (void**) malloc(sizeof(void*) * size);
    } else {
	da->vals = (void**) realloc(da->vals, sizeof(void*) * size);
    }	
    ASSERT(da->vals);
    da->max = size;
    return TCL_OK;
}

/*--------------------------------------------------------------*/

int
DA_Num (tDArray* da)
{
    return da->cnt;
}

/*--------------------------------------------------------------*/

int	  
DA_Add (tDArray* da, int idx, void* val)
{
    int		i;

    if (da->cnt >= da->max)
    {
	int	sz = (da->max >= 8) ? da->max*2 : 8;
	int	res = sDA_Resize(da,sz);
	ASSERT(!res);
    }
    if (idx >= da->cnt)
	idx = da->cnt;
    else {
	/* printf("DA_Add: copy\n"); */
/*	memmove(&da->vals[idx+1],&da->vals[idx],
 *		sizeof(void*)*(da->cnt - idx));
 */ /* hard to believe that some systems still don't have this! */
	for (i= da->cnt; i > idx; i--)
	    da->vals[i] = da->vals[i-1];
    }
    da->cnt++;
    DA_Set(da,idx,val);
    return TCL_OK;
}

/*--------------------------------------------------------------*/

void*
DA_Get (tDArray* da, int idx)
{
    ASSERT(idx >= 0 || idx < da->cnt);
    return da->vals[idx];
}

/*--------------------------------------------------------------*/

void
DA_Set (tDArray* da, int idx, void* val)
{
    ASSERT(idx >= 0 && idx < da->cnt);
    da->vals[idx] = val;
}

/*--------------------------------------------------------------*/

int
DA_Del (tDArray* da, int idx)
{
    /* void**  addr = &da->vals[idx];
     */
    int	    i;

    ASSERT(idx >= 0 && idx < da->cnt);
    if (idx < da->cnt-1) {
	/* memmove(addr,addr+1,sizeof(void*)*(da->cnt - idx - 1));
	 */ /* hard to believe that some systems still don't have this! */
	for (i= idx; i < da->cnt; i++)
	    da->vals[i] = da->vals[i+1];
    }
    da->cnt --;
    return TCL_OK;
}

/*--------------------------------------------------------------*/
/* return: TCL_OK if val found in array, else TCL_ERROR.
 * retIdx set to the result of a binary search for element val, Using
 * comparison function compFP.  The result is the index of the element
 * if found or the closest element greater then the target value.
 */

int
DA_BinSearch (tDArray* da, void* val, tDA_CompFunc compFP, int* retIdx)
{
    int		beg, end;
    ASSERT(compFP);
    
    for (beg= 0, end= da->cnt-1; beg <= end; ) {
	int	mid = (beg + end) / 2;
	int	result;
	result = compFP(da->vals[mid],val);
/*	printf("DA_BinSearch: [%d - %d - %d]\n", beg, mid, end);
 */
	if (result == 0) {
	    *retIdx = mid;
	    return 0;
	}
	if (result > 0)
	    beg = mid + 1;
	else
	    end = mid - 1;
    }
    *retIdx = beg;
    return TCL_ERROR;
}

/*--------------------------------------------------------------*/

void
DA_Nullify (tDArray* da)
{
    da->cnt = 0;
    Gen_ForgetPtr(da->vals);
}

/*--------------------------------------------------------------*/
/* End Of File */

