#--------------------------------------------------------
#
# cmt_switch_expand
#
# Parameters:
#   str : string to expand.
#   str_list : list of possible expansions.
#
# Actions:
#   This procedure expands $str to one of the strings in
#   $str_list if $str is a proper prefix for any of the
#   strings and returns the expanded string. 
#   The string is expanded to the first string 
#   that matches. If none match, $str is returned.
#---------------------------------------------------------

proc cmt_switch_expand {str str_list} {
    foreach i $str_list {
	if {[string first $str $i] == 0} {
	    return $i
	}
    }
    return $str;
}

proc cminfo {cmd args} {

    if {$cmd == "set"} {
	eval cminfo_set $args;
    } else {
	if {$cmd == "get"} {
	    eval cminfo_get $args;
	} else {
	    if {$cmd == "add"} {
		eval cminfo_add $args;
	    } else {
		if {$cmd == "del"} {
		    eval cminfo_del $args;
		}
	    }
	}
    }
}

proc cminfo_set {cat args} {
    global cminfoData;

    set cat [cmt_switch_expand $cat {services version medias formats objects object_info}];

    switch $cat {
	services {
	    if {[llength $args] != 1} {
		error {wrong # args, should be: "cminfo set services <service_name>"}
	    }
	    set cminfoData(service) [lindex $args 0];
	}
	version {
	    if {[llength $args] != 1} {
		error {wrong # args, should be: "cminfo set version <version_number>"}
	    }
	    set cminfoData(version) [lindex $args 0];
	}
	medias {
	    if {[llength $args] != 1} {
		error {wrong # args, should be: "cminfo set medias <media_type_list>"}
	    }
	    set cminfoData(media) [lindex $args 0];
	}
	formats {
	    if {[llength $args] != 2} {
		error {wrong # args, should be: "cminfo set formats <media_type> <format_list>"}
	    }
	    set media_type [lindex $args 0];
	    set cminfoData($media_type,formats) [lindex $args 1];
	}
	objects {
	    if {[llength $args] != 3} {
		error {wrong # args should be: "cminfo set objects <media_type> <format_type> <object_list>"}
	    }
	    set media_type [lindex $args 0];
	    set format_type [lindex $args 1];
	    set cminfoData($media_type,$format_type,objects) [lindex $args 2];
	}
	object_info {
	    if {[llength $args] != 2} {
		error {wrong # args, should be: "cminfo set object_info <object_type> <keyword_list>"}
	    }
	    set object_type [lindex $args 0];
	    set cminfoData(object_info,$object_type) [lindex $args 1];
	}
	default {
	    error "bad info type $cat";
	}
    }
}

proc cminfo_get {cat args} {
    global cminfoData;

    set cat [cmt_switch_expand $cat {services version medias formats objects object_info}];

    switch $cat {
	services {
	    if [catch {set ret_val $cminfoData(service)}] {
		return "";
	    }
	    return $ret_val;
	}
	version {
	    if [catch {set ret_val $cminfoData(version)}] {
		return "";
	    }
	    return $ret_val;
	}
	medias {
	    if [catch {set ret_val $cminfoData(media)}] {
		return "";
	    } 
	    return $ret_val;
	}
	formats {
	    if {[llength $args] != 1} {
		error {wrong # args, should be: "cminfo get formats <media_type>"}
	    }
	    set media_type [lindex $args 0];
	    if [catch {set ret_val $cminfoData($media_type,formats)}] {
		return "";
	    } 
	    return $ret_val;
	}
	objects {
	    if {[llength $args] != 2} {
		error {wrong # args should be: "cminfo get objects <media_type> <format_type>"}
	    }
	    set media_type [lindex $args 0];
	    set format_type [lindex $args 1];
	    if [catch {set ret_val $cminfoData($media_type,$format_type,objects)}] {
		return "";
	    } 
	    return $ret_val;
	}
	object_info {
	    if {[llength $args] != 1} {
		error {wrong # args, should be: "cminfo get object_info <object_type>"}
	    }
	    set object_type [lindex $args 0];
	    if [catch {set ret_val $cminfoData(object_info,$object_type)}] {
		return "";
	    }
	    return $ret_val;
	}
	default {
	    error "bad info type $cat";
	}
    }
}

proc cminfo_add {cat args} {
    global cminfoData;

    set cat [cmt_switch_expand $cat {services version medias formats objects object_info}];

    switch $cat {
	medias {
	    if {[llength $args] != 1} {
		error {wrong # args, should be: "cminfo add medias <media_type_list>"}
	    }
	    if {![catch {set cminfoData(media)}]} {
		foreach i [lindex $args 0] {
		    if {[lsearch $cminfoData(media) $i] == -1} {
			eval lappend cminfoData(media) $i;
		    }
		}
	    } else {
		set cminfoData(media) [lindex $args 0];
	    }
	}
	formats {
	    if {[llength $args] != 2} {
		error {wrong # args, should be: "cminfo add formats <media_type> <format_list>"}
	    }
	    set media_type [lindex $args 0];
	    if {![catch {set cminfoData($media_type,formats)}]} {
		foreach i [lindex $args 1] {
		    if {[lsearch $cminfoData($media_type,formats) $i] == -1} {
		    eval lappend cminfoData($media_type,formats) $i;
		    }
		}
	    } else {
		set cminfoData($media_type,formats) [lindex $args 1];
	    }
	}
	objects {
	    if {[llength $args] != 3} {
		error {wrong # args should be: "cminfo add objects <media_type> <format_type> <object_list>"}
	    }
	    set media_type [lindex $args 0];
	    set format_type [lindex $args 1];
	    if {![catch {set cminfoData($media_type,$format_type,objects)}]} {
		foreach i [lindex $args 2] {
		    if {[lsearch $cminfoData($media_type,$format_type,objects) $i] == -1} {
			eval lappend cminfoData($media_type,$format_type,objects) $i;
		    }
		}
	    } else {
		set cminfoData($media_type,$format_type,objects) [lindex $args 2];
	    }
	}
	object_info {
	    if {[llength $args] != 2} {
		error {wrong # args, should be: "cminfo add object_info <object_type> <keyword_list>"}
	    }
	    set object_type [lindex $args 0];
	    if {![catch {set cminfoData(object_info,$object_type)}]} {
		foreach i [lindex $args 1] {
		    if {[lsearch $cminfoData(object_info,$object_type) $i] == -1} {
			eval lappend cminfoData(object_info,$object_type) $i;
		    }
		}
	    } else {
		set cminfoData(object_info,$object_type) [lindex $args 1];
	    }
	}
	default {
	    error "bad info type $cat";
	}
    }
}

proc cminfo_del {cat args} {
    global cminfoData;

    set cat [cmt_switch_expand $cat {services version medias formats objects object_info}];

    switch $cat {
	medias {
	    if {[llength $args] != 1} {
		error {wrong # args, should be: "cminfo del medias <media_type_list>"}
	    }
	    if [catch {set cminfoData(media)}] {
		return;
	    }
	    foreach i [lindex $args 0] {
		set idx [lsearch $cminfoData(media) $i];
		if {$idx != -1} {
		    set cminfoData(media) [lreplace $cminfoData(media) $idx $idx];
		}
	    }
	    return;
	}
	formats {
	    if {[llength $args] != 2} {
		error {wrong # args, should be: "cminfo del formats <media_type> <format_list>"}
	    }
	    set media_type [lindex $args 0];
	    if [catch {set cminfoData($media_type,formats)}] {
		return;
	    }
	    foreach i [lindex $args 1] {
		set idx [lsearch $cminfoData($media_type,formats) $i];
		if {$idx != -1} {
		    set cminfoData($media_type,formats) [lreplace $cminfoData($media_type,formats) $idx $idx];
		}
	    }
	    return;
	}
	objects {
	    if {[llength $args] != 3} {
		error {wrong # args should be: "cminfo del objects <media_type> <format_type> <object_list>"}
	    }
	    set media_type [lindex $args 0];
	    set format_type [lindex $args 1];
	    if [catch {set cminfoData($media_type,$format_type,objects)}] {
		return;
	    }
	    foreach i [lindex $args 2] {
		set idx [lsearch $cminfoData($media_type,$format_type,objects) $i];
		if {$idx != -1} {
		    set cminfoData($media_type,$format_type,objects) [lreplace $cminfoData($media_type,$format_type,objects) $idx $idx];
		}
	    }
	    return;
	}
	object_info {
	    if {[llength $args] != 2} {
		error {wrong # args, should be: "cminfo del object_info <object_type> <keyword_list>"}
	    }
	    set object_type [lindex $args 0];
	    if [catch {set cminfoData(object_info,$object_type)}] {
		return;
	    }
	    foreach i [lindex $args 1] {
		set idx [lsearch $cminfoData(object_info,$object_type) $i];
		if {$idx != -1} {
		    set cminfoData(object_info,$object_type) [lreplace $cminfoData(object_info,$object_type) $idx $idx];
		}
	    }
	    return;
	}
	default {
	    error "bad info type $cat";
	}
    }
}

set CID_counter 0;

proc CID_Init {} {
    global CID_counter;

    incr CID_counter;
    return $CID_counter;
}
