# ------{tcl2doc HEAD}----------------------------------------------
#
# Author: Eric14@cs.berkeley.edu
#
# Date: 02-27-95
#
# Purpose: Generally useful network handling routines
#
# ------------------------------------------------------------------

set gCMAppL_LocalHost		{}
set kCMApp_DefaultCMTPPort	4567
set gCMAPP_MyPort		{}

set gCMAppConnections(purgeList) {}
set gCMAppConnections(cacheMax)  5

# ------{tcl2doc FUNCTION}--------------------------------------
# Returns the name of the current host.

proc CMAPP_GetMyHostName {} {
    error "CMAPP_GetMyHostName -- DEAD, use dp_hostname"
}    
    
# ------{tcl2doc FUNCTION}--------------------------------------
# Makes the process a server.  The server will attempt to launch with
#   the given port number if possible.  gCMAPP_MyPort is set to the
#   resulting port.  If a server has already been opened, then
#   return the existing port (if the numbers match, else error).

proc CMAPP_MakeServer {{portNum 0}} {
    global  gCMAPP_MyPort
    if {$gCMAPP_MyPort == {}} {
	set gCMAPP_MyPort [dp_MakeRPCServer $portNum none cmapp_CheckCmd]
	return $gCMAPP_MyPort
    }
    if {$portNum != 0 && $portNum != $gCMAPP_MyPort} {
	error "Server already exists with different number"
    }
    return $gCMAPP_MyPort
}

#
# This is the default check command on RPC connections
# It only allows CMApp function calls through
#
proc cmapp_CheckCmd {cmd args} {

# Ketan hacked here.
# I've essentially turned security off here since
# non secure demos were breaking and I really wanted
# them to work for now. What the right thing to do is 
# not completely known yet, but for now, this function
# just returns.

    return;

    case $cmd in {
	CMApp_* return
	eval {return -code continue}
	catch {return -code continue}
    }
    return -code break
}

# ------{tcl2doc FUNCTION}--------------------------------------
# Opens a Tcl-DP pier connection to the given host and port.  The
#   advantage of this over the standard dp_MakeRPCClient is that the
#   connection is cached and latter attempts to connect to the same
#   host:port recieve the same connection.  Close the returned
#   connection with CMApp_CloseHost.  This insures that connections
#   are properly referance counted and purged only when no longer
#   used.
# Returns: file id of new client socket (just like dp_MakeRPCClient).

proc CMApp_OpenHost {host port {checkCmd cmapp_CheckCmd}} {
    upvar #0 gCMAppConnections con

    if {[info exists con($host,$port)]} {
	set serv $con($host,$port)
	if {[incr con($host,$port,ccnt)] == 1} {
	    CMAppRemoveHostFromPurge "$host,$port"
	}
	#puts "**<[pid]> reuse connection($host,$port) <$con($host,$port,ccnt)>"
    } else {
	set serv [dp_MakeRPCClient $host $port $checkCmd]
	dp_SetCheckCmd $serv $checkCmd
	dp_atclose $serv append "CMAppConnectionClosed $host,$port"
	set con($host,$port) $serv
	set con($host,$port,ccnt) 1
	set con($serv) "$host,$port"
	#puts "**<[pid]> make  connection($host,$port,[pid])"
    }
    return $serv
}

proc CMAppRemoveHostFromPurge {hostPort} {
    upvar #0 gCMAppConnections con

    # puts "removeHost(before): $hostPort - $con(purgeList)"
    set indx [lsearch $con(purgeList) $hostPort]
    if {$indx == -1} {
	error "hostPort not found: $hostPort"
    }
    set con(purgeList) \
	[lreplace $con(purgeList) $indx $indx]
    # puts "removeHost(after ): $hostPort - $con(purgeList)"
}

# --------------------------------------------------------------
    
proc CMApp_CloseHost {serv} {
    upvar #0 gCMAppConnections con
    set hostPort $con($serv)

    if {[incr con($hostPort,ccnt) -1] <= 0} {
	lappend con(purgeList) $hostPort
	if {[llength $con(purgeList)] > $con(cacheMax)} {
	    CMApp_PurgeConnections 1
	}
    }
    # puts "**<[pid]> close connection($hostPort)"
}

# ------{tcl2doc FUNCTION}--------------------------------------
# Removes some or all of the currently unused cached connections.  The
#   $num argument specifies the number of connections to purge.  By
#   default (ie no argument) all connections are purged.  The
#   keyword all may also be used.

proc CMApp_PurgeConnections {{num all}} {
    upvar #0 gCMAppConnections con

    # puts "**<[pid]> purge: $num"
    if {$num == "all"} {
	set num [llength $con(purgeList)]
    } 
    incr num -1

    foreach I [lrange $con(purgeList) 0 $num] {
	set serv $con($I)
	dp_CloseRPC $serv
	# puts "**<[pid]> PURGE - $I - $serv"
    }
    # puts "**<[pid]> purge: resulting purgeList - $con(purgeList)"
}


proc CMAppConnectionClosed {hostPort} {
    upvar #0 gCMAppConnections con

    catch {CMAppRemoveHostFromPurge $hostPort}
    catch {
	set serv $con($hostPort)
	unset con($hostPort)
	unset con($hostPort,ccnt)
	unset con($serv)
    }
}

# ------{tcl2doc FUNCTION}--------------------------------------
# This function sets the location to use when attempting to resolve
#   local file referances.  The keyword -SAME is used as a host name
#   if local file refferances should be resolved in the same
#   proccess. 

proc CMApp_SetLocalHost {host {port {}}} {
    global  kCMApp_DefaultCMTPPort  gCMAppL_LocalHost
    if {$host == "-SAME"} {
	set host localhost
	set port 0
    }
    if {$port == {}} {
	set port $kCMApp_DefaultCMTPPort
    }
    set gCMAppL_LocalHost "$host:$port"
    # puts "<pid = [pid]> local host = $gCMAppL_LocalHost"
}

# ------{tcl2doc FUNCTION}--------------------------------------
# Examines the file specified in the tupple (returned from
#   CMApp_TranslateURL) and returns a list containing
#   {type duration}.  If the file investigated is a script, then the
#   type returned is 'movie/cmt' and the duration is not valid.  For
#   further details, see CMP_ExamineFile.

proc CMApp_SourceFileInfo {tupple} {
    # puts "CMApp_SourceFileInfo {$tupple}"
    CMApp_ListSet {host port file} $tupple
    return [CMApp_RPC $host $port CMP_ExamineFile $file]
}

# ------{tcl2doc FUNCTION}--------------------------------------
# Takes a host port pair and executes a RPC on the refferanced
#   process.  Assume that a connection is opened and closed around
#   the RPC, but connections are cached for efficiency.

proc CMApp_RPC {host port args} {
    # puts "@@@ CMApp_RPC <$host,$port> $args"
    if {$host == "localhost" && $port == 0} {
	return [eval $args]
    }
    set serv [CMApp_OpenHost $host $port]
    if {[catch {eval "dp_RPC $serv $args"} out]} {
	global errorInfo
	set saveInfo $errorInfo
	catch {CMApp_CloseHost $serv}
	error $out $saveInfo
    }
    CMApp_CloseHost $serv
    return $out
}

# ------{tcl2doc FUNCTION}--------------------------------------
# Takes a host port pair and executes a RDO on the refferanced
#   process.  Assume that a connection is opened and closed around
#   the RPC, but connections are cached for efficiency.

proc CMApp_RDO {host port args} {
    # puts "@@@ CMApp_RPC <$host,$port> $args"
    if {$host == "localhost" && $port == 0} {
	return [eval $args]
    }
    set serv [CMApp_OpenHost $host $port]
    if {[catch {eval "dp_RDO $serv $args"} out]} {
	global  errorInfo
	set saveInfo $errorInfo
	catch {CMApp_CloseHost $serv}
	error $out $saveInfo
    }
    CMApp_CloseHost $serv
    return $out
}

# ------{tcl2doc FUNCTION}--------------------------------------
# Resolves the given URL or file into a {host port file}-tupple which
# points to a CMS process.  This resolution is done by contacting the
# host:port in the URL and issuing a CMTP_Translate command.  This
# returns either another URL or the needed tupple.  If an URL is
# returned, then the process repeats.

proc CMApp_TranslateURL {aURL args} {
    global  gCMAppL_LocalHost  kCMApp_DefaultCMTPPort tcl_msWindows
    # puts "CMApp_TranslateURL IN  - $url"

    if {![CMApp_IsFileOrURL $aURL url]} {
	error "bad URL name: $url"
    }

    set lastURL {}
    for {set i 0} {[llength $url] <= 1} {incr i} {
	if {$i > 20} {
	    error "$i redirects is too many"
	}
	CMApp_ListSet {x host port x} [CMApp_SplitURL $url]
	if {$port == {}} {
	    set port $kCMApp_DefaultCMTPPort
	}
	if {$host == {} || $port == {}} {
	    error "Could not derive host or port for: $url"
	}

	set command "CMApp_RPC $host $port CMTP_Translate $url $args"
	if {[catch $command out]} {
	    if {$lastURL == {}} {
		error "$out\nOn First hop"
	    }
	    CMApp_ListSet {x host port x} [CMApp_SplitURL $lastURL]
	    if {$port == {}} {
		set port $kCMApp_DefaultCMTPPort
	    }
	    if {$host == {} || $port == {}} {
		error "bad host:port spec - <$host>:<$port>"
	    }
	    set command "CMApp_RPC $host $port CMTP_Translate $lastURL $args -retry $url"
	    set out [eval $command]
	}
	set lastURL $url
	set url $out
        # puts "++  $lastURL --> $url"
    }
    # puts "CMApp_TranslateURL OUT -> $url"
    return $url
}

# ------{tcl2doc FUNCTION}--------------------------------------
# Parses an URL into {protocol host port file} with {} filling
#   unspecifed slots.
# Error: iff host spec is not correct.


proc CMApp_SplitURL {url} {
    set mach {}
    set port {}
    set file {}
    set drive {}
    if {![regexp -nocase -- {^([a-z]+:)(.*)$} $url  zip proto rest]} {
	error "Bad URL, no protocol: $url"
    }
    if {[regexp {^(//[^/]+/)(.*)$} $rest  zip mach rest]} {
	# puts "host+ -- $mach $rest"
	if {![regexp {^//([^/:]+)(:([0-9]+))?([a-zA-Z]:)?/$} $mach  \
		  zip mach zip port drive]} {
	    error "Bad host spec in URL: $url"
	}
    }
    if {[string index $rest 0] != "~"} {
	set file $drive/$rest
    } else {
	set file $drive$rest
    }

    return [list $proto $mach $port $file]
}

# ------{tcl2doc FUNCTION}--------------------------------------
# returns 1 iff url appears to be an URL, else 0.

proc CMApp_IsURL {url} {
    global tcl_msWindows
    if {$tcl_msWindows} {
	if {[string index $url 1] == ":"} {
	    return 0
	}
    }
    return [expr ![catch {CMApp_SplitURL $url}]]
}

# ------{tcl2doc FUNCTION}--------------------------------------
# returns 1 iff the given name is either a well formatted URL or names
# an existing file.  If $varName is given, then upon return it
# contains a URL form of the file name.

proc CMApp_IsFileOrURL {name {varName {}}} {
    global  gCMAppL_LocalHost  kCMApp_DefaultCMTPPort tcl_msWindows

    if {$varName != {}} {
	upvar $varName url
    }
    set url $name

    if {[CMApp_IsURL $url]} {return 1}
    if {![file exists $url]} {return 0}

    if {$tcl_msWindows} {
	regsub -all {[\\]} $url / tmp_url;
	set url $tmp_url;
    }
    if {$varName != {}} {
	set url [glob -- $url]
	if {[string index $url 0] != "/"} {
	    # Windows NT filename check
	    set hasurl 0
	    if {$tcl_msWindows && [string index $url 1] == ":"} {
		if {[string index $url 2] != "/"} {
		    set drive [string range $url 0 1]
		    set url [string range $url 2 end]
		    set oldpwd [pwd]
		    cd $drive
		    set pwd $pwd
		    cd $oldpwd
		    set url $pwd/$url
		}
		set hasurl 1
	    }
	    if {! $hasurl} {
		set url "[pwd]/$url"
	    }
	}
	set url "cmtp://$gCMAppL_LocalHost$url"
	# puts "-+ $url --"
    }
    return 1
}
