# ------------------------------------------------------------------
#
#	tags.tcl
#
# {tcl2doc AUTHOR Eric Baldeschwieler, eric14@cs.berkeley.edu}
#
# date:	05-02-94
#
# {tcl2doc DESCRIPTION}
# Implements simple property lists.  A plist is a list of name
# value pairs of the form {?{<tag> <value>}?...}.  
#
# For example:
#	{{title "Batman, the movie"} {length 120min} {format betaCam}}
#
# ----------------------------------------------------------------


# ------{tcl2doc FUNCTION}--------------------------------------
# returns a new pList with any old value of Prop replaced by the new
#   value.

proc CMApp_SetProp {pList prop value} {
    linsert [CMApp_RemoveProp $pList $prop] 0 [list $prop $value]
}

# ------{tcl2doc FUNCTION}--------------------------------------
# Returns:  The value associated with the prop in pList or {} if
#     there is no such value.

proc CMApp_GetProp {pList prop} {
    foreach I $pList {
	if {$prop == [lindex $I 0]} {
	    return [lindex $I 1]
	}
    }
    return {}
}

# ------{tcl2doc FUNCTION}--------------------------------------
# Returns:  pList with all entries with the given prop removed.  Note:
#     prop can be a glob style patern.

proc CMApp_RemoveProp {pList prop} {
    set out {}
    foreach I $pList {
	if {![string match $prop [lindex $I 0]]} {
	    lappend out $I
	}
    }
    return $out
}
