# ------------------------------------------------------------------
#
# Author: Eric14@cs.berkeley.edu
#
# Date: 02-27-95
#
# Purpose: Special functions to initialize proccesses which have
#     access to the ../cmp library (cmx & cmwish).
#
# ------------------------------------------------------------------

# ------------------------------------------------------------------
# Magic constant set to indicate that a process has access rights to
#     the cmp library.  (Mainly for debugging)

set kCMProcess YES

# ------------------------------------------------------------------
# code that is called at CMWISH startup

proc CMP_InitCMWish {} {
    global  kCMProcess 

    set kCMProcess CMWISH
    # puts stderr "CMP_InitCMWish CALLED!!!"

    CMApp_IfError {CMP_ProcArguments} rethrow {
	puts "** err - $_errorTxt\n$errorInfo"
    }
}

# ------------------------------------------------------------------
# code that is called at CMX startup

proc CMP_InitCMX {} {
    global   kCMProcess

    set kCMProcess CMX
    # puts stderr "CMP_InitCMX CALLED!!!"

    CMP_ProcArguments
}

# ------------------------------------------------------------------
# code that is called at CMX startup

proc CMP_InitNameService {srvc} {
    global gCMAPP_MyPort gCMAPP_MyTicket
    
    # Get the service name from argv
    set host [NS_GetHostName]

    # Call ns_Init to set up nsHandle (global connection to the nameserver)
    ns_Init

    if {$srvc != {}} {
	ns_AdvertiseService $srvc $host $gCMAPP_MyPort [pid] $gCMAPP_MyTicket
    }
}

# ------------------------------------------------------------------
# code that is called at CMX startup

proc CMP_InitTicket {} {
    global gCMAPP_MyTicket
    
    if {[lindex [dp_isready stdin] 0]} {
	set tickets [gets stdin]
	set gCMAPP_MyTicket [lindex $tickets 1]
	ns_RecordTicket $tickets
    } else {
	set gCMAPP_MyTicket {}
    }
}    

# ------------------------------------------------------------------
# code that is called at CMX startup

proc CMP_ProcArguments {} {
    global    argv argc env dp_library

    set hostname [dp_hostname]
    CMApp_SetLocalHost -SAME

    set commands {}

    #puts "--argv = $argv"

    for {set i 0} {$i < [llength $argv]} {} {
	switch -exact -- [lindex $argv $i] {
	    -service {
		puts "SERVICE"
		set serviceName [lindex $argv [expr $i+1]]
		set argv [lreplace $argv $i [expr $i+1]]
		if {![catch {expr $serviceName+1}]} {
		    # name is really a number
		    CMAPP_MakeServer $serviceName
		    set serviceName {}
		} elseif {$serviceName != {}} {
		    puts "getting ticket"
		    CMP_InitTicket
		    puts "Daemonizing"
		    if {[catch {ns_daemonize} result]} {
			puts stderr $result
			exit 1
		    }
		    CMAPP_MakeServer

		    if {![info exists env(NS_HOME_DIR)]} {
			uplevel \#0 {source $dp_library/nslib.tcl}
		    } else {
			uplevel \#0 {source $env(NS_HOME_DIR)/nslib.tcl}
		    }
		    CMP_InitNameService $serviceName

		    # global gCMAPP_MyPort
		    # set host [CMAPP_GetMyHostName]
		    # NS_SrvcInit $serviceName $host $gCMAPP_MyPort
		}
	    }
	    -CM_DEBUG {
		puts stderr "<pid = [pid]> CMP_ProcArguments CALLED!!!"
		set argv [lreplace $argv $i $i]
	    }
	    -command {
		lappend commands [lindex $argv [expr $i+1]]
		set argv [lreplace $argv $i [expr $i+1]]
	    }
	    -slave {
		CMApp_BeSlave [lindex $argv [expr $i+1]]
		set argv [lreplace $argv $i [expr $i+1]]
	    }
	    -- {
		set argv [lreplace $argv $i $i]
		set i $argc
	    }
	    default {incr i}
	}
    }
    set argc [llength $argv]

    foreach C $commands {
	uplevel #0 $C
    }
}


