#
# This file contains tcl code for the format/length resolving function.
#

#---------------------------------------------------------------
# magic debugging test to see if non-privaleged application is
#     sourcing this file.

if {![info exists kCMProcess]} {
    error "non-CMProcess attempted to use cmp library function!"
}

#---------------------------------------------------------------
# The procedure CMP_ExamineFile takes a filename as a parameter
# and returns a list of the following value:
#
# {media/format length}
#
# where media/format is one of:
#       video/mpeg
#       video/mjpeg
#       audio/au
#       other
#
# Length is a floating point number indicating the length of the clip
# in seconds.
#
# Several special file returns exists:
#	movie/cmt  - The file is a cmt movie (script)
#	DIRECTORY  - The file is a directory
#	UNKNOWN    - Could not resolve the type.
#
# All of these types return with a length of -1.

proc CMP_ExamineFile {filename {typeHint {}}} {

    if {[file isdirectory $filename]} {
	return "DIRECTORY -1"
    }
    if {[CMApp_IsMovie $filename]} {
	return "movie/cmt -1"
    }
    if {[CMApp_IsCollection $filename]} {
	return "collection/cmt -1"
    }
    set mpeg_cmd mpegSegment
    set mjpeg_cmd mjpegSegment
    set vq_cmd vqFile
    set au_cmd auSegment

    set cf "";

    set cf [$mpeg_cmd ""];
    if {![catch {$cf conf -filename $filename}]} {
        $cf configure -clipEnd end;
        set length [lindex [$cf configure -clipEnd] 2];
        set mf_type "video/mpeg"

	$cf destroy;
	return "$mf_type $length";
    } else {
	if {$cf != ""} {
	    $cf destroy;
	    set cf "";
	}
    }

    set cf [$mjpeg_cmd ""];
    if {![catch {$cf conf -filename $filename}]} {
        $cf configure -clipEnd end;
        set length [lindex [$cf configure -clipEnd] 2];
	set mf_type "video/mjpeg"

	$cf destroy;
	return "$mf_type $length";
    } else {
	if {$cf != ""} {
	    $cf destroy;
	    set cf "";
	}
    }

    set cf [$vq_cmd ""];
    if {![catch {$cf conf -filename $filename}]} {
	set info [$cf info];
	set mf_type "video/vq"
	set length [lindex $info 1];

	$cf destroy;
	return "$mf_type $length";
    } else {
	if {$cf != ""} {
	    $cf destroy;
	    set cf "";
	}
    }

    set cf [$au_cmd ""];
    if {![catch {$cf conf -filename $filename}]} {
        $cf configure -clipEnd end;
        set length [lindex [$cf configure -clipEnd] 2];
	set mf_type "audio/au"

	$cf destroy;
	return "$mf_type $length";
    } else {
	if {$cf != ""} {
	    $cf destroy;
	    set cf "";
	}
    }

    return "UNKNOW -1"
}
