#
# MJPEG video widget
#
# Todo: Make a video widget that can interleave MPEG and JPEG
#
# Creation:
#     mjpegVideo <name> ?<width> <height>? -- creates a video widget
#     $name destroy -- destroys the widget
#     $name accept  -- returns the accept command for the widget
#     $name ready   -- makes the widget "ready"
#     $name configure
#	read-only slots: -obj -accept
#	read-write slots: -width -height -background -device -lts
#

set mjpegVWInfo(names) ""
proc mjpegVideo {wid {w 320} {h 240}} {
    global mjpegVWInfo

    #
    # First make the parent frame
    #
    set realWidth [expr $w+20]
    set realHeight [expr $h+20]
    frame $wid -relief ridge -borderwidth 10
    pack $wid -fill both -expand yes -padx 10 -pady 10
    frame $wid.disp -background black  -width $realWidth -height $realHeight
    pack $wid.disp -fill both -expand yes
    update

    set obj [mjpegPlay ""]
    $obj config -xid [winfo id $wid.disp]
    $obj config -width $realWidth -height $realHeight -xpos -1 -ypos -1
    bind $wid.disp <Expose> "$obj refresh"
    bind $wid.disp <Configure> \
	"$obj configure -width %w -height %h; catch \"$obj ready\""

    set mjpegVWInfo($wid,obj) $obj
    set mjpegVWInfo($wid,accept) @$obj.accept
    set mjpegVWInfo($wid,width) $w
    set mjpegVWInfo($wid,height) $h
    set mjpegVWInfo($wid,background) black
    set mjpegVWInfo($wid,lts) ""
    set mjpegVWInfo($wid,device) default
    set mjpegVWInfo($wid,ready) 0
    lappend mjpegVWInfo(names) $wid

    rename $wid $wid-orig

    proc $wid {args} "eval mjpegVideoWidgetCommand $wid \$args"

    return $wid
}

proc mjpegVideoWidgetCommand {wid command args} {
    global mjpegVWInfo
    switch -glob $command {
	acc* {return $mjpegVWInfo($wid,accept)}
	des* {DestroyMJPEGWidget $wid}
	con* {eval ConfigureMJPEGWidget $wid $args}
	rea* {ReadyMJPEGWidget $wid}
    }
}

proc DestroyMJPEGWidget {wid} {
    global mjpegVWInfo

    set obj $mjpegVWInfo($wid,obj)
    $obj destroy

    unset mjpegVWInfo($wid,obj)
    unset mjpegVWInfo($wid,accept)
    unset mjpegVWInfo($wid,width)
    unset mjpegVWInfo($wid,height)
    unset mjpegVWInfo($wid,background)
    unset mjpegVWInfo($wid,lts)
    unset mjpegVWInfo($wid,device)
    unset mjpegVWInfo($wid,ready)

    $wid-orig destroy
}

proc ReadyMJPEGWidget {wid} {
    global mjpegVWInfo
    set obj $mjpegVWInfo($wid,obj)
    $obj ready
    set mjpegVWInfo($wid,ready) 1
}

proc ConfigureMJPEGWidget {wid args} {
    global mjpegVWInfo
    set len [llength $args]

    #
    # Case 1: .vid config
    #
    if {$len == 0} {
	set rv {}
	lappend rv [list -width $mjpegVWInfo($wid,width)]
	lappend rv [list -height $mjpegVWInfo($wid,height)]
	lappend rv [list -background $mjpegVWInfo($wid,background)]
	lappend rv [list -lts $mjpegVWInfo($wid,lts)]
	lappend rv [list -device $mjpegVWInfo($wid,device)]
    	lappend rv [list -obj $mjpegVWInfo($wid,obj)]
    	lappend rv [list -accept $mjpegVWInfo($wid,accept)]
	return $rv
    }

    #
    # Case 2: .vid config -slot
    #
    if {$len == 1} {
	set rv {}
	set slot $args
	switch -- $slot {
	  -width {return [list -width $mjpegVWInfo($wid,width)]}
	  -height {return [list -height $mjpegVWInfo($wid,height)]}
	  -background {return [list -background $mjpegVWInfo($wid,background)]}
	  -lts {return [list -lts $mjpegVWInfo($wid,lts)]}
	  -device {return [list -device $mjpegVWInfo($wid,device)]}
	  -obj {return [list -obj $mjpegVWInfo($wid,obj)]}
	  -accept {return [list -accept $mjpegVWInfo($wid,accept)]}
	  default {error "$wid configure: unknown option $slot"}
	}
    }

    #
    # Case 3: .vid config -slot value <-slot value>...
    #
    for {set i 0} {$i < $len} {incr i 2} {
	set slot [lindex $args $i]
	set value [lindex $args [expr 1+$i]]
	switch -- $slot {
	  -width {
	      $wid.disp configure -width [expr $value+20]
	      set mjpegVWInfo($wid,width) $value
	      }
	  -height {
	      $wid.disp configure -height [expr $value+20]
	      set mjpegVWInfo($wid,height) $value
	      }
	  -background {
	      $wid.disp configure -background $value
	      set mjpegVWInfo($wid,background) $value
	      }
	  -lts {
	      set obj $mjpegVWInfo($wid,obj)
	      $obj configure -lts $value
	      set mjpegVWInfo($wid,lts) $value
	      if {$mjpegVWInfo($wid,ready)} {
		  $obj ready
	      }
	  }
	  -device {
	      set obj $mjpegVWInfo($wid,obj)
	      $obj configure -device $value
	      set mjpegVWInfo($wid,device) $value
	      if {$mjpegVWInfo($wid,ready)} {
		  $obj ready
	      }
	  }
	  -obj {
		error "$wid configure: option $slot is read-only"
	  }
	  -accept {
		error "$wid configure: option $slot is read-only"
	  }
	  default {
		error "$wid configure: unknown option $slot"
	  }
	}
    }
}
