/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


/*
#define DEBUG
#define DEBUG_FRAMES
*/
#define DO_REFRESH

/* Device functions high level descriptions
 *
 *	decodeProc	Frame data: do nothing.
 *	refreshProc	Resend most recent frame.
 *	displayProc	Send frame.  Send codebook first if necessary.
 *	initProc	
 *	reinitProc
 *	shutdownProc
 *	freeFrameProc
 *	errorProc
 */

#define _VQIPDEVICE_MASTER_
#include "vqIpDevice.h"
#include "ipnproto.h"
#include "ansi.h"

#define FRAME_HEIGHT	240
#define FRAME_WIDTH	128
#if 0
#define MAX_PAD_PACKET_SIZE	68
#endif
#define MAX_VIDEO_PACKET_LONGS	200

#ifndef min
#define min(A,B)	(((A)<(B))?(A):(B))
#endif

/**********************************************************************/
/* Define the format we expect to get as input */

typedef char block[4][4];	/* order is [i][j].  could be only 6-bits used */

typedef block codebook[256];

typedef struct {
  codebook	y, i, q;
} wholeCodebook;

/* wholeCodebook size = 4kB */
   
typedef struct {
  char	y[2][FRAME_WIDTH/4];
  char	i[FRAME_WIDTH/8];
  char	q[FRAME_WIDTH/8];
} swath;

typedef swath frame[FRAME_HEIGHT/8];

/* frame size = 2880 B = 2.8125 kB */

/* @30 fps, bandwidth = 84.375 kB/s = 675 kb/s */

/**********************************************************************/
/* Output format */

struct {
  int	unused	:21;
  int	flipNow	:1;
  int	type	:1;	/* 0=Y, 1=IQ */
  int	address	:9;
} frameHeader;

/* a frame consists of a header followed by I and Q interleaved data, I first */

struct {
  int	unused	:20;
  int	type	:2;	/* 00=Y, 01=I, 10=Q */
  int	entry	:8;
  int	row	:2;
} codebookHeader;

/* a codebook packet consists of a header followed by some number of
   entries.  Each entry is 4 6-bit values. */

/**********************************************************************/

typedef struct {
  int		padID;
} IPVideoDevice;
   
typedef struct {
  PORTFILE	*gwConnection;
  int	lastCodebookIndex;
#ifdef DO_REFRESH
  char	*lastFrame;
  int	lastFrameLength;
  int	lastFrameBufSize;
#endif
} IPVideoDeviceData;


/* Translation tables.  Need to get init'ed once for all devices */

int		translationTablesInited = 0;
u_char		iq_xlat[256];	/* Does offset binary->sign mag w/ flip & shift */
u_char		y_xlat[256];	/* Just a simple shifting */

/**********************************************************************/
/* Helper functions: */

/* Stolen from Jeff's send_vq program.  type should be 1 for frames, 0 for codebook */

void send_video_data_pad(PORTFILE *psPortPtr, int type, int num_longs,
			 int start_addr, u_char *data, int flip_on_last_packet)
{
  u_long			headl;
  int				words_left;
  int				words_this_xfer;
  int				cur_addr;
  u_char			*data_ptr;
  static u_char	msg_buf[MAX_VIDEO_PACKET_LONGS*4 + 4];
  int				bytesWritten;

  cur_addr = start_addr;
  data_ptr = data;

#ifdef DEBUG
      fprintf(stderr, "svdp: addr %x\tlongs %x\ttype %c\tflip %d\n",
	      start_addr, num_longs, (type) ? ('f'):('c'),
	      flip_on_last_packet);
#endif

  for (words_left=num_longs; words_left>0; words_left-=words_this_xfer)
    {
      words_this_xfer = min(words_left, MAX_VIDEO_PACKET_LONGS);

      headl = cur_addr;

      if (flip_on_last_packet && (words_left == words_this_xfer))
	{
	  headl |= 0x0400;
	}

      msg_buf[0] = headl & 0xff;
      msg_buf[1] = (headl >> 8) & 0xff;
      msg_buf[2] = (headl >> 16) & 0xff;
      msg_buf[3] = (headl >> 24) & 0xff;

#ifdef OLDTYPES
      msg_buf[3] |= type&1;
#endif
      
      memcpy(msg_buf+4, data_ptr, words_this_xfer*4);
      bytesWritten = IPNWriteTyped(msg_buf, words_this_xfer*4+4, 1, psPortPtr,
				   (type) ? (T_VIDEOFRAME) : (T_VIDEOCODE));
      if (bytesWritten != words_this_xfer*4+4) {
	fprintf(stderr, "Couldn't send %d bytes to pad server"
		" (only sent %d bytes)\n",
		words_this_xfer*4+4, bytesWritten);
      }

      data_ptr += words_this_xfer * 4;
      cur_addr += words_this_xfer;
    }
}

void send_codebook_entries(PORTFILE *gwConnection,
			   u_int first_entry, 
			   u_int num_entries, 
			   u_char *codebook_data)
{
  static u_long	codebook_hw[3][4][256];	/* The codebooks in the HW */
  static int		codebook_hw_initted=0;
  int				i,j;
  u_char			A, B, C, D;
  u_char			*xlat; /* Points into Y or IQ xlat */
  u_char			*data_ptr;
  int				all_val;
  int				which_cb;

  /* in send_vq, these can be overridden on the command line */
  int			all_y_code=-1;
  int			all_i_code=-1;
  int			all_q_code=-1;

  if (!codebook_hw_initted)
    {
      memset((u_char *)codebook_hw, 0, 12288);
      codebook_hw_initted = 1;
    }

  /* We have to reformat codebook data to pack it into */
  /* 6 bit quantities, switch it from entry-major order to */
  /* entry-row major order, and finally switch I and Q into sign */
  /* magnitude format */
  /* Hardware wants reverse of:
     00000000 DDDDDDCC CCCCBBBB BBAAAAAA
     which is:
     BBAAAAAA CCCCBBBB DDDDDDCC 00000000
     
     In code, t0=AAAAAA, t1=BBBBBB, t2=CCCCCC, t3=DDDDDD
     */

  if ((first_entry + num_entries > 768) || (num_entries == 0))
    {
      fprintf(stderr, "Invalid codebook region (%d entries starting @ %d) to update\n",
	      num_entries, first_entry);
      return;
    }

  /* Make sure that one call only covers a single codebook */
  if ((first_entry < 256) && (first_entry + num_entries > 256))
    {
      send_codebook_entries(gwConnection, first_entry, 256-first_entry, codebook_data);
      if (codebook_data)
	{
	  codebook_data += (256-first_entry) * 16;
	}
      num_entries -= 256 - first_entry;
      first_entry = 256;
    }

  if ((first_entry < 512) && (first_entry + num_entries > 512))
    {
      send_codebook_entries(gwConnection, first_entry, 512-first_entry, codebook_data);
      if (codebook_data)
	{
	  codebook_data += (512-first_entry) * 16;
	}
      num_entries -= 512 - first_entry;
      first_entry = 512;
    }

  if (first_entry < 256)
    {
      xlat = y_xlat;
    }
  else
    {
      xlat = iq_xlat;
    }

  which_cb = first_entry >> 8;

  /* The if all_X_code is not -1 then set the whole codebook to */
  /* that value */
  switch(which_cb)
    {
    case 0:
      all_val = all_y_code;
      break;

    case 1:
      all_val = all_i_code;
      break;

    case 2:
      all_val = all_q_code;
      break;

    default:
      all_val = -1;
    }

  /* Reformat data into codebook_hw array if new data specified */
  if (codebook_data)
    {
      data_ptr = codebook_data;
      for (i=first_entry & 0xff; i<num_entries+(first_entry & 0xff); i++)
	{
	  for (j=0; j<4; j++)
	    {
	      if (all_val == -1)
		{
		  A = xlat[*data_ptr++];
		  B = xlat[*data_ptr++];
		  C = xlat[*data_ptr++];
		  D = xlat[*data_ptr++];
		}
	      else
		{
		  A = B = C = D = all_val;
		}
	      codebook_hw[which_cb][j][i] = (A << 24) |
		((B & 0x3c) << 14) |
		  ((B & 0x03) << 30) |
		    ((C & 0x30) <<  4) |
		      ((C & 0x0f) << 20) |
			(D << 10);
	    }
	}
    }

  /* Send in 4 transfers - one for each line of codebook entries */
  for (i=0; i<4; i++)
    {
      send_video_data_pad(gwConnection, 0, num_entries, 
			  which_cb*1024 + (first_entry & 0xff) + i*256, 
			  (u_char *)&codebook_hw[which_cb][i][first_entry & 0xff], 0);
    }
}

static void
SendCodebook(VQResource *resourcePtr, int index)
{
  IPVideoDeviceData *devDataPtr;
  
  devDataPtr = (IPVideoDeviceData *) resourcePtr->devicePtr->deviceData;
  if ((index != devDataPtr->lastCodebookIndex) &&
      (index < resourcePtr->codebookArraySize)) {
    char	*dataPtr;
    int		length;

    dataPtr = resourcePtr->codebookArray[index].data;
    length = resourcePtr->codebookArray[index].length;

#ifdef DEBUG
    fprintf(stderr, "SendCodebook: sending cb %d (%10d bytes)\n",
	    index, length);
#endif
    send_codebook_entries(devDataPtr->gwConnection, 0, length/16, dataPtr);
    
    devDataPtr->lastCodebookIndex = index;
  }
}
			 
static void
SendFrame(IPVideoDeviceData *devDataPtr, char *dataPtr, int length)
{
  const int ySize = 1920;	/* in bytes */
  const int iqSize = 960;	/* in bytes */
  
  if ((length % sizeof(long)) != 0) {
    fprintf(stderr, "vqIpDevice(sendFrame): length (%d) not multiple of sizeof(long) (%ld)\n",
	    length, sizeof(long));
  }

  /* need to send Y part and IQ part separately.  For now, at least, assume
   * all frames are the size we want */
  send_video_data_pad(devDataPtr->gwConnection, 1, ySize/4,
		      0, dataPtr, 0);
  send_video_data_pad(devDataPtr->gwConnection, 1, iqSize/4,
		      0x200, dataPtr+ySize, 1);

#ifdef DO_REFRESH
  if (devDataPtr->lastFrameBufSize < length) {
    if (devDataPtr->lastFrame != NULL) {
      free(devDataPtr->lastFrame);
    }
    devDataPtr->lastFrame = (char *) ckalloc(length);
    if (devDataPtr->lastFrame == NULL) {
      fprintf(stderr, "ckalloc failed at %s:%d\n", __FILE__, __LINE__);
      return;
    }
    devDataPtr->lastFrameBufSize = length;
  }

  memcpy(devDataPtr->lastFrame, dataPtr, length);
  devDataPtr->lastFrameLength = length;
#endif
}
			 
/************************************************************
 *
 * vqIpDecode
 *
 * Parameters:
 *	framePtr	Pointer to the frame to decode
 *
 * Return:
 *	0 if ok, non-0 on error
 *
 * Action:
 *	Translate from file format to Pad format.
 *
 ************************************************************/

int
vqIpDecode(VQFrame *framePtr)
{
  int		result = 0;
#ifdef UNNECESSARY
  VQDevice	*vqDevPtr;
  IPDevice	*ipDevPtr;
  
  if (dataLength != sizeof(frame)) {
    result = IPVIDEO_BAD_FRAME_SIZE;
  }
  else {
    vqDevPtr = framePtr->objPtr->devicePtr;
    ipDev = vqDevPtr->deviceData;
    
  }
#endif
  
  return result;
}

int
vqIpRefresh(VQResource *resourcePtr)
{
#ifdef DO_REFRESH
  IPVideoDeviceData	*devDataPtr;

  devDataPtr = (IPVideoDeviceData *) resourcePtr->devicePtr->deviceData;
  if (devDataPtr->lastFrame != NULL) {
    SendFrame(devDataPtr, devDataPtr->lastFrame, devDataPtr->lastFrameLength);
  }
#endif

  return IPVIDEO_OK;
}

int
vqIpDisplay(VQFrame *framePtr)
{
  IPVideoDeviceData *devDataPtr;
  
  devDataPtr = (IPVideoDeviceData *) framePtr->deviceData;
  if (framePtr->frameHeader.codebookIndex != devDataPtr->lastCodebookIndex) {
    SendCodebook(framePtr->objPtr, framePtr->frameHeader.codebookIndex);
  }
#ifdef DEBUG_FRAMES
  fprintf(stderr, "vqIpDevice:vqIpDisplay: sending %10d bytes @ %f\n",
	  framePtr->dataLength, framePtr->logStart);
#endif
  SendFrame(devDataPtr, framePtr->data, framePtr->dataLength);
  return IPVIDEO_OK;
}

int
vqIpInit(VQResource *resourcePtr)
{
  IPVideoDeviceData *devDataPtr;
  
  devDataPtr = (IPVideoDeviceData *) resourcePtr->devicePtr->deviceData;
  devDataPtr->lastCodebookIndex = -1;
  /* shouldn't need this, but I want to be sure this gets set somewhere */
  devDataPtr->gwConnection = NULL;
#ifdef DO_REFRESH
  devDataPtr->lastFrame		= NULL;
  devDataPtr->lastFrameLength	= 0;
  devDataPtr->lastFrameBufSize	= 0;
#endif

  if ((devDataPtr->gwConnection = 
#ifdef USE_PROXY
       VideoConnectToProxy(resourcePtr->xid)
#else
       VideoConnectToPS(resourcePtr->xid)
#endif       
       ) == NULL) {
    return IPVIDEO_CONNECTION_FAILED;
  }

  if (!translationTablesInited) {
    int j;
    
    /* Initialize I / Q get sign mag bit flipping array - */
    /* also shifts down to 6 bits.  Y xlat just shifts */
    for (j=0; j<128; j++)
      {
	y_xlat[j] = j >> 2;
	y_xlat[j + 0x80] = (j >> 2) | 0x20;
	iq_xlat[j] 	= ((j >> 2)^0x1f) | 0x20;
	iq_xlat[j + 0x80] =  (j >> 2);
      }
  }
  
  return IPVIDEO_OK;
}

int
vqIpReinit(VQResource *resourcePtr)
{
  IPVideoDeviceData *devDataPtr;
  
  devDataPtr = (IPVideoDeviceData *) resourcePtr->devicePtr->deviceData;
  devDataPtr->lastCodebookIndex = -1;
#ifdef DO_REFRESH
  devDataPtr->lastFrameLength = 0;
#endif

  if (devDataPtr->gwConnection == NULL) {
    if ((devDataPtr->gwConnection =
#ifdef USE_PROXY
       VideoConnectToProxy(resourcePtr->xid)
#else
       VideoConnectToPS(resourcePtr->xid)
#endif       
	 ) == NULL) {
      return IPVIDEO_CONNECTION_FAILED;
    }
  }

  return IPVIDEO_OK;
}

int
vqIpShutdown(VQResource *resourcePtr)
{
  IPVideoDeviceData *devDataPtr;
  
  devDataPtr = (IPVideoDeviceData *) resourcePtr->devicePtr->deviceData;
#ifdef DO_REFRESH
  ckfree(devDataPtr->lastFrame);
#endif

  if (devDataPtr->gwConnection != NULL) {
    IPNClose(devDataPtr->gwConnection);
  }

  return IPVIDEO_OK;
}

int
vqIpFreeFrame(VQFrame *framePtr)
{
  return IPVIDEO_OK;
}

char
*vqIpError(VQResource *resourcePtr, int errNum)
{
  return (char *) IPVideoErrorString[errNum];
}

VQDevice
*GetDefaultVQDevice(VQResource *objPtr)
{
  VQDevice		*result;
  IPVideoDeviceData	*dataPtr;

  result = (VQDevice *) malloc(sizeof(VQDevice));
  if (result == NULL)
    return NULL;
  result->deviceData = (IPVideoDeviceData *) malloc(sizeof(IPVideoDeviceData));
  if (result->deviceData == NULL)
    return NULL;

  result->asyncDecodeFlag	= 0;
  result->decodeProc		= vqIpDecode;
  result->refreshProc		= vqIpRefresh;
  result->displayProc		= vqIpDisplay;
  result->initProc		= vqIpInit;
  result->reinitProc		= vqIpReinit;
  result->shutdownProc		= vqIpShutdown;
  result->freeFrameProc		= vqIpFreeFrame;
  result->errorProc		= vqIpError;

  dataPtr = (IPVideoDeviceData *) result->deviceData;
  dataPtr->gwConnection = NULL;
  dataPtr->lastCodebookIndex = -1;
#ifdef DO_REFRESH
  dataPtr->lastFrame		= NULL;
  dataPtr->lastFrameLength	= 0;
  dataPtr->lastFrameBufSize	= 0;
#endif
    
  return result;
}
