/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


#ifndef _VQPlay_
#define _VQPlay_

#include <stdio.h>
#include <tcl.h>
#include <tk.h>
#include "cmCookies.h"
#include "cmBufferlist.h"

struct VQDevice;

typedef struct {
  int	length;
  char	*data;
} VQCodebook;

typedef struct VQResource {
  Tcl_Interp	*interp;	/* Tcl interpreter that created object. */
  char		*name;		/* Tcl string name of this object. */
  int		width;		/* Width of window. */
  int		height;		/* Height of window. */
  int		xpos;		/* X offset of image in window. If  -1,
				 * device should center. */
  int		ypos;		/* Y offset of image in window. If -1, device
				 * should center. */
  int		squeeze;	/* Squeeze flag.  */
  int		xid;		/* Xid of window (or padID of pad) */
  ClientData	lts;		/* LTS for object */
  int		ltsTriggerNum;
  char		*ltsName;
  int		maxFrameRate;	/* max frame rate, if 0, means don't */
  char		*ctrlCallback;	/* Frame rate control callback. */
  double	ctrlCycleTime;	/* Control cycle time. */
  struct VQDevice	*devicePtr;	/* Device control structure. */
  char		*devtype;	/* Name of device */
  double	statStartTime;	/* Used for statistics and feedback control. */
  int		missed;		/* Number of frames missed on arrival. */
  int		late;		/* Number of frames late after decode. */
  int		played;		/* Number of frames queued for play. */
  int		bytes;		/* Number of bytes sent to decode. */
  int		ready;		/* Ready flag. */
  int		inited;		/* Flag that indicates device init proc. has
				 * been called. */
  int		inDecode;	/* Flag that indicates device currenly
				 * decoding. */
  SB_List	*queue;		/* Queue of frames to be decoded and played */
  VQCodebook	*codebookArray;		/* array of codebooks */
  int		codebookArraySize;	/* # of entries in codebookArray */
  char		*cntrlEveryHandle;	/* Every handle for feedback control. */
  int		destroyFlag;
  int		*at_tokens;	/* At tokens. */
  int		num_at_tokens;	/* Size of at token list. */
  char		*acceptHandle;
  char		*acceptCodebookHandle;
} VQResource;

typedef struct VQFrame {
  VQBufferHeader	frameHeader;
  double		logStart;
  double		logEnd;
  ScatterBuffer		*sb;
  Buffer		*dataBuffer;
  char			*data;
  int			dataLength;
  ClientData		deviceData;	/* (IPVideoDeviceData *) */
  int			at_token;
  VQResource		*objPtr;
} VQFrame;

typedef struct VQDevice {
  ClientData	deviceData;
  int		asyncDecodeFlag;	/* set by device if frame is being decoded */
  int		(*decodeProc)(VQFrame *framePtr);
  int		(*refreshProc)(VQResource *objPtr);
  int		(*displayProc)(VQFrame *framePtr);
  int		(*initProc)(VQResource *objPtr);
  int		(*reinitProc)(VQResource *objPtr);
  int		(*shutdownProc)(VQResource *objPtr);
  int		(*freeFrameProc)(VQFrame *framePtr);
  char		*(*errorProc)(VQResource *objPtr, int errNum);
} VQDevice;

extern int	VQ_RSRC_DecodeDone(VQFrame *framePtr);
extern VQDevice	*GetDefaultVQDevice(VQResource *objPtr);

/* these will go away */
extern VQDevice *GetParallaxDevice(VQResource *objPtr);
extern VQDevice *GetJ300Device(VQResource *objPtr);
extern VQDevice *GetSoftwareJpegDevice(VQResource *objPtr);
extern VQDevice *GetSunVideoJpegDevice(VQResource *objPtr);

/* helpful access functions */
extern int	VQ_RSRC_CodebookExists(VQResource *resourcePtr,
				       int codebookIndex);

#endif	/* _VQPlay_ */
