/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


#ifndef _VQRAWFORMAT_H_
#define _VQRAWFORMAT_H_

#include <stdio.h>

#define VQ_RAW_MAGIC_COOKIE	0x49505651

typedef unsigned int	uint32;
typedef unsigned short	uint16;

#ifdef Jeff
#define VQ_MAGIC_NUM			0x49505651

/* All numbers should be in net order */

typedef struct _VQ_file_header {
	u_long		magic_number;
	u_short		major_version;
	u_short		minor_version;
	u_long		width;
	u_long		height;
	u_long		frame_size;
	u_long		codebook_entries;
	u_long		codebook_entry_size;
	u_long		extra_data_len;
} VQ_file_header;
#endif

typedef struct {
  uint32	magicCookie;	/* should be VQ_RAW_MAGIC_COOKIE */
  uint16	majorVersion;
  uint16	minorVersion;
  uint32	width;		/* in VQ IQ entries - i.e. 128/8=16 for us */
  uint32	height;		/* (240/8=30 for us) */
  uint32	frameSize;	/* = width*height*6 */
  uint32	codebookSize;	/* # of entries in codebook ?? (768) */
  uint32	codebookEntrySize;	/* (16) */
  uint32	framesPerSecond;
  uint32	extraHeaderSize;	/* in bytes */
} VQRF_fileHeader;

typedef struct {
  VQRF_fileHeader	header;
  FILE			*file;
} VQRF_file;

typedef struct {
  int		numEntries;
  int		firstEntryIndex;
  uint32	*data;			/* 4 elements/entry, arranged by row */
} VQRF_codebookUpdate;

typedef struct {
  int			numCodebookUpdates;
  VQRF_codebookUpdate	*codebookUpdates;
  int			frameSize;		/* in bytes */
  char			*frameData;
} VQRF_frameHunk;

/* a file contains a VQRF_fileHeader, followed by some number of
 * VQRF_frameHunks. */

#define VQRF_GetFilePtr(F)	((F)->file)
#define VQRF_GetHeader(F)	((F)->header)

VQRF_file		*VQRF_Open(char *name, char *options, char **buffer);
/*
 * buffer == NULL	Don't read the extra data
 * *buffer == NULL	Alloc our own space for it
 * *buffer != NULL	Use provided space
 */

VQRF_file		*VQRF_Reopen(FILE *f, char **buffer);
/* same as VQRF_Open, except use already open file f */

void			VQRF_Close(VQRF_file *f);

uint32			VQRF_ReadUint32(VQRF_file *f);
/* read a uint32 from the file, doing byte-order conversion */

VQRF_frameHunk		VQRF_ReadFrameHunk(VQRF_file *f, char **frameData);
/*
 * Read all codebooks plus frame data.
 *
 * frameData == NULL	Do nothing
 * *frameData == NULL	Alloc space
 * *frameData != NULL	Don't alloc space
 */

void			VQRF_FreeFrameHunk(VQRF_frameHunk);

/*
 *	Low level routines
 */
   
VQRF_codebookUpdate	VQRF_ReadCodebookUpdate(VQRF_file *f, char **codebookData);
/*
 * Read a codebook update.  When it's called, the file pointer should be at
 * the size of the codebook update (which may be 0).  When it returns, the
 * fp is at just after the codebook update.
 *
 * codebookData == NULL		Alloc space
 * *codebookData == NULL	Alloc space, make *codebookData point to it
 * *codebookData != NULL	Don't alloc space (if possible)
 */

int			VQRF_ReadFrameData(VQRF_file *f, char **frameData);
/*
 * frameData == NULL	Do nothing
 * *frameData == NULL	Alloc space
 * *frameData != NULL	Don't alloc space
 */

#endif
