#ifdef _WINDOWS
#  define WIN32_LEAN_AND_MEAN
#  include <windows.h>
#  include <winsock.h>
#  undef WIN32_LEAN_AND_MEAN
#endif

#include <tcl.h>

/*
 *------------------------------------------------------------------------------
 * Tcl_ExtensionCmd --
 *
 *	Load a Tcl extension
 *
 * Results:
 *	A standard TCL result
 *
 * Side Effects
 *	A dll gets loaded into the sytem and an attempt is made to initialize it.
 *	If the initialization attempt fails, the DLL is unloaded and an error is
 *	returned.
 *------------------------------------------------------------------------------
 */
int
Tcl_ExtensionCmd(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
{
    HINSTANCE hLib;
    Tcl_AppInitProc *appInitProc;

    if (argc != 3) {
	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
			 "\" dllname initRoutine", (char *) NULL);
	return TCL_ERROR;
    }

    hLib = LoadLibrary(argv[1]);
    if (hLib == NULL) {
	Tcl_AppendResult(interp, "unable to load dll \"", argv[1], "\"",
			 (char *) NULL);
	return TCL_ERROR;
    }

    appInitProc = (Tcl_AppInitProc *) GetProcAddress(hLib, argv[2]);
    if (appInitProc == NULL) {
	FreeLibrary(hLib);
	Tcl_AppendResult(interp, "unable to find initialization routine \"",
			 argv[2], "\" in \"", argv[1], "\"", (char *) NULL);
	return TCL_ERROR;
    }
    if (appInitProc(interp) == TCL_ERROR) {
	FreeLibrary(hLib);
	return TCL_ERROR;
    }

    return TCL_OK;
}

