/* 
 * memory.c (taken from tclCkalloc.c) --
 *
 *    Interface to malloc and free
 *
 * Copyright (c) 1991-1994 The Regents of the University of California.
 * Copyright (c) 1994-1995 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * This code contributed by Karl Lehenbauer and Mark Diekhans
 *
 */

#include "tclInt.h"
#ifdef SMARTHEAP
#  include <smrtheap.h>
#endif

/*
 *----------------------------------------------------------------------
 *
 * Tknt_Ckalloc --
 *     Interface to malloc when TCL_MEM_DEBUG is disabled.  It does check
 *     that memory was actually allocated.
 *
 *----------------------------------------------------------------------
 */
void *
Tknt_Ckalloc(unsigned int size)
{
    char *result;

    result = malloc(size);
    if (result == NULL) 
	panic("unable to alloc %d bytes", size);
    return result;
}

/*
 *----------------------------------------------------------------------
 *
 * Tknt_Ckfree --
 *     Interface to free when TCL_MEM_DEBUG is disabled.  Done here rather
 *     in the macro to keep some modules from being compiled with 
 *     TCL_MEM_DEBUG enabled and some with it disabled.
 *
 *----------------------------------------------------------------------
 */
void
Tknt_Ckfree(void *ptr)
{
    free (ptr);
}

/*
 *----------------------------------------------------------------------
 *
 * Tknt_Ckrealloc --
 *     Interface to realloc when TCL_MEM_DEBUG is disabled.  It does check
 *     that memory was actually allocated.
 *
 *----------------------------------------------------------------------
 */
void *
Tknt_Ckrealloc (void *ptr, unsigned int size)
{
    char *result;
    
    result = realloc(ptr, size);
    if (result == NULL) 
	panic("unable to realloc %d bytes", size);
    return result;
}

/*
 *----------------------------------------------------------------------
 *
 * Tknt_Ckstrdup --
 *     Interface to strdup when TCL_MEM_DEBUG is disabled.  It does check
 *     that memory was actually allocated.
 *
 *----------------------------------------------------------------------
 */
char *
Tknt_Ckstrdup(char *str)
{
    int len;
    char *newstr;

    len = strlen(str) + 1;
    newstr = Tknt_Ckalloc(len);
    memcpy((VOID *) newstr, (VOID *) str, (size_t) len);
    return newstr;
}
