#include <windows.h>
#include <stdio.h>
#include <wchar.h>

#ifndef APPLICATION_NAME
#  define APPLICATION_NAME "Plateau Multimedia Research Group"
#endif

#define REGISTRY_BASE   "SOFTWARE\\" APPLICATION_NAME

typedef struct {
    char *key;
    char *valueName;
    char *value;
} InstallEntry;

InstallEntry Entries[] = {
    {REGISTRY_BASE "",		"CMT_ROOT",	"lib"},
};


void
main(int argc, char **argv)
{
    LONG ret;
    HKEY hive, subkey;
    char *hiveName;
    DWORD dwDisposition;
    char pathName[4096];
    char *libPath;
    int libLen;
    int i;

    if (argc != 2) {
	fprintf(stderr, "Usage: %s <Tknt Library dir>\n", argv[0]);
	exit(1);
    }

    libPath = argv[1];

    libLen = strlen(libPath);
    if (libLen == 0) {
	fprintf(stderr, "Please specify a pathname\n");
	exit(1);
    }
    if (libLen + 50 > sizeof(pathName)) {
	fprintf(stderr, "Specified path %s is too long\n", libPath);
	exit(1);
    }
    strcpy(pathName, libPath);
    for (i = 0; i < libLen; i++) {
	if (pathName[i] == '\\') {
	    pathName[i] = '/';
	}
    }
    if (pathName[libLen-1] != '/') {
	pathName[libLen++] = '/';
    }

    hive = HKEY_LOCAL_MACHINE;
    hiveName = "HKEY_LOCAL_MACHINE";

    ret = RegCreateKeyEx(hive, REGISTRY_BASE, 0, "",
			 REG_OPTION_NON_VOLATILE,
			 KEY_ALL_ACCESS, NULL, &subkey,
			 &dwDisposition);
    if (ret != ERROR_SUCCESS) {
	hive = HKEY_CURRENT_USER;
	hiveName = "HKEY_CURRENT_USER";
	ret = RegCreateKeyEx(hive, REGISTRY_BASE, 0, "",
			     REG_OPTION_NON_VOLATILE,
			     KEY_ALL_ACCESS, NULL, &subkey,
			     &dwDisposition);
	if (ret != ERROR_SUCCESS) {
	    fprintf(stderr, "Unable to create registry key %s\\%s\n",
		    hiveName, REGISTRY_BASE);
	    exit(1);
	}
    }
    RegCloseKey(subkey);

    for (i = 0; i < (sizeof(Entries) / sizeof(InstallEntry)); i++) {
	ret = RegCreateKeyEx(hive, Entries[i].key, 0, "",
			     REG_OPTION_NON_VOLATILE,
			     KEY_ALL_ACCESS, NULL, &subkey,
			     &dwDisposition);
	if (ret != ERROR_SUCCESS) {
	    fprintf(stderr, "Unable to create registry key %s\\%s\n",
		    hiveName, Entries[i].key);
	    RegCloseKey(hive);
	    exit(1);
	}

	strcpy(&pathName[libLen], Entries[i].value);
	ret = RegSetValueEx(subkey, Entries[i].valueName, 0L, REG_SZ,
			    pathName, strlen(pathName) + 1);
	if (ret != ERROR_SUCCESS) {
	    fprintf(stderr, "Unable to set %s\\%s\n",
		    hiveName, Entries[i].valueName);
	    exit(1);
	}
	RegCloseKey(subkey);
    }
}
