/*
 *----------------------------------------------------------------------
 * File: man2hlp.c
 *
 *	This file is a complete hack.  Just about everything here was made
 *	to work with no documentation on man macros, so it won't work
 *	perfectly.  For most stuff, it works pretty well.  It has gone
 *	way beyond its initial goal of simply providing a translator for
 *	Tcl and Tk man pages.
 *
 * 	If you want to handle tbl tables, you'll need to run the man page
 *	through manformat.tcl on Unix.  manformat.tcl run nroff -man on the
 *	man page, and it captures the output.  This will generate a version
 *	of the man with some special format characters in it for the captured
 *	output.  The table will be displayed in a fixed size font.
 *
 * Copyright (c) 1995 The Regents of the University of California.
 * Copyright (c) 1994 Fujitsu Networking and Transmission Systems
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *----------------------------------------------------------------------
 */
 

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>

#define TCL_OK 0
#define TCL_ERROR 1

#define SCALE_i  720.0		/* Inch, multiply by 720 */
#define SCALE_c  290.0		/* Centimeter, multiply by 290 */
#define SCALE_P  120.0		/* Pica, multiply by 120 */
#define SCALE_m  72.0		/* Em, multiply by 72 to get 10ths of points */
#define SCALE_n  144.0		/* Em, multiply by 144 */
#define SCALE_p  10.0		/* Multiply by 10 to get 10ths of points */
#define SCALE_u  1.0		/* Not a measurement, just a value */
#define SCALE_v  10.0		/* Is this the correct factor for defaults? */


int currIndent;			/* Indent for next paragraph */
int prevailIndent;		/* The prevailing indent for a paragraph */
int currFirstLineIndent;	/* Offset from indent for first line of
				 * next paragraph */
int prevailFirstLineIndent;	/* Prevailing offset from indent of first
				 * line of paragraph */
int sectionHdrs;
int centerSection;
int linedSection;
int fC;
int bold;
int italic;
int prevailAfter;
int currAfter;
int prevailBefore;
int currBefore;
int centerLines;

int defining;
int nofill;
int parStarted;
int parEnded;
int needBlankLine;
int haveBlankLine;
int needBlankSpace;
int doBreak;
int superScript;

int tabs;
unsigned long tablist[1000];

typedef struct {
    char *topic;
    char *context;
} Record;


Record topics[2000];

int topicno = 0;
int cycle;

char *currFileName;
char *sequenceId;
int sequenceLen;

/*
 *----------------------------------------------------------------------
 * ResetPageVars --
 *
 *	Reset all variables that need to be set for each man page
 *----------------------------------------------------------------------
 */
void
ResetPageVars(FILE *fout)
{
    int i;
    if (fout) {
	if (italic) fprintf(fout, "\\i0");
	if (bold) fprintf(fout, "\\b0");
	if (superScript) fprintf(fout, "}");
    }
    sectionHdrs = 0;
    prevailIndent = 360;
    currIndent = prevailIndent;
    prevailFirstLineIndent = 0;
    currFirstLineIndent = prevailFirstLineIndent;
    prevailAfter = 0;
    currAfter = prevailAfter;
    prevailBefore = 150;
    currBefore = 0;
    haveBlankLine = 1;
    needBlankLine = 0;
    needBlankSpace = 0;
    parStarted = 0;
    parEnded = 1;
    doBreak = 0;
    superScript = 0;
    centerSection = 0;
    linedSection = 0;
    centerLines = 0;
    fC = 0;
    bold = 0;
    italic = 0;
    defining = 0;
    nofill = 0;
    tabs = 10;
    tablist[0] = 360;
    for (i = 1; i < 10; i++) {
	tablist[i] = (int) (.8 * (double) i * 720.0);
    }
}

/*
 *----------------------------------------------------------------------
 * GetArgcArgv --
 *
 *	Break up a line to argc/argv
 *----------------------------------------------------------------------
 */
int
GetArgcArgv(char *s, char **argv)
{
    int quote = 0;
    int argc = 0;
    char *bp;

    bp = s;
    while (1) {
	while (isspace(*bp)) {
	    bp++;
	}
	if (*bp == '\n' || *bp == '\0') {
	    *bp = '\0';
	    return argc;
	}
	if (*bp == '\"' /*\"" (Anti-emacs color formatting) */) {
	    quote = 1;
	    bp++;
	}
	argv[argc++] = bp;

	while (*bp != '\0') {
	    if (quote) {
		if (*bp == '\"'/*\"" (Anti-emacs color formatting) */) {
		    quote = 0;
		    *bp = '\0';
		    bp++;
		    break;
		}
		bp++;
		continue;
	    }
	    if (isspace(*bp)) {
		*bp = '\0';
		bp++;
		break;
	    }
	    bp++;
	}
    }
}


/*
 *----------------------------------------------------------------------
 * GetScaleValue --
 *
 *	Get a value from the passed in string.  If a value is given
 *	without a specifier of length (i, c, p), then use the default
 *	that is passed in.  Gets a value in 10th of a point.
 *
 * Results:
 *	TCL_OK if the value was properly parsed, TCL_ERROR otherwise.
 *----------------------------------------------------------------------
 */
int
GetScaleValue(char *string, double scale, int currVal, int *intPtr)
{
    char *end;
    double d;
    int absolute;
    char *p;

    errno = 0;
    if (*string == '|') {
	absolute = 1;
	p = string + 1;
    } else {
	absolute = 0;
	p = string;
    }

    d = strtod(p, &end);
    if (end == p) {
	badDouble:
	fprintf(stderr, "GetScaleValue: Bad double\n");
	return TCL_ERROR;
    }
    if (errno != 0) {
	fprintf(stderr, "GetScaleValue: Bad value\n");
	return TCL_ERROR;
    }
    while ((*end != 0) && isspace(*end)) {
	end++;
    }
    if (*end != 0) {
	if (*end == 'i') {
	    d *= 720.0;
	} else if (*end == 'c') {
	    d *= 290.0;
	} else if (*end == 'p') {
	    d *= 10.0;
	} else if (*end == 'm') {
	    d *= 72.0;
	} else {
	    goto badDouble;
	}
	end++;
    } else {
	d = d * scale;
    }

    if (!absolute) {
	d = currVal + d;
    }

    if (*end != 0) {
	goto badDouble;
    }
    
    *intPtr = (int) d;
    return TCL_OK;
}

/*
 *----------------------------------------------------------------------
 * ParagraphReset --
 *
 *	We are starting a new paragraph.  Reset all the attributes
 *	that affect paragraphs.
 *----------------------------------------------------------------------
 */
void
ParagraphReset(void)
{
    centerSection = 0;
    centerLines = 0;
    linedSection = 0;
    nofill = 0;
    fC = 0;
}

 
/*
 *----------------------------------------------------------------------
 * Check if we need to end an old paragraph
 *----------------------------------------------------------------------
 */
void
OutputParEnd(FILE *fout)
{
    if (!parEnded) {
	parStarted = 0;
	parEnded = 1;
	if (superScript) {
	    fprintf(fout, "}");
	    superScript = 0;
	}
	fprintf(fout, "\\par\n");
	needBlankLine = 0;
	needBlankSpace = 0;
	haveBlankLine = 1;
    }
}

/*
 *----------------------------------------------------------------------
 * Check if we need to start a new paragraph
 *----------------------------------------------------------------------
 */
int
OutputParStart(FILE *fout)
{
    int i;

    OutputParEnd(fout);
    if (!parStarted) {
	parStarted = 1;
	parEnded = 0;
	fprintf(fout, "\\pard\\sa%d\\sb%d",
		currAfter, currBefore);
	for (i = 0; i < tabs; i++) {
	    fprintf(fout, "\\tx%lu", tablist[i]);
	}
	fprintf(fout, "\n");

	currAfter = prevailAfter;
	currBefore = prevailBefore;

	needBlankLine = 0;
	needBlankSpace = 0;
	haveBlankLine = 1;

	return 1;
    }
    return 0;
}


/*
 *----------------------------------------------------------------------
 * Check if we need to end an old paragraph and start a new one.
 *----------------------------------------------------------------------
 */
void
OutputParCheck(FILE *fout)
{
    /*
     * XXX: Check if we need to start a new paragraph.  If yes,
     * end the old paragraph and generate the new paragraph type.
     */
    if (doBreak) {
	OutputParEnd(fout);
	if (OutputParStart(fout)) {
	    fprintf(fout, "\\fi%d\\li%d\n",
		    currFirstLineIndent, currIndent);
	    currIndent = prevailIndent;
	    currFirstLineIndent = prevailFirstLineIndent;
	}
	doBreak = 0;
	haveBlankLine = 1;
	needBlankSpace = 0;
    } else if (needBlankLine) {
	fprintf(fout, "\\line\n");
	if (!haveBlankLine) {
	    fprintf(fout, "\\line\n");
	}
	haveBlankLine = 1;
	needBlankSpace = 0;
    } else if (needBlankSpace) {
	fprintf(fout, " ");
	needBlankSpace = 0;
    }
    needBlankLine = 0;
}

/*
 *----------------------------------------------------------------------
 * Output a character after doing new paragraph check
 *----------------------------------------------------------------------
 */
void
OutputChar(FILE *fout, char c)
{
    /*
     * XXX: Check if we need to start a new paragraph.  If yes,
     * end the old paragraph and generate the new paragraph type.
     */
    OutputParCheck(fout);
    fputc(c, fout);
    haveBlankLine = 0;
    needBlankSpace = 0;
}

/*
 *----------------------------------------------------------------------
 * Output a string after doing new paragraph check
 *----------------------------------------------------------------------
 */
void
OutputString(FILE *fout, char *s)
{
    OutputParCheck(fout);
    fprintf(fout, "%s", s);
    haveBlankLine = 0;
    needBlankSpace = 0;
}

/*
 *----------------------------------------------------------------------
 * We've hit an end of an input line.  For some formatting commands, this
 * can have special significance.  If it does, them here.
 *----------------------------------------------------------------------
 */
void
OutputEndInputLine(FILE *fout)
{
    /*
     * Depending on the current formatting needs, we may need to
     * start a new paragraph at the next output. XXX
     */
    if (nofill || centerSection || linedSection) {
	needBlankLine = 1;
	haveBlankLine = 1;
    } else if (centerLines) {
	centerLines--;
	needBlankLine = 1;
	haveBlankLine = 1;
    } else {
	if (!haveBlankLine && !needBlankLine) {
	    needBlankSpace = 1;
	}
    }
}

/*
 *----------------------------------------------------------------------
 * Output a page break and reset all the parameters.
 *----------------------------------------------------------------------
 */
void
OutputEOP(FILE *fout)
{
    ResetPageVars(fout);
    fprintf(fout, "\\page\n");
}

/*
 *----------------------------------------------------------------------
 * OutputUncleanText --
 *
 * Arguments:
 *	line: Input line to parse.  It will not have any dot commands in it.
 *	fout: The output file.
 *	partialLine: If this is only part of a line in a dot command,
 *	   set this value.  It can be set to 0 for the last part of the
 *	   partial line.
 *----------------------------------------------------------------------
 */
void
OutputUncleanText(FILE *fout, char *line, int partialLine)
{
    char *bp;

    bp = line;

    if (centerSection || fC || centerLines) {
	OutputString(fout, "\\tab ");
    }
    while (*bp != '\0' && *bp != '\n') {
	if (*bp == '\\') {
	    bp++;
	    if (*bp == 'f') {
		int count;

		count = 1;
		while (count) {
		    count--;
		    bp++;
		    if (*bp == '(') {
			count += 2;
		    } else if (*bp == 'I') {
			if (bold == 1) {
			    OutputString(fout, "\\b0");
			    bold = 0;
			}
			OutputString(fout, "\\i ");
			italic = 1;
		    } else if (*bp == 'B') {
			OutputString(fout, "\\b ");
			bold = 1;
		    } else if (*bp == 'R' || *bp == 'P') {
			if (italic) {
			    italic = 0;
			    OutputString(fout, "\\i0 ");
			}
			if (bold) {
			    bold = 0;
			    OutputString(fout, "\\b0 ");
			}
			if (fC) {
			    fC = 0;
			}
		    } else if (*bp == 'C') {
			fC = 1;
			OutputString(fout, "\\line\\b\\tab ");
			bold = 1;
		    } else {
			fprintf(stderr, "%s: Warning: unknown formatting symbol \'\\f%c\n", currFileName, *bp);
		    }
		    bp++;
		}
		continue;
	    } else if (*bp == '*') {
		bp++;
		if (*bp == '=') {
		    OutputString(fout, "=");
		    bp++;
		} else if (*bp == '(') {
		    bp++;
		    if ((*bp == 'r' || *bp == 'l') && bp[1] == 'q') {
			OutputString(fout, "\"");
			bp += 2;
		    }
		}
		continue;
	    } else if (*bp == '0') {
		OutputString(fout, " ");
		bp++;
		continue;
	    } else if (*bp == 'e') {
		bp++;
		continue;
	    } else if (*bp == 'u') /* Superscript */{
		if (! superScript) {
		    superScript = 1;
		    OutputString(fout, "{\\super ");
		}
		bp++;
	    } else if (*bp == 'd') /* Back to normal */{
		if (superScript) {
		    superScript = 0;
		    OutputString(fout, "}");
		}
		bp++;
	    } else if (*bp == '&') {
		bp++;
	    }
	} else if (*bp == '{' || *bp == '}') {
	    OutputChar(fout, '\\');
	} else if (*bp == '\t') {
	    OutputString(fout, "\\tab ");
	    bp++;
	    continue;
	} else if (*bp == '|') {
	    bp++;
	    continue;
#if 0
	} else if (*bp == '`' && bp[1] == '`') {
	    bp += 2;
	    s = bp;
	    while (isalpha(*bp)) {
		bp++;
	    }
	    if (*bp == '\'' && bp[1] == '\'') {
		*bp = '\0';
		bp += 2;
		OutputString(fout, "\n{\\uldb %s}{\\v %s}{\\footnote %s} \n",
			s, s, s);
		continue;
	    } else {
		bp = s - 2;
	    }
#endif
	}
	OutputChar(fout, *bp);
	bp++;
    }
    
    OutputEndInputLine(fout);

    if (partialLine) {
	needBlankSpace = 1;
    }
}

/*
 *----------------------------------------------------------------------
 * GenerateTH --
 *
 *	Generate a title header for the page
 *----------------------------------------------------------------------
 */
void
GenerateTH(FILE *fout, char *commandName, char *section,
	   char *version, char *module, char *title)
{
    int i = 0;
    char *ip, *op, *context;
    int len;

    len = strlen(commandName);
    context = (char *) malloc(sequenceLen + 5 + len);
    strcpy(context, sequenceId);
    op = &context[sequenceLen];
    *op++ = '_';
    *op++ = '_';
    *op++ = '_';
    *op++ = '_';
    strcpy(op, commandName);

    for (ip = op; *ip; ip++) {
	if (isalnum(*ip)) {
	    *op++ = *ip;
	} else if (*ip == '_' || *ip == '&' || *ip == '-') {
	    *op++ = '_';
	}
    }
    *op = 0;


    if (cycle == 0) {
	topics[topicno].topic = strdup(commandName);
	topics[topicno].context = context;
	topicno++;
	i = topicno;
    } else {
	for (; i < topicno; i++) {
	    if (strcmp(topics[i].topic, commandName) == 0) {
		break;
	    }
	}
    }
    fprintf(fout, "#{\\footnote %s}\n", context);
    fprintf(fout, "${\\footnote %s}\n", commandName);
    fprintf(fout, "K{\\footnote %s}\n", commandName);
    fprintf(fout, "+{\\footnote %s%04d}\n", sequenceId, i);

    fprintf(fout, "\n\\pard\\tx3000\\sb100\\sa100\\fs24\\keepn");
    fprintf(fout, "\\b %s\\tab %s\\b0\\par\n", 
	    commandName, title);
    OutputString(fout, "\\fs20\n");
}

/*
 *----------------------------------------------------------------------
 * GenerateSH --
 *
 *----------------------------------------------------------------------
 */
void
GenerateSH(FILE *fout, int argc, char **argv, int subjIndent)
{
    int i;
    char buf[100];

    if (sectionHdrs != 0) {
	OutputParEnd(fout);
    }
    sectionHdrs++;
    currBefore = prevailBefore;
    doBreak = 0;
    OutputParStart(fout);
    sprintf(buf, "\\li%d\\b ", subjIndent);
    OutputString(fout, buf);

    for (i = 0; i < argc; i++) {
	OutputString(fout, argv[i]);
	OutputChar(fout, ' ');
    }
    OutputString(fout, "\\b0");
    currBefore = 0;
}

/*
 *----------------------------------------------------------------------
 * GenerateFormattedTB --
 *
 *	Parse a tbl table
 *----------------------------------------------------------------------
 */
void
GenerateFormattedTB(FILE *fout, FILE *fin)
{
    char buf[1024];

    fprintf(fout, "{\\f1\n");
    while (fgets(buf, sizeof(buf), fin) != NULL) {
	if (buf[0] == '.' && buf[1] == 't' && buf[2] == 'a' &&
	    buf[3] == 'b' && buf[4] == 'l' && buf[5] == 'e' &&
	    buf[6] == 'e')
	{
	    fprintf(fout, "}\n");
	    fprintf(fout, "\\line\n");
	    OutputParStart(fout);
	    return;
	}
	fprintf(fout, "%s", buf);
	fprintf(fout, "\\line\n");
    }
}

/*
 *----------------------------------------------------------------------
 * GenerateTB --
 *
 *	Parse a tbl table
 *----------------------------------------------------------------------
 */
void
GenerateTB(FILE *fout, FILE *fin)
{
#if 1
    char buf[1024];

    while (fgets(buf, sizeof(buf), fin) != NULL) {
	if (buf[0] == '.' && buf[1] == 'T' && buf[2] == 'E') {
	    return;
	}
    }
    fprintf(fout, "\\line\n");
    /* Do table here */
    fprintf(fout, "\\line\n");

#else
    char *bp;
    char *s, *e;
    int argc;
    char **argv;
    char *heading, *table;
    char buf[1024];
    int first = 1;
    int maxLines = 10;
    char **lines;

    lines = (char **) malloc(maxLines * sizeof(char *));

    /*
     * Get the table format
     */
    while (fgets(buf, sizeof(buf), fin) != NULL) {
	while (*bp != '\n' && *bp != '.') {
	    bp++;
	}
	if (first && *bp != '.') {
	    *bp = 0;
	    heading = strdup(buf);
	} else {
	    *bp = 0;
	    table = strdup(buf);
	}
	if (first) {
	    first = 0;
	}
    }

    /*
     * Now its time to parse the table
     */
    while (fgets(buf, sizeof(buf), fin) != NULL) {
	
    }
#endif
}

/*
 *----------------------------------------------------------------------
 * ReadNextLine --
 *
 *	Reads the next line from the file.  Returns NULL if there
 *	is no more space.  Returns the passed in buffer if something
 *	was read.
 *----------------------------------------------------------------------
 */
char *
ReadNextLine(FILE *fin, char *buf, int size)
{
    char *bp, *bcont;

    bcont = NULL;
    if (fgets(buf, size, fin) == NULL) {
	return NULL;
    }
    bcont = buf;
    while (1) {
	bp = bcont;
	while (*bp && *bp != '\n') {
	    bp++;
	}
	bp--;
	if (*bp != '\\') {
	    break;
	}
	bcont = bp;
	if (fgets(buf, sizeof(buf) - (bcont-buf), fin) == NULL) {
	    break;
	}
    }
    return buf;
}

/*
 *----------------------------------------------------------------------
 * ReadFile --
 *
 *	Reads a man page and checks the dot formatting characters.
 *	It then sends output to OutputUncleanText() for further
 *	processing.
 *----------------------------------------------------------------------
 */
void
ReadFile(FILE *fin, FILE *fout, char *filename)
{
    char buf[8192];
    char *bp, *spos;
    char *cmd;
    char *largv[30];
    char *s;
    int largc;
    int i;
    long val;
    int haveTH = 0;
    int v;

    currFileName = filename;
    while (ReadNextLine(fin, buf, sizeof(buf)) != NULL) {
	bp = buf;
	if (defining) {
	    if (*bp == '.' && bp[1] == '.') {
		defining = 0;
	    }
	    goto nextLine;
	}
	if (haveTH && *bp == '\n') {
	    needBlankLine = 1;
	}

	/* See if we have a formatting command or something */
	switch (*bp) {
	case '.':
	    if (bp[1] == '.' && bp[2] == '.') {
		break;
	    }
	    if (isdigit(bp[1])) {
		break;
	    }
	    /* Time to check up on out formatting characters. */
	    bp++;
	    spos = bp;
	    while (!isspace(*bp)) {
		bp++;
	    }
	    if (bp - spos == 0) {
		goto nextLine;
	    }

	    if (*bp != '\0') {
		*bp = '\0';
		bp++;
	    }
	    cmd = spos;

	    while (isspace(*bp)) {
		bp++;
	    }

	    /* fprintf(stderr, "Processing \'.%s\'\n", cmd); */
	    /* ------ .\" ----- */ /*\"" */
	    if (cmd[0] == '\\' && cmd[1] == '\"' /*\"" */) {
	    }

	    /*
	     *--------------------------------------------------
	     *--------------------------------------------------
	     * 3. Page Control
	     * .pl, .bp, .pn, .po, .ne, .mk, .rt
	     *--------------------------------------------------
	     *--------------------------------------------------
	     */

	    /* ------ .ne (Need N vertical space) */
	    else if (strcmp(cmd, "ne") == 0) {
		/* XXX: Ignore ? */
	    }

	    /* ------ .bp (Eject current page) */
	    else if (strcmp(cmd, "bp") == 0) {
		needBlankLine = 1;
	    }


	    /*
	     *--------------------------------------------------
	     *--------------------------------------------------
	     * 4. Text Filling, Adjusting, and Centering Commands
	     * .br, .fi, .nf, .ad, .na, .ce
	     *--------------------------------------------------
	     *--------------------------------------------------
	     */

	    /* ------ .br (Break) */
	    else if (strcmp(cmd, "br") == 0) {
		needBlankLine = 1;
		haveBlankLine = 1;
	    }

	    /* ------ .fi (Resume filling or adjusting output lines). */
	    else if (strcmp(cmd, "fi") == 0) {
		doBreak = 1;
		nofill = 0;
	    }

	    /* ------ .nf (No filling or adjusting output lines) */
	    else if (strcmp(cmd, "nf") == 0) {
		/* End of line means newline */
		doBreak = 1;
		nofill = 1;
	    }

	    /* ------ .ad (Adjust output lines w/ mode c; c=l,r,c,b, none) */
	    else if (strcmp(cmd, "ad") == 0) {
	    }

	    /* ------ .na (No output line adjusting ) */
	    else if (strcmp(cmd, "na") == 0) {
	    }

	    /* ------ .ce (Center next N input lines (1 by default) */
	    else if (strcmp(cmd, "ce") == 0) {
		largc = GetArgcArgv(bp, largv);
		if (largc == 0) {
		    centerLines = 1;
		} else {
		    if (GetScaleValue(largv[0], SCALE_m, 0, &v) == TCL_OK) {
			centerLines = v;
		    } else {
			centerLines = 1;
		    }
		}
	    }



	    /*
	     *--------------------------------------------------
	     *--------------------------------------------------
	     * 5. Vertical Spacing
	     * .vs, .ls, .sp, .sv, .os, .ns, .rs
	     *--------------------------------------------------
	     *--------------------------------------------------
	     */
	    /* ------ .sp ----- */
	    else if (strcmp(cmd, "sp") == 0) {
		needBlankLine = 1;
	    }

	    /* ------ .ns ----- */
	    else if (strcmp(cmd, "ns") == 0 /* Turn no-space mode on */) {
	    }

	    /* ------ .rs ----- */
	    else if (strcmp(cmd, "rs") == 0 /* Turn no-space mode off */) {
	    }


	    /*
	     *--------------------------------------------------
	     *--------------------------------------------------
	     * 6. Line Length and Indenting
	     * .ll, .in, .ti
	     *--------------------------------------------------
	     *--------------------------------------------------
	     */
	    /* ------ .ll ----- */
	    else if (strcmp(cmd, "ll") == 0) {
	    }

	    /* ------ .in ----- */
	    else if (strcmp(cmd, "in") == 0) {
		if (!doBreak) {
		    currBefore = 0;
		}
		doBreak = 1;
		largc = GetArgcArgv(bp, largv);
		if (largc > 0 &&
		    GetScaleValue(largv[0], SCALE_m, prevailIndent, &v) == TCL_OK) {
		    currIndent = prevailIndent = v;
		} else {
		    currIndent = prevailIndent = 360;
		}
	    }

	    /* ------ .ti ----- */
	    else if (strcmp(cmd, "ti") == 0) {
		if (!doBreak) {
		    currBefore = 0;
		}
		doBreak = 1;
		largc = GetArgcArgv(bp, largv);
		if (largc > 0 &&
		    GetScaleValue(largv[0], SCALE_m, prevailFirstLineIndent, &v) == TCL_OK) {
		    currFirstLineIndent = v;
		}
	    }



	    /*
	     *--------------------------------------------------
	     *--------------------------------------------------
	     * 7. Macros, Strings, Diversion, and Position Traps
	     * .de, .am, .ds, .as, .rm, .rn, .di, .da, .wh, .ch,
	     * .dt, .it, .em
	     *--------------------------------------------------
	     *--------------------------------------------------
	     */

	    /* ------ .de (Define or redefine a macro) */
	    else if (strcmp(cmd, "de") == 0) {
		defining = 1;
	    }

	    /* ------ .am (Append to a macro) */
	    else if (strcmp(cmd, "am") == 0) {
	    }

	    /* ------ .ds (Define a string xx containing string) */
	    else if (strcmp(cmd, "ds") == 0) {
	    }

	    /* ------ .as (Append string to string xx) */
	    else if (strcmp(cmd, "as") == 0) {
	    }

	    /* ------ .rm (Remvoe request, macro, or string) */
	    else if (strcmp(cmd, "rm") == 0) {
	    }

	    /* ------ .rn (Rename request, macro, or string) */
	    else if (strcmp(cmd, "rn") == 0) {
	    }

	    /* ------ .di (Divert output to macro xx) */
	    else if (strcmp(cmd, "di") == 0) {
	    }

	    /* ------ .da (Divert and append to xx) */
	    else if (strcmp(cmd, "da") == 0) {
	    }

	    /* ------ .wh (Set location trap; negative is w.r.t page bottom) */
	    else if (strcmp(cmd, "wh") == 0) {
	    }

	    /* ------ .ch (Change trap location) */
	    else if (strcmp(cmd, "ch") == 0) {
	    }

	    /* ------ .dt (Set a diversion trap) */
	    else if (strcmp(cmd, "dt") == 0) {
	    }

	    /* ------ .it (Set an input-line count trap) */
	    else if (strcmp(cmd, "it") == 0) {
	    }

	    /* ------ .em (End macro is xx) */
	    else if (strcmp(cmd, "em") == 0) {
	    }



	    /*
	     *--------------------------------------------------
	     *--------------------------------------------------
	     * 8. Number Registers
	     * .nr, .af, .rr
	     * XXX: Nothing is done with these
	     *--------------------------------------------------
	     *--------------------------------------------------
	     */

	    /* ------ .nr ----- */
	    else if (strcmp(cmd, "nr") == 0) {
	    }

	    /* ------ .af ----- */
	    else if (strcmp(cmd, "af") == 0) {
	    }

	    /* ------ .rr ----- */
	    else if (strcmp(cmd, "rr") == 0) {
	    }



	    /*
	     *--------------------------------------------------
	     *--------------------------------------------------
	     * 9. Tabs, Leaders, and Fields
	     * .ta, .tc, .lc, .fc
	     *--------------------------------------------------
	     *--------------------------------------------------
	     */

	    /* ------ .ta (Tab settings) */
	    else if (strcmp(cmd, "ta") == 0) {
		unsigned long n;

		/* Tab list */
		tabs = GetArgcArgv(bp, largv);
		for (i = 0; i < tabs; i++) {
		    if (GetScaleValue(largv[0], SCALE_m, 0, &tablist[i]) != TCL_OK) {
			tablist[i] = 0;
		    }
		}
		if (tabs == 2) {
		    n = tablist[0];
		    tablist[0] = tablist[1];
		    for (i = 1; i < 10; i++) {
			tablist[i] = (i * n) + tablist[0];
		    }
		    tabs = 10;
		} else if (tabs == 1) {
		    n = tablist[0];
		    for (i = tabs; i < 10; i++) {
			tablist[i] = (i + 1) * n;
		    }
		    tabs = 10;
		}
	    }

	    /* ------ .tc (Tab repetition character) */
	    else if (strcmp(cmd, "tc") == 0) {
	    }

	    /* ------ .lc (Leader repetition character) */
	    else if (strcmp(cmd, "lc") == 0) {
	    }

	    /* ------ .fc (Set filed delimiter a and pad character b) */
	    else if (strcmp(cmd, "fc") == 0) {
	    }



	    /*
	     *--------------------------------------------------
	     *--------------------------------------------------
	     * 13. Hyphenation
	     * .nh, .hy, .hc, .hw
	     *--------------------------------------------------
	     *--------------------------------------------------
	     */

	    /* ------ .nh (No hyphenation) */
	    else if (strcmp(cmd, "nh") == 0) {
	    }
	    /* ------ .hy (Hyphenate) */
	    else if (strcmp(cmd, "hy") == 0) {
	    }
	    /* ------ .hc (Hyphenation indicator character) */
	    else if (strcmp(cmd, "hc") == 0) {
	    }
	    /* ------ .hw (Add words to hyphenation dictionary) */
	    else if (strcmp(cmd, "hw") == 0) {
	    }


	    /*
	     *--------------------------------------------------
	     *--------------------------------------------------
	     * 16. Conditional Acceptance of Input
	     * .if, .ie, .el
	     *--------------------------------------------------
	     *--------------------------------------------------
	     */

	    /* ------ .if ----- */
	    else if (strcmp(cmd, "if") == 0) {
	    }

	    /* ------ .ie ----- */
	    else if (strcmp(cmd, "ie") == 0) {
	    }

	    /* ------ .el ----- */
	    else if (strcmp(cmd, "el") == 0) {
	    }



	    /*
	     *--------------------------------------------------
	     *--------------------------------------------------
	     * 19. Conditional Acceptance of Input
	     * .so, .nx, .sy, .pi, .cf
	     *--------------------------------------------------
	     *--------------------------------------------------
	     */

	    /* ------ .so (Source a file) */
	    else if (strcmp(cmd, "so") == 0) {
	    }



	    /* ------ .AP ----- */
	    else if (strcmp(cmd, "AP") == 0) {
		largc = GetArgcArgv(bp, largv);
		OutputParEnd(fout);
		OutputParStart(fout);
		val = 1800;
		fprintf(fout, "\\fi%d\\li%d\\sa20\\tx%d\\tx%d\\tx%d\n",
			-val*3, prevailIndent + val*3,
			prevailIndent + val, prevailIndent + val*2, prevailIndent + val*3);
		if (largc > 0) {
		    fprintf(fout, "%s\\tab ", largv[0]);
		} else {
		    fprintf(fout, "\\tab ");
		}
		if (largc > 1) {
		    fprintf(fout, "%s\\tab ", largv[1]);
		} else {
		    fprintf(fout, "\\tab ");
		}
		if (largc > 2) {
		    fprintf(fout, "(%s)\\tab ", largv[2]);
		} else {
		    fprintf(fout, "\\tab ");
		}
		doBreak = 1;
		linedSection = 0;
		centerSection = 0;
	    }

	    /* ------ .AS ----- */
	    else if (strcmp(cmd, "AS") == 0) {
		/* Ignore this for now */
	    }

	    /* ------ .B (Make rest of this line or the next line bold) */
	    else if (strcmp(cmd, "B") == 0) {
		OutputString(fout, "\\b ");
		if (*bp == '\n') {
		    bp++;
		}
		OutputUncleanText(fout, bp, 1);
		OutputString(fout, "\\b0 ");
	    }

	    /* ------ .BS ----- */
	    else if (strcmp(cmd, "BS") == 0) {
	    }

	    /* ------ .BE ----- */
	    else if (strcmp(cmd, "BE") == 0) {
	    }

	    /* ------ .CS ----- */
	    else if (strcmp(cmd, "CS") == 0) {
		doBreak = 1;
		ParagraphReset();
		centerSection = 1;
		linedSection = 1;
	    }

	    /* ------ .CE ----- */
	    else if (strcmp(cmd, "CE") == 0) {
		doBreak = 1;
		ParagraphReset();
	    }

	    /* ------ .DS ----- */
	    else if (strcmp(cmd, "DS") == 0) {
		/* Supposed to handle some centering, but does not: XXX */
		doBreak = 1;
		ParagraphReset();
		centerSection = 1;
		linedSection = 1;
	    }

	    /* ------ .DE ----- */
	    else if (strcmp(cmd, "DE") == 0) {
		doBreak = 1;
		ParagraphReset();
	    }

	    /* ------ .HS ----- */
	    else if (strcmp(cmd, "HS") == 0) {
		largc = GetArgcArgv(bp, largv);
		if (largc >= 2) {
		    if (strcmp(largv[1], "cmds") == 0) {
			s = "Command";
		    } else if (strcmp(largv[1], "lib") == 0) {
			s = "Library Function";
		    } else if (strcmp(largv[1], "tcl") == 0) {
			s = "Tcl Built-In Commands";
		    } else if (strcmp(largv[1], "tk") == 0) {
			s = "Tk Built-In Commands";
		    } else if (strcmp(largv[1], "tclc") == 0) {
			s = "Tcl Library Procedures";
		    } else if (strcmp(largv[1], "tkc") == 0) {
			s = "Tk Library Procedures";
		    } else if (strcmp(largv[1], "tclcmds") == 0) {
			s = "Tcl Applications";
		    } else if (strcmp(largv[1], "tkcmds") == 0) {
			s = "Tk Applications";
		    } else if (strcmp(largv[1], "tcldp") == 0) {
			s = "Tcl-DP Commands";
		    } else if (strcmp(largv[1], "tcldpc") == 0) {
			s = "Tcl-DP Library Procedures";
		    } else if (strcmp(largv[1], "tcldpcmds") == 0) {
			s = "Tcl-DP Applications";
		    } else if (strcmp(largv[1], "tcldpintro") == 0) {
			s = "Introduction to Tcl-DP";
		    } else if (strcmp(largv[1], "tix") == 0) {
			s = "Tix Commands";
		    } else if (strcmp(largv[1], "tixcmds") == 0) {
			s = "Tix Applications";
		    } else if (strcmp(largv[1], "cm") == 0) {
			s = "CMT";
		    } else {
			fprintf(stderr, "%s: Warning: .HS has unknown type for arg2: \'%s\'\n", currFileName, largv[1]);
			s = NULL;
		    }
		} else {
		    s = NULL;
		}

		GenerateTH(fout, largv[0], "", "", "", s);
		haveTH = 1;
	    }

	    /* ------ .I ------ */
	    else if (strcmp(cmd, "I") == 0) {
		if (*bp == '\n') {
		    bp++;
		}
		OutputString(fout, "\\i ");
		OutputUncleanText(fout, bp, 0);
		OutputString(fout, "\\i0 ");
	    }

	    /* ------ .IR ------ */
	    else if ((strcmp(cmd, "IR") == 0) ||
		     (strcmp(cmd, "RI") == 0)) {
		if (*bp == '\n') {
		    bp++;
		}
		OutputString(fout, "\\b\\i ");
		OutputUncleanText(fout, bp, 0);
		OutputString(fout, "\\b0\\i0 ");
	    }

	    /* ------ .IP (Begin indented paragraph with tag x-- .IP x i) */
	    else if (strcmp(cmd, "IP") == 0) {
		largc = GetArgcArgv(bp, largv);
		if (largc > 0) {
		    if (largc >= 2 &&
			GetScaleValue(largv[1], SCALE_m, 0, &v) == TCL_OK) {
			currIndent = prevailIndent = v - 360;
		    }
		}

		doBreak = 1;
		if (largc != 0) {
		    OutputUncleanText(fout, largv[0], 0);
		}

		currBefore = 0;
		currIndent = prevailIndent + 360;
		doBreak = 1;
		
		ParagraphReset();
	    }

	    /* ------ .NF (No filling or adjusting output lines?) */
	    else if (strcmp(cmd, "NF") == 0) {
		/* End of line means newline */
		doBreak = 1;
		nofill = 1;
	    }

	    /* ------ .FI (Resume filling or adjusting output lines?). */
	    else if (strcmp(cmd, "FI") == 0) {
		doBreak = 1;
		nofill = 0;
	    }

	    /* ------ .TP (Begin indented paragraph.  Tag on next line) */
	    else if (strcmp(cmd, "TP") == 0) {
		char tbuf[8192];

		largc = GetArgcArgv(bp, largv);
		if (largc >= 1 &&
		    GetScaleValue(largv[0], SCALE_m, prevailIndent, &v) == TCL_OK) {
		    currIndent = prevailIndent = v - 360;
		}
		
		if (ReadNextLine(fin, tbuf, sizeof(tbuf)) == NULL) {
		    return;
		}
		ParagraphReset();

		doBreak = 1;
		OutputUncleanText(fout, tbuf, 0);

		currBefore = 0;
		currIndent = prevailIndent + 360;

		doBreak = 1;
	    }

	    /* ------ .LP ----- */
	    /* ------ .PP (Same as LP) */
	    else if (strcmp(cmd, "LP") == 0 ||
		     strcmp(cmd, "PP") == 0) {
		doBreak = 1;
		ParagraphReset();
		currIndent = prevailIndent = 360;
	    }

	    /* ------ .P ----- */
	    else if (strcmp(cmd, "P") == 0) {
	    }


	    /* ------ .RS ----- */
	    else if (strcmp(cmd, "RS") == 0) {
		/* XXX: Should handle an argument for indent */
		doBreak = 1;
		ParagraphReset();
		prevailIndent += 360;
		currIndent = prevailIndent;
	    }

	    /* ------ .RE ----- */
	    else if (strcmp(cmd, "RE") == 0) {
		doBreak = 1;
		ParagraphReset();
		prevailIndent -= 360;
		currIndent = prevailIndent;
	    }

	    /* ------ .SH ----- */
	    else if (strcmp(cmd, "SH") == 0) {
		haveBlankLine = 1;
		largc = GetArgcArgv(bp, largv);
		if (largc < 1) {
		    goto nextLine;
		}
		GenerateSH(fout, largc, largv, 0);
		ParagraphReset();
		currIndent = prevailIndent = 360;
		doBreak = 1;
	    }

	    /* ------ .SS ----- */
	    else if (strcmp(cmd, "SS") == 0 || strcmp(cmd, "SB") == 0) {
		largc = GetArgcArgv(bp, largv);
		if (largc < 1) {
		    fprintf(stderr, "Error: Wrong # args for .SS, need 1, got %d\n", largc);
		    goto nextLine;
		}
		GenerateSH(fout, largc, largv, 200);
		ParagraphReset();
		doBreak = 1;
	    }

	    /* ------ .tables ----- */
	    else if (strcmp(cmd, "tables") == 0) {
		/* Beginning of a preformatted table */
		GenerateFormattedTB(fout, fin);
	    }

	    /* ------ .TE ----- */
	    else if (strcmp(cmd, "TE") == 0) {
	    }

	    /* ------ .TH ----- */
	    else if (strcmp(cmd, "TH") == 0) {
		largc = GetArgcArgv(bp, largv);
		if (largc < 1) {
		    fprintf(stderr, "Error: .TH has %d args, needs 1\n", largc);
		    goto nextLine;
		}

		/* command is arg 0 */
		/* section is arg 1 */
		/* version is arg 2 */
		/* module  is arg 3 */
		/* title   is arg 4 */
		GenerateTH(fout, largv[0],
			   (largc >= 1) ? largv[1] : "",
			   (largc >= 2) ? largv[2] : "",
			   (largc >= 3) ? largv[3] : "",
			   (largc >= 4) ? largv[4] : "");
		haveTH = 1;
	    }

	    /* ------ .TS ----- */
	    else if (strcmp(cmd, "TS") == 0) {
		GenerateTB(fout, fin);
	    }

	    /* ------ .UL ----- */
	    /* XXX: Make like the .B command */
	    else if (strcmp(cmd, "UL") == 0) {
		largc = GetArgcArgv(bp, largv);
		for (i = 0; i < largc; i++) {
		    fprintf(fout, "\\ul %s\\ul0 ", largv[i]);
		}
	    }

	    /* ------ .VS ----- */
	    else if (strcmp(cmd, "VS") == 0) {
		/* Vertical bar start */
	    }

	    /* ------ .VE ----- */
	    else if (strcmp(cmd, "VE") == 0) {
		/* Vertical bar start */
	    }

	    /* ------ Unknown ----- */
	    else {
		fprintf(stderr, "%s: Warning: Unknown command \'.%s\', skipping...\n", currFileName, cmd);
	    }

	    goto nextLine;

	case '\'':
	    goto nextLine;
	default:
	    break;
	}

	OutputUncleanText(fout, bp, 0);

    nextLine:
	;
    }
}

/*
 *----------------------------------------------------------------------
 * SortFunc --
 *
 *	Function to sort the title strings for our index.
 *----------------------------------------------------------------------
 */
static int
SortFunc(const void *first, const void *second)
{
    const Record *f, *s;
    /* const char **f, **s; */
    char fc, sc;

    f = first;
    s = second;

    fc = f->topic[0];
    sc = s->topic[0];
    if (isupper(fc) && islower(sc)) {
	return 1;
    } else if (islower(fc) && isupper(sc)) {
	return -1;
    }
    return strcmp(f->topic, s->topic);
}

/*
 *----------------------------------------------------------------------
 * main --
 *
 *	Takes all the command line arguments and generates a large RTF
 *	file based on all their inputs.  The .TH or .HS macro needs to
 *	be used to specify the title of each page.  If a filename begins
 *	with '@', this indicates that the rest of the command line will
 *	be read that file (minus the '@').
 *----------------------------------------------------------------------
 */
void
main(int argc, char **argv)
{
    char *fargv[1000];
    char cmdline[10000];
    int i, args;
    FILE *fin, *fout, *fargs;
    char *inname, *outname;
    int pos;

    ResetPageVars(NULL);

    fargs = NULL;
    for (args = 0; args < argc; args++) {
	if (argv[args][0] == '@') {
	    fargs = fopen(&argv[args][1], "r");
	    if (fargs == NULL) {
		fprintf(stderr, "Unable to open \'%s\' for reading:\n",
			argv[args]);
		perror("");
		exit(1);
	    }
	    pos = 0;
	    for (i = 0; i < args; i++) {
		strcpy(&cmdline[pos], argv[i]);
		pos += strlen(&cmdline[pos]) + 1;
		fargv[i] = argv[i];
	    }
	    fgets(&cmdline[pos], sizeof(cmdline), fargs);
	    fclose(fargs);
	    i += GetArgcArgv(&cmdline[pos], &fargv[i]);
	    argc = i;
	    argv = fargv;
	    break;
	}
    }
	
    if (argc == 1) {
    Usage:
	fprintf(stderr, "Usage: %s [-sequence id] [-out outfile] <infile> [<infile> ...\n", argv[0]);
	exit(1);
    }

    for (cycle = 0; cycle < 2; cycle++) {
	fout = stdout;
	args = 1;
	sequenceId = "seq";
	while (argv[args][0] == '-') {
	    if (strcmp(argv[args], "-out") == 0) {
		if (argc - args < 3) {
		    goto Usage;
		}
		args++;
		outname = argv[args];
		fout = fopen(outname, "w");
		if (fout == NULL) {
		    fprintf(stderr, "Unable to open \'%s\' for writing:\n", outname);
		    perror("");
		    exit(1);
		}
	    } else if (strcmp(argv[args], "-sequence") == 0) {
		if (argc - args < 3) {
		    goto Usage;
		}
		args++;
		sequenceId = argv[args];
	    }
	    args++;
	}
	sequenceLen = strlen(sequenceId);

	fprintf(fout, "{\\rtf1\\ansi\n");
	fprintf(fout, "{\\fonttbl {\\f0\\fswiss Arial;}"
		" {\\f1\\fmodern Courier New;}}\n");

	while (args < argc) {
	    inname = argv[args];
	    fin = fopen(inname, "r");
	    if (fin == NULL) {
		fprintf(stderr, "Unable to open \'%s\' for reading:\n", inname);
		perror("");
		exit(1);
	    }

	    ReadFile(fin, fout, inname);

	    fclose(fin);
	    OutputEOP(fout);
	    args++;
	}

	qsort(topics, topicno, sizeof(Record), SortFunc);
	fprintf(fout, "\\pard\\fs20\n");
	fprintf(fout, "#{\\footnote %s_main_contents}\n", sequenceId);
	fprintf(fout, "${\\footnote %s_main_contents}\n", sequenceId);
	fprintf(fout, "K{\\footnote %s_main_contents}\n", sequenceId);
	for (i = 0; i < topicno; i++) {
	    fprintf(fout, "{\\uldb %s}{\\v %s}\\line\n",
		    topics[i].topic, topics[i].context);
	}
	fprintf(fout, "\\page\n");

	fprintf(fout, "}\n");
	if (fout != stdout) {
	    fclose(fout);
	}
    }
}
