#include <windows.h>
#include <stdio.h>
#include <wchar.h>

#ifndef APPLICATION_NAME
#  define APPLICATION_NAME "Tknt"
#endif

#define REGISTRY_BASE   "SOFTWARE\\" APPLICATION_NAME

typedef struct {
    char *key;
    char *valueName;
    char *value;
} InstallEntry;

InstallEntry Entries[] = {
#if 0
    {REGISTRY_BASE "\\Tcl 7.3",		"TCL_LIBRARY",	"tcl7.3"},
    {REGISTRY_BASE "\\Tk 3.6",		"TK_LIBRARY",   "tk3.6"},
    {REGISTRY_BASE "\\BLT 1.7",		"BLT_LIBRARY",	"blt1.7"},
#else
    {REGISTRY_BASE "\\Tcl 7.4",		"TCL_LIBRARY",	"tcl7.4"},
    {REGISTRY_BASE "\\Tk 4.0",		"TK_LIBRARY",	"tk4.0"},
    {REGISTRY_BASE "\\BLT 1.9",		"BLT_LIBRARY",	"blt1.9"},
    {REGISTRY_BASE "\\Tix 4.0",		"TIX_LIBRARY",	"tix4.0"},
#endif
    {REGISTRY_BASE "\\Tcl-DP 3.3", 	"DP_LIBRARY",	"dp3.3"},
    {REGISTRY_BASE "\\Itcl 1.5", 	"ITCL_LIBRARY", "itcl1.5"},
};


void
main(int argc, char **argv)
{
    LONG ret;
    HKEY hive, subkey;
    char *hiveName;
    DWORD dwDisposition;
    char pathName[4096];
    char *libPath;
    int libLen;
    int i;

    if (argc != 2) {
	fprintf(stderr, "Usage: %s <Tknt Library dir>\n", argv[0]);
	exit(1);
    }

    libPath = argv[1];

    libLen = strlen(libPath);
    if (libLen == 0) {
	fprintf(stderr, "Please specify a pathname\n");
	exit(1);
    }
    if (libLen + 50 > sizeof(pathName)) {
	fprintf(stderr, "Specified path %s is too long\n", libPath);
	exit(1);
    }
    strcpy(pathName, libPath);
    for (i = 0; i < libLen; i++) {
	if (pathName[i] == '\\') {
	    pathName[i] = '/';
	}
    }
    if (pathName[libLen-1] != '/') {
	pathName[libLen++] = '/';
    }

    hive = HKEY_LOCAL_MACHINE;
    hiveName = "HKEY_LOCAL_MACHINE";

    ret = RegCreateKeyEx(hive, REGISTRY_BASE, 0, "",
			 REG_OPTION_NON_VOLATILE,
			 KEY_ALL_ACCESS, NULL, &subkey,
			 &dwDisposition);
    if (ret != ERROR_SUCCESS) {
	hive = HKEY_CURRENT_USER;
	hiveName = "HKEY_CURRENT_USER";
	ret = RegCreateKeyEx(hive, REGISTRY_BASE, 0, "",
			     REG_OPTION_NON_VOLATILE,
			     KEY_ALL_ACCESS, NULL, &subkey,
			     &dwDisposition);
	if (ret != ERROR_SUCCESS) {
	    fprintf(stderr, "Unable to create registry key %s\\%s\n",
		    hiveName, REGISTRY_BASE);
	    exit(1);
	}
    }
    RegCloseKey(subkey);

    for (i = 0; i < (sizeof(Entries) / sizeof(InstallEntry)); i++) {
	ret = RegCreateKeyEx(hive, Entries[i].key, 0, "",
			     REG_OPTION_NON_VOLATILE,
			     KEY_ALL_ACCESS, NULL, &subkey,
			     &dwDisposition);
	if (ret != ERROR_SUCCESS) {
	    fprintf(stderr, "Unable to create registry key %s\\%s\n",
		    hiveName, Entries[i].key);
	    RegCloseKey(hive);
	    exit(1);
	}

	strcpy(&pathName[libLen], Entries[i].value);
	ret = RegSetValueEx(subkey, Entries[i].valueName, 0L, REG_SZ,
			    pathName, strlen(pathName) + 1);
	if (ret != ERROR_SUCCESS) {
	    fprintf(stderr, "Unable to set %s\\%s\n",
		    hiveName, Entries[i].valueName);
	    exit(1);
	}
	RegCloseKey(subkey);
    }
}
