/*
 * callback.c --
 *
 * Copyright (c) 1994 Software Research Associates, Inc.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Software Research Associates not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  Software Research
 * Associates makes no representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or implied
 * warranty.
 */

#include "xlibInt.h" 

static HHOOK hNextHook;

int lx_PointerGrabbed = 0;
Window lx_GrabbedWindow = None;
static Window pointerWindow = HWND_DESKTOP;

static int keyboardGrabbed = 0;
static Window keyboardWindow = None;

static int pointerX = 0;
static int pointerY = 0;


#if 0
static void
DebugPrintEvent(Window w, int type, int x, int y, int detail, int mode, int focus, unsigned long serial)
{
    char *detailStr;
    char *typeStr;
    char *modeStr;

    switch (type) {
    case EnterNotify:
	typeStr = "EnterNotify";
	break;
    case LeaveNotify:
	typeStr = "LeaveNotify";
	break;
    case FocusIn:
	typeStr = "FocusIn";
	break;
    case FocusOut:
	typeStr = "FocusOut";
	break;
    }

    switch (detail) {
    case NotifyAncestor:
	detailStr = "NotifyAncestor";
	break;
    case NotifyInferior:
	detailStr = "NotifyInferior";
	break;
    case NotifyVirtual:
	detailStr = "NotifyVirtual";
	break;
    case NotifyNonlinear:
	detailStr = "NotifyNonlinear";
	break;
    case NotifyNonlinearVirtual:
	detailStr = "NotifyNonlinearVirtual";
	break;
    }

    switch (mode) {
    case NotifyGrab:
	modeStr = "NotifyGrab";
	break;
    case NotifyUngrab:
	modeStr = "NotifyUngrab";
	break;
    case NotifyNormal:
	modeStr = "NotifyNormal";
	break;
    }
    printf("%s: 0x%08x (%d, %d) %s %s focus=%d ser#=%ld\n", typeStr, w, x, y, detailStr, modeStr, focus, serial);
}
#endif


/*
 *----------------------------------------------------------------------
 *
 * CreateCrossingEvent -- 
 *
 *	Actually create an EnterNotify event and add it to the
 *	X event queue.  Post a message to make sure that the
 *	GetMessage() call will know that something is waiting
 *	in the X event queue.
 *
 * Results:
 *	None
 *
 * Side Effects:
 *	None
 *
 *----------------------------------------------------------------------
 */
static void
CreateCrossingEvent(Display *display, HWND hwnd, int type,
		    int x, int y, int detail, int mode)
{
    XEvent e;
    POINT sPt;
    Window parent;
    int focus;
    char className[256];

    if (! Lx_CheckSendMessage(hwnd, type)) {
	return;
    }

    focus = FALSE;
    parent = hwnd;
    while (parent != HWND_DESKTOP) {
	if (display->focusWin == parent) {
	    focus = TRUE;
	    break;
	}

	GetClassName(parent, className, sizeof(className));
	if (strcmp(className, "TopLevel") == 0) {
	    break;
	}
	parent = GetParent(parent);
    }

    e.type = type;
    e.xany.display = display;
    e.xany.window = (Window) hwnd;
    e.xany.serial = display->request;
    e.xany.send_event = False;
    e.xcrossing.subwindow = None;
    e.xcrossing.time = GetCurrentTime();
    e.xcrossing.x_root = sPt.x = x;
    e.xcrossing.y_root = sPt.y = y; 
    ScreenToClient(hwnd, &sPt);
    e.xcrossing.x = sPt.x;
    e.xcrossing.y = sPt.y;
    e.xcrossing.mode = mode;
    e.xcrossing.detail = detail;
    e.xcrossing.same_screen = True;
    e.xcrossing.focus = focus;
    e.xcrossing.state = Lx_MsgGetState(0);

    Lx_PutEvent(display, &e);
    PostMessage(hwnd, EVENTADDED, 0, 0L);

#if 0
    DebugPrintEvent(hwnd, type, x, y, detail, mode, focus, e.xany.serial);
#endif
}

/*
 *----------------------------------------------------------------------
 * CreateCrossingAncestorEvents --
 *
 *	Recursively create crossing events until we hit the stop window
 *----------------------------------------------------------------------
 */
static void
CreateCrossingAncestorEvents(Display *display, HWND stop, HWND start,
			     int x, int y, int detail, int mode)
{
    char className[256];

#if 0
    printf("CreateCrossingAncestorEvents: start=0x%p, stop=0x%p\n", start, stop);
#endif

    if (start == stop) {
	return;
    }
    if (start == HWND_DESKTOP) {
	return;
    }

    GetClassName(start, className, sizeof(className));
    if (strcmp(className, "TopLevel") == 0) {
	CreateCrossingEvent(display, start, EnterNotify, x, y, detail, mode);
	return;
    }

    CreateCrossingAncestorEvents(display, stop, GetParent(start), x, y, detail, mode);
    CreateCrossingEvent(display, start, EnterNotify, x, y, detail, mode);
}

/*
 *----------------------------------------------------------------------
 * CreateCrossingInEvents
 *
 *	Create crossing in events for the given range
 *----------------------------------------------------------------------
 */
static void
CreateCrossingInEvents(Display *display, HWND stop, HWND start,
		       int x, int y, Bool linear, int mode)
{
    int detail1, detail2;
    char className[256];

#if 0
    printf("CreateCrossingInEvents: stop=0x%p, start=0x%p\n", stop, start);
#endif
    if (start == stop) {
	return;
    }

    if (linear) {
	detail1 = NotifyAncestor;
	detail2 = NotifyVirtual;
    } else {
	detail1 = NotifyNonlinear;
	detail2 = NotifyNonlinearVirtual;
    }

    GetClassName(start, className, sizeof(className));
    if (strcmp(className, "TopLevel") == 0) {
	CreateCrossingEvent(display, start, EnterNotify, x, y, detail1, mode);
	return;
    }

    CreateCrossingAncestorEvents(display, stop, GetParent(start), 
				 x, y, detail2, mode);
    CreateCrossingEvent(display, start, EnterNotify, x, y, detail1, mode);
}
    
/*
 *----------------------------------------------------------------------
 * CreateCrossingOutEvents
 *
 *	Create crossing events up the tree.
 *----------------------------------------------------------------------
 */
static void
CreateCrossingOutEvents(Display *display, HWND start, HWND stop,
			int x, int y, Bool linear, int mode)
{
    char className[256];
    HWND curr;
    int detail1, detail2;

    if (start == stop) {
	return;
    }

    if (linear) {
	detail1 = NotifyAncestor;
	detail2 = NotifyVirtual;
    } else {
	detail1 = NotifyNonlinear;
	detail2 = NotifyNonlinearVirtual;
    }

    CreateCrossingEvent(display, start, LeaveNotify, x, y, detail1, mode);
    GetClassName(start, className, sizeof(className));
    if (strcmp(className, "TopLevel") == 0) {
	return;
    }

    for (curr = GetParent(start); curr != stop; curr = GetParent(curr)) {
	CreateCrossingEvent(display, curr, LeaveNotify, x, y, detail2, mode);
	GetClassName(curr, className, sizeof(className));
	if (strcmp(className, "TopLevel") == 0) {
	    break;
	}
    }
}


/*
 *----------------------------------------------------------------------
 * Lx_GenerateCrossingEvents --
 *
 *	Generate the events that go along with a crossing change.  This
 *	code is very similar to that in callback.c which generates
 *	EnterNotify and LeaveNotify events.
 *----------------------------------------------------------------------
 */
static void
Lx_GenerateCrossingEvents(Display *display, HWND hprevwnd, HWND hcurwnd,
		       int x, int y, int mode)
{
    HWND hwnd1, hwnd2;
    int nomem = 0;
    char className[256];
    int nmWin;
    static int szHwndChain = 0;
    static HWND *hwndChain = NULL;

    if (hprevwnd == hcurwnd) {
	return;
    }
    if (hcurwnd == NULL) {
	hcurwnd = HWND_DESKTOP;
    }

#if 0
    printf("\nGenerateCrossingEvents: prev=%x, curr=%x\n", hprevwnd, hcurwnd);
#endif

    if (hcurwnd == HWND_DESKTOP) {
	if (hprevwnd != HWND_DESKTOP) {
	    CreateCrossingOutEvents(display, hprevwnd, HWND_DESKTOP, x, y,
				    True, mode);
	}
	return;
    }

    if (hprevwnd == HWND_DESKTOP) {
	CreateCrossingInEvents(display, hprevwnd, hcurwnd, x, y, True, mode);
	return;
    }

    /* Save a chain so we can check for a common ancestor later */
    nmWin = 0;
    hwnd1 = hcurwnd;
    while (1) {
	if (nmWin >= szHwndChain) {
	    HWND *newChain;
	    int szNewChain;

	    szNewChain = szHwndChain + 20;
	    newChain = (HWND *) ckalloc(sizeof(HWND) * szNewChain);
	    if (newChain == NULL) {
		nomem = 1;
	    } else {
		if (hwndChain) {
		    memcpy(newChain, hwndChain, szHwndChain * sizeof(HWND));
		    ckfree(hwndChain);
		}
		szHwndChain = szNewChain;
		hwndChain = newChain;
	    }
	}
	hwndChain[nmWin] = hwnd1;
	nmWin++;
	if (hwnd1 == HWND_DESKTOP || hwnd1 == hprevwnd) {
	    break;
	}
	GetClassName(hwnd1, className, sizeof(className));
	if (strcmp(className, "TopLevel") == 0) {
	    hwnd1 = HWND_DESKTOP;
	} else {
	    hwnd1 = GetParent(hwnd1);
	}
    }

    for (hwnd2 = hprevwnd; hwnd2 != HWND_DESKTOP && hwnd2 != hcurwnd;
	 hwnd2 = GetParent(hwnd2))
    {
	GetClassName(hwnd2, className, sizeof(className));
	if (strcmp(className, "TopLevel") == 0) {
	    hwnd2 = HWND_DESKTOP;
	    break;
	}
    }

    if (hwnd1 == HWND_DESKTOP && hwnd2 == HWND_DESKTOP) {
	/* This is a crossing between two non-ancestorial windows.
	 * We still need to generate the NotifyNonlinearVirtual
	 * Leave and Enter events
	 */
	HWND hwndCommon;
	int i;

	/* Find a common ancestor */
	for (hwndCommon = NULL, hwnd2 = hprevwnd;
	     hwnd2 != HWND_DESKTOP; hwnd2 = GetParent(hwnd2))
	{
	    GetClassName(hwnd2, className, sizeof(className));
	    if (strcmp(className, "TopLevel") == 0) {
		break;
	    }
	    for (i = 0; i < nmWin; i++) {
		if (hwndChain[i] == hwnd2) {
		    hwndCommon = hwnd2;
		    break;
		}
	    }
	    if (hwndCommon) {
		break;
	    }
	}
	if (! hwndCommon) {
	    hwndCommon = HWND_DESKTOP;
	}

	CreateCrossingOutEvents(display, hprevwnd, hwndCommon, x, y, False, mode);
	CreateCrossingInEvents(display, hwndCommon, hcurwnd, x, y, False, mode);
	return;
    }

    /* Check if the current window is higher in the tree */
    if (hwnd1 == HWND_DESKTOP) {
	CreateCrossingOutEvents(display, hprevwnd, hcurwnd, x, y, True, mode);
	CreateCrossingEvent(display, hcurwnd, EnterNotify, x, y, NotifyInferior, mode);
	return;
    }

    /* Check if the previous window is higher in the tree */
    if (hwnd2 == HWND_DESKTOP) {
	CreateCrossingEvent(display, hprevwnd, LeaveNotify, x, y, NotifyInferior, mode);
	CreateCrossingInEvents(display, hprevwnd, hcurwnd, x, y, True, mode);
	return;
    }

    /* Should never get here */
}

/*
 *----------------------------------------------------------------------
 *
 * Lx_EnterLeave --
 *
 *	This routine generates EnterNotify and LeaveNotify events.
 *	Normally, it is called when the mouse has been moved, but it
 *	is also called when a window moves to check if the window has
 *	moved from being under the cursor to outside the cursor.
 *
 * Return:
 *	None
 *
 * Side Effects:
 *	X events may be added to the X event queue
 *
 *----------------------------------------------------------------------
 */
void
Lx_EnterLeave(HWND hcurwnd, int x, int y, int windowMoved)
{
    DWORD mytask = GetCurrentThreadId();
    RECT rect;
    HWND hgrab, hwnd;
    POINT pt;
    static int szHwndChain = 0;
    static HWND *hwndChain = NULL;
    Display *display;

    display = XOpenDisplay(NULL);

    if (windowMoved) {
	if (hcurwnd != pointerWindow) {
	    return;
	}
	hwnd = XWindowFromPoint(display, pointerX, pointerY);
	if (hwnd == pointerWindow) {
	    return;
	}
	hcurwnd = hwnd;
	x = pointerX;
	y = pointerY;
    }

    if (pointerX == x && pointerY == y && hcurwnd == pointerWindow) {
	return;
    }

    hgrab = GetCapture();
    if (hgrab) {
	/* Take care of the case where a button is pressed in a window and the
	 * mouse is moved outside the window (while the mouse is captured)
	 */
	GetWindowRect(hgrab, &rect);
	if (pointerWindow != hgrab) {
	    if (x >= rect.left && x <= rect.right &&
		y >= rect.top && y <= rect.bottom)
	    {
		CreateCrossingEvent(display, hgrab, EnterNotify,
				    x, y, NotifyNonlinear, NotifyNormal);
		hcurwnd = hgrab;
	    } else {
		hcurwnd = pointerWindow;
	    }
	} else if (pointerWindow == hcurwnd) {
	    if (x < rect.left || x > rect.right ||
		y < rect.top || y > rect.bottom)
	    {
		if (GetWindowThreadProcessId(pointerWindow, NULL) == mytask) {
		    CreateCrossingEvent(display, pointerWindow, LeaveNotify,
					x, y, NotifyNonlinear, NotifyNormal);
		    hcurwnd = GetDesktopWindow();
		}
	    }
	}
    } else {
	if (hcurwnd != HWND_DESKTOP) {
	    pt.x = x;
	    pt.y = y;
	    ScreenToClient(hcurwnd, &pt);
	    if (pt.x < 0 || pt.y < 0) {
		hcurwnd = HWND_DESKTOP;
	    }
	}
	Lx_GenerateCrossingEvents(display, pointerWindow, hcurwnd, x, y, NotifyNormal);
    }

    pointerWindow = hcurwnd;
    pointerX = x;
    pointerY = y;

    return;
}

/*
 *----------------------------------------------------------------------
 *
 * DeliverEnterLeave --
 * 
 * 	This is a callback for SetWindowsHookEx(WH_MOUSE, ...).
 *
 *----------------------------------------------------------------------
 */

LRESULT CALLBACK
DeliverEnterLeave(int code, WPARAM wParam, LPARAM lParam)
{
    MOUSEHOOKSTRUCT * lpmhs = (MOUSEHOOKSTRUCT *)lParam;
    POINT pt = lpmhs->pt;
    HWND hcurwnd = lpmhs->hwnd;

    if (!(code < 0 || (wParam != WM_MOUSEMOVE && wParam != WM_NCMOUSEMOVE))) {
	Lx_EnterLeave(hcurwnd, pt.x, pt.y, 0 /* windowMoved */);
    }

    return CallNextHookEx(hNextHook, code, wParam, lParam);
}

/*
 *----------------------------------------------------------------------
 * XGrabPointer --
 *
 *	Grab the pointer and generate any Enter and Leave events that
 *	are needed.
 *----------------------------------------------------------------------
 */
int 
XGrabPointer(Display *display, Window grab_window, Bool owner_event,
	     unsigned long event_mask, int pointer_mode, int keybord_mode,
	     Window confine_to, Cursor cursor, Time time)
{
    Window prevWindow;

    display->request++;

    if (GetCapture()) {
	ReleaseCapture();
    }
    
    if (lx_PointerGrabbed) {
	prevWindow = lx_GrabbedWindow;
    } else {
	prevWindow = pointerWindow;
    }

    lx_GrabbedWindow = grab_window;
    Lx_GenerateCrossingEvents(display, prevWindow, lx_GrabbedWindow,
			   pointerX, pointerY, NotifyGrab);
    lx_PointerGrabbed = 1;

    return Success;
}

/*
 *----------------------------------------------------------------------
 * XUngrabPointer --
 *
 *	Ungrab the pointer and generate any Enter and Leave events that
 *	are needed.
 *----------------------------------------------------------------------
 */
void
XUngrabPointer(Display *display, Time time)
{
    display->request++;

    if (lx_PointerGrabbed) {
	Lx_GenerateCrossingEvents(display, lx_GrabbedWindow, pointerWindow,
			       pointerX, pointerY, NotifyUngrab);
    }
    lx_PointerGrabbed = 0;
    lx_GrabbedWindow = None;
    ReleaseCapture();
}

/*
 *----------------------------------------------------------------------
 *
 * SetHookDeliverEL --
 *
 *	Sets the next hook to be called after the Windows hook 
 *	for WH_MOUSE events has been installed.
 *
 * Results:
 *	None
 *
 * Side Effects:
 *	None
 *
 *----------------------------------------------------------------------
 */
void
SetHookDeliverEL(HHOOK handle)
{
    hNextHook = handle;
}


/*
 *----------------------------------------------------------------------
 *
 * CreateFocusEvent -- 
 *
 *	Actually create an FocusIn or FocusOut event and add it to the
 *	X event queue.  Post a message to make sure that the
 *	GetMessage() call will know that something is waiting
 *	in the X event queue.
 *
 * Results:
 *	None
 *
 * Side Effects:
 *	None
 *
 *----------------------------------------------------------------------
 */
static void
CreateFocusEvent(Display *display, HWND hwnd, int type, int detail, int mode)
{
    XEvent e;

    if (! Lx_CheckSendMessage(hwnd, type)) {
	return;
    }

    e.type = type;
    e.xany.window = (Window) hwnd;
    e.xany.serial = display->request;
    e.xany.send_event = False;
    e.xany.display = display;
    e.xfocus.mode = mode;
    e.xfocus.detail = detail;
    Lx_PutEvent(display, &e);
    PostMessage(hwnd, EVENTADDED, 0, 0L);
    
#if 0
    DebugPrintEvent(hwnd, type, 0, 0, detail, mode, 0, e.xany.serial);
#endif

    if (type == FocusIn &&
	detail != NotifyNonlinearVirtual && detail != NotifyVirtual)
    {
	/* printf("Calling SetFocus(%04x)\n", hwnd); */
	SetFocus(hwnd);
    }
}

/*
 *----------------------------------------------------------------------
 * CreateFocusAncestorEvents --
 *
 *	Recursively create focus events until we hit the stop window
 *----------------------------------------------------------------------
 */
void
CreateFocusAncestorEvents(Display *display, HWND stop, HWND start, int type,
			  int mode)
{
    char className[256];

    if (start == stop) {
	return;
    }
    if (start == HWND_DESKTOP) {
	return;
    }

    GetClassName(start, className, sizeof(className));
    if (strcmp(className, "TopLevel") == 0) {
	CreateFocusEvent(display, start, FocusIn, type, mode);
	return;
    }

    CreateFocusAncestorEvents(display, GetParent(start), stop, type, mode);
    CreateFocusEvent(display, start, FocusIn, type, mode);
}

/*
 *----------------------------------------------------------------------
 * CreateFocusInEvents
 *
 *	Create focus in events for the given range
 *----------------------------------------------------------------------
 */
void
CreateFocusInEvents(Display *display, HWND stop, HWND start, Bool linear,
		    int mode)
{
    int detail1, detail2;
    char className[256];

    if (start == stop) {
	return;
    }

    if (linear) {
	detail1 = NotifyAncestor;
	detail2 = NotifyVirtual;
    } else {
	detail1 = NotifyNonlinear;
	detail2 = NotifyNonlinearVirtual;
    }

    GetClassName(start, className, sizeof(className));
    if (strcmp(className, "TopLevel") == 0) {
	CreateFocusEvent(display, start, FocusIn, detail1, mode);
	return;
    }

    CreateFocusAncestorEvents(display, stop, GetParent(start), detail2, mode);
    CreateFocusEvent(display, start, FocusIn, detail1, mode);
}
    
/*
 *----------------------------------------------------------------------
 * CreateFocusOutEvents
 *
 *	Create focus events up the tree.
 *----------------------------------------------------------------------
 */
void
CreateFocusOutEvents(Display *display, HWND start, HWND stop, Bool linear,
		     int mode)
{
    char className[256];
    HWND curr;
    int detail1, detail2;

    if (start == stop) {
	return;
    }

    if (linear) {
	detail1 = NotifyInferior;
	detail2 = NotifyVirtual;
    } else {
	detail1 = NotifyNonlinear;
	detail2 = NotifyNonlinearVirtual;
    }

    CreateFocusEvent(display, start, FocusOut, detail1, mode);
    GetClassName(start, className, sizeof(className));
    if (strcmp(className, "TopLevel") == 0) {
	return;
    }

    for (curr = GetParent(start); curr != stop; curr = GetParent(curr))
    {
	CreateFocusEvent(display, curr, FocusOut, detail2, mode);
	GetClassName(curr, className, sizeof(className));
	if (strcmp(className, "TopLevel") == 0) {
	    break;
	}
    }
}


/*
 *----------------------------------------------------------------------
 * Lx_GenerateFocusEvents --
 *
 *	Generate the events that go along with a focus change.  This
 *	code is very similar to that in callback.c which generates
 *	FocusIn and FocusOut events.
 *----------------------------------------------------------------------
 */
void
Lx_GenerateFocusEvents(Display *display, HWND hprevwnd, HWND hcurwnd, int mode)
{
    HWND hwnd1, hwnd2;
    int nomem = 0;
    char className[256];
    int nmWin;
    static int szHwndChain = 0;
    static HWND *hwndChain = NULL;

    if (hprevwnd == hcurwnd) {
	return;
    }
    if (hcurwnd == NULL) {
	hcurwnd = HWND_DESKTOP;
    }

#if 0
    printf("\nGenerateFocusEvents: prev=%x, curr=%x\n", hprevwnd, hcurwnd);
#endif

    if (hcurwnd == HWND_DESKTOP) {
	if (hprevwnd != HWND_DESKTOP) {
	    CreateFocusOutEvents(display, hprevwnd, HWND_DESKTOP, True, mode);
	}
	return;
    }

    if (hprevwnd == HWND_DESKTOP) {
	CreateFocusInEvents(display, hprevwnd, hcurwnd, True, mode);
	return;
    }

    /* Save a chain so we can check for a common ancestor later */
    nmWin = 0;
    hwnd1 = hcurwnd;
    while (1) {
	if (nmWin >= szHwndChain) {
	    HWND *newChain;
	    int szNewChain;

	    szNewChain = szHwndChain + 20;
	    newChain = (HWND *) ckalloc(sizeof(HWND) * szNewChain);
	    if (newChain == NULL) {
		nomem = 1;
	    } else {
		if (hwndChain) {
		    memcpy(newChain, hwndChain, szHwndChain * sizeof(HWND));
		    ckfree(hwndChain);
		}
		szHwndChain = szNewChain;
		hwndChain = newChain;
	    }
	}
	hwndChain[nmWin] = hwnd1;
	nmWin++;
	if (hwnd1 == HWND_DESKTOP || hwnd1 == hprevwnd) {
	    break;
	}
	GetClassName(hwnd1, className, sizeof(className));
	if (strcmp(className, "TopLevel") == 0) {
	    hwnd1 = HWND_DESKTOP;
	} else {
	    hwnd1 = GetParent(hwnd1);
	}
    }

    for (hwnd2 = hprevwnd; hwnd2 != HWND_DESKTOP && hwnd2 != hcurwnd;
	 hwnd2 = GetParent(hwnd2))
    {
	GetClassName(hwnd2, className, sizeof(className));
	if (strcmp(className, "TopLevel") == 0) {
	    hwnd2 = HWND_DESKTOP;
	    break;
	}
    }

    if (hwnd1 == HWND_DESKTOP && hwnd2 == HWND_DESKTOP) {
	/* This is a crossing between two non-ancestorial windows.
	 * We still need to generate the NotifyNonlinearVirtual
	 * FocusOut and FocusIn events
	 */
	HWND hwndCommon;
	int i;

	/* Find a common ancestor */
	for (hwndCommon = NULL, hwnd2 = hprevwnd;
	     hwnd2 != HWND_DESKTOP; hwnd2 = GetParent(hwnd2))
	{
	    for (i = 0; i < nmWin; i++) {
		if (hwndChain[i] == hwnd2) {
		    hwndCommon = hwnd2;
		    break;
		}
	    }
	    if (hwndCommon) {
		break;
	    }
	}
	if (! hwndCommon) {
	    hwndCommon = HWND_DESKTOP;
	}

	CreateFocusOutEvents(display, hprevwnd, hwndCommon, False, mode);
	CreateFocusInEvents(display, hwndCommon, hcurwnd, False, mode);
	return;
    }

    /* Check if the current window is higher in the tree */
    if (hwnd1 == HWND_DESKTOP) {
	CreateFocusOutEvents(display, hprevwnd, hcurwnd, True, mode);
	CreateFocusEvent(display, hcurwnd, FocusIn, NotifyInferior, mode);
	return;
    }

    /* Check if the previous window is higher in the tree */
    if (hwnd2 == HWND_DESKTOP) {
	CreateFocusEvent(display, hprevwnd, FocusOut, NotifyInferior, mode);
	CreateFocusInEvents(display, hprevwnd, hcurwnd, True, mode);
	return;
    }

    /* Should never get here */
}

/*
 *----------------------------------------------------------------------
 * Lx_CheckAndResetFocus --
 *
 *	If a window that had the focus is destroyed, we need to put out
 *	some FocusOut events before the window goes away for good.
 *----------------------------------------------------------------------
 */
void
Lx_CheckAndResetFocus(Display *display, HWND destroy)
{
    HWND curr;
    HWND newFocus;
    HWND focus = display->focusWin;
    char className[256];

    for (curr = focus; curr != HWND_DESKTOP; curr = GetParent(curr))
    {
	/* This is always reverting to an ancestor, not the correct
	 * behavior necessarily */
	if (curr == destroy) {
	    newFocus = GetParent(curr);
	    CreateFocusOutEvents(display, focus, newFocus, True, NotifyNormal);
	    display->focusWin = newFocus;
	    break;
	}

	GetClassName(curr, className, sizeof(className));
	if (strcmp(className, "TopLevel") == 0) {
	    break;
	}
    }
}

/*
 *----------------------------------------------------------------------
 * XSetInputFocus --
 *
 *	Set the keyboard focus window
 *
 * Notes:
 *	If the focus window is None, then all keyboard message are
 *	dropped until another window gets keyboard focus.
 *
 *	revert_to can be one of RevertToParent, RevertToPointer, or
 *	RevertToNone.  This is currently ignored.
 *
 *	time can be either CurrentTime or a time when the focus should
 *	be changed.   For now, focus changes always occur immediately.
 *----------------------------------------------------------------------
 */
void
XSetInputFocus(Display *display, Window focus, int revert_to, Time time)
{
#if 0
    printf("XSetInputFocus Window=%04x\n", focus);
#endif
    if (focus == None) {
	Lx_GenerateFocusEvents(display, display->focusWin, HWND_DESKTOP, NotifyNormal);
	display->focusWin = HWND_DESKTOP;
    } else {
	Lx_GenerateFocusEvents(display, display->focusWin, focus, NotifyNormal);
	display->focusWin = focus;
    }
}

/*
 *----------------------------------------------------------------------
 * XGetInputFocus --
 *
 *	Get the keyboard focus window
 *----------------------------------------------------------------------
 */
void
XGetInputFocus(Display *display, Window *focus_return, int *revert_to_return)
{
    *focus_return = display->focusWin;
    *revert_to_return = RevertToParent;
}


/*
 *----------------------------------------------------------------------
 * XGrabKeyboard --
 *
 *	Grab the keyboard and generate FocusIn and FocusOut events
 *----------------------------------------------------------------------
 */
int
XGrabKeyboard(Display *display, Window grab_window, Bool owner_events,
	      int pointer_mode, int keyboard_mode, Time time)
{
    Window prevWindow;

    display->request++;

    if (keyboardGrabbed) {
	prevWindow = keyboardWindow;
    } else {
	prevWindow = pointerWindow;
    }

    keyboardWindow = grab_window;
    Lx_GenerateFocusEvents(display, prevWindow, keyboardWindow, NotifyGrab);
    keyboardGrabbed = 1;

    return Success;
}

/*
 *----------------------------------------------------------------------
 * XUngrabKeyboard --
 *
 *	Ungrab the keyboard and generate FocusIn and FocusOut events
 *----------------------------------------------------------------------
 */
void
XUngrabKeyboard(Display *display, Time time)
{
    display->request++;

    if (keyboardGrabbed) {
	Lx_GenerateFocusEvents(display, keyboardWindow, pointerWindow,
			    NotifyUngrab);
    }
    keyboardWindow = None;
    keyboardGrabbed = 0;
}	
