/*
 * msg2evt.c --
 *
 * Copyright (c) 1994 Software Research Associates, Inc.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Software Research Associates not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  Software Research
 * Associates makes no representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or implied
 * warranty.
 */

#include "xlibInt.h" 
#ifdef KANJI
#include <mbctype.h>
#endif /* KANJI */

static POINT mousePt;

/*
 *----------------------------------------------------------------------
 *
 * Lx_MsgGetState --
 *
 *	Get the state of the modifier keys on the keyboard and,
 *	if desired, the mouse.
 *	'flag' needs to be non-zero for the mouse state to be
 *	included in the current mouse/keyboard state.
 *
 * Results:
 *	A short with the bits set for which modifier keys/buttons
 *	are currently pressed.
 *
 * Notes:
 *	Keep these consistent with the modifier keys in xkeysym.c
 *	routine XGetModifierMapping
 *
 * Side Effects:
 *	None
 *
 *----------------------------------------------------------------------
 */
short int
Lx_MsgGetState(int flag)
{
    int state = 0;

    if (GetKeyState(VK_SHIFT) & 0x8000)   state |= ShiftMask;
    if (GetKeyState(VK_CAPITAL) & 0x0001) state |= LockMask; 
    if (GetKeyState(VK_CONTROL) & 0x8000) state |= ControlMask; 
    if (GetKeyState(VK_MENU) & 0x8000)    state |= Mod2Mask;
    if (GetKeyState(VK_NUMLOCK) & 0x0001) state |= Mod3Mask; 
    if (GetKeyState(VK_SCROLL) & 0x0001)  state |= Mod4Mask; 
    if (flag) return (short) state;

    if (GetKeyState(VK_LBUTTON) & 0x8000) state |= Button1Mask;
    if (GetKeyState(VK_MBUTTON) & 0x8000) state |= Button3Mask; 
    if (GetKeyState(VK_RBUTTON) & 0x8000) state |= 
	(GetKeyState(VK_MENU) & 0x8000)?Button3Mask:Button2Mask;
    return (short) state;
}

/*
 *----------------------------------------------------------------------
 *
 * Lx_MsgToEvent --
 *
 *	This routine generates most X events.  It converts Windows
 *	messages received by GetMessage() into X events if possible.
 *
 *	The serial number is placed in the lParam field for 
 *	CONFIGURENOTIFY messages because we need very accurate events.
 *	Really, all events should get their serial number when they
 *	are posted as opposed to here, but most don't matter that much.
 *
 * Results:
 *	0 if the message was handled, 1 if it was not.
 *	
 * Side Effects:
 *	Fills in the applicable fields of the passed in event if
 *	the message is handled.
 *
 *----------------------------------------------------------------------
 */
int					
Lx_MsgToEvent(Display *display, MSG *mPtr, XEvent *ePtr)
{
    POINT sPt;
    DWORD dw;
    char className[256];
    static mbflag = 0;
    Lx_WindowEntry *winEntry;
    XSetWindowAttributes *pXSWAttrs;

    ePtr->xany.window = (Window) mPtr->hwnd;
    ePtr->xany.serial = display->request;
    ePtr->xany.send_event = False;
    ePtr->xany.display = display;

    switch(mPtr->message) {
      case SOCKETNOTIFY:
	ePtr->type = SocketNotify;
	ePtr->xsocket.socket = mPtr->wParam;
	ePtr->xsocket.message = mPtr->lParam;
	break;

      case INPUTNOTIFY:
	if (mPtr->hwnd == Tknt_GetPostWindow()) {
	    ePtr->type = InputNotify;
	    ePtr->xinput.io_type = InputTypeConsole;
	    ePtr->xinput.id = mPtr->wParam;
	    ePtr->xinput.data = mPtr->lParam;
	} else {
	    return 0;
	}
	break;
		
      case PIPENOTIFY:
	ePtr->type = InputNotify;
	ePtr->xinput.io_type = InputTypePipe;
	ePtr->xinput.id = mPtr->wParam;
	ePtr->xinput.data = mPtr->lParam;
	break;

      case NAMEDPIPENOTIFY:
	ePtr->type = InputNotify;
	ePtr->xinput.io_type = InputTypeNamedPipe;
	ePtr->xinput.id = mPtr->wParam;
	ePtr->xinput.data = mPtr->lParam;
	break;
		
      case TIMERNOTIFY:
	KillTimer(Tknt_GetPostWindow(), mPtr->wParam);
	ePtr->type = TimerNotify; 
	ePtr->xtimer.id = mPtr->wParam;
	break;

      case ACTIVATEAPP:
    	ePtr->type = ActivateApp;
	break;
				 
      case CREATENOTIFY:
	ePtr->type = CreateNotify;
	GetClassName(mPtr->hwnd, className, sizeof(className));
	if (strcmp(className, "TopLevel") == 0) {
	    ePtr->xcreatewindow.parent = (Window) HWND_DESKTOP;
	} else {
	    ePtr->xcreatewindow.parent = (Window) GetParent(mPtr->hwnd);
	}
	ePtr->xcreatewindow.window = mPtr->hwnd;
	ePtr->xcreatewindow.x = ((CREATESTRUCT *) mPtr->lParam)->x;
	ePtr->xcreatewindow.y = ((CREATESTRUCT *) mPtr->lParam)->y;
	ePtr->xcreatewindow.width = ((CREATESTRUCT *) mPtr->lParam)->cx;
	ePtr->xcreatewindow.height = ((CREATESTRUCT *) mPtr->lParam)->cy;
	ePtr->xcreatewindow.border_width = 1;
	ePtr->xcreatewindow.override_redirect = False;
	break;

      case DESTROYNOTIFY:
	ePtr->type = DestroyNotify;
	ePtr->xdestroywindow.event =
	    ePtr->xdestroywindow.window = (Window) mPtr->wParam; 
	break;

      case WM_MOUSEMOVE:   			/* MotionNotify */
	ePtr->type = MotionNotify;
	ePtr->xmotion.root = HWND_DESKTOP;	  
	ePtr->xmotion.subwindow = None;
	ePtr->xmotion.time = mPtr->time;
	ePtr->xmotion.x = sPt.x = (int) ((short) LOWORD(mPtr->lParam));
	ePtr->xmotion.y = sPt.y = (int) ((short) HIWORD(mPtr->lParam));
	ClientToScreen(mPtr->hwnd, &sPt);
	if (sPt.x == mousePt.x && sPt.y == mousePt.y) {
	    return 0;
	}
	ePtr->xmotion.x_root = mousePt.x = sPt.x;
	ePtr->xmotion.y_root = mousePt.y = sPt.y;
	ePtr->xmotion.state = Lx_MsgGetState(0);
	ePtr->xmotion.is_hint = NotifyNormal;
	ePtr->xmotion.same_screen = True;
	break;
        
#if 0
	/* These should never be seen here */
      case FOCUSIN:			/* FocusIn Event */
	ePtr->type = FocusIn;
	ePtr->xfocus.mode = NotifyNormal;
	break;

      case FOCUSOUT:			/* FocusOut Event */
	ePtr->type = FocusOut;
	ePtr->xfocus.mode = NotifyNormal;
	break;
#endif

      case WM_CHAR: 
#ifdef KANJI    
	if(mbflag == 0 && _ismbblead((unsigned short)mPtr->wParam)) {
	    ePtr->xany.flag = 0x1234;
	    mbflag = 1;
	} else if(mbflag == 1 && _ismbbtrail((unsigned short)mPtr->wParam)) {
	    ePtr->xany.flag = 0x5678;
	    mbflag = 0;
	} else {
	    mbflag = 0;
	    ePtr->xany.flag = 0xABCD;
	}       
#else 	
	mbflag = 0;
	ePtr->xany.flag = 0xABCD;
#endif /* KANJI */

      case WM_KEYDOWN:			/* KeyPress Event */
      case WM_SYSKEYDOWN:
	ePtr->type = KeyPress;
	goto common_keyevent;

      case WM_KEYUP:			/* KeyRelease Event */
      case WM_SYSKEYUP:
	ePtr->type = KeyRelease;
      common_keyevent:
	ePtr->xkey.window = ePtr->xkey.subwindow = (Window) mPtr->hwnd;
	ePtr->xkey.root = HWND_DESKTOP; 	
	ePtr->xkey.time = mPtr->time;
	dw = GetMessagePos();
	sPt.x = (int) ((short) LOWORD(dw));
	sPt.y = (int) ((short) HIWORD(dw));
	ePtr->xkey.x_root = sPt.x;
	ePtr->xkey.y_root = sPt.y;
	ScreenToClient(mPtr->hwnd, &sPt);
	ePtr->xkey.x = sPt.x;
	ePtr->xkey.y = sPt.y; 
	ePtr->xkey.state = Lx_MsgGetState(0);
	ePtr->xkey.keycode = mPtr->wParam;
	ePtr->xkey.same_screen = True;
	break;

      case MAPNOTIFY:			/* MapNotify, UnmapNotify Event */
	ePtr->type = MapNotify;
	ePtr->xmap.event = ePtr->xmap.window = (Window) mPtr->hwnd;
	ePtr->xmap.override_redirect= False;
	break;

      case UNMAPNOTIFY:
	ePtr->type = UnmapNotify;
	ePtr->xunmap.event = ePtr->xmap.window = (Window) mPtr->hwnd;
	ePtr->xunmap.from_configure = False;
	break;

      case CLOSEWINMESSAGE:
    	ePtr->type = ClientMessage;
	ePtr->xclient.message_type = XInternAtom(display, "WM_PROTOCOLS", False);
	ePtr->xclient.format = 32;
	ePtr->xclient.data.l[0] = XInternAtom(display, "WM_DELETE_WINDOW", False);
	break;

      case CLIENTMESSAGE:
    	ePtr->type = ClientMessage;
	ePtr->xclient.message_type = mPtr->wParam;
	ePtr->xclient.format = 32;
	ePtr->xclient.data.l[0] = mPtr->lParam;
	break;

      case SYSCOLORNOTIFY:
	ePtr->type = SyscolorNotify;
	break;

      case DUMMY_NOTIFY:
	ePtr->type = DummyNotify;
	break;

      default:
	return 0;
    }

    if (Lx_CheckSendMessage(ePtr->xany.window, ePtr->type)) {
	return 1;
    } else {
	return 0;
    }
}

/*
 *----------------------------------------------------------------------
 * XQueryPointer --
 *
 *	Get the current mouse location
 *
 * Note:
 *	This does not accurately return child_out.  It should only
 *	return a value if *child_out is a child of 'window'.
 *----------------------------------------------------------------------
 */
Bool
XQueryPointer(Display *display, Window window,
	      Window *root_out, Window *child_out, 
	      int *root_x_out, int *root_y_out,
	      int *win_x_out, int *win_y_out,
	      unsigned int *mask_return)
{
    POINT sPt, currPt;

    display->request++;

    *root_out = HWND_DESKTOP;
    GetCursorPos(&currPt);
    *child_out = (Window) WindowFromPoint(currPt);
    *root_x_out = mousePt.x;
    *root_y_out = mousePt.y;
    sPt.x = mousePt.x;
    sPt.y = mousePt.y;
    ScreenToClient(window, &sPt);
    *win_x_out = sPt.x;
    *win_y_out = sPt.y;
    *mask_return = Lx_MsgGetState(0);

    return True;
}
