/*
 * wndproc.c --
 *
 * Copyright (c) 1994 Software Research Associates, Inc.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Software Research Associates not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  Software Research
 * Associates makes no representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or implied
 * warranty.
 */

#include "xlibInt.h" 

/*
 *----------------------------------------------------------------------
 * RegisterWidgetClass --
 *
 *	Register the window type "Widget" and sets its window
 *	procedure
 *
 * Results:
 *	None
 *
 * Side Effects:
 *	None
 *----------------------------------------------------------------------
 */
void
RegisterWidgetClass(HINSTANCE hInstance)
{
    WNDCLASS wndclass;

    wndclass.style	   = CS_HREDRAW  | CS_VREDRAW;
    wndclass.lpfnWndProc   = WidgetProc;
    wndclass.cbClsExtra    = 0;
    wndclass.cbWndExtra    = LAST_OFFSET;
    wndclass.hInstance     = hInstance;
    wndclass.hIcon	   = NULL;
    wndclass.hCursor       = NULL;
    wndclass.hbrBackground = NULL;
    wndclass.lpszMenuName  = NULL;
    wndclass.lpszClassName = "Widget";

    RegisterClass(&wndclass);
}

/*
 *----------------------------------------------------------------------
 * RegisterTopLevelClass -- 
 *
 *	Register the window type "TopLevel" and sets its window
 *	procedure
 *
 * Results:
 *	None
 *
 *----------------------------------------------------------------------
 */

void
RegisterTopLevelClass(HINSTANCE hInstance)
{
    WNDCLASS wndclass;

    wndclass.style	   = 0; /* CS_HREDRAW | CS_VREDRAW */
    wndclass.lpfnWndProc   = TopLevelProc;
    wndclass.cbClsExtra    = 0;
    wndclass.cbWndExtra    = LAST_OFFSET;
    wndclass.hInstance     = hInstance;
    wndclass.hIcon	   = NULL;
    wndclass.hCursor       = NULL;
    wndclass.hbrBackground = NULL;
    wndclass.lpszMenuName  = NULL;
    wndclass.lpszClassName = "TopLevel";

    RegisterClass(&wndclass);
}

/*
 *----------------------------------------------------------------------
 * SendActivate	--
 *
 *	Callback function for EnumChildWindows.
 *----------------------------------------------------------------------
 */
static BOOL 
SendActivate(HWND hwnd, LPARAM lParam)	
{
    SendMessage(hwnd, WM_ACTIVATE,  0, 0L);
    return 1;
}

/* 
 *----------------------------------------------------------------------
 *
 * DeleteProperty --
 * 
 *	Callback function for EnumProp
 *
 *----------------------------------------------------------------------
 */
static BOOL CALLBACK
DeleteProperty(HWND hwnd, LPCTSTR lpsz, HANDLE hData)	
{
    RemoveProp(hwnd, lpsz);
    GlobalFree(hData);
    return 1;
}

/*
 *----------------------------------------------------------------------
 *
 * WindowProc --
 *
 *	All Windows messages pass through this routine.
 *
 *----------------------------------------------------------------------
 */
int
WindowProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam, int toplevel)
{
    HGLOBAL hWork;
    Lx_WindowEntry *winEntry;
    XSetWindowAttributes *pXSWAttrs;
    XWindowChanges *pXWChanges;
    WINDOWPOS *lpwp;
    XEvent e;
    Display *display;
    char propName[8];
    char *ptr;
    HWND hparent;
    HDC hdc;
    BOOLEAN bChanged;
    HCURSOR hcs;
    int nc;
    POINT sPt;
    int retval;
    Lx_DrawableDCState state;
    static BOOLEAN flag;

    winEntry = Lx_GetWindowHashEntry(hwnd);
    if (winEntry) {
	pXWChanges = &winEntry->winChanges;
	pXSWAttrs = &winEntry->setWinAttrs;
	if (pXSWAttrs->native_widget) {
	    return 0;
	}
    }
    display = XOpenDisplay(NULL);

    retval = 0;
    switch(message) {
      case WM_SYSCOLORCHANGE:
	if (Lx_CheckSendMessage(hwnd, SyscolorNotify)) {
	    PostMessage(hwnd, SYSCOLORNOTIFY, 0, 0L);
	}
	break;

      case WM_SYSCOMMAND:
	switch (wParam & 0xFFF0) {
	  case SC_MINIMIZE:
	    if (Lx_CheckSendMessage(hwnd, UnmapNotify)) {
		PostMessage(hwnd, UNMAPNOTIFY, 0, 0L);
	    }
	    break;
	  case SC_MAXIMIZE:
	  case SC_RESTORE:
	    if (IsIconic(hwnd)) {
		if (Lx_CheckSendMessage(hwnd, MapNotify)) {
		    PostMessage(hwnd, MAPNOTIFY, 0, 0L);
		}
	    }
	    break;
	}
	break;

      case WM_CREATE:
	Lx_NewWindowHashEntry(hwnd);
	if (Lx_CheckSendMessage(GetParent(hwnd), CreateNotify)) {
	    PostMessage(hwnd, CREATENOTIFY, wParam, lParam);
	}
	retval = 1;
	break;

      case WM_QUERYNEWPALETTE:
	/* This next call causes the palette to be selected and realized */
	hdc = Lx_GetDrawableDC(hwnd, &state);
	Lx_ReleaseDrawableDC(hwnd, hdc, &state);

	retval = 1;
	break;

      case WM_PALETTECHANGED:
	if (lx_ScreenDepth == 8) {
	    Lx_UpdatePaletteColors(hwnd);
	}
	break;

      case WM_ACTIVATE:
	if (wParam != WA_INACTIVE) { 
#if 0
	    if (Lx_CheckSendMessage(hwnd, FocusIn)) {
		PostMessage(hwnd, FOCUSIN, wParam, 0L);
		printf("1. Posting FOCUSIN on %04x\n", hwnd);
	    }
	    if (Lx_CheckSendMessage(hwnd, Expose)) {
		PostMessage(hwnd, EXPOSE, 0, 0L);
	    }

 	    EnumChildWindows(hwnd, MakeProcInstance(SendActivate,
						    Tknt_GetAppInstance()), 0L);
#endif
	}
	break;

      case WM_SETFOCUS:
	for (hparent = display->focusWin;
	     hparent != HWND_DESKTOP && hparent != NULL;
	     hparent = GetParent(hparent))
	{
	    if (hparent == hwnd) {
		/* Focus is already in this toplevel */
		break;
	    }
	}

	if (display->focusWin != hwnd) {
	    Lx_GenerateFocusEvents(display, display->focusWin, hwnd,
				   NotifyNormal);
	    display->focusWin = hwnd;
	}
	break;
	
      case WM_KILLFOCUS:
	break;

      case WM_WINDOWPOSCHANGED:
	if (! toplevel) {
	    lpwp = (WINDOWPOS *)  lParam;
	    if ((lpwp->flags & SWP_NOMOVE) && (lpwp->flags & SWP_NOSIZE)) {
		break;
	    }

	    if (! (lpwp->flags & SWP_NOMOVE)) {
		pXWChanges->x = lpwp->x;
		pXWChanges->y = lpwp->y;
	    }
	    if (! (lpwp->flags & SWP_NOSIZE)) {
		pXWChanges->width = lpwp->cx;
		pXWChanges->height = lpwp->cy;
	    }

	    if (! Lx_CheckSendMessage(hwnd, ConfigureNotify)) {
		WINDOWPOS *wpos = (WINDOWPOS *) lParam;
		Lx_AddConfigureNotify(hwnd, wpos->hwndInsertAfter,
				      pXWChanges->x, pXWChanges->y,
				      pXWChanges->width, pXWChanges->height);
	    }
	    retval = 0;
	}
	break;
	
      case WM_WINDOWPOSCHANGING:
	if (toplevel) {
	    if (IsIconic(hwnd)) {
		break;
	    }

	    lpwp = (WINDOWPOS *) lParam;
	    if ((lpwp->flags & SWP_NOMOVE) && (lpwp->flags & SWP_NOSIZE)) {
		break;
	    }

	    bChanged = FALSE;
	    if (! (lpwp->flags & SWP_NOMOVE)) {
		pXWChanges->x = Lx_ReAdjustX(lpwp->x, pXSWAttrs->override_redirect);
		pXWChanges->y = Lx_ReAdjustY(lpwp->y, pXSWAttrs->override_redirect);
	    }
	    if (! (lpwp->flags & SWP_NOSIZE)) {
		pXWChanges->width =
		    Lx_ReAdjustWidth(lpwp->cx, pXSWAttrs->override_redirect);
		pXWChanges->height =
		    Lx_ReAdjustHeight(lpwp->cy, pXSWAttrs->override_redirect);
	    }

	    if (Lx_CheckSendMessage(hwnd, ConfigureNotify)) {
		WINDOWPOS *wpos = (WINDOWPOS *) lParam;

		Lx_AddConfigureNotify(hwnd, wpos->hwndInsertAfter,
				      pXWChanges->x, pXWChanges->y,
				      pXWChanges->width, pXWChanges->height);
	    }
	}
	break;

      case WM_GETMINMAXINFO:
	sprintf(propName, "#0x%0.4x", XA_WM_NORMAL_HINTS);
	hWork = GetProp(hwnd, propName);
    	if (hWork != NULL) {
	    XSizeHints *pHints;
	    MINMAXINFO *lpmmi;

	    lpmmi = (MINMAXINFO *) lParam;
	    ptr = ((char *) GlobalLock(hWork)) + sizeof(Atom);
	    pHints = (XSizeHints *) ptr;

	    if (!(pHints->flags & USSize)) {
	    	if (pHints->flags & PMinSize) {
	    	    lpmmi->ptMinTrackSize.x = 
	    	    	    Lx_AdjustWidth(pHints->min_width, pXSWAttrs->override_redirect);
		    lpmmi->ptMinTrackSize.y = 
			    Lx_AdjustHeight(pHints->min_height, pXSWAttrs->override_redirect);
	    	}
	    	if (pHints->flags & PMaxSize) {
	    	    lpmmi->ptMaxTrackSize.x = 
			    Lx_AdjustWidth(pHints->max_width, pXSWAttrs->override_redirect);
		    lpmmi->ptMaxTrackSize.y = 
			    Lx_AdjustHeight(pHints->max_height, pXSWAttrs->override_redirect);
	    	}
	    }
	    GlobalUnlock(hWork);
	}
	break;

      case WM_SIZE:
    	switch (wParam) {
    	case SIZE_MINIMIZED:
	    Lx_UpdateIcon(display, hwnd);
	    sprintf(propName, "#0x%0.4x", XA_WM_ICON_NAME);
	    hWork = GetProp(hwnd, propName);
	    break;
	case SIZE_MAXIMIZED:
	case SIZE_RESTORED: 
	    sprintf(propName, "#0x%0.4x", XA_WM_NAME);
	    hWork = GetProp(hwnd, propName);
	    break;
	default:
	    hWork = NULL;
	}
	if (hWork != NULL) {
	    char * p;

	    p = ((char *) GlobalLock(hWork)) + sizeof(Atom);
	    SetWindowText(hwnd, p);
	    GlobalUnlock(hWork);
	}
	retval = 1;
	break;

      case WM_MOUSEMOVE: 
	hcs = Lx_FindCursor(hwnd);
	if (hcs) {
	    SetCursor(hcs);
	}
	retval = 1; 
	break;

      case WM_ACTIVATEAPP:
    	if (wParam == 0) {
	    if (Lx_CheckSendMessage(hwnd, ActivateApp)) {
		PostMessage(hwnd, ACTIVATEAPP, 0, 0L);
	    }
	}
	retval = 1;
	break;

#if 1 /* in msg2evt.c */
      case WM_NCLBUTTONDOWN:
      case WM_NCLBUTTONDBLCLK:
      case WM_NCMBUTTONDOWN:
      case WM_NCMBUTTONDBLCLK:
      case WM_NCRBUTTONDOWN:
      case WM_NCRBUTTONDBLCLK:
	if (lx_PointerGrabbed) {
	    e.type = ButtonPress;
	    e.xbutton.state = Lx_MsgGetState(1);
	    nc = 1;
	    goto common_buttonevent;
	}
	break;

      case WM_NCLBUTTONUP:
      case WM_NCMBUTTONUP:
      case WM_NCRBUTTONUP:
	if (lx_PointerGrabbed) {
	    e.type = ButtonRelease;
	    if (GetCapture() != NULL) {
		ReleaseCapture();
	    }
	    e.xbutton.state = Lx_MsgGetState(1);
	    nc = 1;
	    goto common_buttonevent;
	}
	break;

      case WM_LBUTTONDOWN:
      case WM_MBUTTONDOWN:
      case WM_RBUTTONDOWN:
      case WM_LBUTTONDBLCLK:
      case WM_MBUTTONDBLCLK:
      case WM_RBUTTONDBLCLK:
	if (GetCapture() != NULL) {
	    ReleaseCapture();
	}
	SetCapture(hwnd);
	e.type = ButtonPress;
	e.xbutton.state = Lx_MsgGetState(1);
	nc = 0;
	goto common_buttonevent;

      case WM_LBUTTONUP:
      case WM_MBUTTONUP:
      case WM_RBUTTONUP:
	e.type = ButtonRelease;
	if (GetCapture() != NULL) {
	    ReleaseCapture();
	}
	e.xbutton.state = Lx_MsgGetState(1);
	nc = 0;

      common_buttonevent:
	if (Lx_CheckSendMessage(hwnd, e.type)) {
	    /* Note: the wParam may not be set if this event was generated
	     * in topproc.c
	     */
	    e.xany.window = hwnd;
	    e.xany.serial = display->request;
	    e.xany.send_event = False;
	    e.xany.display = display;
	    e.xbutton.root = HWND_DESKTOP;
	    e.xbutton.time = GetCurrentTime();
	    e.xbutton.same_screen = True;
	    if (! nc) {
		e.xbutton.x = sPt.x = (int) ((short) LOWORD(lParam));
		e.xbutton.y = sPt.y = (int) ((short) HIWORD(lParam));
		ClientToScreen(hwnd, &sPt);
		e.xbutton.x_root = sPt.x;
		e.xbutton.y_root = sPt.y;
	    } else {
		e.xbutton.x_root = sPt.x =
		    (int) ((short) LOWORD(lParam));
		e.xbutton.y_root = sPt.y =
		    (int) ((short) HIWORD(lParam));
		ScreenToClient(hwnd, &sPt);
		e.xbutton.x = sPt.x;
		e.xbutton.y = sPt.y;
	    }
	    switch (message) {
	      case WM_LBUTTONDOWN:
	      case WM_NCLBUTTONDOWN:
	      case WM_LBUTTONDBLCLK:
	      case WM_NCLBUTTONDBLCLK:
		e.xbutton.button = Button1;
		e.xbutton.state &= ~Button1Mask;
		break;
	      case WM_LBUTTONUP:
	      case WM_NCLBUTTONUP:
		e.xbutton.button = Button1;
		e.xbutton.state |= Button1Mask;
		break;
	      case WM_MBUTTONDOWN:
	      case WM_MBUTTONDBLCLK:
	      case WM_NCMBUTTONDBLCLK:
	      case WM_NCMBUTTONDOWN:
		e.xbutton.button = Button2;
		e.xbutton.state &= ~Button2Mask;
		break;

	      case WM_MBUTTONUP:
	      case WM_NCMBUTTONUP:
		e.xbutton.button = Button2;
		e.xbutton.state |= Button2Mask;
		break;
	      case WM_RBUTTONDOWN:
	      case WM_RBUTTONDBLCLK:
	      case WM_NCRBUTTONDOWN:
	      case WM_NCRBUTTONDBLCLK:
		e.xbutton.button = 
		    (GetKeyState(VK_MENU) & 0x8000)?Button3:Button2; break;
		e.xbutton.button = Button3;
		e.xbutton.state &= ~Button3Mask;
		break;

	      case WM_RBUTTONUP:
	      case WM_NCRBUTTONUP:
		e.xbutton.button = 
		    (GetKeyState(VK_MENU) & 0x8000)?Button3:Button2; break;
		e.xbutton.button = Button3;
		e.xbutton.state |= Button3Mask;
		break;
	    }

	    e.xbutton.state &= ~Mod2Mask;
	    Lx_PutEvent(display, &e);

	    switch (message) {
	      case WM_NCLBUTTONDOWN:
	      case WM_NCLBUTTONDBLCLK:
		e.xbutton.button = Button1;
		e.xbutton.state |= Button1Mask;
		Lx_PutEvent(display, &e);
		break;
	      case WM_NCMBUTTONDOWN:
	      case WM_NCMBUTTONDBLCLK:
		e.xbutton.button = Button2;
		e.xbutton.state |= Button2Mask;
		Lx_PutEvent(display, &e);
		break;
	      case WM_NCRBUTTONDOWN:
	      case WM_NCRBUTTONDBLCLK:
		e.xbutton.button = Button3;
		e.xbutton.state |= Button3Mask;
		Lx_PutEvent(display, &e);
		break;
	    }
	    PostMessage(hwnd, EVENTADDED, 0, 0L);
	}
	break;
#endif

      case WM_PAINT:
	if (pXSWAttrs->native_widget != 0) {
	    printf("Native widget expose: 0x%x\n", hwnd);
	    retval = 0;
	    break;
	}
	if (!IsIconic(hwnd)) {
	    HBRUSH hBrush;
	    COLORREF bg_cref;
	    HPALETTE hPrevPal, hPal;
	    PAINTSTRUCT ps;
	    int clear_bg;

	    bg_cref = LX_CONVERTPIXEL(pXSWAttrs->background_pixel);
	    clear_bg = pXSWAttrs->clear_on_expose;

	    hdc = BeginPaint(hwnd, &ps);
	    if (clear_bg) {
		hBrush = CreateSolidBrush(bg_cref);
		hPal = Lx_GetDrawablePalette(hwnd);
		if (hPal) {
		    hPrevPal = SelectPalette(hdc, hPal, FALSE);
		}

		FillRect(hdc, &ps.rcPaint, hBrush);
		if (hPal) {
		    SelectPalette(hdc, hPrevPal, FALSE);
		}
		DeleteObject(hBrush);
	    }

	    e.xany.window = (Window) hwnd;
	    e.xany.serial = display->request;
	    e.xany.send_event = False;
	    e.xany.display = display;
	    e.xexpose.type = Expose;
	    e.xexpose.x = (int) ps.rcPaint.left;
	    e.xexpose.y = (int) ps.rcPaint.top;
	    e.xexpose.width = (int) (ps.rcPaint.right - ps.rcPaint.left);
	    e.xexpose.height = (int) (ps.rcPaint.bottom - ps.rcPaint.top);

	    /* Really should find the true expose count.  To do this, we
	     * could peek at the messages before they are dispatched.
	     * If the events are put at the front of the queue instead
	     * of at the end, the count can be incremented for each PAINT
	     * message that we get for the window.
	     */
	    e.xexpose.count = 0;

	    Lx_PutEvent(display, &e);
	    PostMessage(hwnd, EVENTADDED, 0, 0L);

	    EndPaint(hwnd, &ps);
	    retval = 0;
	    break;
	}

      case WM_PAINTICON:
	display = XOpenDisplay(NULL);
	Lx_UpdateIcon(display, hwnd);
	retval = 1;
	break;

      case WM_CLOSE:
	if (Lx_CheckSendMessage(hwnd, ClientMessage)) {
	    PostMessage(hwnd, CLOSEWINMESSAGE, (WPARAM) 0, 0L);
	}
	retval = 1;
	break;

      case WM_DESTROY: 
	Lx_FreeWindowHashEntry(hwnd);
	EnumProps(hwnd, DeleteProperty);
	retval = 1;
	break;

      case WM_ERASEBKGND:
	if (IsIconic(hwnd)) {
	    retval = 1;
	}
	break;

      case WM_QUERYDRAGICON:
	display = XOpenDisplay(NULL);
	return ((LRESULT) (UINT) Lx_GetIcon(display, hwnd));

      case TKW_SELECTIONSAVE:
	/* This message gets posted by TbSetClipboard() in tkwinutil.c */
        flag = 1;
	retval = 1;
	break;

      case WM_DESTROYCLIPBOARD:
	if (flag) {
	    flag = 0;
	} else {
	    if (Lx_CheckSendMessage(hwnd, SelectionClear)) {
		Lx_GenerateSelectionClear(XOpenDisplay(NULL), hwnd);
	    }
	}
	break;
    }

    if (retval) {
#if 0
	if (pXSWAttrs->native_widget) {
	    return 0;
	}
#endif
	return retval;
    }
    return 0;
}

/*
 *----------------------------------------------------------------------
 *
 * TopLevelProc --
 *
 *	Windows messages for Toplevel windows pass through this
 *	routine.
 *
 *----------------------------------------------------------------------
 */
LRESULT CALLBACK
TopLevelProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    int ret;

    ret = WindowProc(hwnd, message, wParam, lParam, TRUE);
    if (ret) {
	return ret;
    }
    return DefWindowProc(hwnd, message, wParam, lParam);
}
 	
/*
 *----------------------------------------------------------------------
 * WidgetProc --
 *
 *	Process Windows messages for an individual widget here.
 *	Toplevel Windows messages do not pass through here.  If
 *	the message cannot be handled here, it is passed on to the
 *	Windows libraries.
 *
 * Side Effects:
 *	Depends on the message.  For some messages, the structure
 *	associated the window may be modified.  This structure
 *	maintains info about its width, height, and position.
 *----------------------------------------------------------------------
 */
LRESULT CALLBACK
WidgetProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    int ret;

    ret = WindowProc(hwnd, message, wParam, lParam, FALSE);
    if (ret) {
	return ret;
    }
    return DefWindowProc(hwnd, message, wParam, lParam);
}

/*
 *----------------------------------------------------------------------
 * ScrollbarProc --
 *
 *	Process Windows messages for a Microsoft scrollbar here.
 *
 * Side Effects:
 *	Depends on the message.  For some messages, the structure
 *	associated the window may be modified.  This structure
 *	maintains info about its width, height, and position.
 *----------------------------------------------------------------------
 */
LRESULT CALLBACK
ScrollbarProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    int ret;
    Lx_WindowEntry *winEntry;
    FARPROC defaultWinProc;

    winEntry = Lx_GetWindowHashEntry(hwnd);
    defaultWinProc = winEntry->defaultWinProc;

    ret = WindowProc(hwnd, message, wParam, lParam, FALSE);
    if (ret) {
	return ret;
    }
    return CallWindowProc(defaultWinProc, hwnd, message, wParam, lParam);
}

