/*
 * xatom.c --
 *
 * Copyright (c) 1994 Software Research Associates, Inc.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Software Research Associates not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  Software Research
 * Associates makes no representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or implied
 * warranty.
 */
 
#include "xlibInt.h" 

Atom
XInternAtom(Display *display, const char *atom_name, Bool exists)
{
    Atom atom;

    display->request++;
    if (exists == True) {
	return  GlobalFindAtom(atom_name);
    } else {
	if ((atom = GlobalFindAtom(atom_name)) != 0) {
	    return atom;
	} else {
	    return GlobalAddAtom(atom_name);
	}
    }
}

char *
XGetAtomName(Display *display, Atom atom)
{
    char buffer[256];
    char *p, *s;
    int len;

    display->request++;
    p = NULL;
    /* All Windows generated atoms are in the range of 0xC000 - 0xFFFF */
    if (atom < 0xC000) {
	switch (atom) {
	case XA_PRIMARY:
	    p = "PRIMARY";

	case XA_SECONDARY:
	    p = "SECONDARY";

	case XA_ARC:
	    p = "ARC";

	case XA_ATOM:
	    p = "ATOM";

	case XA_BITMAP:
	    p = "BITMAP";

	case XA_CARDINAL:
	    p = "CARDINAL";

	case XA_COLORMAP:
	    p = "COLORMAP";

	case XA_CURSOR:
	    p = "CURSOR";

	case XA_CUT_BUFFER0:
	    p = "CUT_BUFFER0";

	case XA_CUT_BUFFER1:
	    p = "CUT_BUFFER1";

	case XA_CUT_BUFFER2:
	    p = "CUT_BUFFER2";

	case XA_CUT_BUFFER3:
	    p = "CUT_BUFFER3";

	case XA_CUT_BUFFER4:
	    p = "CUT_BUFFER4";

	case XA_CUT_BUFFER5:
	    p = "CUT_BUFFER5";

	case XA_CUT_BUFFER6:
	    p = "CUT_BUFFER6";

	case XA_CUT_BUFFER7:
	    p = "CUT_BUFFER7";

	case XA_DRAWABLE:
	    p = "DRAWABLE";

	case XA_FONT:
	    p = "FONT";

	case XA_INTEGER:
	    p = "INTEGER";

	case XA_PIXMAP:
	    p = "PIXMAP";

	case XA_POINT:
	    p = "POINT";

	case XA_RECTANGLE:
	    p = "RECTANGLE";

	case XA_RESOURCE_MANAGER:
	    p = "RESOURCE_MANAGER";

	case XA_RGB_COLOR_MAP:
	    p = "RGB_COLOR_MAP";

	case XA_RGB_BEST_MAP:
	    p = "RGB_BEST_MAP";

	case XA_RGB_BLUE_MAP:
	    p = "RGB_BLUE_MAP";

	case XA_RGB_DEFAULT_MAP:
	    p = "RGB_DEFAULT_MAP";

	case XA_RGB_GRAY_MAP:
	    p = "RGB_GRAY_MAP";

	case XA_RGB_GREEN_MAP:
	    p = "RGB_GREEN_MAP";

	case XA_RGB_RED_MAP:
	    p = "RGB_RED_MAP";

	case XA_STRING:
	    p = "STRING";

	case XA_VISUALID:
	    p = "VISUALID";

	case XA_WINDOW:
	    p = "WINDOW";

	case XA_WM_COMMAND:
	    p = "WM_COMMAND";

	case XA_WM_HINTS:
	    p = "WM_HINTS";

	case XA_WM_CLIENT_MACHINE:
	    p = "WM_CLIENT_MACHINE";

	case XA_WM_ICON_NAME:
	    p = "WM_ICON_NAME";

	case XA_WM_ICON_SIZE:
	    p = "WM_ICON_SIZE";

	case XA_WM_NAME:
	    p = "WM_NAME";

	case XA_WM_NORMAL_HINTS:
	    p = "WM_NORMAL_HINTS";

	case XA_WM_SIZE_HINTS:
	    p = "WM_SIZE_HINTS";

	case XA_WM_ZOOM_HINTS:
	    p = "WM_ZOOM_HINTS";

	case XA_MIN_SPACE:
	    p = "MIN_SPACE";

	case XA_NORM_SPACE:
	    p = "NORM_SPACE";

	case XA_MAX_SPACE:
	    p = "MAX_SPACE";

	case XA_END_SPACE:
	    p = "END_SPACE";

	case XA_SUPERSCRIPT_X:
	    p = "SUPERSCRIPT_X";

	case XA_SUPERSCRIPT_Y:
	    p = "SUPERSCRIPT_Y";

	case XA_SUBSCRIPT_X:
	    p = "SUBSCRIPT_X";

	case XA_SUBSCRIPT_Y:
	    p = "SUBSCRIPT_Y";

	case XA_UNDERLINE_POSITION:
	    p = "UNDERLINE_POSITION";

	case XA_UNDERLINE_THICKNESS:
	    p = "UNDERLINE_THICKNESS";

	case XA_STRIKEOUT_ASCENT:
	    p = "STRIKEOUT_ASCENT";

	case XA_STRIKEOUT_DESCENT:
	    p = "STRIKEOUT_DESCENT";

	case XA_ITALIC_ANGLE:
	    p = "ITALIC_ANGLE";

	case XA_X_HEIGHT:
	    p = "X_HEIGHT";

	case XA_QUAD_WIDTH:
	    p = "QUAD_WIDTH";

	case XA_WEIGHT:
	    p = "WEIGHT";

	case XA_POINT_SIZE:
	    p = "POINT_SIZE";

	case XA_RESOLUTION:
	    p = "RESOLUTION";

	case XA_COPYRIGHT:
	    p = "COPYRIGHT";

	case XA_NOTICE:
	    p = "NOTICE";

	case XA_FONT_NAME:
	    p = "FONT_NAME";

	case XA_FAMILY_NAME:
	    p = "FAMILY_NAME";

	case XA_FULL_NAME:
	    p = "FULL_NAME";

	case XA_CAP_HEIGHT:
	    p = "CAP_HEIGHT";

	case XA_WM_CLASS:
	    p = "WM_CLASS";

	case XA_WM_TRANSIENT_FOR:
	    p = "WM_TRANSIENT_FOR";

	default:
	    return NULL;
	}
    }

    if (p) {
	len = strlen(p) + 1;
	s = ckalloc(len);
	if (! s) {
	    return NULL;
	}
	memcpy(s, p, len);
	return s;
    }

    len = (int) GlobalGetAtomName(atom, buffer, sizeof(buffer));
    if ((p = ckalloc(len + 1)) == NULL) {
    	return NULL;
    } else {
    	strcpy(p, buffer);
    }
    return p;
}
