/* 
 * xcolor.c --
 *
 * Copyright (c) 1990-1993 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
 * CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
/*
 * Copyright (c) 1994 Software Research Associates, Inc.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Software Research Associates not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  Software Research
 * Associates makes no representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or implied
 * warranty.
 */

#include "xlibInt.h"

#define NEAREST_COLOR
/*
 * use management color
 */
int lx_NoColorUpdate = 0;

static int built = 0;

static Lx_HashTable valueTable;
typedef struct {
    unsigned long red, green, blue;	/* Values for desired color. */
    Colormap colormap;			/* Colormap from which color will be
				 	 * allocated. */
} ValueKey;

static Lx_HashTable ceDBTable;

typedef struct {
    unsigned char red;
    unsigned char green;
    unsigned char blue;
    char *name;
} ColorEntry;

static ColorEntry colorDB[] = {
#include "xcolor2.h"
      0,   0,   0, NULL
};

typedef struct {
    char *name;
    int index;
} SystemColorEntry;

static SystemColorEntry sysColorEntries[] = {
    "SystemActiveBorder",		COLOR_ACTIVEBORDER,
    "SystemActiveCaption",		COLOR_ACTIVECAPTION,
    "SystemAppWorkspace",		COLOR_APPWORKSPACE,
    "SystemBackground",			COLOR_BACKGROUND,
    "SystemButtonFace",			COLOR_BTNFACE,
    "SystemButtonHighlight",		COLOR_BTNHIGHLIGHT,
    "SystemButtonShadow",		COLOR_BTNSHADOW,
    "SystemButtonText",			COLOR_BTNTEXT,
    "SystemCaptionText",		COLOR_CAPTIONTEXT,
    "SystemDisabledText",		COLOR_GRAYTEXT,
    "SystemHighlight",			COLOR_HIGHLIGHT,
    "SystemHighlightText",		COLOR_HIGHLIGHTTEXT,
    "SystemInactiveBorder",		COLOR_INACTIVEBORDER,
    "SystemInactiveCaption",		COLOR_INACTIVECAPTION,
    "SystemInactiveCaptionText",	COLOR_INACTIVECAPTIONTEXT,
    "SystemMenu",			COLOR_MENU,
    "SystemMenuText",			COLOR_MENUTEXT,
    "SystemScrollbar",			COLOR_SCROLLBAR,
    "SystemWindow",			COLOR_WINDOW,
    "SystemWindowFrame",		COLOR_WINDOWFRAME,
    "SystemWindowText",			COLOR_WINDOWTEXT,
    NULL,				0
};

static ColorEntry sysColorDB[] = {
    0, 0, 0, "SystemActiveBorder",
    0, 0, 0, "SystemActiveCaption",
    0, 0, 0, "SystemAppWorkspace",
    0, 0, 0, "SystemBackground",
    0, 0, 0, "SystemButtonFace",
    0, 0, 0, "SystemButtonHighlight",
    0, 0, 0, "SystemButtonShadow",
    0, 0, 0, "SystemButtonText",
    0, 0, 0, "SystemCaptionText",
    0, 0, 0, "SystemDisabledText",
    0, 0, 0, "SystemHighlight",
    0, 0, 0, "SystemHighlightText",
    0, 0, 0, "SystemInactiveBorder",
    0, 0, 0, "SystemInactiveCaption",
    0, 0, 0, "SystemInactiveCaptionText",
    0, 0, 0, "SystemMenu",
    0, 0, 0, "SystemMenuText",
    0, 0, 0, "SystemScrollbar",
    0, 0, 0, "SystemWindow",
    0, 0, 0, "SystemWindowFrame",
    0, 0, 0, "SystemWindowText",
    0, 0, 0, NULL
};

typedef struct _LxColormap LxColormap;
struct _LxColormap {
    LxColormap *next;
    int type;
    int size;
    int *colors;
    int *refcount;
};
    
/*
 *----------------------------------------------------------------------
 * SetSystemColorDB
 * Purpose:
 *    Set all the system colors in our database
 *----------------------------------------------------------------------
 */
void
LxSetSystemColors()
{
    SystemColorEntry *sPtr;
    ColorEntry *ePtr;
    COLORREF color;

    for (ePtr = sysColorDB, sPtr = sysColorEntries;
	 sPtr->name != NULL; ePtr++, sPtr++)
    {
	color = GetSysColor(sPtr->index);
	ePtr->red = GetRValue(color) << 8;
	ePtr->green = GetGValue(color) << 8;
	ePtr->blue = GetBValue(color) << 8;
    }
}

/*
 *----------------------------------------------------------------------
 * BuildColorDB --
 *
 *	Build the color name database
 *----------------------------------------------------------------------
 */
static void
BuildColorDB()
{
    ColorEntry *ePtr;
    Lx_HashEntry *hPtr;
    int dummy;

    built = 1;

    Lx_InitHashTable(&ceDBTable, LX_STRING_KEYS);
    for (ePtr = colorDB; ePtr->name != NULL; ePtr++) {
        hPtr = Lx_CreateHashEntry(&ceDBTable, strlwr(ePtr->name), &dummy);
        Lx_SetHashValue(hPtr, ePtr);
    }

    LxSetSystemColors();

    for (ePtr = sysColorDB; ePtr->name != NULL; ePtr++) {
        hPtr = Lx_CreateHashEntry(&ceDBTable, strlwr(ePtr->name), &dummy);
        Lx_SetHashValue(hPtr, ePtr);
    }
}

static int enumeratedColors = 0;
/*
 *----------------------------------------------------------------------
 *
 * EnumSolidColors --
 *
 *	Finds all the solid colors in the system palette
 *
 * Results:
 *	1
 *
 *----------------------------------------------------------------------
 */
int CALLBACK
EnumSolidColors(LPVOID obj, LPARAM data)
{
    LOGPEN *lpen = obj;
    XColor *color = (XColor *) data;

    color += enumeratedColors;

    if (lpen->lopnStyle == PS_SOLID) {
	color->red = GetRValue(lpen->lopnColor) << 8;
	color->green = GetGValue(lpen->lopnColor) << 8;
	color->blue = GetBValue(lpen->lopnColor) << 8;
	enumeratedColors++;
    }
    return 1;
}
 
XCMapInfo **ColorMapInfo = NULL;
Colormap colormapId = 1;
	
/*
 *----------------------------------------------------------------------
 * XCreateColormap --
 *
 *	Create a colormap.
 *
 * Results:
 *	The created colormap or 0 if this fails.
 *
 * Notes:
 *	For an 8 bit visual that matches the root depth, a logical
 *	palette is created with 256 entries.  If this is the default
 *	colormap for the screen, black and white are allocated in it.
 *
 *	Nothing happens for 32 bit visuals.
 *----------------------------------------------------------------------
 */
Colormap
XCreateColormap(Display *display, Window w, Visual *visual, int alloc)
{
    XCMapInfo **newInfo, *cmapInfo;
    int i;
    PALETTEENTRY pe;

    display->request++;

    newInfo = (XCMapInfo **) ckalloc(sizeof(XCMapInfo *) * (colormapId + 1));
    if (newInfo == NULL) {
	return 0;
    }
    if (ColorMapInfo != NULL) {
	memcpy(newInfo, ColorMapInfo, (sizeof(XCMapInfo *) * colormapId));
	ckfree((char *) ColorMapInfo);
    }
    ColorMapInfo = newInfo;

    if (display->screens[0].root_depth == 8) {
	LOGPALETTE *palette;
	HPALETTE hPal;

	palette = (LOGPALETTE *)
	    ckalloc(sizeof(LOGPALETTE) + 256 * sizeof(PALETTEENTRY));
	if (palette == NULL) {
	    return 0;
	}
	palette->palVersion = 0x300;
	palette->palNumEntries = 256;
	memset(&palette->palPalEntry[0], 0, 256 * sizeof(PALETTEENTRY));
	for (i = 0; i < 256; i++) {
	    palette->palPalEntry[i].peFlags = 0; /* PC_RESERVED */;
	}

	hPal = CreatePalette(palette);
	ckfree((char *) palette);

	if (hPal == NULL) {
	    return 0;
	}

	cmapInfo = (XCMapInfo *)
	    ckalloc(sizeof(XCMapInfo) + 256 * sizeof(XCMColor));
	if (cmapInfo == NULL) {
	    return 0;
	}
	cmapInfo->hPalette = hPal;
	ColorMapInfo[colormapId] = cmapInfo;
	memset(&cmapInfo->cindex[0], 0, 256 * sizeof(XCMColor));

	cmapInfo->cmapid = colormapId;
	colormapId++;

	cmapInfo->created_depth = 8;
	cmapInfo->actual_depth = 8;
	cmapInfo->ncolors = 256;

	/* If this is the root colormap, allocate black and white pixels */
	if (colormapId == 2) {
	    XColor colors[256];
	    HDC hdc;

	    hdc = GetDC(NULL);
	    EnumObjects(hdc, OBJ_PEN, EnumSolidColors, colors);
	    ReleaseDC((HWND) NULL, hdc);

	    for (i = 0; i < enumeratedColors; i++) {
		XAllocColor(display, cmapInfo->cmapid, &colors[i]);
		if (colors[i].red == 0 && colors[i].green == 0 &&
		    colors[i].blue == 0) {
		    display->screens[0].black_pixel = i;
		} else if (colors[i].red == 0xff00 &&
			   colors[i].green == 0xff00 &&
			   colors[i].blue == 0xff00) {
		    display->screens[0].white_pixel = i;
		}
		cmapInfo->cindex[colors[i].pixel].flags =
		    ALLOC_PERMANENT|ALLOC_READONLY;
		pe.peRed = colors[i].red >> 8;
		pe.peGreen = colors[i].green >> 8;
		pe.peBlue = colors[i].blue >> 8;
		pe.peFlags = 0;
		SetPaletteEntries(cmapInfo->hPalette, i, 1, &pe);
	    }
	}

    } else {
	cmapInfo = (XCMapInfo *) ckalloc(sizeof(XCMapInfo));
	if (cmapInfo == NULL) {
	    return 0;
	}
	ColorMapInfo[colormapId] = cmapInfo;

	cmapInfo->cmapid = colormapId;
	colormapId++;

	cmapInfo->hPalette = NULL;
	cmapInfo->created_depth = 32;
	cmapInfo->actual_depth = 24;
	cmapInfo->ncolors = 2 << 24;
    }

    return cmapInfo->cmapid;
}

/*
 *----------------------------------------------------------------------
 * XFreeColormap --
 *
 *	Free up a colormap.
 *----------------------------------------------------------------------
 */
void
XFreeColormap(Display *display, Colormap colormap)
{
    XCMapInfo *cmapInfo;

    display->request++;

    /* Don't delete a bogus colormap or the default one */
    if (colormap < 2 || colormap >= colormapId) {
	return;
    }
    cmapInfo = ColorMapInfo[colormap];
    if (cmapInfo == NULL) {
	return;
    }

    if (cmapInfo->created_depth == 8) {
	if (cmapInfo->hPalette) {
	    DeleteObject(cmapInfo->hPalette);
	}
    }
    ColorMapInfo[colormap] = NULL;
    ckfree((char *) cmapInfo);
}

/*
 *----------------------------------------------------------------------
 * XGetWMColormapWindows --
 *
 *	Get a window's WM_COLORMAP_WINDOWS property.
 *
 * Notes:
 *	XXX: Not yet implemented
 *----------------------------------------------------------------------
 */
Status
XGetWMColormapWindows(Display *display, Window w, Window **windows_return,
		      int *count_return)
{
    display->request++;
    return Success;
}

/*
 *----------------------------------------------------------------------
 * XSetWMColormapWindows --
 *
 *	Set a window's WM_COLORMAP_WINDOWS property.  
 *
 * Notes:
 *	XXX: Not yet implemented
 *----------------------------------------------------------------------
 */
Status
XSetWMColormapWindows(Display *display, Window w, Window *colormap_windows,
		      int count)
{
    display->request++;
    return Success;
}

/*
 *----------------------------------------------------------------------
 * XLookupColor --
 *
 *	Get database RGB values and closest hardware-supported RGB
 *	values from color name.
 *
 * Results:
 *	0 on failure, non-zero on success.
 *----------------------------------------------------------------------
 */
Status
XLookupColor(Display *display, Colormap colormap, const char *colorname, 
	     XColor *exact_def_return, XColor *screen_def_return)
{
    Lx_HashEntry *hPtr;
    ColorEntry *cPtr;
    HDC hdc;
    XColor *col1, *col2;
    static char *name = NULL;
    BYTE red, green, blue;
    unsigned long pixel;
    XCMapInfo *cmapInfo;
    int len;

    display->request++;

    if (colormap == 0 || colormap >= colormapId) {
	return 0;
    }
    cmapInfo = ColorMapInfo[colormap];
    if (cmapInfo == NULL) {
	return 0;
    }

    if (!built) {
        BuildColorDB();
    }

    if (name != NULL) {
	ckfree(name);
    }

    len = strlen(colorname) + 1;
    name = ckalloc(len);
    if (name == NULL) {
	return 0;
    }
    memcpy(name, colorname, len);

    hPtr = Lx_FindHashEntry(&ceDBTable, (char *) strlwr(name));

    if (hPtr == NULL) {
        return 0;
    }
    col1 = exact_def_return;
    col2 = screen_def_return;

    col1->pad = 0;

    cPtr = (ColorEntry *) Lx_GetHashValue(hPtr);
    red   = (BYTE) cPtr->red;
    green = (BYTE) cPtr->green;
    blue  = (BYTE) cPtr->blue;

    col1->red   = red   << 8;
    col1->green = green << 8;
    col1->blue  = blue << 8;

    if (cmapInfo->created_depth != 8) {
	hdc = GetDC(NULL);
	pixel = GetNearestColor(hdc, RGB(red, green, blue));
	ReleaseDC(NULL, hdc);

	col1->pixel = pixel;
	col2->pixel = pixel;
	col2->red = GetRValue(pixel) << 8;
	col2->green = GetGValue(pixel) << 8;
	col2->blue = GetBValue(pixel) << 8;

    } else {
	*col2 = *col1;
    }    
    return 1;
}

/*
 *----------------------------------------------------------------------
 * XAllocNamedColor --
 *
 *	Allocate a read-only colorcell from color name
 *
 * Results:
 *	0 on failure, non-zero on success.
 *----------------------------------------------------------------------
 */

Status
XAllocNamedColor(Display *display, Colormap colormap, const char *colorname, 
		 XColor *screen_def_return, XColor *exact_def_return)
{		 
    Status ret;

    display->request++;

    ret = XLookupColor(display, colormap, colorname,
		       exact_def_return, screen_def_return);
    if (ret == 0) {
	return 0;
    }
    ret = XAllocColor(display, colormap, screen_def_return);
    exact_def_return->pixel = screen_def_return->pixel;
    return ret;
}

/*
 *----------------------------------------------------------------------
 * XParseColor --
 *
 *	Look up RGB values from ASCII color name or translate 
 *	hexadecimal value.
 *----------------------------------------------------------------------
 */
Status
XParseColor(Display *display, Colormap colormap, const char *spec,
	    XColor *color)
{
    char fmt[16];
    int i, ret;
    XColor ignore;

    if (spec[0] != '#') {
	return XLookupColor(display, colormap, spec, color, &ignore);
    }

    if ((i = strlen(spec+1))%3)
	return 0;
    i /= 3;

    sprintf(fmt, "%%%dx%%%dx%%%dx", i, i, i);
    ret = sscanf(spec+1, fmt, &color->red, &color->green, &color->blue);
    if (ret != 3) {
	return 0;
    }
    color->pad = 0;

    if (i == 2) {
	color->red <<= 8;
	color->green <<= 8;
	color->blue <<= 8;
    }

    return 1;
}

/*
 *----------------------------------------------------------------------
 * XAllocColor --
 *
 *	Allocate a read-only colormap cell with closest
 *	hardware-supported color
 *
 * Results:
 *	0 on failure, non-zero on success
 *
 * Side Effects:
 *	A read only color is allocated or the reference count
 *	of the color is incremented (pixel field).
 *
 * Notes:
 *	The search to find a matching color may not work properly in
 *	all circumstances for 8 bit mode
 *----------------------------------------------------------------------
 */
Status
XAllocColor(Display *display, Colormap colormap, XColor *color)
{
    HDC hdc;
    BYTE red, green, blue;
    XCMapInfo *cmapInfo;
    COLORREF colorref;
    PALETTEENTRY pe;
    int i, ret, new;
    int attempt;
    ValueKey valueKey;
    Lx_HashEntry *valueHashPtr;
    static int initialized = 0;

    display->request++;

    if (! initialized) {
	Lx_InitHashTable(&valueTable, sizeof(ValueKey)/sizeof(int));
	initialized = 1;
    }

    if (colormap == 0 || colormap >= colormapId) {
	return 0;
    }
    cmapInfo = ColorMapInfo[colormap];
    if (cmapInfo == NULL) {
	return 0;
    }

    red   = (BYTE) (color->red   >> 8);
    green = (BYTE) (color->green >> 8);
    blue  = (BYTE) (color->blue  >> 8);
    colorref = RGB(red, green, blue);

    if (cmapInfo->created_depth != 8) {
	hdc = GetDC(NULL);
	color->pixel = GetNearestColor(hdc, colorref);
	ReleaseDC(NULL, hdc);
	color->red = GetRValue(color->pixel) << 8;
	color->green = GetGValue(color->pixel) << 8;
	color->blue = GetBValue(color->pixel) << 8;
	return 1;
    }

    valueKey.red = color->red & 0xff00;
    valueKey.blue = color->blue & 0xff00;
    valueKey.green = color->green & 0xff00;
    valueKey.colormap = colormap;
    valueHashPtr = Lx_CreateHashEntry(&valueTable, (char *) &valueKey, &new);
    if (!new) {
	color->pixel = (unsigned long) Lx_GetHashValue(valueHashPtr);
	cmapInfo->cindex[color->pixel].pixel++;
	if (cmapInfo->cindex[color->pixel].flags == ALLOC_FREED) {
	    cmapInfo->cindex[color->pixel].flags = ALLOC_READONLY;
	}
	return 1;
    }
    
    for (attempt = 0; attempt < 2; attempt++) {
	for (i = 0; i < 256; i++) {
	    if ((attempt == 0 && cmapInfo->cindex[i].flags == ALLOC_NONE) ||
		(attempt == 1 && (cmapInfo->cindex[i].flags == ALLOC_FREED ||
				  cmapInfo->cindex[i].flags == ALLOC_FREED)))
	    {
		if (cmapInfo->cindex[i].ptr) {
		    Lx_DeleteHashEntry(cmapInfo->cindex[i].ptr);
		}

		cmapInfo->cindex[i].flags = ALLOC_READONLY;
		cmapInfo->cindex[i].red   = color->red & 0xff00;
		cmapInfo->cindex[i].green = color->green & 0xff00;
		cmapInfo->cindex[i].blue  = color->blue & 0xff00;
		cmapInfo->cindex[i].pixel = 1;
		cmapInfo->cindex[i].ptr   = valueHashPtr;
		Lx_SetHashValue(valueHashPtr, i);

		pe.peRed = red;
		pe.peGreen = green;
		pe.peBlue = blue;
		pe.peFlags = 0;
		ret = SetPaletteEntries(cmapInfo->hPalette, i, 1, &pe);
		color->pixel = i;

		/* printf("%d: (%02x,%02x,%02x)\t", i, red, green, blue); */
		return 1;
	    }
	}
    }

    /* printf("Alloc failed\n"); */
    Lx_DeleteHashEntry(valueHashPtr);
    return 0;
}

/*
 *----------------------------------------------------------------------
 * Lx_UpdatePaletteColors --
 *
 *	When a palette gets changed, all the windows that use that
 *	palette need to update their colors.  This routine goes through
 *	all the children of the given toplevel window and updates 
 *	their colors.
 *
 * Results:
 *	None
 *----------------------------------------------------------------------
 */
void
Lx_UpdatePaletteColors(HWND hwnd)
{
    HDC hdc;
    HPALETTE hPal, hPalOld;
    int n;

    hPal = Lx_GetDrawablePalette(hwnd);
    hdc = GetDC(hwnd);
    if (hPal) {
	hPalOld = SelectPalette(hdc, hPal, FALSE);
	n = RealizePalette(hdc);
	if (n > 0) {
	    UpdateColors (hdc);
	}
	SelectPalette(hdc, hPalOld, FALSE);
    }
    ReleaseDC(hwnd, hdc);
}

/*
 *----------------------------------------------------------------------
 * XFreeColors --
 *
 *	Free colormap cells or planes.
 *
 * Results:
 *	None
 *
 * Side Effects:
 *	Reference counts are decremented and colors are freed from the
 *	color table if a reference count reaches 0.
 *----------------------------------------------------------------------
 */
void
XFreeColors(Display *display, Colormap colormap, unsigned long pixels[],
	    int npixels, unsigned long planes)
{
    XCMapInfo *cmapInfo;
    int i, j, bit, pixel, flags, cnt, noplanes;

    display->request++;

    if (colormap == 0 || colormap >= colormapId) {
	return;
    }
    cmapInfo = ColorMapInfo[colormap];
    if (cmapInfo == NULL) {
	return;
    }

    if (cmapInfo->created_depth != 8) {
	return;
    }

    if (planes == 0) {
	noplanes = 1;
	bit = 0;
    } else {
	noplanes = 8;
	bit = 1;
    }

    for (j = 0; j < noplanes; j++, bit = bit << 1) {
	if ((bit == 0) || (bit & planes)) {
	    for (i = 0; i < npixels; i++) {
		pixel = pixels[i] | bit;

		flags = cmapInfo->cindex[pixel].flags;
		if (flags & ALLOC_PERMANENT) {
		    continue;
		}
		if (flags & ALLOC_READONLY) {

		    /* Reference count is maintained in the pixel field */
		    cnt = cmapInfo->cindex[pixel].pixel;
		    if (cnt > 0) {
			cnt--;
			cmapInfo->cindex[pixel].pixel = cnt;
			if (cnt == 0) {
			    cmapInfo->cindex[pixel].flags = ALLOC_FREED;
			}
		    }
		} else if (flags & ALLOC_READWRITE) {
		    cmapInfo->cindex[pixel].flags = ALLOC_FREED;
		}
	    }
	}
    }
}

/*
 *----------------------------------------------------------------------
 * XQueryColor --
 *
 *	Obtain RGB values for a colorcell.
 *
 *----------------------------------------------------------------------
 */
void
XQueryColor(Display *display, Colormap colormap, XColor *color)
{
    XCMapInfo *cmapInfo;

    display->request++;

    if (colormap == 0 || colormap >= colormapId) {
	return;
    }
    cmapInfo = ColorMapInfo[colormap];
    if (cmapInfo == NULL) {
	return;
    }

    if (cmapInfo->created_depth != 8) {
	color->red = GetRValue(color->pixel) << 8;
	color->green = GetRValue(color->pixel) << 8;
	color->blue = GetRValue(color->pixel) << 8;
	return;
    }

    color->red = cmapInfo->cindex[color->pixel].red;
    color->green = cmapInfo->cindex[color->pixel].green;
    color->blue = cmapInfo->cindex[color->pixel].blue;
}
    
/*
 *----------------------------------------------------------------------
 *
 * XQueryColorPalette --
 *
 *	Obtain information about a color palette.  The current allocation
 *	colors and flags are copied into the color array.
 *	These are the flags definitions.
 *
 *	#define ALLOC_NONE		0x00   Not allocated
 *	#define ALLOC_READONLY		0x01   Just like XAllocColor does
 *	#define ALLOC_READWRITE		0x02   Just like XAllocColorCells does
 *	#define ALLOC_PERMANENT		0x04   Permanent colors we don't touch
 *	#define ALLOC_FREED		0x08   Previously allocated, but not
 *					       now
 *
 * Results:
 *	# of colors in palette.
 *
 *----------------------------------------------------------------------
 */
int
XQueryColorPalette(Display *display, Colormap colormap, XColor *color,
		   int ncolors)
{
    XCMapInfo *cmapInfo;
    int i, n;

    display->request++;

    if (colormap == 0 || colormap >= colormapId) {
	return 0;
    }
    cmapInfo = ColorMapInfo[colormap];
    if (cmapInfo == NULL) {
	return 0;
    }

    if (cmapInfo->created_depth != 8) {
	return (2 << 24);
    }
    if (ncolors != 0 && color != NULL) {
	n = (ncolors > 256) ? 256 : ncolors;

	for (i = 0; i < n; i++) {
	    color[i].red = cmapInfo->cindex[i].red;
	    color[i].green = cmapInfo->cindex[i].green;
	    color[i].blue = cmapInfo->cindex[i].blue;
	    color[i].flags = cmapInfo->cindex[i].flags;
	    color[i].pixel = i;
	}
    }
    return 256;
}
    
/*
 *----------------------------------------------------------------------
 * XGetWindowsPalette --
 *
 *	Return the Windows color palette for a given colormap
 *
 * Results:
 *	The palette if found, NULL if not.
 *
 *----------------------------------------------------------------------
 */
void *
XGetWindowsPalette(Display *display, Colormap colormap)
{
    XCMapInfo *cmapInfo;
    if (colormap == 0 || colormap >= colormapId) {
	return NULL;
    }
    cmapInfo = ColorMapInfo[colormap];
    if (cmapInfo == NULL) {
	return NULL;
    }
    return cmapInfo->hPalette;
}

/*
 *----------------------------------------------------------------------
 * XQueryColors --
 *
 *	Obtain RGB values for an array of colorcells
 * 
 *----------------------------------------------------------------------
 */
void
XQueryColors(Display *display, Colormap	colormap, XColor *colors,
	     int ncolors)
{
    XCMapInfo *cmapInfo;
    int i;

    display->request++;

    if (colormap == 0 || colormap >= colormapId) {
	return;
    }
    cmapInfo = ColorMapInfo[colormap];
    if (cmapInfo == NULL) {
	return;
    }

    for (i = 0; i < ncolors; i++) {
	XQueryColor(display, colormap, &colors[i]);
    }
}

/*
 *----------------------------------------------------------------------
 * XAllocColorCells --
 *
 *	Allocate read/write (non-shared) colorcells.  Not implemented
 *	currently, but the structures are in place to do so.
 *
 * Results:
 *	0 on failure, and we are always failing
 *----------------------------------------------------------------------
 */
Status
XAllocColorCells(Display *display, Colormap colormap, Bool contig,
		 unsigned long *plane_masks_return, unsigned int nplanes,
		 unsigned long *pixels_return, unsigned int npixels)
{
    return 0;
}

/*
 *----------------------------------------------------------------------
 * XStoreColor --
 *
 *	Set or change the RGB value of a read/write colorcell to the
 *	closest possible hardware colors.  Not implemented.
 *
 * Results:
 *	None
 *----------------------------------------------------------------------
 */
void
XStoreColor(Display *display, Colormap colormap,
	    XColor *color)
{
}

/*
 *----------------------------------------------------------------------
 * XStoreColors --
 *
 *	Set or change the RGB values of read/write colorcells to the
 *	closest possible hardware colors.  Not implemented.
 *
 * Results:
 *	None
 *----------------------------------------------------------------------
 */
void
XStoreColors(Display *display, Colormap colormap,
	     XColor *color, int ncolors)
{
}

/*
 *----------------------------------------------------------------------
 * XDefaultColormap --
 *
 *	Returns the default colormap for a screen
 *----------------------------------------------------------------------
 */
Colormap
XDefaultColormap(Display *display, int screen_number)
{
    return display->screens[screen_number].cmap;
}
