/*
 * xdefcur.c --
 *
 * Copyright (c) 1994 Software Research Associates, Inc.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Software Research Associates not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  Software Research
 * Associates makes no representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or implied
 * warranty.
 */

#include "xlibInt.h" 

static HCURSOR hDefaultCursor;

void
XDefineCursor(Display *display, Window w, Cursor cursor)
{
    POINT pt;
    XSetWindowAttributes attrs;
    unsigned char result;

    display->request++;

    attrs.cursor = cursor;
    result = Lx_SetWindowAttributes(w, CWCursor, &attrs);
    if (result != Success) {
	Lx_PostError(result, w);
	return;
    }
    GetCursorPos(&pt);
    SetCursorPos(pt.x, pt.y);
}

void
XUndefineCursor(Display *display, Window w)
{
    POINT pt;
    XSetWindowAttributes attrs;
    unsigned char result;
    Lx_WindowEntry *winEntry;

    display->request++;

    winEntry = Lx_GetWindowHashEntry(w);
    if (! winEntry) {
	return;
    }
    attrs.cursor = winEntry->prevCursor;
    result = Lx_SetWindowAttributes(w, CWCursor, &attrs);
    if (result != Success) {
	Lx_PostError(result, w);
	return;
    }
    winEntry->prevCursor = NULL;
    GetCursorPos(&pt);
    SetCursorPos(pt.x, pt.y);
}

/*
 *----------------------------------------------------------------------
 *
 * Lx_SetDefaultCursor --
 *
 *	Set the default cursor for a window when no cursor for the
 *	window or its ancestors has been set.
 *
 * Results:
 * 	None
 *
 * Side Effects:
 *	None
 *
 *----------------------------------------------------------------------
 */
void
Lx_SetDefaultCursor(HCURSOR hcs)
{
    hDefaultCursor = hcs;
}

/*
 *----------------------------------------------------------------------
 *
 * Lx_FindCursor --
 *
 *	Find the cursor shape to be displayed when the cursor is
 *	over the given window.  If no cursor is set for the particular
 *	window, then the parent is checked recursively until a
 *	cursor is found or until the desktop window is hit.  In this
 *	case, the default cursor shape is used.
 *
 * Results:
 *	A Windows cursor object
 *
 * Side Effects:
 *	None
 *
 *----------------------------------------------------------------------
 */
HCURSOR
Lx_FindCursor(HWND hwnd)
{
    Lx_WindowEntry *winEntry;

    if (hwnd == HWND_DESKTOP) {
    	return hDefaultCursor;
    }

    winEntry = Lx_GetWindowHashEntry(hwnd);
    if (! winEntry) {
    	return hDefaultCursor;
    }

    if (winEntry->setWinAttrs.cursor) {
	return winEntry->setWinAttrs.cursor;
    }

    return Lx_FindCursor(GetParent(hwnd));
}
