/*
 * xevent.c --
 *
 * Copyright (c) 1994 Software Research Associates, Inc.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Software Research Associates not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  Software Research
 * Associates makes no representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or implied
 * warranty.
 */
 
#include "xlibInt.h" 

int
XFileEventsQueued(Display *display, int mode, int flags)
{
    MSG msg;
    XEvent event;
    int len;

    if (flags == 0) {
	flags = X_Queue;
    }

    len = 0;
    if (mode != QueuedForceReading) {
	if (flags & X_Queue) {
	    len += display->qlen;
	}
	if (flags & File_Queue) {
	    len += display->fileqlen;
	}
	if (flags & Timer_Queue) {
	    len += display->timerqlen;
	}
	if (len > 0 || mode == QueuedAlready) {
	    return len;
	}
    }
    
    while (1) {
	len = 0;
	if (flags & X_Queue) {
	    len += display->qlen;
	}
	if (flags & File_Queue) {
	    len += display->fileqlen;
	}
	if (flags & Timer_Queue) {
	    len += display->timerqlen;
	}

#if 0
	/*
	 * This is an attempt to not eat up events needed by
	 * OLE drag & drop.  Unfortunately, is did not work
	 * as wanted.  The problem is that in X, we want to
	 * queue up as many events as possible.  This takes
	 * the Windows events out of the message queue.  Drag
	 * and drop expects the events to still be their, so
	 * we end up with problems.
	 */
	if ((flags & X_Queue) && (display->qlen > 0)) {
	    return len;
	}
#endif
	if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE) == 0) {
	    return len;
	}
	if (msg.hwnd == Tknt_GetPostWindow() 
	   || (msg.message == WM_KEYDOWN 
	       && (msg.wParam == VK_NUMLOCK
		   || msg.wParam == VK_CAPITAL
		   || msg.wParam == VK_SCROLL))) {
	    if (msg.message == EVENTADDED) {
		continue;
	    }
	    TranslateMessage(&msg);
	}

	DispatchMessage(&msg);
	if (Lx_MsgToEvent(display, &msg, &event)) {
	    if (event.type == SocketNotify || event.type == InputNotify) {
		Lx_PutFileEvent(display, &event);
	    } else if (event.type == TimerNotify) {
		Lx_PutTimerEvent(display, &event);
	    } else {
		Lx_PutEvent(display, &event);
	    }
	}
    }
    /* NOTREACHED */
    return 1;
}

int
XEventsQueued(Display *display, int mode)
{
    return XFileEventsQueued(display, mode, X_Queue);
}

void
XNextFileEvent(Display *display, XEvent *report, int flags)
{
    MSG msg;

    if (flags == 0) {
	flags = X_Queue;
    }

    while (1) {
	if (flags & X_Queue && display->qlen > 0) {
	    Lx_GetEvent(display, report);
	    display->last_request_read = report->xany.serial;
	    return;
	}
	if (flags & File_Queue && display->fileqlen > 0) {
	    Lx_GetFileEvent(display, report);
	    display->last_request_read = report->xany.serial;
	    return;
	}
	if (flags & Timer_Queue && display->timerqlen > 0) {
	    Lx_GetTimerEvent(display, report);
	    display->last_request_read = report->xany.serial;
	    return;
	}

	GetMessage(&msg, NULL, 0, 0);
	if (msg.hwnd == Tknt_GetPostWindow() 
	   || (msg.message == WM_KEYDOWN 
	       && (msg.wParam == VK_NUMLOCK
		   || msg.wParam == VK_CAPITAL
		   || msg.wParam == VK_SCROLL))) {
	    if (msg.message == EVENTADDED) {
		continue;
	    }
	    TranslateMessage(&msg);
	}
	DispatchMessage(&msg);
	if (Lx_MsgToEvent(display, &msg, report)) {
	    /* DummyNotify is used to force a return to the Tk event loop.
	     * This is used for Tk's 'send' command since it bypasses the
	     * event loop altogether and needs to force idle events to occur
	     */
	    if (report->type == DummyNotify) {
		display->last_request_read = report->xany.serial;
		return;
	    }
	    if ((report->type == SocketNotify || report->type == InputNotify)
		&& !(flags & File_Queue))
	    {
		Lx_PutFileEvent(display, report);
	    } else if (report->type == TimerNotify && !(flags & Timer_Queue)) {
		Lx_PutTimerEvent(display, report);
	    } else if (report->type != SocketNotify &&
		       report->type != InputNotify &&
		       report->type != TimerNotify &&
		       ! (flags & X_Queue))
	    {
		Lx_PutEvent(display, report);
	    } else {
		display->last_request_read = report->xany.serial;
		return;
	    }
	}
    }
}   

void
XNextEvent(Display *display, XEvent *report)
{
    XNextFileEvent(display, report, X_Queue);
}

/*
 * Process a windows message.  This function was going to be used by
 * the socket blocking hook, but that hook never got called.  Now it
 * is not used, but there may be reason to leave it here for now.
 */
void
XNextWindowsMessage(Display *display)
{
    MSG msg;
    XEvent event;

    GetMessage(&msg, NULL, 0, 0);
    if (msg.hwnd == Tknt_GetPostWindow() 
       || (msg.message == WM_KEYDOWN 
	   && (msg.wParam == VK_NUMLOCK
	       || msg.wParam == VK_CAPITAL
	       || msg.wParam == VK_SCROLL))) {
	if (msg.message == EVENTADDED) {
	    return;
	}
	TranslateMessage(&msg);
    }

    DispatchMessage(&msg);
    if (Lx_MsgToEvent(display, &msg, &event)) {
	if (event.type == SocketNotify || event.type == InputNotify) {
	    Lx_PutFileEvent(display, &event);
	} else if (event.type == TimerNotify) {
	    Lx_PutTimerEvent(display, &event);
	} else {
	    
	    Lx_PutEvent(display, &event);
	}
    }
    return;
}
    
Bool
XCheckIfEvent(Display *display, XEvent *event_return, 	
	Bool (*predicate)( Display *, XEvent *, char * ),
	char *arg)
{
    _XQEvent *head, *chain;

    head = display->head;
    for (chain = head->next; chain != head; chain = chain->next) {
	if( !(*predicate)( display, &chain->event, arg ) )
	    /* unmatch */
	    continue;
	/* match, pick it up and discard */
	Lx_PeekEvent(display, chain, event_return);
	Lx_DiscardEvent(display, chain);
	return TRUE;
    }
    /* No match */
    return FALSE;
}

static long eventMasks[LASTEvent] = {
    0,
    0,
    KeyPressMask,			/* KeyPress */
    KeyReleaseMask,			/* KeyRelease */
    ButtonPressMask,			/* ButtonPress */
    ButtonReleaseMask,			/* ButtonRelease */
    PointerMotionMask|PointerMotionHintMask|ButtonMotionMask
	    |Button1MotionMask|Button2MotionMask|Button3MotionMask
	    |Button4MotionMask|Button5MotionMask,
					/* MotionNotify */
    EnterWindowMask,			/* EnterNotify */
    LeaveWindowMask,			/* LeaveNotify */
    FocusChangeMask,			/* FocusIn */
    FocusChangeMask,			/* FocusOut */
    KeymapStateMask,			/* KeymapNotify */
    ExposureMask,			/* Expose */
    ExposureMask,			/* GraphicsExpose */
    ExposureMask,			/* NoExpose */
    VisibilityChangeMask,		/* VisibilityNotify */
    SubstructureNotifyMask,		/* CreateNotify */
    StructureNotifyMask,		/* DestroyNotify */
    StructureNotifyMask,		/* UnmapNotify */
    StructureNotifyMask,		/* MapNotify */
    SubstructureRedirectMask,		/* MapRequest */
    StructureNotifyMask,		/* ReparentNotify */
    StructureNotifyMask,		/* ConfigureNotify */
    SubstructureRedirectMask,		/* ConfigureRequest */
    StructureNotifyMask,		/* GravityNotify */
    ResizeRedirectMask,			/* ResizeRequest */
    StructureNotifyMask,		/* CirculateNotify */
    SubstructureRedirectMask,		/* CirculateRequest */
    PropertyChangeMask,			/* PropertyNotify */
    0,					/* SelectionClear */
    0,					/* SelectionRequest */
    0,					/* SelectionNotify */
    ColormapChangeMask,			/* ColormapNotify */
    0,					/* ClientMessage */
    0,					/* Mapping Notify */
#ifdef _WINDOWS
    0,					/* Activate App */
    ColormapChangeMask,			/* System color change */
    0,					/* InputNotify */
    0,					/* TimerNotify */
    0,					/* SocketNotify */
    0,					/* DummyNotify */
#endif
};

/*
 *----------------------------------------------------------------------
 * XWindowEvent --
 *
 *	Remove the next event that matches the specified mask and window
 *----------------------------------------------------------------------
 */
void
XWindowEvent(Display *display, Window w, long event_mask, XEvent *event)
{
    _XQEvent *head, *cPtr;
    MSG msg;

  rescan:
    head = display->head;
    for (cPtr = head->next; cPtr != head; cPtr = cPtr->next) {
	if (eventMasks[cPtr->event.type] & event_mask) {
	    Lx_PeekEvent(display, cPtr, event);
	    Lx_DiscardEvent(display, cPtr);
	    return;
	}
    }

    while (1) {
	GetMessage(&msg, NULL, 0, 0);
	if (msg.hwnd == Tknt_GetPostWindow() 
	    || (msg.message == WM_KEYDOWN 
		&& (msg.wParam == VK_NUMLOCK
		    || msg.wParam == VK_CAPITAL
		    || msg.wParam == VK_SCROLL))) {
	    if (msg.message == EVENTADDED) {
		goto rescan;
	    }
	    TranslateMessage(&msg);
	}

	DispatchMessage(&msg);
	if (Lx_MsgToEvent(display, &msg, event)) {
	    if (event->type == SocketNotify || event->type == InputNotify) {
		Lx_PutFileEvent(display, event);
	    } else if (event->type == TimerNotify) {
		Lx_PutTimerEvent(display, event);
	    } else {
		if (eventMasks[event->type] == event_mask) {
		    return;
		}
		Lx_PutEvent(display, event);
	    }
	}
    }
}

void
XPutBackEvent(Display *display, XEvent *event)
{
    Lx_PutbackEvent(display, event);
}

    
int	 
XCheckWindowEvent(Display *display, Window w, long mask, XEvent *event)
{
    int type;
    _XQEvent *cPtr, *head;

    head = display->head;
    if (XEventsQueued(display, QueuedForceReading)) {
	for (cPtr = head->next; cPtr != head; cPtr = cPtr->next) {
	    type = cPtr->event.type;
	    if (eventMasks[type] & mask) {
		Lx_PeekEvent(display, cPtr, event);
		Lx_DiscardEvent(display, cPtr);
		return 1;
	    }
	}
    }
    return 0;
}	
     
Status
XSendEvent(Display *display, Window w, Bool propagate, long event_mask, XEvent *event_send)
{
    XEvent e;

    display->request++;

    memcpy(&e, event_send, sizeof(e));
    e.xany.send_event = TRUE;
    Lx_PutEvent(display, &e);
    return Success;
}

/*
 *----------------------------------------------------------------------
 * Lx_CheckSendMessage --
 *
 *	Check if the message should be posted to the window or one of
 *	its ancestors.
 *
 * Results:
 *	True if the message should be posted, False if it should not be.
 *
 *----------------------------------------------------------------------
 */
int
Lx_CheckSendMessage(Window w, int type)
{
    Lx_WindowEntry *winEntry;
    XSetWindowAttributes *pXSWAttrs;
    long event_mask;
    long do_not_propagate_mask;
    char className[256];
    int mask;

    if (type == 0) {
	return True;
    }

    while (w && w != HWND_DESKTOP) {
	winEntry = Lx_GetWindowHashEntry(w);
	if (winEntry == NULL) {
	    /* Probably the terminal window */
	    return True;
	}
	pXSWAttrs = &winEntry->setWinAttrs;

	mask = eventMasks[type];
	if (mask == 0) {
	    return True;
	}

	event_mask = pXSWAttrs->event_mask;
	do_not_propagate_mask = pXSWAttrs->do_not_propagate_mask;

	if (mask & event_mask) {
	    return True;
	}
	if (mask & do_not_propagate_mask) {
	    return False;
	}

	GetClassName(w, className, sizeof(className));
	if (strcmp(className, "TopLevel") == 0) {
	    break;
	}
	w = GetParent(w);
    }
    return False;
}

/*
 *----------------------------------------------------------------------
 * Lx_AddConfigureNotify --
 *
 *	Add a configure notify message to the queue.
 *	its ancestors.
 *
 * Results:
 *	None
 *
 *----------------------------------------------------------------------
 */
void
Lx_AddConfigureNotify(Window w, Window above, int x, int y, int width, int height)
{
    XEvent e;
    Display *display;

    display = XOpenDisplay(NULL);

    e.type = ConfigureNotify;
    e.xany.window = w;
    e.xany.send_event = False;
    e.xany.display = display;
    e.xany.serial = display->request;
    e.xconfigure.event = e.xconfigure.window = w;
    e.xconfigure.x = x;
    e.xconfigure.y = y;
    e.xconfigure.width = width;
    e.xconfigure.height = height;
    e.xconfigure.border_width = 1;	
    e.xconfigure.above = above;
    e.xconfigure.override_redirect = False;

    Lx_PutEvent(display, &e);
    PostMessage(w, EVENTADDED, 0, 0L);
}

