/*
 * xgc.c --
 *
 * Copyright (c) 1994 Software Research Associates, Inc.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Software Research Associates not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  Software Research
 * Associates makes no representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or implied
 * warranty.
 */
 
#include "xlibInt.h" 

/* 
 * 	XCreateGC -- 
 *
 */

GC
XCreateGC(Display *display, Drawable d, unsigned long mask, XGCValues *values)
{
    GC gc;
    XGCValues *gcv;
    unsigned long fg, bg;
	
    display->request++;

    gc = (GC) ckalloc(sizeof(struct _XGC));
    if (gc == NULL) {
	return None;
    }

    gc->rects = False;
    gcv = &gc->values;

    if (lx_ScreenDepth == 8) {
	fg = display->screens[0].black_pixel;
	bg = display->screens[0].white_pixel;
    } else {
	fg = RGB(0, 0, 0);
	fg = RGB(255, 255, 255);
    }
    gcv->function = 	(mask & GCFunction) 	?values->function	:GXcopy;
    gcv->plane_mask = 	(mask & GCPlaneMask) 	?values->plane_mask 	:~0;
    gcv->foreground = 	(mask & GCForeground) 	?values->foreground 	:fg;
    gcv->background = 	(mask & GCBackground) 	?values->background 	:bg;
    gcv->line_width = 	(mask & GCLineWidth)	?values->line_width	:0;	
    gcv->line_style = 	(mask & GCLineStyle)	?values->line_style	:LineSolid;
    gcv->cap_style =  	(mask & GCCapStyle)	?values->cap_style	:0;
    gcv->join_style = 	(mask & GCJoinStyle)	?values->join_style	:0;
    gcv->fill_style =  	(mask & GCFillStyle)	?values->fill_style	:FillSolid;
    gcv->fill_rule =  	(mask & GCFillRule)	?values->fill_rule	:WindingRule;
    gcv->arc_mode = 	(mask & GCArcMode)	?values->arc_mode	:ArcPieSlice;
    gcv->tile = 	(mask & GCTile)		?values->tile		:None;
    gcv->stipple = 	(mask & GCStipple)	?values->stipple	:None;
    gcv->ts_x_origin = 	(mask & GCTileStipXOrigin)	?values->ts_x_origin:0;
    gcv->ts_y_origin = 	(mask & GCTileStipYOrigin)	?values->ts_y_origin:0;
    gcv->font = 	(mask & GCFont)		?values->font		:None;
    gcv->subwindow_mode = (mask & GCSubwindowMode)?values->subwindow_mode:ClipByChildren;
    gcv->graphics_exposures = (mask & GCGraphicsExposures)?values->graphics_exposures:True;
    gcv->clip_x_origin = (mask & GCClipXOrigin)	?values->clip_x_origin	:0;
    gcv->clip_y_origin = (mask & GCClipYOrigin)	?values->clip_y_origin	:0;
    gcv->clip_mask = 	(mask & GCClipMask)	?values->clip_mask	:None;
    gcv->dash_offset = 	(mask & GCDashOffset)	?values->dash_offset	:0;
    gcv->dashes = 	(mask & GCDashList)	?values->dashes		:4;

    return gc;
}  

/*
 *----------------------------------------------------------------------
 * XCopyGC --
 *
 *      Copy the selected elements of one graphics context to another
 *----------------------------------------------------------------------
 */
 
void
XCopyGC(Display *display, GC src, unsigned long mask, GC dest)
{
 
    display->request++;

    if (mask & GCFunction) dest->values.function = src->values.function;
    if (mask & GCPlaneMask) dest->values.plane_mask = src->values.plane_mask;
    if (mask & GCForeground) dest->values.foreground = src->values.foreground;
    if (mask & GCBackground) dest->values.background = src->values.background;
    if (mask & GCLineWidth) dest->values.line_width = src->values.line_width;
    if (mask & GCLineStyle) dest->values.line_style = src->values.line_style;
    if (mask & GCCapStyle) dest->values.cap_style = src->values.cap_style;
    if (mask & GCJoinStyle) dest->values.join_style = src->values.join_style;
    if (mask & GCFillStyle) dest->values.fill_style = src->values.fill_style;
    if (mask & GCFillRule) dest->values.fill_rule = src->values.fill_rule;
    if (mask & GCTile) dest->values.tile = src->values.tile;
    if (mask & GCStipple) dest->values.stipple = src->values.stipple;
    if (mask & GCTileStipXOrigin) dest->values.ts_x_origin = src->values.ts_x_origin;
    if (mask & GCTileStipYOrigin) dest->values.ts_x_origin = src->values.ts_y_origin;
    if (mask & GCFont) dest->values.font = src->values.font;
    if (mask & GCSubwindowMode) dest->values.subwindow_mode = src->values.subwindow_mode;
    if (mask & GCGraphicsExposures) dest->values.graphics_exposures = src->values.graphics_exposures;
    if (mask & GCClipXOrigin) dest->values.clip_x_origin = src->values.clip_x_origin;
    if (mask & GCClipYOrigin) dest->values.clip_x_origin = src->values.clip_y_origin;
    if (mask & GCClipMask) dest->values.clip_mask = src->values.clip_mask;
    if (mask & GCDashOffset) dest->values.dash_offset = src->values.dash_offset;
    if (mask & GCDashList) dest->values.dashes = src->values.dashes;
    if (mask & GCArcMode) dest->values.arc_mode = src->values.arc_mode;

    if (mask & GCClipMask && src->rects) {
        dest->values.clip_mask = Lx_CopyRegion((Region) src->values.clip_mask);
        dest->rects = True;
    } else {
	dest->rects = False;
    }
}


/*
 *----------------------------------------------------------------------
 * XFreeGC --
 *
 *	Destroy a current graphics context and free up the region if one
 *	is currently in use.
 *----------------------------------------------------------------------
 */

void
XFreeGC(Display *display, GC gc)
{
    display->request++;

    if (gc != None) {
	if (gc->rects == True) {
	    XDestroyRegion((Region) gc->values.clip_mask);
	}
	ckfree(gc);
    }
}   


void
XSetForeground(Display *display, GC gc, unsigned long foreground)
{
    display->request++;

    gc->values.foreground = foreground;
}

void 
XSetBackground(Display *display, GC gc, unsigned long background)
{
    display->request++;

    gc->values.background = background;
}

void
XSetFunction(Display *display, GC gc, int function)
{
    display->request++;

    gc->values.function = function;
}

void
XSetFillRule(Display *display, GC gc, int fill_rule)
{
    display->request++;

    gc->values.fill_rule = fill_rule;
}

void
XSetFillStyle(Display *display, GC gc, int fill_style)
{
    display->request++;

    gc->values.fill_style = fill_style;
}


void
XSetFont(Display *display, GC gc, HFONT font)
{
    display->request++;

    gc->values.font = font;
}

void
XSetTSOrigin(Display *display, GC gc, int x, int y)
{
    display->request++;

    gc->values.ts_x_origin = x;
    gc->values.ts_y_origin = y;
}

void
XSetArcMode(Display *display, GC gc, int arc_mode)
{
    display->request++;

    gc->values.arc_mode = arc_mode;
}

void
XSetStipple(Display *display, GC gc, Pixmap stipple)
{
    display->request++;

    gc->values.stipple = stipple;
}


void
XSetLineAttributes(Display *display, GC gc, unsigned int line_width,
		   int line_style, int cap_style, int join_style)
{
    display->request++;

    gc->values.line_style = line_style;
    gc->values.cap_style = cap_style;
    gc->values.join_style = join_style;
}

void
XSetClipOrigin(Display *display, GC gc, int clip_x_origin, int clip_y_origin)
{
    display->request++;

    gc->values.clip_x_origin = clip_x_origin;
    gc->values.clip_y_origin = clip_y_origin;
}

void
XSetClipMask(Display *display, GC gc, Pixmap pixmap)
{
    if (gc->rects) {
	XDestroyRegion((Region) gc->values.clip_mask);
    }
    gc->rects = False;
    gc->values.clip_mask = pixmap;
}

/* XSetRegion: in xregion.c */

/*
 *----------------------------------------------------------------------
 * XChangeGC --
 *
 *	Change the components of a given graphics context
 *----------------------------------------------------------------------
 */
void
XChangeGC(Display *display, GC gc, unsigned long mask, XGCValues *attr)
{

    display->request++;

    mask &= (1L << (GCLastBit + 1)) - 1;
    if (!mask) {
	return;
    }

    if (mask & GCFunction) {
	gc->values.function = attr->function;
    }

    if (mask & GCPlaneMask) {
	gc->values.plane_mask = attr->plane_mask;
    }

    if (mask & GCForeground) {
	gc->values.foreground = attr->foreground;
    }

    if (mask & GCBackground) {
	gc->values.background = attr->background;
    }

    if (mask & GCLineWidth) {
	gc->values.line_width = attr->line_width;
    }

    if (mask & GCLineStyle) {
	gc->values.line_style = attr->line_style;
    }

    if (mask & GCCapStyle) {
	gc->values.cap_style = attr->cap_style;
    }

    if (mask & GCJoinStyle) {
	gc->values.join_style = attr->join_style;
    }

    if (mask & GCFillStyle) {
	gc->values.fill_style = attr->fill_style;
    }

    if (mask & GCFillRule) {
	gc->values.fill_rule = attr->fill_rule;
    }

    if (mask & GCArcMode) {
	gc->values.arc_mode = attr->arc_mode;
    }

    if (mask & GCTile) {
        gc->values.tile = attr->tile;
    }

    if (mask & GCStipple) {
        gc->values.stipple = attr->stipple;
    }

    if (mask & GCTileStipXOrigin) {
	gc->values.ts_x_origin = attr->ts_x_origin;
    }

    if (mask & GCTileStipYOrigin) {
	gc->values.ts_y_origin = attr->ts_y_origin;
    }

    if (mask & GCFont) {
	gc->values.font = attr->font;
    }

    if (mask & GCSubwindowMode) {
	gc->values.subwindow_mode = attr->subwindow_mode;
    }

    if (mask & GCGraphicsExposures) {
	gc->values.graphics_exposures = attr->graphics_exposures;
    }

    if (mask & GCClipXOrigin) {
	gc->values.clip_x_origin = attr->clip_x_origin;
    }

    if (mask & GCClipYOrigin) {
	gc->values.clip_y_origin = attr->clip_y_origin;
    }

    if (mask & GCClipMask) {
	if (gc->rects) {
	    XDestroyRegion((Region) gc->values.clip_mask);
	}
        gc->values.clip_mask = attr->clip_mask;
        gc->rects = False;
    }

    if (mask & GCDashOffset) {
	gc->values.dash_offset = attr->dash_offset;
    }

    if (mask & GCDashList) {
	gc->values.dashes = attr->dashes;
    }
    return;
}
