/*
 * xkeysym.c --
 *
 * Copyright (c) 1994 Software Research Associates, Inc.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Software Research Associates not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  Software Research
 * Associates makes no representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or implied
 * warranty.
 */
 
#include "xlibInt.h" 
#include "x11/keysym.h"

typedef struct {
    char *name;				/* Name of keysym. */
    KeySym value;			/* Numeric identifier for keysym. */
} KeySymInfo;
static KeySymInfo keyArray[] = {
#include "ks_names.h"
    {(char *) NULL, 0}
};

static Lx_HashTable keySymTable;	/* Hashed form of above structure. */
static int initialized = 0;

static char *keyCodeToString[] = {
    "",				/* 0x000 */
    "",				/* 0x001, VK_LBUTTON */
    "",				/* 0x002, VK_RBUTTON */
    "",				/* 0x003, VK_CANCEL */
    "",				/* 0x004, VK_MBUTTON */
    "",				/* 0x005 */
    "",				/* 0x006 */
    "",				/* 0x007 */
    "\015",			/* 0x008, VK_BACK */
    "\t",			/* 0x009, VK_TAB */
    "",				/* 0x00A */
    "",				/* 0x00B */
    "",				/* 0x00C, VK_CLEAR */
    "\r",			/* 0x00D, VK_RETURN */
    "",				/* 0x00E */
    "",				/* 0x00F */
    "",				/* 0x010, VK_SHIFT */
    "",				/* 0x011, VK_CONTROL */
    "",				/* 0x012, VK_MENU */
    "",				/* 0x013, VK_PAUSE */
    "",				/* 0x014, VK_CAPITAL */
    "",				/* 0x015, VK_KANA */
    "",				/* 0x016 */
    "",				/* 0x017 */
    "",				/* 0x018 */
    "",				/* 0x019, VK_KANJI */
    "",				/* 0x01A */
    "",				/* 0x01B, VK_ESCAPE */
    "",				/* 0x01C, VK_CONVERT */
    "",				/* 0x01D, VK_NONCONVERT */
    "",				/* 0x01E */
    "",				/* 0x01F */
    " ",			/* 0x020, VK_SPACE */
    "",				/* 0x021, VK_PRIOR */
    "",				/* 0x022, VK_NEXT */
    "",				/* 0x023, VK_END */
    "",				/* 0x024, VK_HOME */
    "",				/* 0x025, VK_LEFT */
    "",				/* 0x026, VK_UP */
    "",				/* 0x027, VK_RIGHT */
    "",				/* 0x028, VK_DOWN */
    "",				/* 0x029, VK_SELECT */
    "",				/* 0x02A, VK_PRINT */
    "",				/* 0x02B, VK_EXECUTE */
    "",				/* 0x02C, VK_SNAPSHOT */
    "",				/* 0x02D, VK_INSERT */
    "\037",			/* 0x02E, VK_DELETE */
    "",				/* 0x02F, VK_HELP */
};

static KeySym keyCodeToKeySym[] = {
    NoSymbol,			/* 0x000 */
    NoSymbol,			/* 0x001, VK_LBUTTON */
    NoSymbol,			/* 0x002, VK_RBUTTON */
    XK_Cancel,			/* 0x003, VK_CANCEL */
    NoSymbol,			/* 0x004, VK_MBUTTON */
    NoSymbol,			/* 0x005 */
    NoSymbol,			/* 0x006 */
    NoSymbol,			/* 0x007 */
    XK_BackSpace,		/* 0x008, VK_BACK */
    XK_Tab,			/* 0x009, VK_TAB */
    NoSymbol,			/* 0x00A */
    NoSymbol,			/* 0x00B */
    XK_Clear,			/* 0x00C, VK_CLEAR */
    XK_Return,			/* 0x00D, VK_RETURN */
    NoSymbol,			/* 0x00E */
    NoSymbol,			/* 0x00F */
    XK_Shift_L,			/* 0x010, VK_SHIFT */
    XK_Control_L,		/* 0x011, VK_CONTROL */
    XK_Alt_L,			/* 0x012, VK_MENU */
    XK_Pause,			/* 0x013, VK_PAUSE */
    XK_Caps_Lock,		/* 0x014, VK_CAPITAL */
    XK_Kana_Shift,		/* 0x015, VK_KANA */
    NoSymbol,			/* 0x016 */
    NoSymbol,			/* 0x017 */
    NoSymbol,			/* 0x018 */
    XK_Kanji,			/* 0x019, VK_KANJI */
    NoSymbol,			/* 0x01A */
    XK_Escape,			/* 0x01B, VK_ESCAPE */
    XK_Henkan,			/* 0x01C, VK_CONVERT */
    XK_Muhenkan,		/* 0x01D, VK_NONCONVERT */
    NoSymbol,			/* 0x01E */
    NoSymbol,			/* 0x01F */
    XK_space,			/* 0x020, VK_SPACE */
    XK_Prior,			/* 0x021, VK_PRIOR */
    XK_Next,			/* 0x022, VK_NEXT */
    XK_End,			/* 0x023, VK_END */
    XK_Home,			/* 0x024, VK_HOME */
    XK_Left,			/* 0x025, VK_LEFT */
    XK_Up,			/* 0x026, VK_UP */
    XK_Right,			/* 0x027, VK_RIGHT */
    XK_Down,			/* 0x028, VK_DOWN */
    XK_Select,			/* 0x029, VK_SELECT */
    XK_Print,			/* 0x02A, VK_PRINT */
    XK_Execute,			/* 0x02B, VK_EXECUTE */
    NoSymbol,			/* 0x02C, VK_SNAPSHOT */
    XK_Insert,			/* 0x02D, VK_INSERT */
    XK_Delete,			/* 0x02E, VK_DELETE */
    XK_Help,			/* 0x02F, VK_HELP */
    XK_VoidSymbol,		/* 0x030 */
    XK_VoidSymbol,		/* 0x031 */
    XK_VoidSymbol,		/* 0x032 */
    XK_VoidSymbol,		/* 0x033 */
    XK_VoidSymbol,		/* 0x034 */
    XK_VoidSymbol,		/* 0x035 */
    XK_VoidSymbol,		/* 0x036 */
    XK_VoidSymbol,		/* 0x037 */
    XK_VoidSymbol,		/* 0x038 */
    XK_VoidSymbol,		/* 0x039 */
    XK_VoidSymbol,		/* 0x03A */
    XK_VoidSymbol,		/* 0x03B */
    XK_VoidSymbol,		/* 0x03C */
    XK_VoidSymbol,		/* 0x03D */
    XK_VoidSymbol,		/* 0x03E */
    XK_VoidSymbol,		/* 0x03F */
    XK_VoidSymbol,		/* 0x040 */
    XK_VoidSymbol,		/* 0x041 */
    XK_VoidSymbol,		/* 0x042 */
    XK_VoidSymbol,		/* 0x043 */
    XK_VoidSymbol,		/* 0x044 */
    XK_VoidSymbol,		/* 0x045 */
    XK_VoidSymbol,		/* 0x046 */
    XK_VoidSymbol,		/* 0x047 */
    XK_VoidSymbol,		/* 0x048 */
    XK_VoidSymbol,		/* 0x049 */
    XK_VoidSymbol,		/* 0x04A */
    XK_VoidSymbol,		/* 0x04B */
    XK_VoidSymbol,		/* 0x04C */
    XK_VoidSymbol,		/* 0x04D */
    XK_VoidSymbol,		/* 0x04E */
    XK_VoidSymbol,		/* 0x04F */
    XK_VoidSymbol,		/* 0x050 */
    XK_VoidSymbol,		/* 0x051 */
    XK_VoidSymbol,		/* 0x052 */
    XK_VoidSymbol,		/* 0x053 */
    XK_VoidSymbol,		/* 0x054 */
    XK_VoidSymbol,		/* 0x055 */
    XK_VoidSymbol,		/* 0x056 */
    XK_VoidSymbol,		/* 0x057 */
    XK_VoidSymbol,		/* 0x058 */
    XK_VoidSymbol,		/* 0x059 */
    XK_VoidSymbol,		/* 0x05A */
    NoSymbol,			/* 0x05B, VK_LWIN */
    NoSymbol,			/* 0x05C, VK_RWIN */
    NoSymbol,			/* 0x05D, VK_APPS */
    XK_VoidSymbol,		/* 0x05E */
    XK_VoidSymbol,		/* 0x05F */
    XK_KP_0,			/* 0x060, VK_NUMPAD0 */
    XK_KP_1,			/* 0x061, VK_NUMPAD1 */
    XK_KP_2,			/* 0x062, VK_NUMPAD2 */
    XK_KP_3,			/* 0x063, VK_NUMPAD3 */
    XK_KP_4,			/* 0x064, VK_NUMPAD4 */
    XK_KP_5,			/* 0x065, VK_NUMPAD5 */
    XK_KP_6,			/* 0x066, VK_NUMPAD6 */
    XK_KP_7,			/* 0x067, VK_NUMPAD7 */
    XK_KP_8,			/* 0x068, VK_NUMPAD8 */
    XK_KP_9,			/* 0x069, VK_NUMPAD9 */
    XK_KP_Multiply,		/* 0x06A, VK_MULTIPLY */
    XK_KP_Add,			/* 0x06B, VK_ADD */
    XK_KP_Separator,		/* 0x06C, VK_SEPARATOR */
    XK_KP_Subtract,		/* 0x06D, VK_SUBTRACT */
    XK_VoidSymbol,		/* 0x06E, VK_DECIMAL */
    XK_KP_Divide,		/* 0x06F, VK_DIVIDE */
    XK_F1,			/* 0x070, VK_F1 */
    XK_F2,			/* 0x071, VK_F2 */
    XK_F3,			/* 0x072, VK_F3 */
    XK_F4,			/* 0x073, VK_F4 */
    XK_F5,			/* 0x074, VK_F5 */
    XK_F6,			/* 0x075, VK_F6 */
    XK_F7,			/* 0x076, VK_F7 */
    XK_F8,			/* 0x077, VK_F8 */
    XK_F9,			/* 0x078, VK_F9 */
    XK_F10,			/* 0x079, VK_F10 */
    XK_F11,			/* 0x07A, VK_F11 */
    XK_F12,			/* 0x07B, VK_F12 */
    XK_F13,			/* 0x07C, VK_F13 */
    XK_F14,			/* 0x07D, VK_F14 */
    XK_F15,			/* 0x07E, VK_F15 */
    XK_F16,			/* 0x07F, VK_F16 */
    XK_F17,			/* 0x080, VK_F17 */
    XK_F18,			/* 0x081, VK_F18 */
    XK_F19,			/* 0x082, VK_F19 */
    XK_F20,			/* 0x083, VK_F20 */
    XK_F21,			/* 0x084, VK_F21 */
    XK_F22,			/* 0x085, VK_F22 */
    XK_F23,			/* 0x086, VK_F23 */
    XK_F24,			/* 0x087, VK_F24 */
    NoSymbol,			/* 0x088 */
    NoSymbol,			/* 0x089 */
    NoSymbol,			/* 0x08A */
    NoSymbol,			/* 0x08B */
    NoSymbol,			/* 0x08C */
    NoSymbol,			/* 0x08D */
    NoSymbol,			/* 0x08E */
    NoSymbol,			/* 0x08F */
    XK_Num_Lock,		/* 0x090, VK_NUMLOCK */
    XK_Scroll_Lock,		/* 0x091, VK_SCROLL */
    NoSymbol,			/* 0x092 */
    NoSymbol,			/* 0x093 */
    NoSymbol,			/* 0x094 */
    NoSymbol,			/* 0x095 */
    NoSymbol,			/* 0x096 */
    NoSymbol,			/* 0x097 */
    NoSymbol,			/* 0x098 */
    NoSymbol,			/* 0x099 */
    NoSymbol,			/* 0x09A */
    NoSymbol,			/* 0x09B */
    NoSymbol,			/* 0x09C */
    NoSymbol,			/* 0x09D */
    NoSymbol,			/* 0x09E */
    NoSymbol,			/* 0x09F */
    XK_Shift_L,			/* 0x0A0, VK_LSHIFT */
    XK_Shift_R,			/* 0x0A1, VK_RSHIFT */
    XK_Control_L,		/* 0x0A2, VK_LCONTROL */
    XK_Control_R,		/* 0x0A3, VK_RCONTROL */
    XK_Alt_L,			/* 0x0A4, VK_LMENU */
    XK_Alt_R,			/* 0x0A5, VK_RMENU */
    NoSymbol,			/* 0x0A6 */
    NoSymbol,			/* 0x0A7 */
    NoSymbol,			/* 0x0A8 */
    NoSymbol,			/* 0x0A9 */
    NoSymbol,			/* 0x0AA */
    NoSymbol,			/* 0x0AB */
    NoSymbol,			/* 0x0AC */
    NoSymbol,			/* 0x0AD */
    NoSymbol,			/* 0x0AE */
    NoSymbol,			/* 0x0AF */
    NoSymbol,			/* 0x0B0 */
    NoSymbol,			/* 0x0B1 */
    NoSymbol,			/* 0x0B2 */
    NoSymbol,			/* 0x0B3 */
    NoSymbol,			/* 0x0B4 */
    NoSymbol,			/* 0x0B5 */
    NoSymbol,			/* 0x0B6 */
    NoSymbol,			/* 0x0B7 */
    NoSymbol,			/* 0x0B8 */
    NoSymbol,			/* 0x0B9 */
    XK_semicolon,		/* 0x0BA */
    XK_equal,			/* 0x0BB */
    XK_comma,			/* 0x0BC */
    XK_minus,			/* 0x0BD */
    XK_period,			/* 0x0BE */
    XK_slash,			/* 0x0BF */
    XK_grave,			/* 0x0C0 */
    NoSymbol,			/* 0x0C1 */
    NoSymbol,			/* 0x0C2 */
    NoSymbol,			/* 0x0C3 */
    NoSymbol,			/* 0x0C4 */
    NoSymbol,			/* 0x0C5 */
    NoSymbol,			/* 0x0C6 */
    NoSymbol,			/* 0x0C7 */
    NoSymbol,			/* 0x0C8 */
    NoSymbol,			/* 0x0C9 */
    NoSymbol,			/* 0x0CA */
    NoSymbol,			/* 0x0CB */
    NoSymbol,			/* 0x0CC */
    NoSymbol,			/* 0x0CD */
    NoSymbol,			/* 0x0CE */
    NoSymbol,			/* 0x0CF */
    NoSymbol,			/* 0x0D0 */
    NoSymbol,			/* 0x0D1 */
    NoSymbol,			/* 0x0D2 */
    NoSymbol,			/* 0x0D3 */
    NoSymbol,			/* 0x0D4 */
    NoSymbol,			/* 0x0D5 */
    NoSymbol,			/* 0x0D6 */
    NoSymbol,			/* 0x0D7 */
    NoSymbol,			/* 0x0D8 */
    NoSymbol,			/* 0x0D9 */
    NoSymbol,			/* 0x0DA */
    XK_bracketleft,		/* 0x0DB */
    XK_backslash,		/* 0x0DC */
    XK_bracketright,		/* 0x0DD */
    XK_apostrophe,		/* 0x0DE */
    NoSymbol,			/* 0x0DF */
    NoSymbol,			/* 0x0E0 */
    NoSymbol,			/* 0x0E1 */
    NoSymbol,			/* 0x0E2 */
    NoSymbol,			/* 0x0E3 */
    NoSymbol,			/* 0x0E4 */
    NoSymbol,			/* 0x0E5 */
    NoSymbol,			/* 0x0E6 */
    NoSymbol,			/* 0x0E7 */
    NoSymbol,			/* 0x0E8 */
    NoSymbol,			/* 0x0E9 */
    NoSymbol,			/* 0x0EA */
    NoSymbol,			/* 0x0EB */
    NoSymbol,			/* 0x0EC */
    NoSymbol,			/* 0x0ED */
    NoSymbol,			/* 0x0EE */
    NoSymbol,			/* 0x0EF */
    NoSymbol,			/* 0x0F0 */
    NoSymbol,			/* 0x0F1 */
    NoSymbol,			/* 0x0F2 */
    NoSymbol,			/* 0x0F3 */
    NoSymbol,			/* 0x0F4 */
    NoSymbol,			/* 0x0F5 */
    XK_3270_Attn,		/* 0x0F6, VK_ATTN */
    XK_3270_CursorSelect,	/* 0x0F7, VK_CRSEL */
    XK_3270_ExSelect,		/* 0x0F8, VK_EXSEL */
    XK_3270_EraseEOF,		/* 0x0F9, VK_EREOF */
    XK_3270_Play,		/* 0x0FA, VK_PLAY */
    NoSymbol,			/* 0x0FB, VK_ZOOM */
    NoSymbol,			/* 0x0FC, VK_NONAME */
    XK_3270_PA1,		/* 0x0FD, VK_PA1 */
    NoSymbol,			/* 0x0FE, VK_OEM_CLEAR */
    NoSymbol,			/* 0x0FF */

    /* Now start the shifted keysyms */

    NoSymbol,			/* 0x100 */
    NoSymbol,			/* 0x101, VK_LBUTTON */
    NoSymbol,			/* 0x102, VK_RBUTTON */
    XK_Cancel,			/* 0x103, VK_CANCEL */
    NoSymbol,			/* 0x104, VK_MBUTTON */
    NoSymbol,			/* 0x105 */
    NoSymbol,			/* 0x106 */
    NoSymbol,			/* 0x107 */
    XK_BackSpace,		/* 0x108, VK_BACK */
    XK_Tab,			/* 0x109, VK_TAB */
    NoSymbol,			/* 0x10A */
    NoSymbol,			/* 0x10B */
    XK_Clear,			/* 0x10C, VK_CLEAR */
    XK_Return,			/* 0x10D, VK_RETURN */
    NoSymbol,			/* 0x10E */
    NoSymbol,			/* 0x10F */
    NoSymbol,			/* 0x110, VK_SHIFT */
    XK_Control_L,		/* 0x111, VK_CONTROL */
    XK_Alt_L,			/* 0x112, VK_MENU */
    XK_Break,			/* 0x113, VK_PAUSE */
    XK_Caps_Lock,		/* 0x114, VK_CAPITAL */
    XK_Kana_Shift,		/* 0x115, VK_KANA */
    NoSymbol,			/* 0x116 */
    NoSymbol,			/* 0x117 */
    NoSymbol,			/* 0x118 */
    XK_Kanji,			/* 0x119, VK_KANJI */
    NoSymbol,			/* 0x11A */
    XK_Escape,			/* 0x11B, VK_ESCAPE */
    XK_Henkan,			/* 0x11C, VK_CONVERT */
    XK_Muhenkan,		/* 0x11D, VK_NONCONVERT */
    NoSymbol,			/* 0x11E */
    NoSymbol,			/* 0x11F */
    XK_space,			/* 0x120, VK_SPACE */
    XK_Prior,			/* 0x121, VK_PRIOR */
    XK_Next,			/* 0x122, VK_NEXT */
    XK_End,			/* 0x123, VK_END */
    XK_Home,			/* 0x124, VK_HOME */
    XK_Left,			/* 0x125, VK_LEFT */
    XK_Up,			/* 0x126, VK_UP */
    XK_Right,			/* 0x127, VK_RIGHT */
    XK_Down,			/* 0x128, VK_DOWN */
    XK_Select,			/* 0x129, VK_SELECT */
    XK_Sys_Req,			/* 0x12A, VK_PRINT */
    XK_Execute,			/* 0x12B, VK_EXECUTE */
    NoSymbol,			/* 0x12C, VK_SNAPSHOT */
    XK_Insert,			/* 0x12D, VK_INSERT */
    XK_Delete,			/* 0x12E, VK_DELETE */
    XK_Help,			/* 0x12F, VK_HELP */
    XK_parenright,		/* 0x130 */
    XK_exclam,			/* 0x131 */
    XK_at,			/* 0x132 */
    XK_numbersign,		/* 0x133 */
    XK_dollar,			/* 0x134 */
    XK_percent,			/* 0x135 */
    XK_upcaret,			/* 0x136 */
    XK_ampersand,		/* 0x137 */
    XK_asterisk,		/* 0x138 */
    XK_parenleft,		/* 0x139 */
    NoSymbol,			/* 0x13A */
    NoSymbol,			/* 0x13B */
    XK_less,			/* 0x13C */
    NoSymbol,			/* 0x13D */
    XK_greater,			/* 0x13E */
    NoSymbol,			/* 0x13F */
    NoSymbol,			/* 0x140 */
    XK_A,			/* 0x141 */
    XK_B,			/* 0x142 */
    XK_C,			/* 0x143 */
    XK_D,			/* 0x144 */
    XK_E,			/* 0x145 */
    XK_F,			/* 0x146 */
    XK_G,			/* 0x147 */
    XK_H,			/* 0x148 */
    XK_I,			/* 0x149 */
    XK_J,			/* 0x14A */
    XK_K,			/* 0x14B */
    XK_L,			/* 0x14C */
    XK_M,			/* 0x14D */
    XK_N,			/* 0x14E */
    XK_O,			/* 0x14F */
    XK_P,			/* 0x150 */
    XK_Q,			/* 0x151 */
    XK_R,			/* 0x152 */
    XK_S,			/* 0x153 */
    XK_T,			/* 0x154 */
    XK_U,			/* 0x155 */
    XK_V,			/* 0x156 */
    XK_W,			/* 0x157 */
    XK_X,			/* 0x158 */
    XK_Y,			/* 0x159 */
    XK_Z,			/* 0x15A */
    NoSymbol,			/* 0x15B, VK_LWIN */
    NoSymbol,			/* 0x15C, VK_RWIN */
    NoSymbol,			/* 0x15D, VK_APPS */
    XK_VoidSymbol,		/* 0x15E */
    XK_VoidSymbol,		/* 0x15F */
    XK_KP_0,			/* 0x160, VK_NUMPAD0 */
    XK_KP_1,			/* 0x161, VK_NUMPAD1 */
    XK_KP_2,			/* 0x162, VK_NUMPAD2 */
    XK_KP_3,			/* 0x163, VK_NUMPAD3 */
    XK_KP_4,			/* 0x164, VK_NUMPAD4 */
    XK_KP_5,			/* 0x165, VK_NUMPAD5 */
    XK_KP_6,			/* 0x166, VK_NUMPAD6 */
    XK_KP_7,			/* 0x167, VK_NUMPAD7 */
    XK_KP_8,			/* 0x168, VK_NUMPAD8 */
    XK_KP_9,			/* 0x169, VK_NUMPAD9 */
    XK_KP_Multiply,		/* 0x16A, VK_MULTIPLY */
    XK_KP_Add,			/* 0x16B, VK_ADD */
    XK_KP_Separator,		/* 0x16C, VK_SEPARATOR */
    XK_KP_Subtract,		/* 0x16D, VK_SUBTRACT */
    XK_VoidSymbol,		/* 0x16E, VK_DECIMAL */
    XK_KP_Divide,		/* 0x16F, VK_DIVIDE */
    XK_F1,			/* 0x170, VK_F1 */
    XK_F2,			/* 0x171, VK_F2 */
    XK_F3,			/* 0x172, VK_F3 */
    XK_F4,			/* 0x173, VK_F4 */
    XK_F5,			/* 0x174, VK_F5 */
    XK_F6,			/* 0x175, VK_F6 */
    XK_F7,			/* 0x176, VK_F7 */
    XK_F8,			/* 0x177, VK_F8 */
    XK_F9,			/* 0x178, VK_F9 */
    XK_F10,			/* 0x179, VK_F10 */
    XK_F11,			/* 0x17A, VK_F11 */
    XK_F12,			/* 0x17B, VK_F12 */
    XK_F13,			/* 0x17C, VK_F13 */
    XK_F14,			/* 0x17D, VK_F14 */
    XK_F15,			/* 0x17E, VK_F15 */
    XK_F16,			/* 0x17F, VK_F16 */
    XK_F17,			/* 0x180, VK_F17 */
    XK_F18,			/* 0x181, VK_F18 */
    XK_F19,			/* 0x182, VK_F19 */
    XK_F20,			/* 0x183, VK_F20 */
    XK_F21,			/* 0x184, VK_F21 */
    XK_F22,			/* 0x185, VK_F22 */
    XK_F23,			/* 0x186, VK_F23 */
    XK_F24,			/* 0x187, VK_F24 */
    NoSymbol,			/* 0x188 */
    NoSymbol,			/* 0x189 */
    NoSymbol,			/* 0x18A */
    NoSymbol,			/* 0x18B */
    NoSymbol,			/* 0x18C */
    NoSymbol,			/* 0x18D */
    NoSymbol,			/* 0x18E */
    NoSymbol,			/* 0x18F */
    XK_Num_Lock,		/* 0x190, VK_NUMLOCK */
    XK_Scroll_Lock,		/* 0x191, VK_SCROLL */
    NoSymbol,			/* 0x192 */
    NoSymbol,			/* 0x193 */
    NoSymbol,			/* 0x194 */
    NoSymbol,			/* 0x195 */
    NoSymbol,			/* 0x196 */
    NoSymbol,			/* 0x197 */
    NoSymbol,			/* 0x198 */
    NoSymbol,			/* 0x199 */
    NoSymbol,			/* 0x19A */
    NoSymbol,			/* 0x19B */
    NoSymbol,			/* 0x19C */
    NoSymbol,			/* 0x19D */
    NoSymbol,			/* 0x19E */
    NoSymbol,			/* 0x19F */
    XK_Shift_L,			/* 0x1A0, VK_LSHIFT */
    XK_Shift_R,			/* 0x1A1, VK_RSHIFT */
    XK_Control_L,		/* 0x1A2, VK_LCONTROL */
    XK_Control_R,		/* 0x1A3, VK_RCONTROL */
    XK_Alt_L,			/* 0x1A4, VK_LMENU */
    XK_Alt_R,			/* 0x1A5, VK_RMENU */
    NoSymbol,			/* 0x1A6 */
    NoSymbol,			/* 0x1A7 */
    NoSymbol,			/* 0x1A8 */
    NoSymbol,			/* 0x1A9 */
    NoSymbol,			/* 0x1AA */
    NoSymbol,			/* 0x1AB */
    NoSymbol,			/* 0x1AC */
    NoSymbol,			/* 0x1AD */
    NoSymbol,			/* 0x1AE */
    NoSymbol,			/* 0x1AF */
    NoSymbol,			/* 0x1B0 */
    NoSymbol,			/* 0x1B1 */
    NoSymbol,			/* 0x1B2 */
    NoSymbol,			/* 0x1B3 */
    NoSymbol,			/* 0x1B4 */
    NoSymbol,			/* 0x1B5 */
    NoSymbol,			/* 0x1B6 */
    NoSymbol,			/* 0x1B7 */
    NoSymbol,			/* 0x1B8 */
    NoSymbol,			/* 0x1B9 */
    XK_colon,			/* 0x1BA */
    XK_plus,			/* 0x1BB */
    XK_less,			/* 0x1BC */
    XK_underscore,		/* 0x1BD */
    XK_greater,			/* 0x1BE */
    XK_question,		/* 0x1BF */
    XK_grave,			/* 0x1C0 */
    NoSymbol,			/* 0x1C1 */
    NoSymbol,			/* 0x1C2 */
    NoSymbol,			/* 0x1C3 */
    NoSymbol,			/* 0x1C4 */
    NoSymbol,			/* 0x1C5 */
    NoSymbol,			/* 0x1C6 */
    NoSymbol,			/* 0x1C7 */
    NoSymbol,			/* 0x1C8 */
    NoSymbol,			/* 0x1C9 */
    NoSymbol,			/* 0x1CA */
    NoSymbol,			/* 0x1CB */
    NoSymbol,			/* 0x1CC */
    NoSymbol,			/* 0x1CD */
    NoSymbol,			/* 0x1CE */
    NoSymbol,			/* 0x1CF */
    NoSymbol,			/* 0x1D0 */
    NoSymbol,			/* 0x1D1 */
    NoSymbol,			/* 0x1D2 */
    NoSymbol,			/* 0x1D3 */
    NoSymbol,			/* 0x1D4 */
    NoSymbol,			/* 0x1D5 */
    NoSymbol,			/* 0x1D6 */
    NoSymbol,			/* 0x1D7 */
    NoSymbol,			/* 0x1D8 */
    NoSymbol,			/* 0x1D9 */
    NoSymbol,			/* 0x1DA */
    XK_braceleft,		/* 0x1DB */
    XK_bar,			/* 0x1DC */
    XK_braceright,		/* 0x1DD */
    XK_quotedbl,		/* 0x1DE */
    NoSymbol,			/* 0x1DF */
    NoSymbol,			/* 0x1E0 */
    NoSymbol,			/* 0x1E1 */
    NoSymbol,			/* 0x1E2 */
    NoSymbol,			/* 0x1E3 */
    NoSymbol,			/* 0x1E4 */
    NoSymbol,			/* 0x1E5 */
    NoSymbol,			/* 0x1E6 */
    NoSymbol,			/* 0x1E7 */
    NoSymbol,			/* 0x1E8 */
    NoSymbol,			/* 0x1E9 */
    NoSymbol,			/* 0x1EA */
    NoSymbol,			/* 0x1EB */
    NoSymbol,			/* 0x1EC */
    NoSymbol,			/* 0x1ED */
    NoSymbol,			/* 0x1EE */
    NoSymbol,			/* 0x1EF */
    NoSymbol,			/* 0x1F0 */
    NoSymbol,			/* 0x1F1 */
    NoSymbol,			/* 0x1F2 */
    NoSymbol,			/* 0x1F3 */
    NoSymbol,			/* 0x1F4 */
    NoSymbol,			/* 0x1F5 */
    XK_3270_Attn,		/* 0x1F6, VK_ATTN */
    XK_3270_CursorSelect,	/* 0x1F7, VK_CRSEL */
    XK_3270_ExSelect,		/* 0x1F8, VK_EXSEL */
    XK_3270_EraseEOF,		/* 0x1F9, VK_EREOF */
    XK_3270_Play,		/* 0x1FA, VK_PLAY */
    NoSymbol,			/* 0x1FB, VK_ZOOM */
    NoSymbol,			/* 0x1FC, VK_NONAME */
    XK_3270_PA1,		/* 0x1FD, VK_PA1 */
    NoSymbol,			/* 0x1FE, VK_OEM_CLEAR */
    NoSymbol,			/* 0x1FF */
};

static KeySym keyCodeForKeySym[] = {
    NoSymbol,			/* 0x000 */
    NoSymbol,			/* 0x001, VK_LBUTTON */
    NoSymbol,			/* 0x002, VK_RBUTTON */
    XK_Cancel,			/* 0x003, VK_CANCEL */
    NoSymbol,			/* 0x004, VK_MBUTTON */
    NoSymbol,			/* 0x005 */
    NoSymbol,			/* 0x006 */
    NoSymbol,			/* 0x007 */
    XK_BackSpace,		/* 0x008, VK_BACK */
    XK_Tab,			/* 0x009, VK_TAB */
    NoSymbol,			/* 0x00A */
    NoSymbol,			/* 0x00B */
    XK_Clear,			/* 0x00C, VK_CLEAR */
    XK_Return,			/* 0x00D, VK_RETURN */
    NoSymbol,			/* 0x00E */
    NoSymbol,			/* 0x00F */
    XK_Shift_L,			/* 0x010, VK_SHIFT */
    XK_Control_L,		/* 0x011, VK_CONTROL */
    XK_Alt_L,			/* 0x012, VK_MENU */
    XK_Pause,			/* 0x013, VK_PAUSE */
    XK_Caps_Lock,		/* 0x014, VK_CAPITAL */
    XK_Kana_Shift,		/* 0x015, VK_KANA */
    NoSymbol,			/* 0x016 */
    NoSymbol,			/* 0x017 */
    NoSymbol,			/* 0x018 */
    XK_Kanji,			/* 0x019, VK_KANJI */
    NoSymbol,			/* 0x01A */
    XK_Escape,			/* 0x01B, VK_ESCAPE */
    XK_Henkan,			/* 0x01C, VK_CONVERT */
    XK_Muhenkan,		/* 0x01D, VK_NONCONVERT */
    NoSymbol,			/* 0x01E */
    NoSymbol,			/* 0x01F */
    XK_space,			/* 0x020, VK_SPACE */
    XK_Prior,			/* 0x021, VK_PRIOR */
    XK_Next,			/* 0x022, VK_NEXT */
    XK_End,			/* 0x023, VK_END */
    XK_Home,			/* 0x024, VK_HOME */
    XK_Left,			/* 0x025, VK_LEFT */
    XK_Up,			/* 0x026, VK_UP */
    XK_Right,			/* 0x027, VK_RIGHT */
    XK_Down,			/* 0x028, VK_DOWN */
    XK_Select,			/* 0x029, VK_SELECT */
    XK_Print,			/* 0x02A, VK_PRINT */
    XK_Execute,			/* 0x02B, VK_EXECUTE */
    NoSymbol,			/* 0x02C, VK_SNAPSHOT */
    XK_Insert,			/* 0x02D, VK_INSERT */
    XK_Delete,			/* 0x02E, VK_DELETE */
    XK_Help,			/* 0x02F, VK_HELP */
    XK_0,			/* 0x030 */
    XK_1,			/* 0x031 */
    XK_2,			/* 0x032 */
    XK_3,			/* 0x033 */
    XK_4,			/* 0x034 */
    XK_5,			/* 0x035 */
    XK_6,			/* 0x036 */
    XK_7,			/* 0x037 */
    XK_8,			/* 0x038 */
    XK_9,			/* 0x039 */
    NoSymbol,			/* 0x03A */
    NoSymbol,			/* 0x03B */
    XK_less,			/* 0x03C */
    NoSymbol,			/* 0x03D */
    XK_greater,			/* 0x03E */
    NoSymbol,			/* 0x03F */
    XK_at,			/* 0x040 */
    XK_a,			/* 0x041 */
    XK_b,			/* 0x042 */
    XK_c,			/* 0x043 */
    XK_d,			/* 0x044 */
    XK_e,			/* 0x045 */
    XK_f,			/* 0x046 */
    XK_g,			/* 0x047 */
    XK_h,			/* 0x048 */
    XK_i,			/* 0x049 */
    XK_j,			/* 0x04A */
    XK_k,			/* 0x04B */
    XK_l,			/* 0x04C */
    XK_m,			/* 0x04D */
    XK_n,			/* 0x04E */
    XK_o,			/* 0x04F */
    XK_p,			/* 0x050 */
    XK_q,			/* 0x051 */
    XK_r,			/* 0x052 */
    XK_s,			/* 0x053 */
    XK_t,			/* 0x054 */
    XK_u,			/* 0x055 */
    XK_v,			/* 0x056 */
    XK_w,			/* 0x057 */
    XK_x,			/* 0x058 */
    XK_y,			/* 0x059 */
    XK_z,			/* 0x05A */
    NoSymbol,			/* 0x05B, VK_LWIN */
    NoSymbol,			/* 0x05C, VK_RWIN */
    NoSymbol,			/* 0x05D, VK_APPS */
    XK_VoidSymbol,		/* 0x05E */
    XK_VoidSymbol,		/* 0x05F */
    XK_VoidSymbol,		/* 0x060, VK_NUMPAD0 */
    XK_VoidSymbol,		/* 0x061, VK_NUMPAD1 */
    XK_VoidSymbol,		/* 0x062, VK_NUMPAD2 */
    XK_VoidSymbol,		/* 0x063, VK_NUMPAD3 */
    XK_VoidSymbol,		/* 0x064, VK_NUMPAD4 */
    XK_VoidSymbol,		/* 0x065, VK_NUMPAD5 */
    XK_VoidSymbol,		/* 0x066, VK_NUMPAD6 */
    XK_VoidSymbol,		/* 0x067, VK_NUMPAD7 */
    XK_VoidSymbol,		/* 0x068, VK_NUMPAD8 */
    XK_VoidSymbol,		/* 0x069, VK_NUMPAD9 */
    XK_KP_Multiply,		/* 0x06A, VK_MULTIPLY */
    XK_KP_Add,			/* 0x06B, VK_ADD */
    XK_KP_Separator,		/* 0x06C, VK_SEPARATOR */
    XK_KP_Subtract,		/* 0x06D, VK_SUBTRACT */
    XK_VoidSymbol,		/* 0x06E, VK_DECIMAL */
    XK_KP_Divide,		/* 0x06F, VK_DIVIDE */
    XK_F1,			/* 0x070, VK_F1 */
    XK_F2,			/* 0x071, VK_F2 */
    XK_F3,			/* 0x072, VK_F3 */
    XK_F4,			/* 0x073, VK_F4 */
    XK_F5,			/* 0x074, VK_F5 */
    XK_F6,			/* 0x075, VK_F6 */
    XK_F7,			/* 0x076, VK_F7 */
    XK_F8,			/* 0x077, VK_F8 */
    XK_F9,			/* 0x078, VK_F9 */
    XK_F10,			/* 0x079, VK_F10 */
    XK_F11,			/* 0x07A, VK_F11 */
    XK_F12,			/* 0x07B, VK_F12 */
    XK_F13,			/* 0x07C, VK_F13 */
    XK_F14,			/* 0x07D, VK_F14 */
    XK_F15,			/* 0x07E, VK_F15 */
    XK_F16,			/* 0x07F, VK_F16 */
    XK_F17,			/* 0x080, VK_F17 */
    XK_F18,			/* 0x081, VK_F18 */
    XK_F19,			/* 0x082, VK_F19 */
    XK_F20,			/* 0x083, VK_F20 */
    XK_F21,			/* 0x084, VK_F21 */
    XK_F22,			/* 0x085, VK_F22 */
    XK_F23,			/* 0x086, VK_F23 */
    XK_F24,			/* 0x087, VK_F24 */
    NoSymbol,			/* 0x088 */
    NoSymbol,			/* 0x089 */
    NoSymbol,			/* 0x08A */
    NoSymbol,			/* 0x08B */
    NoSymbol,			/* 0x08C */
    NoSymbol,			/* 0x08D */
    NoSymbol,			/* 0x08E */
    NoSymbol,			/* 0x08F */
    XK_Num_Lock,		/* 0x090, VK_NUMLOCK */
    XK_Scroll_Lock,		/* 0x091, VK_SCROLL */
    NoSymbol,			/* 0x092 */
    NoSymbol,			/* 0x093 */
    NoSymbol,			/* 0x094 */
    NoSymbol,			/* 0x095 */
    NoSymbol,			/* 0x096 */
    NoSymbol,			/* 0x097 */
    NoSymbol,			/* 0x098 */
    NoSymbol,			/* 0x099 */
    NoSymbol,			/* 0x09A */
    NoSymbol,			/* 0x09B */
    NoSymbol,			/* 0x09C */
    NoSymbol,			/* 0x09D */
    NoSymbol,			/* 0x09E */
    NoSymbol,			/* 0x09F */
    XK_Shift_L,			/* 0x0A0, VK_LSHIFT */
    XK_Shift_R,			/* 0x0A1, VK_RSHIFT */
    XK_Control_L,		/* 0x0A2, VK_LCONTROL */
    XK_Control_R,		/* 0x0A3, VK_RCONTROL */
    XK_Alt_L,			/* 0x0A4, VK_LMENU */
    XK_Alt_R,			/* 0x0A5, VK_RMENU */
    NoSymbol,			/* 0x0A6 */
    NoSymbol,			/* 0x0A7 */
    NoSymbol,			/* 0x0A8 */
    NoSymbol,			/* 0x0A9 */
    NoSymbol,			/* 0x0AA */
    NoSymbol,			/* 0x0AB */
    NoSymbol,			/* 0x0AC */
    NoSymbol,			/* 0x0AD */
    NoSymbol,			/* 0x0AE */
    NoSymbol,			/* 0x0AF */
    NoSymbol,			/* 0x0B0 */
    NoSymbol,			/* 0x0B1 */
    NoSymbol,			/* 0x0B2 */
    NoSymbol,			/* 0x0B3 */
    NoSymbol,			/* 0x0B4 */
    NoSymbol,			/* 0x0B5 */
    NoSymbol,			/* 0x0B6 */
    NoSymbol,			/* 0x0B7 */
    NoSymbol,			/* 0x0B8 */
    NoSymbol,			/* 0x0B9 */
    XK_semicolon,		/* 0x0BA */
    XK_equal,			/* 0x0BB */
    XK_comma,			/* 0x0BC */
    XK_minus,			/* 0x0BD */
    XK_period,			/* 0x0BE */
    XK_slash,			/* 0x0BF */
    XK_grave,			/* 0x0C0 */
    NoSymbol,			/* 0x0C1 */
    NoSymbol,			/* 0x0C2 */
    NoSymbol,			/* 0x0C3 */
    NoSymbol,			/* 0x0C4 */
    NoSymbol,			/* 0x0C5 */
    NoSymbol,			/* 0x0C6 */
    NoSymbol,			/* 0x0C7 */
    NoSymbol,			/* 0x0C8 */
    NoSymbol,			/* 0x0C9 */
    NoSymbol,			/* 0x0CA */
    NoSymbol,			/* 0x0CB */
    NoSymbol,			/* 0x0CC */
    NoSymbol,			/* 0x0CD */
    NoSymbol,			/* 0x0CE */
    NoSymbol,			/* 0x0CF */
    NoSymbol,			/* 0x0D0 */
    NoSymbol,			/* 0x0D1 */
    NoSymbol,			/* 0x0D2 */
    NoSymbol,			/* 0x0D3 */
    NoSymbol,			/* 0x0D4 */
    NoSymbol,			/* 0x0D5 */
    NoSymbol,			/* 0x0D6 */
    NoSymbol,			/* 0x0D7 */
    NoSymbol,			/* 0x0D8 */
    NoSymbol,			/* 0x0D9 */
    NoSymbol,			/* 0x0DA */
    XK_bracketleft,		/* 0x0DB */
    XK_backslash,		/* 0x0DC */
    XK_bracketright,		/* 0x0DD */
    XK_apostrophe,		/* 0x0DE */
    NoSymbol,			/* 0x0DF */
    NoSymbol,			/* 0x0E0 */
    NoSymbol,			/* 0x0E1 */
    NoSymbol,			/* 0x0E2 */
    NoSymbol,			/* 0x0E3 */
    NoSymbol,			/* 0x0E4 */
    NoSymbol,			/* 0x0E5 */
    NoSymbol,			/* 0x0E6 */
    NoSymbol,			/* 0x0E7 */
    NoSymbol,			/* 0x0E8 */
    NoSymbol,			/* 0x0E9 */
    NoSymbol,			/* 0x0EA */
    NoSymbol,			/* 0x0EB */
    NoSymbol,			/* 0x0EC */
    NoSymbol,			/* 0x0ED */
    NoSymbol,			/* 0x0EE */
    NoSymbol,			/* 0x0EF */
    NoSymbol,			/* 0x0F0 */
    NoSymbol,			/* 0x0F1 */
    NoSymbol,			/* 0x0F2 */
    NoSymbol,			/* 0x0F3 */
    NoSymbol,			/* 0x0F4 */
    NoSymbol,			/* 0x0F5 */
    XK_3270_Attn,		/* 0x0F6, VK_ATTN */
    XK_3270_CursorSelect,	/* 0x0F7, VK_CRSEL */
    XK_3270_ExSelect,		/* 0x0F8, VK_EXSEL */
    XK_3270_EraseEOF,		/* 0x0F9, VK_EREOF */
    XK_3270_Play,		/* 0x0FA, VK_PLAY */
    NoSymbol,			/* 0x0FB, VK_ZOOM */
    NoSymbol,			/* 0x0FC, VK_NONAME */
    XK_3270_PA1,		/* 0x0FD, VK_PA1 */
    NoSymbol,			/* 0x0FE, VK_OEM_CLEAR */
    NoSymbol,			/* 0x0FF */

    /* Now start the shifted keysyms */

    NoSymbol,			/* 0x100 */
    NoSymbol,			/* 0x101, VK_LBUTTON */
    NoSymbol,			/* 0x102, VK_RBUTTON */
    XK_Cancel,			/* 0x103, VK_CANCEL */
    NoSymbol,			/* 0x104, VK_MBUTTON */
    NoSymbol,			/* 0x105 */
    NoSymbol,			/* 0x106 */
    NoSymbol,			/* 0x107 */
    XK_BackSpace,		/* 0x108, VK_BACK */
    XK_Tab,			/* 0x109, VK_TAB */
    NoSymbol,			/* 0x10A */
    NoSymbol,			/* 0x10B */
    XK_Clear,			/* 0x10C, VK_CLEAR */
    XK_Return,			/* 0x10D, VK_RETURN */
    NoSymbol,			/* 0x10E */
    NoSymbol,			/* 0x10F */
    NoSymbol,			/* 0x110, VK_SHIFT */
    XK_Control_L,		/* 0x111, VK_CONTROL */
    XK_Alt_L,			/* 0x112, VK_MENU */
    XK_Break,			/* 0x113, VK_PAUSE */
    XK_Caps_Lock,		/* 0x114, VK_CAPITAL */
    XK_Kana_Shift,		/* 0x115, VK_KANA */
    NoSymbol,			/* 0x116 */
    NoSymbol,			/* 0x117 */
    NoSymbol,			/* 0x118 */
    XK_Kanji,			/* 0x119, VK_KANJI */
    NoSymbol,			/* 0x11A */
    XK_Escape,			/* 0x11B, VK_ESCAPE */
    XK_Henkan,			/* 0x11C, VK_CONVERT */
    XK_Muhenkan,		/* 0x11D, VK_NONCONVERT */
    NoSymbol,			/* 0x11E */
    NoSymbol,			/* 0x11F */
    XK_space,			/* 0x120, VK_SPACE */
    XK_Prior,			/* 0x121, VK_PRIOR */
    XK_Next,			/* 0x122, VK_NEXT */
    XK_End,			/* 0x123, VK_END */
    XK_Home,			/* 0x124, VK_HOME */
    XK_Left,			/* 0x125, VK_LEFT */
    XK_Up,			/* 0x126, VK_UP */
    XK_Right,			/* 0x127, VK_RIGHT */
    XK_Down,			/* 0x128, VK_DOWN */
    XK_Select,			/* 0x129, VK_SELECT */
    XK_Sys_Req,			/* 0x12A, VK_PRINT */
    XK_Execute,			/* 0x12B, VK_EXECUTE */
    NoSymbol,			/* 0x12C, VK_SNAPSHOT */
    XK_Insert,			/* 0x12D, VK_INSERT */
    XK_Delete,			/* 0x12E, VK_DELETE */
    XK_Help,			/* 0x12F, VK_HELP */
    XK_parenright,		/* 0x130 */
    XK_exclam,			/* 0x131 */
    XK_at,			/* 0x132 */
    XK_numbersign,		/* 0x133 */
    XK_dollar,			/* 0x134 */
    XK_percent,			/* 0x135 */
    XK_upcaret,			/* 0x136 */
    XK_ampersand,		/* 0x137 */
    XK_asterisk,		/* 0x138 */
    XK_parenleft,		/* 0x139 */
    NoSymbol,			/* 0x13A */
    NoSymbol,			/* 0x13B */
    XK_less,			/* 0x13C */
    NoSymbol,			/* 0x13D */
    XK_greater,			/* 0x13E */
    NoSymbol,			/* 0x13F */
    NoSymbol,			/* 0x140 */
    XK_A,			/* 0x141 */
    XK_B,			/* 0x142 */
    XK_C,			/* 0x143 */
    XK_D,			/* 0x144 */
    XK_E,			/* 0x145 */
    XK_F,			/* 0x146 */
    XK_G,			/* 0x147 */
    XK_H,			/* 0x148 */
    XK_I,			/* 0x149 */
    XK_J,			/* 0x14A */
    XK_K,			/* 0x14B */
    XK_L,			/* 0x14C */
    XK_M,			/* 0x14D */
    XK_N,			/* 0x14E */
    XK_O,			/* 0x14F */
    XK_P,			/* 0x150 */
    XK_Q,			/* 0x151 */
    XK_R,			/* 0x152 */
    XK_S,			/* 0x153 */
    XK_T,			/* 0x154 */
    XK_U,			/* 0x155 */
    XK_V,			/* 0x156 */
    XK_W,			/* 0x157 */
    XK_X,			/* 0x158 */
    XK_Y,			/* 0x159 */
    XK_Z,			/* 0x15A */
    NoSymbol,			/* 0x15B, VK_LWIN */
    NoSymbol,			/* 0x15C, VK_RWIN */
    NoSymbol,			/* 0x15D, VK_APPS */
    XK_VoidSymbol,		/* 0x15E */
    XK_VoidSymbol,		/* 0x15F */
    XK_KP_0,			/* 0x160, VK_NUMPAD0 */
    XK_KP_1,			/* 0x161, VK_NUMPAD1 */
    XK_KP_2,			/* 0x162, VK_NUMPAD2 */
    XK_KP_3,			/* 0x163, VK_NUMPAD3 */
    XK_KP_4,			/* 0x164, VK_NUMPAD4 */
    XK_KP_5,			/* 0x165, VK_NUMPAD5 */
    XK_KP_6,			/* 0x166, VK_NUMPAD6 */
    XK_KP_7,			/* 0x167, VK_NUMPAD7 */
    XK_KP_8,			/* 0x168, VK_NUMPAD8 */
    XK_KP_9,			/* 0x169, VK_NUMPAD9 */
    XK_KP_Multiply,		/* 0x16A, VK_MULTIPLY */
    XK_KP_Add,			/* 0x16B, VK_ADD */
    XK_KP_Separator,		/* 0x16C, VK_SEPARATOR */
    XK_KP_Subtract,		/* 0x16D, VK_SUBTRACT */
    XK_VoidSymbol,		/* 0x16E, VK_DECIMAL */
    XK_KP_Divide,		/* 0x16F, VK_DIVIDE */
    XK_F1,			/* 0x170, VK_F1 */
    XK_F2,			/* 0x171, VK_F2 */
    XK_F3,			/* 0x172, VK_F3 */
    XK_F4,			/* 0x173, VK_F4 */
    XK_F5,			/* 0x174, VK_F5 */
    XK_F6,			/* 0x175, VK_F6 */
    XK_F7,			/* 0x176, VK_F7 */
    XK_F8,			/* 0x177, VK_F8 */
    XK_F9,			/* 0x178, VK_F9 */
    XK_F10,			/* 0x179, VK_F10 */
    XK_F11,			/* 0x17A, VK_F11 */
    XK_F12,			/* 0x17B, VK_F12 */
    XK_F13,			/* 0x17C, VK_F13 */
    XK_F14,			/* 0x17D, VK_F14 */
    XK_F15,			/* 0x17E, VK_F15 */
    XK_F16,			/* 0x17F, VK_F16 */
    XK_F17,			/* 0x180, VK_F17 */
    XK_F18,			/* 0x181, VK_F18 */
    XK_F19,			/* 0x182, VK_F19 */
    XK_F20,			/* 0x183, VK_F20 */
    XK_F21,			/* 0x184, VK_F21 */
    XK_F22,			/* 0x185, VK_F22 */
    XK_F23,			/* 0x186, VK_F23 */
    XK_F24,			/* 0x187, VK_F24 */
    NoSymbol,			/* 0x188 */
    NoSymbol,			/* 0x189 */
    NoSymbol,			/* 0x18A */
    NoSymbol,			/* 0x18B */
    NoSymbol,			/* 0x18C */
    NoSymbol,			/* 0x18D */
    NoSymbol,			/* 0x18E */
    NoSymbol,			/* 0x18F */
    XK_Num_Lock,		/* 0x190, VK_NUMLOCK */
    XK_Scroll_Lock,		/* 0x191, VK_SCROLL */
    NoSymbol,			/* 0x192 */
    NoSymbol,			/* 0x193 */
    NoSymbol,			/* 0x194 */
    NoSymbol,			/* 0x195 */
    NoSymbol,			/* 0x196 */
    NoSymbol,			/* 0x197 */
    NoSymbol,			/* 0x198 */
    NoSymbol,			/* 0x199 */
    NoSymbol,			/* 0x19A */
    NoSymbol,			/* 0x19B */
    NoSymbol,			/* 0x19C */
    NoSymbol,			/* 0x19D */
    NoSymbol,			/* 0x19E */
    NoSymbol,			/* 0x19F */
    XK_Shift_L,			/* 0x1A0, VK_LSHIFT */
    XK_Shift_R,			/* 0x1A1, VK_RSHIFT */
    XK_Control_L,		/* 0x1A2, VK_LCONTROL */
    XK_Control_R,		/* 0x1A3, VK_RCONTROL */
    XK_Alt_L,			/* 0x1A4, VK_LMENU */
    XK_Alt_R,			/* 0x1A5, VK_RMENU */
    NoSymbol,			/* 0x1A6 */
    NoSymbol,			/* 0x1A7 */
    NoSymbol,			/* 0x1A8 */
    NoSymbol,			/* 0x1A9 */
    NoSymbol,			/* 0x1AA */
    NoSymbol,			/* 0x1AB */
    NoSymbol,			/* 0x1AC */
    NoSymbol,			/* 0x1AD */
    NoSymbol,			/* 0x1AE */
    NoSymbol,			/* 0x1AF */
    NoSymbol,			/* 0x1B0 */
    NoSymbol,			/* 0x1B1 */
    NoSymbol,			/* 0x1B2 */
    NoSymbol,			/* 0x1B3 */
    NoSymbol,			/* 0x1B4 */
    NoSymbol,			/* 0x1B5 */
    NoSymbol,			/* 0x1B6 */
    NoSymbol,			/* 0x1B7 */
    NoSymbol,			/* 0x1B8 */
    NoSymbol,			/* 0x1B9 */
    XK_colon,			/* 0x1BA */
    XK_plus,			/* 0x1BB */
    XK_less,			/* 0x1BC */
    XK_underscore,		/* 0x1BD */
    XK_greater,			/* 0x1BE */
    XK_question,		/* 0x1BF */
    XK_grave,			/* 0x1C0 */
    NoSymbol,			/* 0x1C1 */
    NoSymbol,			/* 0x1C2 */
    NoSymbol,			/* 0x1C3 */
    NoSymbol,			/* 0x1C4 */
    NoSymbol,			/* 0x1C5 */
    NoSymbol,			/* 0x1C6 */
    NoSymbol,			/* 0x1C7 */
    NoSymbol,			/* 0x1C8 */
    NoSymbol,			/* 0x1C9 */
    NoSymbol,			/* 0x1CA */
    NoSymbol,			/* 0x1CB */
    NoSymbol,			/* 0x1CC */
    NoSymbol,			/* 0x1CD */
    NoSymbol,			/* 0x1CE */
    NoSymbol,			/* 0x1CF */
    NoSymbol,			/* 0x1D0 */
    NoSymbol,			/* 0x1D1 */
    NoSymbol,			/* 0x1D2 */
    NoSymbol,			/* 0x1D3 */
    NoSymbol,			/* 0x1D4 */
    NoSymbol,			/* 0x1D5 */
    NoSymbol,			/* 0x1D6 */
    NoSymbol,			/* 0x1D7 */
    NoSymbol,			/* 0x1D8 */
    NoSymbol,			/* 0x1D9 */
    NoSymbol,			/* 0x1DA */
    XK_braceleft,		/* 0x1DB */
    XK_bar,			/* 0x1DC */
    XK_braceright,		/* 0x1DD */
    XK_quotedbl,		/* 0x1DE */
    NoSymbol,			/* 0x1DF */
    NoSymbol,			/* 0x1E0 */
    NoSymbol,			/* 0x1E1 */
    NoSymbol,			/* 0x1E2 */
    NoSymbol,			/* 0x1E3 */
    NoSymbol,			/* 0x1E4 */
    NoSymbol,			/* 0x1E5 */
    NoSymbol,			/* 0x1E6 */
    NoSymbol,			/* 0x1E7 */
    NoSymbol,			/* 0x1E8 */
    NoSymbol,			/* 0x1E9 */
    NoSymbol,			/* 0x1EA */
    NoSymbol,			/* 0x1EB */
    NoSymbol,			/* 0x1EC */
    NoSymbol,			/* 0x1ED */
    NoSymbol,			/* 0x1EE */
    NoSymbol,			/* 0x1EF */
    NoSymbol,			/* 0x1F0 */
    NoSymbol,			/* 0x1F1 */
    NoSymbol,			/* 0x1F2 */
    NoSymbol,			/* 0x1F3 */
    NoSymbol,			/* 0x1F4 */
    NoSymbol,			/* 0x1F5 */
    XK_3270_Attn,		/* 0x1F6, VK_ATTN */
    XK_3270_CursorSelect,	/* 0x1F7, VK_CRSEL */
    XK_3270_ExSelect,		/* 0x1F8, VK_EXSEL */
    XK_3270_EraseEOF,		/* 0x1F9, VK_EREOF */
    XK_3270_Play,		/* 0x1FA, VK_PLAY */
    NoSymbol,			/* 0x1FB, VK_ZOOM */
    NoSymbol,			/* 0x1FC, VK_NONAME */
    XK_3270_PA1,		/* 0x1FD, VK_PA1 */
    NoSymbol,			/* 0x1FE, VK_OEM_CLEAR */
    NoSymbol,			/* 0x1FF */
};

/*
 *----------------------------------------------------------------------
 * XKeycodeToKeysym --
 *
 *	Convert a raw keycode to a keysym
 *----------------------------------------------------------------------
 */
KeySym
XKeycodeToKeysym(Display *display, UINT keycode, int index)
{
    KeySym sym;
    int result;
#if 0
    LONG uScanCode;
#endif
    unsigned char kbuf[256];
    unsigned char chb[4];

    display->request++;

    if (keycode >= 0x100) {
	return NoSymbol;
    }

    if (! (index & 1)) {
	sym = keyCodeToKeySym[keycode];
    } else {
	sym = keyCodeToKeySym[keycode+256];
    }
    if (sym != XK_VoidSymbol) {
	return sym;
    }

    /* Looks like we have to do further searching before we have the key */

    memset(kbuf, 0, 256);
#if 0
    uScanCode = GetWindowLong(Tknt_GetPostWindow(), GWL_ID);
#endif
    if (index & 0x02) {
	kbuf[VK_SHIFT] = 0x80;
    }
    if (index & 0x04) {
	kbuf[VK_CAPITAL] = 1;
    }
    if (index & 0x08) {
	kbuf[VK_NUMLOCK] = 1;
    }
    result=ToAscii(keycode, 0, kbuf, (unsigned short *) chb, 0); 
    if (result == 1) {
	return (KeySym) chb[0];
    }
    return NoSymbol;
}

/*
 *----------------------------------------------------------------------
 * XLookupString --
 *
 *	Map a key event to an ASCII string, keysym, and ComposeStatus.
 *
 * Notes:
 *	Compose status is not implemented.
 *----------------------------------------------------------------------
 */
int	   
XLookupString(XKeyEvent *eventPtr, char *buffer_return, int bytes_buffer, 
	      KeySym *keysym_out, XComposeStatus *dummy2)
{					    
    static char mbBuffer[3] = {0, 0, 0};
    static mbflag = 0;
    unsigned char code = 0; 
    int result;
    BYTE kbuf[256];
    int index;
#if 0
    LONG uScanCode;
#endif

#ifdef KANJI
    if(eventPtr->flag == 0x1234 && _ismbblead((unsigned short)eventPtr->keycode)) {
    	mbBuffer[0] = (unsigned char) eventPtr->keycode;
	mbflag = 1;
    	return NoSymbol;
    } else if(eventPtr->flag == 0x5678 && _ismbbtrail((unsigned short)eventPtr->keycode)) {
    	mbBuffer[1] = (unsigned char) eventPtr->keycode;
	mbflag = 0;
        if (buffer_return) strncpy(buffer_return, mbBuffer, bytes_buffer-1); 
	return 2; 
    } else if (eventPtr->flag == 0xABCD) {
	*buffer_return  = (unsigned char) eventPtr->keycode;
	*(buffer_return+1) = '\0';
	return 1;
    }
#else
    if (eventPtr->flag == 0xABCD) {
	*buffer_return  = (unsigned char) eventPtr->keycode;
	*(buffer_return+1) = '\0';
	return 1;
    }
#endif /* KANJI */
    memset(kbuf, 0, 256);
#if 0
    uScanCode = GetWindowLong(Tknt_GetPostWindow(), GWL_ID);
#endif
    index = 0;
    if (eventPtr->state & ShiftMask) {
	kbuf[VK_SHIFT] = 0x80;
	index = 2;
    }
    if (eventPtr->state & LockMask) {
	kbuf[VK_CAPITAL] = 1;
	index += 4;
    }
    if (eventPtr->state & Mod3Mask) {
	kbuf[VK_NUMLOCK] = 1;
	index += 8;
    }
    if (keysym_out) {
	*keysym_out = XKeycodeToKeysym(NULL, eventPtr->keycode, index);
    }
    result = ToAscii(eventPtr->keycode, 0, kbuf,
		     (unsigned short *) buffer_return, 0); 
    if (result == 1) {
	return 1;
    }
    mbflag = 0;

    if (eventPtr->keycode <= 0x2F) {
	strncpy(buffer_return, keyCodeToString[eventPtr->keycode],
		bytes_buffer);
	return min(strlen(keyCodeToString[eventPtr->keycode]),
		   (unsigned)bytes_buffer);
    }
    buffer_return[0] = '\0';
    return 0;
}

/*
 *----------------------------------------------------------------------
 * XRefreshKeyboardMapping --
 *
 *	Read keycode-keysym mapping from server into Xlib.  Don't do 
 *	anything, because no MappingEvents are generated (yet)...
 *----------------------------------------------------------------------
 */
void XRefreshKeyboardMapping(XMappingEvent *event_map)
{
    /* This doesn't generate a request, so the request count stays the same */
}

/*
 *----------------------------------------------------------------------
 * XGetModifierMapping --
 *
 *	Obtain the mapping of modifier keys (Shift, Control, etc).
 *
 *	These need to be kept consistent with Lx_MsgGetState in msg2evt.c.
 *	Better yet, make them changeable just like with X.
 *----------------------------------------------------------------------
 */
XModifierKeymap	*
XGetModifierMapping(Display *display)
{
#define MAX_KEYPERMOD 4
    XModifierKeymap *modmap;
    int size1, size2;

    size1 = 8 * MAX_KEYPERMOD * sizeof(KeyCode);
    size2 = sizeof(XModifierKeymap);

    modmap = (XModifierKeymap *) ckalloc(size1 + size2);
    modmap->max_keypermod = MAX_KEYPERMOD;
    modmap->modifiermap = (KeyCode *) (modmap + 1);
    memset(modmap->modifiermap, 0, size1);

    modmap->modifiermap[ShiftMapIndex*MAX_KEYPERMOD+0]   = VK_SHIFT;
    modmap->modifiermap[ShiftMapIndex*MAX_KEYPERMOD+1]   = VK_LSHIFT;
    modmap->modifiermap[ShiftMapIndex*MAX_KEYPERMOD+2]   = VK_RSHIFT;

    modmap->modifiermap[LockMapIndex*MAX_KEYPERMOD+0]    = VK_CAPITAL;

    modmap->modifiermap[ControlMapIndex*MAX_KEYPERMOD+0] = VK_CONTROL;
    modmap->modifiermap[ControlMapIndex*MAX_KEYPERMOD+1] = VK_LCONTROL;
    modmap->modifiermap[ControlMapIndex*MAX_KEYPERMOD+2] = VK_RCONTROL;

    modmap->modifiermap[Mod2MapIndex*MAX_KEYPERMOD+0]    = VK_MENU;
    modmap->modifiermap[Mod2MapIndex*MAX_KEYPERMOD+1]    = VK_LMENU;
    modmap->modifiermap[Mod2MapIndex*MAX_KEYPERMOD+2]    = VK_RMENU;

    modmap->modifiermap[Mod3MapIndex*MAX_KEYPERMOD+0]    = VK_NUMLOCK;

#ifdef VK_KANA
    modmap->modifiermap[Mod4MapIndex*MAX_KEYPERMOD+0]    = VK_KANA;
#else
    modmap->modifiermap[Mod4MapIndex*MAX_KEYPERMOD+0]    = VK_SCROLL;
#endif

#ifdef VK_KANJI
    modmap->modifiermap[Mod5MapIndex*MAX_KEYPERMOD+0]    = VK_KANJI;
#endif

#ifdef VK_CONVERT
    modmap->modifiermap[Mod1MapIndex*MAX_KEYPERMOD+0]    = VK_CONVERT;
#endif

    return modmap;
#if 0
    modmap->modifiermap[0*8] = VK_NUMLOCK;
    modmap->modifiermap[1*8] = VK_KANA;
    modmap->modifiermap[2*8] = VK_KANJI;
    modmap->modifiermap[3*8] = VK_CONVERT;
    modmap->modifiermap[4*8] = VK_NONCONVERT;
#endif
}

/*
 *----------------------------------------------------------------------
 * XFreeModifiermap --
 *
 *	Free the modifier map allocated by XGetModifierMapping
 *----------------------------------------------------------------------
 */
void
XFreeModifiermap(XModifierKeymap *modmap)
{
    ckfree((char *) modmap);
}

/*
 *----------------------------------------------------------------------
 * InitializeKeySymTable --
 *
 *	Initialize the hash table that will be used to do convert
 *	a keysym name string to a keysym.
 *----------------------------------------------------------------------
 */
static void
InitializeKeySymTable()
{
    register KeySymInfo *kPtr;
    register Lx_HashEntry *hPtr;
    int dummy;

    Lx_InitHashTable(&keySymTable, LX_STRING_KEYS);
    for (kPtr = keyArray; kPtr->name != NULL; kPtr++) {
	hPtr = Lx_CreateHashEntry(&keySymTable, kPtr->name, &dummy);
	Lx_SetHashValue(hPtr, kPtr->value);
    }
    initialized = 1;
}

/*
 *----------------------------------------------------------------------
 * XStringToKeysym --
 *
 *	Convert a keysym name string to a keysym
 *----------------------------------------------------------------------
 */
KeySym
XStringToKeysym(const char *name)
{
    Lx_HashEntry *hPtr;

    if (! initialized) {
	InitializeKeySymTable();
    }
    hPtr = Lx_FindHashEntry(&keySymTable, (char *) name);
    if (hPtr != NULL) {
	return (KeySym) Lx_GetHashValue(hPtr);
    }
    return 0;
}

/*
 *----------------------------------------------------------------------
 * XKeysymToString --
 *
 *	Convert a keysym symbol to a string.
 *----------------------------------------------------------------------
 */
char *
XKeysymToString(KeySym keysym)
{
    register KeySymInfo *kPtr;

    for (kPtr = keyArray; kPtr->name != NULL; kPtr++) {
	if (kPtr->value == keysym) {
	    return kPtr->name;
	}
    }
    return NULL;
}

/*
 *----------------------------------------------------------------------
 * XKeysymToKeycode --
 *
 *	For a given keysym, return its keycode
 *
 * Note:
 *	This routine does not properly convert all codes yet.  For
 *	example, ':' would not be converted.  To do this, we need a
 *	keysym to keycode table that we need to run through to find
 *	mappings.  Probably the best thing would be to use a direct
 *	table for the lowest 256 keysyms.  This is incredibly slow,
 *	but for most things, who cares?  tkTest is the only thing
 *	in Tk that uses it, and I can't see lots of other reasons to
 *	use it.
 *----------------------------------------------------------------------
 */
KeyCode
XKeysymToKeycode(Display *display, KeySym keysym)
{
    int i;

    if (keysym == NoSymbol) {
	return 0;
    }

    /* Check for some quick, standard mappings */
    if ((XK_A <= keysym) && (keysym <= XK_Z)) {
	return (KeyCode) (keysym - XK_A + 0x41);
    }
    if ((XK_a <= keysym) && (keysym <= XK_z)) {
	return (KeyCode) (keysym - XK_a + 0x41);
    }
    if ((XK_0 <= keysym) && (keysym <= XK_9)) {
	return (KeyCode) (keysym - XK_0 + 0x30);
    }

    for (i = 0; i < 512; i++) {
	if (keyCodeForKeySym[i] == keysym) {
	    return (KeyCode) i;
	}
    }

    return 0;
}
