/*****************************************************************************/
/*                                                                           */
/* tcmXProbe.c - TCL interface for X window probe functions.  This file      */
/*            implements the xprobe tcl command, which allows you to find    */
/*            information about the current X display.                       */
/*                                                                           */
/* Authors:  Eric Scharff (escharff)                                         */
/*                                                                           */
/* Created on 29-May-1995 by escharff (Inital Functions)                     */
/* Modified on 30-May-1995 by escharff (Added FindByName)                    */
/*                                                                           */
/* FindByName code was written by Brian Smith.                               */
/*                                                                           */
/* TODO:  Testing                                                            */
/*        Error Checking                                                     */
/*****************************************************************************/

#include <stdio.h>
#include <errno.h>
#include <X11/Xos.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <ctype.h>
#include <tcl.h>
#define MAX_RET_STR 15

char retstr[MAX_RET_STR];  /* Return String for sprintf calls */

Window
FindByName (name, win, dpy)
    char *name;
    Window win;
    Display *dpy;
{
    Window rv;
    Window parent;
    Window root;
    Window *kids;
    int numKids;
    Status status;
    int i;
    char *winName;
    int len;

    /*
     * See if this window is a match
     */
    len = strlen(name);
    XFetchName (dpy, win, &winName);
    if (winName && (strncmp (name, winName, strlen(name)) == 0)) {
        XFree (winName);
        return win;
    }
    XFree (winName);

    /*
     * If not, recurse on all children
     */
    status = XQueryTree (dpy, win, &root, &parent, &kids, &numKids);
    if (!status) {
        return 0;
    }

    for (i=0; i<numKids; i++) {
        rv = FindByName (name, kids[i], dpy);
        if (rv != 0) {
            return rv;
        }
    }

    XFree (kids);
    return 0;
}

int
Tk_XProbeCmd(clientData, interp, argc, argv)
  ClientData clientData;
  Tcl_Interp *interp;
  int argc;
  char *argv[];
/* Dispatch procedure for xprobe.  Currently supports findwin, parentwin, */
/* and chlidwin.  Doesn't really do any error checking.                   */
{
  static Display *dpy = NULL; /* Current Display Pointer */
  Window retwin;              /* Window pointer returned */
  Window curwin;              /* Window passed from TCL  */
  Window root;                /* Root window             */
  Window parent;              /* Parent Window           */
  Window *kids;               /* Pointer to children     */
  int numkids;                /* Number of children      */
  Status status;              /* return status           */
  int childnum;               /* Child # passed from TCL */

  /* Simple check for arg count */
  if (argc < 3) {
     interp->result = "wrong # args";
     return TCL_ERROR;
  }
  /* Open display if not currently open */
  if (dpy == NULL) {
     if ((dpy = XOpenDisplay("")) == NULL) {
        interp->result = "Unable to open display";
        return TCL_ERROR;
     }
  }
  /* Dispatch the differnt operations */
  if (strcmp(argv[1], "findwin") == 0) {
     retwin = FindByName (argv[2], DefaultRootWindow(dpy), dpy);
     (void)sprintf(retstr, "%d", retwin);
     interp->result = retstr; 
  }
  else {
     /* Collect useful information */
     curwin = (Window) atoi(argv[2]);
     status = XQueryTree(dpy, curwin, &root, &parent, &kids, &numkids);
     if (!status) {
        interp->result = "Error Querying X Server";
        return TCL_ERROR;
     }
     if (strcmp(argv[1], "numchildren") == 0) {
        (void)sprintf(retstr, "%d", numkids);
        interp->result = retstr; 
     }
     else if (strcmp(argv[1], "parentwin") == 0) {
        (void)sprintf(retstr, "%d", parent);
        interp->result = retstr; 
     }
     else if (strcmp(argv[1], "childwin") == 0) {
       childnum = atoi(argv[3]);
       (void)sprintf(retstr, "%d", kids[childnum]);
       interp->result = retstr; 
     }
  }
  return TCL_OK;
}
