/*
 * tkInit.c --
 *
 *	Provides for installing tk commands.
 *
 * Copyright 1989 Regents of the University of California.
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  The University of California
 * makes no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without
 * express or implied warranty.
 */

#define NEED_TCM_EVENT_H

#ifdef _WINDOWS
#    define WIN32_LEAN_AND_MEAN
#    include "wndproc.h"
#    include <winsock.h>
#    undef WIN32_LEAN_AND_MEAN
#endif /* _WINDOWS */


#include "tk.h"
#include "patchlevel.h"

/*
 * The following structure defines all of the commands supported by
 * Tk, and the C procedures that execute them.
 */

typedef struct 
{
  char *name;			/* Name of command. */
  int (*cmdProc)  _ANSI_ARGS_((ClientData clientData, Tcl_Interp *interp,
			       int argc, char **argv));
				/* Command procedure. */
} TkCmd;

TkCmd tkInitCommands[] = 
{
    /*
     * Commands that are truly intrinsic:
     */

    {"at",		Tcm_AtCmd},
    {"atq",		Tcm_AtqCmd},
    {"atrm",		Tcm_AtrmCmd},
    {"systime",		Tk_SystimeCmd},
    {"whenidle",	Tk_IdleCmd},

    /*
     * Misc. Commands:
     */
#ifndef NO_X11
#ifndef _WINDOWS
    {"xprobe",		Tk_XProbeCmd},
#endif
#endif

    {(char *) NULL,	(int (*)()) NULL}
};


/*
 *----------------------------------------------------------------------
 *
 * Tkx_Init --
 *
 *	Initialize an interpreter to barebones tk.
 *
 * Results:
 *
 * Side effects:
 *
 * Notes:  DO NOT do a normal TkInit.
 *----------------------------------------------------------------------
 */
	/* ARGSUSED */
int
Tkx_Init (interp)
    Tcl_Interp *interp;		/* Interpreter to use for error reporting. */
{
    register TkCmd *cmdPtr;
    char *libDir;
    Tcl_DString dString;

    if (Tcl_GetVar(interp, "tk_library", TCL_GLOBAL_ONLY) == NULL) {
	/*
	 * A library directory hasn't already been set, so figure out
	 * which one to use.
	 */

	libDir = getenv("CMT_ROOT");
#ifdef _WINDOWS
	if (libDir == NULL) {
	    extern char *Tknt_GetRegistryString(char *, char *);
	    libDir = Tknt_GetRegistryString("SOFTWARE\\" APPLICATION_NAME, "CMT_ROOT");
	}
#endif
	if (libDir == NULL) {
	    libDir = CMT_ROOT;
	}
	Tcl_DStringInit(&dString);
	Tcl_DStringAppend(&dString, libDir, -1);
	Tcl_DStringAppend(&dString, "/tk", -1);
	Tcl_DStringAppend(&dString, TK_VERSION, -1);
	Tcl_SetVar(interp, "tk_library", Tcl_DStringValue(&dString),
		   TCL_GLOBAL_ONLY);
	Tcl_DStringFree(&dString);

	/* Bind in the 'after' command and others */
	Tk_EventInit(interp);
    }
    Tcl_SetVar(interp, "tk_patchLevel", TK_PATCH_LEVEL, TCL_GLOBAL_ONLY);
    Tcl_SetVar(interp, "tk_version", TK_VERSION, TCL_GLOBAL_ONLY);

    /*
     * Bind in Tk commands.
     */
    for (cmdPtr = tkInitCommands; cmdPtr->name != NULL; cmdPtr++) {
        Tcl_CreateCommand (interp, cmdPtr->name, cmdPtr->cmdProc,
			   (ClientData) NULL, (void (*) ()) NULL);
    }
    InitPriorityGroup (interp);

    return TCL_OK;
}
