/* This file contains code for the utility avi2mjpgc which converts
   motion jpeg avi files to motion jpeg chunk files.
*/
/* Usage: avi2mjpgc <input_filename> <output_filename> */


#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/uio.h>
#include <netinet/in.h>
#include <unistd.h>
#include <math.h>
#include "cmChunk.h"
#include "cmCookies.h"

#define ONE_BILLION 1000000000.0
#define CF_READ_BUFFER_SIZE 10000

#define FIND_AVI_LIST(tchar1, tchar2, tchar3, tchar4)                         \
  while ((!((buffer[buf_index] == 'L') && (buffer[buf_index+1] == 'I') &&     \
	  (buffer[buf_index+2] == 'S') && (buffer[buf_index+3] == 'T'))) ||   \
	 (!((buffer[buf_index+8] == tchar1) && (buffer[buf_index+9] == tchar2) && \
	    (buffer[buf_index+10] == tchar3) &&                               \
	    (buffer[buf_index+11] == tchar4)))) {                             \
    buf_index += 4;                                                           \
    junk_size = GetInt32FromAVI(buffer, buf_index);                           \
    buf_index += 4 + junk_size;                                               \
                                                                              \
    if (buf_index > num_read - 12) {                                          \
      lseek(input_fd, buf_index+file_index, SEEK_SET);                   \
      file_index += buf_index;                                                \
      buf_index = 0;                                                          \
      num_read = read(input_fd, buffer, CF_READ_BUFFER_SIZE);            \
      if (num_read < 12) goto error;                                          \
    }                                                                         \
  }

#define FIND_AVI_SUBCHUNK(tchar1, tchar2, tchar3, tchar4)                     \
  while (!((buffer[buf_index] == tchar1) && (buffer[buf_index+1] == tchar2) &&\
	    (buffer[buf_index+2] == tchar3) &&                                \
	    (buffer[buf_index+3] == tchar4))) {                               \
    buf_index += 4;                                                           \
    junk_size = GetInt32FromAVI(buffer, buf_index);                           \
    buf_index += 4 + junk_size;                                               \
                                                                              \
    if (buf_index > num_read - 8) {                                           \
      lseek(input_fd, buf_index+file_index, SEEK_SET);                   \
      file_index += buf_index;                                                \
      buf_index = 0;                                                          \
      num_read = read(input_fd, buffer, CF_READ_BUFFER_SIZE);            \
      if (num_read < 8) goto error;                                           \
    }                                                                         \
  }

static unsigned int
GetInt32FromAVI(unsigned char *buffer,     /* Pointer to actual bytes. */
		int buf_index     /* Index where 32-bit value starts. */)
{
  unsigned int result = 0;

  result = (((unsigned int) buffer[buf_index]) | 
	    (((unsigned int) (buffer[buf_index+1])) << 8) |
	    (((unsigned int) (buffer[buf_index+2])) << 16) | 
	    (((unsigned int) (buffer[buf_index+3])) << 24));

  return result;
}


void
main (int argc, char **argv)
{
  char *input_filename, *output_filename;
  int i, input_fd, junk_size;
  MediaIndepHeader mi_header;
  MJPEGHeader cfInfo;
  int num_read;
  char buffer[CF_READ_BUFFER_SIZE];
  int file_index, buf_index, avi_success, main_avi_hdr_size;
  int strl_size, strl_start, stream_num, found_stream;
  int stream_header_size, stream_scale, stream_rate, stream_length;
  int strf_header_size, stream_width, stream_height;
  char stream_id[32];
  int *rawFOT;
  double *rawFTT;
  int frame_counter, movi_length, movi_end, list_size, frame_length;
  int numFrames;
  double clipLength;
  tChunkFileState *cfState;
  double clength_s, clength_ns;
  int offset, length, fTime_s, fTime_ns;
  int qfactor;
  int cntr;

  input_filename = output_filename = NULL;
  qfactor = 100;

  cntr = 1;
  while (cntr < argc) {
    if (strcmp(argv[cntr], "-qfactor") == 0) {
      cntr++;
      if (cntr == argc) {
	fprintf(stderr, "Must specify an integer qfactor after -qfactor command line option\n");
	exit(1);
      }
      qfactor = atoi(argv[cntr]);
    } else if (strcmp(argv[cntr], "-input") == 0) {
      cntr++;
      if (cntr == argc) {
	fprintf(stderr, "Must specify input file name after -input command line option\n");
	exit(1);
      }
      input_filename = argv[cntr];
    } else if (strcmp(argv[cntr], "-output") == 0) {
      cntr++;
      if (cntr == argc) {
	fprintf(stderr, "Must specify output file name after -output command line option\n");
	exit(1);
      }
      output_filename = argv[cntr];
    } else {
    usage:
      fprintf(stderr, "Usage: %s -input <input_filename> -output <output_filename> [-qfactor <qfactor>] \n", argv[0]);
      exit(1);
    }
    cntr++;
  }

  if ((input_filename == NULL) || (output_filename == NULL)) {
    goto usage;
  }

  /* 
   * Open input filename. 
   */

  input_fd = open(input_filename, O_RDONLY);

  if (input_fd < 0) {
    fprintf(stderr, "Couldn't open %s as input file.\n", input_filename);
    exit(2);
  }

  /* 
   * Confirm that input file is an avi file.
   */
  
  num_read = read(input_fd, buffer, CF_READ_BUFFER_SIZE);
  file_index = 0;
  buf_index = 0;

  if (num_read < 24) goto error;
  avi_success = 0;
  
  while (buf_index < num_read-24) {
    if ((buffer[buf_index] == 'R') && (buffer[buf_index+1] == 'I') &&
	(buffer[buf_index+2] == 'F') && (buffer[buf_index+3] == 'F') &&
	(buffer[buf_index+8] == 'A') && (buffer[buf_index+9] == 'V') &&
	(buffer[buf_index+10] == 'I') && (buffer[buf_index+11] == ' ')) {
      avi_success = 1;
      break;
    }
    buf_index++;
  }

  if (!avi_success) {
    fprintf(stderr, "Input file %s doesn't seem to be an avi file.\n", input_filename);
    exit(1);
  }

  /*
   * Now we know its an avi, lets make sure that there is a mjpeg stream 
   * in the file and find the stream number.
   */
  
  buf_index += 12;
  
  FIND_AVI_LIST('h', 'd', 'r', 'l');
  
  buf_index += 12;
  
  if (buf_index > num_read-8) {
    lseek(input_fd, buf_index+file_index, SEEK_SET);
    file_index += buf_index;
    buf_index = 0;
    num_read = read(input_fd, buffer, CF_READ_BUFFER_SIZE);
    if (num_read < 8) goto error;
  }

  FIND_AVI_SUBCHUNK('a', 'v', 'i', 'h');
  buf_index += 4;

  main_avi_hdr_size = GetInt32FromAVI(buffer, buf_index);
  buf_index += 4;

  buf_index += main_avi_hdr_size;

  if (buf_index > num_read-8) {
    lseek(input_fd, file_index+buf_index, SEEK_SET);
    file_index += buf_index;
    buf_index = 0;
    num_read = read(input_fd, buffer, CF_READ_BUFFER_SIZE);
    if (num_read < 8) goto error;
  }

  FIND_AVI_LIST('s', 't', 'r', 'l');

  buf_index += 4;

  strl_size = GetInt32FromAVI(buffer, buf_index);
  buf_index += 4;
  
  strl_start = file_index + buf_index;
  buf_index += 4;


  if (buf_index > num_read-8) {
    lseek(input_fd, file_index+buf_index, SEEK_SET);
    file_index += buf_index;
    buf_index = 0;
    num_read = read(input_fd, buffer, CF_READ_BUFFER_SIZE);
    if (num_read < 8) goto error;
  }

  stream_num = -1;
  found_stream = 0;

  while (1) {
    FIND_AVI_SUBCHUNK('s', 't', 'r', 'h');
    
    stream_num++;
    buf_index += 4;
    
    stream_header_size = GetInt32FromAVI(buffer, buf_index);

    buf_index += 4;
    
    if ((buffer[buf_index] == 'v') && (buffer[buf_index+1] == 'i') &&
	(buffer[buf_index+2] == 'd') && (buffer[buf_index+3] == 's')) {
      
      stream_scale = GetInt32FromAVI(buffer, buf_index+20);
      stream_rate = GetInt32FromAVI(buffer, buf_index+24);
      stream_length = GetInt32FromAVI(buffer, buf_index+32);
      
      buf_index += stream_header_size;
      
      if (buf_index > num_read - 8) {
	lseek(input_fd, file_index+buf_index, SEEK_SET);
	file_index += buf_index;
	buf_index = 0;
	num_read = read(input_fd, buffer, CF_READ_BUFFER_SIZE);
	if (num_read < 8) goto error;
      }
      
      FIND_AVI_SUBCHUNK('s', 't', 'r', 'f');

      buf_index += 4;
      
      strf_header_size = GetInt32FromAVI(buffer, buf_index);
      buf_index += 4;
      
      if ((buffer[buf_index+16] == 'M') && (buffer[buf_index+17] == 'J') &&
	  (buffer[buf_index+18] == 'P') && (buffer[buf_index+19] == 'G')) {
	
	stream_width = GetInt32FromAVI(buffer, buf_index+4);
	stream_height = GetInt32FromAVI(buffer, buf_index+8);

	found_stream = 1;
      }

      buf_index += strf_header_size;
    
    } else {
      buf_index += stream_header_size;
    }

    if (found_stream) break;
    
    if (buf_index > num_read - 8) {
      lseek(input_fd, file_index+buf_index, SEEK_SET);
      file_index += buf_index;
      buf_index = 0;
      num_read = read(input_fd, buffer, CF_READ_BUFFER_SIZE);
      if (num_read < 8) goto error;
    }
  }

  if (!found_stream) {
    fprintf(stderr, "AVI file did not contain MJPG data stream.\n");
    exit(1);
  }


  /*
   * Found a motion jpeg stream at this point as well as general things like:
   * width, height, how many frames, frame rate.
   * Now construct the data structures by doing:
   * 1) Find all the frames in the avi file, creating an offset table.
   * 2) Adjust number of frames to match number of frames found.
   * 3) Create a frame time table.
   * 4) Fill in cf info structure and general info.
   * 5) Return.
   */

  /* 
   * Find all frames, create FOT.
   */

  sprintf(stream_id, "%.2d", stream_num);

  if (stream_length < 1) {
    /* Stream length does not contain valid info. Heuristically use 128 to start. */

    stream_length = 128;
  }

  rawFOT = (int *) malloc(stream_length*sizeof(int)*2);

  frame_counter = 0;

  lseek(input_fd, strl_start+strl_size, SEEK_SET);
  file_index = strl_start+strl_size;
  buf_index = 0;
  num_read = read(input_fd, buffer, CF_READ_BUFFER_SIZE);
  if (num_read < 12) goto error;

  FIND_AVI_LIST('m', 'o', 'v', 'i');

  buf_index += 4;

  movi_length = GetInt32FromAVI(buffer, buf_index);
  buf_index += 4;
  movi_end = file_index + buf_index + movi_length;
  buf_index += 4;

  if (buf_index > num_read - 12) {
    lseek(input_fd, buf_index+file_index, SEEK_SET);
    file_index += buf_index;
    buf_index = 0;
    num_read = read(input_fd, buffer, CF_READ_BUFFER_SIZE);
    if (num_read < 12) goto error;
  }

  while (buf_index + file_index < movi_end) {
    
    if ((buffer[buf_index] == 'L') && (buffer[buf_index+1] == 'I') &&
	(buffer[buf_index+2] == 'S') && (buffer[buf_index+3] == 'T')) {
      buf_index += 4;

      if ((buffer[buf_index+4] == 'r') && (buffer[buf_index+5] == 'e') &&
	  (buffer[buf_index+6] == 'c') && (buffer[buf_index+7] == ' ')) {
	buf_index += 8;
      } else {
	list_size = GetInt32FromAVI(buffer, buf_index);
	buf_index += 4 + list_size;
      }
    } else {
      
      frame_length = GetInt32FromAVI(buffer, buf_index+4);

      if ((buffer[buf_index] == stream_id[0]) && 
	  (buffer[buf_index+1] == stream_id[1]) &&
	  (buffer[buf_index+2] == 'd') &&
	  (buffer[buf_index+3] == 'c')) {

	if (frame_counter >= stream_length) {
	  stream_length += 128;
	  rawFOT = (int *) realloc(rawFOT, sizeof(int)*2*stream_length);
	}

	rawFOT[frame_counter*2] = file_index + buf_index + 8;
	rawFOT[(frame_counter*2)+1] = frame_length;
	frame_counter++;
      }

      buf_index += 8;
      if (frame_length%2) {
	buf_index += frame_length + 1;
      } else {
	buf_index += frame_length;
      }
    }
  
    if (buf_index > num_read - 12) {
      lseek(input_fd, buf_index+file_index, SEEK_SET);
      file_index += buf_index;
      buf_index = 0;
      num_read = read(input_fd, buffer, CF_READ_BUFFER_SIZE);
      if (num_read < 12) break;
    }
  }

  /* 
   * Done computing FOT, now fill in general info.
   * NOTE: stream_length has number of frames found.
   */

  numFrames = stream_length;
  clipLength = (((double) stream_length) /
		(((double) stream_rate) / ((double) stream_scale)));
  
  cfInfo.width = stream_width;
  cfInfo.height = stream_height;
  cfInfo.device = J300_DEV_CODE;
  cfInfo.subformat = JFIF_NO_HUFFMAN_FRAME_CODE;
  cfInfo.qfactor = qfactor;
  
  /* 
   * Now create FTT.
   */

  rawFTT = (double *) malloc(sizeof(double)*(stream_length+1));

  for (i=0; i<=stream_length; i++) {
    rawFTT[i] = (((double) i) / ((double) stream_length)) * clipLength;
  }


  /*
   * Now write the output file.
   */

  cfState = Chunk_OpenFile(output_filename, "cw");
  if (cfState == NULL) {
    fprintf(stderr, "Couldn't open output file %s for writing\n", output_filename);
    exit(1);
  }

  mi_header.media = htonl(VIDEO_MEDIA_CODE);
  mi_header.format = htonl(MJPEG_FORMAT_CODE);
  mi_header.device = htonl(J300_DEV_CODE);
  mi_header.version = htonl(3);
  mi_header.num_frames = htonl(numFrames);
  clength_s = floor(clipLength);
  clength_ns = ((clipLength -floor(clipLength)) * ONE_BILLION);
  mi_header.length_s = htonl((int) clength_s);
  mi_header.length_ns = htonl((int) clength_ns);

  if (Chunk_WriteHeader(cfState, CM_MEDIA_INDEP_CF, "") != 0) {
    fprintf(stderr, "Error writing media indep. header\n");
    exit(4);
  }

  write(cfState->fd, &mi_header, (sizeof(MediaIndepHeader)));

  cfInfo.width = htonl(cfInfo.width);
  cfInfo.height = htonl(cfInfo.height);
  cfInfo.device = htonl(cfInfo.device);
  cfInfo.subformat = htonl(cfInfo.subformat);
  cfInfo.qfactor = htonl(cfInfo.qfactor);

  if (Chunk_WriteHeader(cfState, CM_MJPEG_CF, "") != 0) {
    fprintf(stderr, "Error writing mjpeg chunk header.\n");
    exit(5);
  }

  write(cfState->fd, &cfInfo, (sizeof(MJPEGHeader)));

  if (Chunk_WriteHeader(cfState, CM_DATA_SEGMENT, "") != 0) {
    fprintf(stderr, "Error writing data chunk header.\n");
    exit(6);
  }

  lseek(input_fd, 0, SEEK_SET);

  num_read = read(input_fd, buffer, CF_READ_BUFFER_SIZE); 
  
  while (num_read != 0) {
    write(cfState->fd, buffer, num_read);
    num_read = read(input_fd, buffer, CF_READ_BUFFER_SIZE);
  }

  free(buffer);

  /* Write frame offset table. */

  if (Chunk_WriteHeader(cfState, CM_FOT, "") != 0) {
    fprintf(stderr, "Error writing frame offset table chunk header.\n");
    exit(7);
  }

  for (i=0; i<numFrames; i++) {
    offset = htonl(rawFOT[i*2]);
    length = htonl(rawFOT[(i*2)+1]);
    write(cfState->fd, &offset, 4);
    write(cfState->fd, &length, 4);
  }

  /* Write frame time table. */
  
  if (Chunk_WriteHeader(cfState, CM_FTT, "") != 0) {
    fprintf(stderr, "Error writing frame time table chunck header.\n");
    exit(9);
  }
  
  for(i=0; i<=numFrames; i++) {
    fTime_s = htonl(((int) (floor(rawFTT[i]))));
    fTime_ns = htonl(((int) ((rawFTT[i] - floor(rawFTT[i])) * ONE_BILLION)));

    write(cfState->fd, &fTime_s, 4);
    write(cfState->fd, &fTime_ns, 4);
  }

  Chunk_CloseFile(cfState);

  exit(0);

  /*
   * Error handling. We jump here if reading data went awry. 
   */

 error:

  fprintf(stderr, "Trouble reading data from avi file.\n");

  if (rawFTT != NULL) {
    free((char *) rawFTT);
    rawFTT = NULL;
  }
  if (rawFOT != NULL) {
    free((char *) rawFOT);
    rawFOT = NULL;
  }
  exit(1);
}
