#include "cmChunk.h"
#include <stdio.h>
#include <assert.h>
#include <assert.h>
#include <fcntl.h>
#include <unistd.h>

/*--------------------------------------------------------------*/

#define ERR_IN		fprintf(stderr,"## ERROR: "); fprintf(stderr,
#define ERR_OUT		); putc('\n',stderr);

#define ERR(s) 		{ ERR_IN s ERR_OUT }
#define DIE(s) 		{ ERR_IN s ERR_OUT exit(1);}
#define DIEIF(c,s)	{if (c) { ERR_IN s ERR_OUT exit(1);}}
#define C_	,

#define BUF_SZ	1024

/*--------------------------------------------------------------*/

void
CopyOutFile (char* fileName, int inFd, int32 dataSz)
{
    int		outFd;
    char	inBuf[BUF_SZ];
    long	cnt, trySz;
    
    if ((outFd = creat(fileName,0666)) == -1)
	DIE("Could not open file '%s'"C_ fileName);

    for (; dataSz > 0; dataSz -= BUF_SZ)
    {
	trySz = (dataSz > BUF_SZ) ? BUF_SZ : dataSz;
	if ((cnt= read(inFd,inBuf,trySz)) != trySz)
	    DIE("CopyOutFile: failed reading src!");
	cnt = write(outFd,inBuf,trySz);
	DIEIF(cnt != trySz,"CopyOutFile: failed writing!");
	assert(cnt == trySz);
    }
    close(outFd);
}

/*--------------------------------------------------------------*/

void
Help (void)
{
    printf("Usage: <inFile> [-type <outFile> | -index <int> <fileName>]...\n");
    exit(0);
}

/*--------------------------------------------------------------*/

int
main (int argc, char* argv[])
{
    char		str[5];
    char*		chunkStr;
    tChunkHeader	hdr;
    int			err, cnt, i;
    tChunk 		cf;
    
    if (argc == 1)
	Help();
    cf = Chunk_OpenFile(argv[1],"r");
    DIEIF(cf == NULL, "Could not open file: %s"C_ argv[1]);

    if (argc == 2)
    {
	printf("IDX - [TYPE] bytes String\n");
	for (cnt = 0; (err = Chunk_ReadHeader(cf,&hdr)) == 0; cnt++)
	{
	    chunkStr = Chunk_ReadString(cf,NULL);
	    assert(chunkStr);
	    Chunk_Type2Str(str,hdr.type);
	    printf("%3d - [%s] ", cnt, str);
	    printf("%-6d %s\n", hdr.chunkSize, chunkStr);
	    free(chunkStr);
	}
	if (err != kChunk_EndOfFile)
	{
	    fprintf(stderr,"Bad End of File\n");
	    exit(1);
	}
    }
    for (i = 2; i < argc; i++)
    {
	if 	(strcmp(argv[i],"-help") == 0 ||
		 strcmp(argv[i],"-h") == 0 || 
		 strcmp(argv[i],"-?") == 0)
	    Help();
	else if (strcmp(argv[i],"-index") == 0 ||
		 strcmp(argv[i],"-i") == 0)
	{
	    int		srcIdx = atoi(argv[++i]);
	    err = Chunk_FindIndex(cf,srcIdx,&hdr);
	    assert(err == 0);
	    CopyOutFile(argv[++i],cf->fd,hdr.chunkSize);
	} 
	else if (strcmp(argv[i],"-type") == 0 ||
		 strcmp(argv[i],"-t") == 0)
	{
	    int32	srcType = Chunk_Str2Type(argv[++i]);
	    err = Chunk_FindType(cf,srcType,&hdr);
	    assert(err == 0);
	    CopyOutFile(argv[++i],cf->fd,hdr.chunkSize);
	} else
	{
	    ERR("Unknown option: %s"C_ argv[i]);
	    Help();
	}
    }
    return 0;
}
