/*
 * Copyright (c) 1995 Cornell University
 * All rights reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for research and educational purposes, without fee, and
 * without written agreement is hereby granted, provided that the above
 * copyright notice and the following two paragraphs appear in all copies
 * of this software.
 *
 * IN NO EVENT SHALL CORNELL UNIVERSITY BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF CORNELL UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * CORNELL UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND CORNELL UNIVERSITY HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 */

/*
	Netvideo version 3.3
	Written by Ron Frederick <frederick@parc.xerox.com>

	Video encode routines
*/

/*
 * Copyright (c) Xerox Corporation 1992. All rights reserved.
 *  
 * License is granted to copy, to use, and to make and to use derivative
 * works for research and evaluation purposes, provided that Xerox is
 * acknowledged in all documentation pertaining to any such copy or derivative
 * work. Xerox grants no other licenses expressed or implied. The Xerox trade
 * name should not be used in any advertising without its written permission.
 *  
 * XEROX CORPORATION MAKES NO REPRESENTATIONS CONCERNING EITHER THE
 * MERCHANTABILITY OF THIS SOFTWARE OR THE SUITABILITY OF THIS SOFTWARE
 * FOR ANY PARTICULAR PURPOSE.  The software is provided "as is" without
 * express or implied warranty of any kind.
 *  
 * These notices must be retained in any copies of any part of this software.
 */

#include <stdio.h>
#include <memory.h>
#include <malloc.h>
#include <sys/types.h>
#include <sys/time.h>
#include <netinet/in.h>
#include "sized_types.h"
#include "rtp.h"
#include "vid_image.h"
#include "vid_code.h"
#include "nv.h"

#define PACKETLEN		1024
#define AGE_THRESHOLD		8
#define XMIT_THRESHOLD		64

static int width, height, block_width, block_height;
static int xmit_color;
static uint8 *curr_y_data=NULL;
static int8 *curr_uv_data=NULL;
static uint8 *xmit=NULL, *age=NULL;

static int start_x=0, start_y=0, age_x=0, age_y=0;
static int frame_ok=0, scan_cycle=0;
static uint8 nv_data[2048];
static uint32 rtp_frame_time;

static struct {
    uint16 width, height;
} nv_hdr;

void NV_SetXmit(int x, int y, int w)
{
	int	offset;
	uint8	*xmitp;

	offset = y*block_width+x;
	xmitp = &xmit[offset];
	while (w--) {
		*xmitp++ = 1;
	}
}

void NV_SetXmitAll()
{
	memset(xmit, 1, block_width*block_height);
}

static uint8 *NV_PutBlockFromImage(vidimage_t *image, uint8 *data, uint8 **runhdrp, int x, int y, int loss)
{
    int i, offset, zcount, rem;
    uint8 b, *yp, *runhdr=*runhdrp;
    uint32 blkw, *blkwp, *blkwLim, block[32];
    int8 *uvp, *blkp;
    static uint8 run_x = 0xFF, run_y = 0xFF;

    if ((runhdr == NULL) || (run_x != x) || (run_y != y)) {
	*runhdrp = runhdr = data;
	runhdr[0] = 1;
	runhdr[1] = run_x = x;
	runhdr[2] = run_y = y;
	data += 3;
    } else {
	runhdr[0]++;
    }

    offset = y*8*width+x*8;
    yp = &((uint8 *)image->y_data)[offset];
    uvp = image->uv_data? &((int8 *)image->uv_data)[offset]:0;

    NV_FwdTransform(yp, xmit_color? uvp : NULL, width, block);
    blkp = (int8 *)block;

    if (loss > 0) {
	blkp = (int8 *)block;
	if ((blkp[2]>=-loss) && (blkp[2]<=loss)) blkp[2] = 0;
	if ((blkp[3]>=-loss) && (blkp[3]<=loss)) blkp[3] = 0;
	if ((blkp[4]>=-loss) && (blkp[4]<=loss)) blkp[4] = 0;
	if ((blkp[5]>=-loss) && (blkp[5]<=loss)) blkp[5] = 0;
	if ((blkp[6]>=-loss) && (blkp[6]<=loss)) blkp[6] = 0;
	if ((blkp[7]>=-loss) && (blkp[7]<=loss)) blkp[7] = 0;
	blkp = ((int8 *)block)+10;
	for (i=10; i<64; i+=6, blkp+=6) {
	    if ((blkp[0]>=-loss) && (blkp[0]<=loss)) blkp[0] = 0;
	    if ((blkp[1]>=-loss) && (blkp[1]<=loss)) blkp[1] = 0;
	    if ((blkp[2]>=-loss) && (blkp[2]<=loss)) blkp[2] = 0;
	    if ((blkp[3]>=-loss) && (blkp[3]<=loss)) blkp[3] = 0;
	    if ((blkp[4]>=-loss) && (blkp[4]<=loss)) blkp[4] = 0;
	    if ((blkp[5]>=-loss) && (blkp[5]<=loss)) blkp[5] = 0;
	}

	if (xmit_color) {
	    blkp = ((int8 *)block)+64;
	    if ((blkp[2]>=-2*loss) && (blkp[2]<=2*loss)) blkp[2] = 0;
	    if ((blkp[3]>=-2*loss) && (blkp[3]<=2*loss)) blkp[3] = 0;
	    if ((blkp[4]>=-2*loss) && (blkp[4]<=2*loss)) blkp[4] = 0;
	    if ((blkp[5]>=-2*loss) && (blkp[5]<=2*loss)) blkp[5] = 0;
	    if ((blkp[6]>=-2*loss) && (blkp[6]<=2*loss)) blkp[6] = 0;
	    if ((blkp[7]>=-2*loss) && (blkp[7]<=2*loss)) blkp[7] = 0;
	    blkp = ((int8 *)block)+74;
	    for (i=10; i<64; i+=6, blkp+=6) {
		if ((blkp[0]>=-2*loss) && (blkp[0]<=2*loss)) blkp[0] = 0;
		if ((blkp[1]>=-2*loss) && (blkp[1]<=2*loss)) blkp[1] = 0;
		if ((blkp[2]>=-2*loss) && (blkp[2]<=2*loss)) blkp[2] = 0;
		if ((blkp[3]>=-2*loss) && (blkp[3]<=2*loss)) blkp[3] = 0;
		if ((blkp[4]>=-2*loss) && (blkp[4]<=2*loss)) blkp[4] = 0;
		if ((blkp[5]>=-2*loss) && (blkp[5]<=2*loss)) blkp[5] = 0;
	    }
	}
    }

    /* XXX: The ntohl() calls here need to be redone as something more
	    efficient for little-endian machines! */
    blkwp = block;
    blkwLim = xmit_color? block+32 : block+16;
    i = 2;
    zcount = 0;
    blkw = ntohl(*blkwp++);
    data[1] = blkw >> 24;
    blkw <<= 8;
    rem = 3;
    do {
	while ((b = blkw >> 24) != 0) {
	    data[i++] = b;
	    blkw <<= 8;
	    rem--;
	    if (i == 4) break;
	    if (rem == 0) {
		if (blkwp == blkwLim) break;
		blkw = ntohl(*blkwp++);
		rem = 4;
	    }
	}

	while (1) {
	    if (zcount+rem >= 63) {
		break;
	    } else if (blkw == 0) {
		zcount += rem;
		rem = 0;
		if (blkwp == blkwLim) break;
		blkw = ntohl(*blkwp++);
		rem = 4;
	    } else if ((blkw >> 24) == 0) {
		zcount++;
		blkw <<= 8;
		rem--;
	    } else break;
	}

	*data = ((i-1) << 6) + zcount;
	data += i;
	i = 1;
	zcount = 0;
    } while ((rem != 0) || (blkwp != blkwLim));

    yp += 8;
    uvp += 8;

    if (++run_x == block_width) run_x = -1;

    return data;
}

static int NV_SaveMotion(int *syncp, vidimage_t *image)
{
    int x, y, offset;
    uint8 *data=nv_data, *runhdr=NULL, *xmitp, *agep;

    offset = start_y*block_width+start_x;
    xmitp = &xmit[offset];
    agep = &age[offset];
    for (y=start_y; y < block_height; y++) {
	for (x=start_x; x < block_width; x++) {
	    if (*xmitp) {
		*xmitp = 0;
		*agep = 1;
		data = NV_PutBlockFromImage(image, data, &runhdr, x, y, 0);
	    } else if ((*agep > 0) && (++*agep == AGE_THRESHOLD)) {
		*agep = 0;
		data = NV_PutBlockFromImage(image, data, &runhdr, x, y, 0);
	    }

	    if (data-nv_data > PACKETLEN) {
		start_x = x;
		start_y = y;
		*syncp = 0;
		return data-nv_data;
	    }

	    xmitp++;
	    agep++;
	}
	start_x = 0;
    }

    start_x = start_y = 0;
    *syncp = 1;
    return data-nv_data;
}

int NV_Save(int fd, vidimage_t *image, saveproc_t *callback)
{
    int bytes, sync=0;

    nv_hdr.width &= htons(NV_WIDTHMASK);
    if (xmit_color) nv_hdr.width |= htons(NV_COLORFLAG);

    while (!sync) {
	bytes = NV_SaveMotion(&sync, image);
        (*callback)(fd, sync, RTPCONT_NV, rtp_frame_time, (uint8 *) &nv_hdr,
		sizeof(nv_hdr), nv_data, bytes);
    }

    return 1;
}

/*ARSGUSED*/
void NV_Encode_Reconfig(int color, int w, int h)
{
    xmit_color = color;
    width = w;
    height = h;

    nv_hdr.width = htons(width);
    nv_hdr.height = htons(height);
    block_width = width/8;
    block_height = height/8;

    if (curr_y_data != NULL) free(curr_y_data);
    curr_y_data = (uint8 *) malloc(width*height);

    if (curr_uv_data != NULL) free(curr_uv_data);
    curr_uv_data = (int8 *) malloc(width*height);

    if (xmit != NULL) free(xmit);
    xmit = (uint8 *) malloc(block_height*block_width);
    memset(xmit, 0, block_height*block_width);

    if (age != NULL) free(age);
    age = (uint8 *) malloc(block_height*block_width);
    memset(age, 0, block_height*block_width);

    scan_cycle = start_x = start_y = age_x = age_y = frame_ok = 0;
}

