/*
 * Copyright (c) 1995 Cornell University
 * All rights reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for research and educational purposes, without fee, and
 * without written agreement is hereby granted, provided that the above
 * copyright notice and the following two paragraphs appear in all copies
 * of this software.
 *
 * IN NO EVENT SHALL CORNELL UNIVERSITY BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF CORNELL UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * CORNELL UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND CORNELL UNIVERSITY HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 */

/*
	Netvideo version 3.3
	Written by Ron Frederick <frederick@parc.xerox.com>

	Video decode routines
*/

/*
 * Copyright (c) Xerox Corporation 1992. All rights reserved.
 *  
 * License is granted to copy, to use, and to make and to use derivative
 * works for research and evaluation purposes, provided that Xerox is
 * acknowledged in all documentation pertaining to any such copy or derivative
 * work. Xerox grants no other licenses expressed or implied. The Xerox trade
 * name should not be used in any advertising without its written permission.
 *  
 * XEROX CORPORATION MAKES NO REPRESENTATIONS CONCERNING EITHER THE
 * MERCHANTABILITY OF THIS SOFTWARE OR THE SUITABILITY OF THIS SOFTWARE
 * FOR ANY PARTICULAR PURPOSE.  The software is provided "as is" without
 * express or implied warranty of any kind.
 *  
 * These notices must be retained in any copies of any part of this software.
 */

#include <sys/types.h>
#include "sized_types.h"
#include "vid_image.h"
#include "vid_code.h"
#include "nv.h"

static uint8 *NV_PrepDoBlock(vidimage_t *image, uint8 *data, uint8 *dataLim,
			  int color)
{
    int i, j, k, lim, x0, y0, w, w0, run, width=image->width/4;

    if (dataLim-data < 3) return 0;

    w0 = w = data[0];
    x0 = data[1];
    y0 = data[2];
    data += 3;

    if ((x0+w > width/2) || (y0 >= image->height/8)) return 0;

    while (w-- > 0) {
	if (data >= dataLim) return 0;

	lim=color? 128 : 64;
	for (i=0; i<lim; ) {
	    run = *data++;
	    j = run >> 6;
	    k = run & 0x3f;
	    if (i+j+k > lim) return 0;
	    i+=j;
	    i+=k;
	    while (j--) data++;
	}
    }

    NV_SetXmit(x0,y0,w0);

    return data;
}

int NV_Prep(vidimage_t *image, uint8 *data, int len)
{
    int color, width, height;
    uint8 *dataLim=data+len;

    if (dataLim-data < 4) return 1;

    width  = (data[0] << 8) + data[1];
    height = (data[2] << 8) + data[3];
    data += 4;

    color = ((width & NV_COLORFLAG) != 0);
    width &= NV_WIDTHMASK;
    if ((width >= NV_MAX_WIDTH) || (height >= NV_MAX_HEIGHT)) return 1;

    while ((data != 0) && (data < dataLim)) {
	data = NV_PrepDoBlock(image, data, dataLim, color);
    }
    return (data != dataLim);
}
