/*
 * Copyright (c) 1995 Cornell University
 * All rights reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for research and educational purposes, without fee, and
 * without written agreement is hereby granted, provided that the above
 * copyright notice and the following two paragraphs appear in all copies
 * of this software.
 *
 * IN NO EVENT SHALL CORNELL UNIVERSITY BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF CORNELL UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * CORNELL UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND CORNELL UNIVERSITY HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 */

/*
        Netvideo version 3.3
        Written by Ron Frederick <frederick@parc.xerox.com>
 
        Video image handling routines
*/

/*
 * Copyright (c) Xerox Corporation 1992. All rights reserved.
 *  
 * License is granted to copy, to use, and to make and to use derivative
 * works for research and evaluation purposes, provided that Xerox is
 * acknowledged in all documentation pertaining to any such copy or derivative
 * work. Xerox grants no other licenses expressed or implied. The Xerox trade
 * name should not be used in any advertising without its written permission.
 *  
 * XEROX CORPORATION MAKES NO REPRESENTATIONS CONCERNING EITHER THE
 * MERCHANTABILITY OF THIS SOFTWARE OR THE SUITABILITY OF THIS SOFTWARE
 * FOR ANY PARTICULAR PURPOSE.  The software is provided "as is" without
 * express or implied warranty of any kind.
 *  
 * These notices must be retained in any copies of any part of this software.
 */

#include <stdio.h>
#include <memory.h>
#include <malloc.h>
#include <sys/types.h>
#include "sized_types.h"
#include "vid_image.h"

/*extern int fprintf(FILE *fp, char *format, ...);*/

extern int color_ok;

static void VidImage_AllocImage(vidimage_t *image, int width, int height)
{
    image->width = width;
    image->height = height;
    image->y_data = (uint8 *) malloc(width*height);
    memset(image->y_data, 128, width*height);
    if (image->flags & VIDIMAGE_ISCOLOR) {
	image->uv_data = (int8 *) malloc(width*height);
	memset(image->uv_data, 0, width*height);
    } else {
	image->uv_data = 0;
    }
}

vidimage_t *VidImage_Create(int color, int width, int height)
{
    vidimage_t *image;

    image = (vidimage_t *)malloc(sizeof(vidimage_t));
    if (image != NULL) {
	image->flags = (color ? VIDIMAGE_ISCOLOR : 0) | VIDIMAGE_WANTCOLOR;
	VidImage_AllocImage(image, width, height);
    }

    return image;
}

void VidImage_Destroy(vidimage_t *image)
{
    free(image->y_data);
    if (image->uv_data) free(image->uv_data);
    free(image);
}

void VidImage_Clear(vidimage_t *image)
{
    int size=image->width*image->height;

    memset(image->y_data, 128, size);
    if (image->uv_data) memset(image->uv_data, 0, size);
}

void VidImage_SetColor(vidimage_t *image, int iscolor, int wantcolor)
{
    int width=image->width, height=image->height;

    image->flags &= ~(VIDIMAGE_ISCOLOR | VIDIMAGE_WANTCOLOR);
    image->flags |= iscolor? VIDIMAGE_ISCOLOR : 0;
    image->flags |= (wantcolor && color_ok)? VIDIMAGE_WANTCOLOR : 0;

    if (iscolor) {
	if (image->uv_data == 0) {
	    image->uv_data = (int8 *) malloc(width*height);
	    memset(image->uv_data, 0, width*height);
	}
    } else {
	if (image->uv_data) {
	    free(image->uv_data);
	    image->uv_data = 0;
	}
    }

}

void VidImage_SetSize(vidimage_t *image, int width, int height)
{
    free(image->y_data);
    if (image->uv_data) free(image->uv_data);
    VidImage_AllocImage(image, width, height);
}


