/*
 * Copyright (c) 1992 Anders Klemets <klemets@sics.se>
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms, etc.
 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <dirent.h>


/* defaults */
char *aIPaddr = "224.2.0.1";
char *vIPaddr = "224.2.1.0";
char *ttl = "16";
char *afileprefix = "audio";
char *vfileprefix = "video";
char *aUDPport = "7878";
char *vUDPport = "4444";
char *adirectory = ".";
char *vdirectory = ".";
char *confid = "0";
char *playoffset = "0";

main(argc, argv)
int argc;
char **argv;
{
  parsearguments(argc, argv);
  startprograms();
}

parsearguments(argc, argv)
int argc;
char **argv;
{
  extern char *optarg;
  int c;
  while ((c = getopt(argc, argv, "i:j:p:q:t:f:g:d:e:c:o:")) != -1) {
    switch (c) {
    case 'i':
      aIPaddr = optarg;
      break;
    case 'j':
      vIPaddr = optarg;
      break;
    case 'p':
      aUDPport = optarg;
      break;
    case 'q':
      vUDPport = optarg;
      break;
    case 't':
      ttl = optarg;
      break;
    case 'f':
      afileprefix = optarg;
      break;
    case 'g':
      vfileprefix = optarg;
      break;
    case 'd':
      adirectory = optarg;
      break;
    case 'e':
      vdirectory = optarg;
      break;
    case 'c':
      confid = optarg;
      break;
    case 'o':
      playoffset = optarg;
      break;
    default:
      fprintf(stderr,"Usage:\t[-i <audio IP address>] [-p <audio UDP port>] [-c <conference id>]\n\t[-j <video IP address>] [-q <video UDP port>] [-t <ttl>]\n\t[-d <audio directory>] [-f <audio filename prefix>]\n\t[-e <video directory> [-g <video filename prefix>] [-o <time offset>]\n");
      exit(1);
    }
  }
}

startprograms()
{
  struct timeval atim, vtim, diftim;
  int a, v;
  int apid, vpid;
  a = first_file(adirectory, afileprefix, &atim);
  v = first_file(vdirectory, vfileprefix, &vtim);
  if(a && v) {
    diftim.tv_sec = atim.tv_sec - vtim.tv_sec;
    diftim.tv_usec = atim.tv_usec - vtim.tv_usec;
    if(diftim.tv_sec > 0 && diftim.tv_usec < 0) {
      diftim.tv_sec--;
      diftim.tv_usec += 1000000;
    }
    if(diftim.tv_sec < 0 && diftim.tv_usec > 0) {
      diftim.tv_sec++;
      diftim.tv_usec -= 1000000;
    }
  }
  if (a)
    if((apid = fork()) == 0) {
      if (a && v) {
	if (diftim.tv_sec > 0)
	  sleep(diftim.tv_sec);
	if (diftim.tv_usec > 0)
	  usleep(diftim.tv_usec);
      }
      execlp("vat_play", "vat_play", "-i", aIPaddr, "-p", aUDPport, "-c", confid,
	     "-t", ttl, "-d", adirectory, "-f", afileprefix, "-o", playoffset,
	     "-k", NULL);
    }
  if (v)
    if((vpid = fork()) == 0) {
      if (a && v) {
	if (diftim.tv_sec < 0)
	  sleep(-diftim.tv_sec);
	if (diftim.tv_usec < 0)
	  usleep(-diftim.tv_usec);
      }
      execlp("nv_play", "nv_play", "-i", vIPaddr, "-p", vUDPport,
	     "-t", ttl, "-d", vdirectory, "-f", vfileprefix, "-o",
	     playoffset, "-k", NULL);
    }
  if (a)
    wait(NULL);
  if (v)
    wait(NULL);
}

int first_file(directory, fileprefix, tp)
char *directory;
char *fileprefix;
struct timeval *tp;
{
  DIR *dirp;
  struct dirent *dp;
  unsigned int mint = ~0, minu;
  unsigned int m1, um1;

  if ((dirp = opendir(directory)) == NULL) {
    perror("opendir");
    exit(1);
  }
  while ((dp = readdir(dirp)) != NULL) {
    if(!strncmp(dp->d_name, fileprefix, strlen(fileprefix))) {
      sscanf(dp->d_name + strlen(fileprefix), ".%x.%x", &m1, &um1);
      if (m1 && (m1 < mint)) {
	mint = m1;
	minu = um1;
      }
      if (m1 && m1 == mint)
	if (um1 < minu)
	  minu = um1;
    }
  }
  closedir(dirp);
  if (mint == ~0)
    return 0;
  tp->tv_sec = mint;
  tp->tv_usec = minu;
  return 1;
}

