/*
 * Copyright (c) 1992 Anders Klemets <klemets@sics.se>
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms, etc.
 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>
#include <unistd.h>
#include "rtp.h"

#define NV_MIN_FLOW	32

#define timediffinms(tp1,tp2)	(((tp1)->tv_sec - (tp2)->tv_sec) * 1000 + ((tp1)->tv_usec - (tp2)->tv_usec) / 1000)

/* defaults */
char *IPaddr = "224.2.1.0";
u_short UDPport = 4444;
int secperfile = 5;

FILE *open_file();
int interrupted = 0;
void sighandler();
int sv;		/* nv socket */

#define NSID_MAX	64
struct src_info {
  struct sockaddr_in sin;
  struct timeval last;
  char *sdescp;
  long pkt_total;
  long pkt_last;
  long lost_total;
  long lost_last;
  long order_total;
  long order_last;
  long dup_total;
  long dup_last;
  u_short seq;
};
struct src_info *sidtable[NSID_MAX];	/* source id table */

main(argc, argv)
int argc;
char **argv;
{
  parsearguments(argc, argv);
  setupsocket();
  dispchosenparams();
  getpackets();
}

parsearguments(argc, argv)
int argc;
char **argv;
{
  extern char *optarg;
  int c, i;
  while ((c = getopt(argc, argv, "i:p:s:f:d:")) != -1) {
    switch (c) {
    case 'i':
      IPaddr = optarg;
      break;
    case 'p':
      UDPport = atoi(optarg);
      break;
    default:
      fprintf(stderr,"Usage:\t[-i <IP address>] [-p <UDP port>]\n");
      exit(1);
    }
  }
}

dispchosenparams()
{
  printf("Receiving packets from %s:%d\n", IPaddr,
	 UDPport);
}

getpackets()
{
  struct timeval tim, tfile;
  fd_set fds;
  struct sockaddr_in sin;
  int slen;
  u_char sid;
  char buf[1500];
  struct rtphdr *rtp;

  tfile.tv_sec = 0;
  FD_ZERO(&fds);
  while(!interrupted) {
    FD_SET (sv,&fds);
    if (select (sv+1, &fds, NULL, NULL, NULL) > 0) {
      slen = sizeof(sin);
      recvfrom(sv, buf, 1500, 0, &sin, &slen);
      rtp = (struct rtphdr *) buf;
      if(rtp->rh_vers != RTP_VERSION)		/* check for wrong version */
	      continue;
      if(rtp->rh_flow < NV_MIN_FLOW)
	      continue;
      sid = register_speaker(&sin,&tim); /* update list of active speakers */
      rtp_parse(buf,sid);
      gettimeofday(&tim, NULL); /* get timestamp for this video packet */
      /* open a new video file for writing */
      if (!tfile.tv_sec || timediffinms(&tim,&tfile) >= 1000 * secperfile) {
	dispstats();
	tfile = tim;
      }
    }
  }
}

setupsocket()
{
  struct ip_mreq mreq;
  struct hostent *h;
  struct sockaddr_in addr;
  int one;
  unsigned char loop = 1;
  
  if((sv = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
      perror("can't open socket");
      exit(1);
    }
  if ((addr.sin_addr.s_addr = inet_addr(IPaddr)) == -1) {
    if ((h = gethostbyname(IPaddr)) == NULL) {
      fprintf(stderr,"Invalid hostname, %s\n",IPaddr);
      return 1;
    }
    addr.sin_addr.s_addr = *((int *) h->h_addr_list[0]);
  }
  if(IN_MULTICAST(ntohl(addr.sin_addr.s_addr))) {
    mreq.imr_multiaddr = addr.sin_addr;
    mreq.imr_interface.s_addr = htonl(INADDR_ANY);
    if (setsockopt(sv, IPPROTO_IP, IP_ADD_MEMBERSHIP, &mreq, sizeof(mreq)) < 0)
      perror("setsockopt");
  }
#ifdef notdef /* not useful anyway */
  one = 1;   /* enable reuse of address */
  if (setsockopt(sa, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(one)) < 0)
    perror("setsockopt2");
  if (setsockopt(sa, IPPROTO_IP, IP_MULTICAST_LOOP, &loop, sizeof(loop)) < 0)
    perror("setsockopt4");
#endif notdef
  addr.sin_family = AF_INET;
  addr.sin_addr.s_addr = htonl(INADDR_ANY);
  addr.sin_port = htons(UDPport);
  if(bind(sv, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
    perror("bind");
    exit(1);
  }
  signal(SIGINT, sighandler);
  signal(SIGTERM, sighandler);
  return 0;
}

int register_speaker(sin, tp)
struct sockaddr_in *sin;
struct timeval *tp;
{
  int i, minuse, minsid;
  for (i=0; i < NSID_MAX; ++i)
    if (sidtable[i] != NULL &&
	sidtable[i]->sin.sin_addr.s_addr == sin->sin_addr.s_addr &&
	sidtable[i]->sin.sin_port == sin->sin_port) {
      sidtable[i]->last = *tp;
      return i;
    }
  for (i=0; i < NSID_MAX; ++i)	/* previously unknown source */
    if (sidtable[i] == NULL) {
      sidtable[i] = (struct src_info *) calloc(1,sizeof(struct src_info));
      sidtable[i]->sin.sin_addr.s_addr = sin->sin_addr.s_addr;
      sidtable[i]->sin.sin_port = sin->sin_port;
      sidtable[i]->last = *tp;
      return i;
    }
  /* table is full, purge old least used entry */
  minuse = ~0;
  for (i=0; i < NSID_MAX; ++i)
    if (sidtable[i]->last.tv_sec < minuse) {
      minuse = sidtable[i]->last.tv_sec;
      minsid = i;
    }
  sidtable[minsid]->sin.sin_addr.s_addr = sin->sin_addr.s_addr;
  sidtable[minsid]->sin.sin_port = sin->sin_port;
  sidtable[minsid]->last = *tp;
  if(sidtable[minsid]->sdescp == NULL) {
	  free(sidtable[minsid]->sdescp);
	  sidtable[minsid]->sdescp = NULL;
  }
  sidtable[minsid]->pkt_total = sidtable[minsid]->pkt_last = 0;
  sidtable[minsid]->lost_total = sidtable[minsid]->lost_last = 0;
  sidtable[minsid]->order_total = sidtable[minsid]->order_last = 0;
  sidtable[minsid]->dup_total = sidtable[minsid]->dup_last = 0;
  return minsid;
}

void sighandler()
{
  interrupted = 1;
}

rtp_parse(buf,sid)
char *buf;
int sid;
{
	struct rtphdr *rtp;
	struct rtpopthdr *ro;
	struct rtpsrchdr *src;
	struct rtpbophdr *bop;
	struct rtcpcdeschdr *cdesc;
	struct rtcpsdeschdr *sdesc;
	struct rtcpbyehdr *bye;
	struct in_addr in;
	u_short seq;
	rtp = (struct rtphdr *) buf;
	if (rtp->rh_content != RTPCONT_NV)
		return;
	seq = ntohs(rtp->rh_seq);
	if (sidtable[sid]->seq != 0) {
		if (seq - sidtable[sid]->seq > 1) {
			sidtable[sid]->lost_total += seq - sidtable[sid]->seq - 1;
			sidtable[sid]->lost_last += seq - sidtable[sid]->seq - 1;
		}
		if (seq == sidtable[sid]->seq) {
			sidtable[sid]->dup_total++;
			sidtable[sid]->dup_last++;
		}
		if (seq - sidtable[sid]->seq < 0) {
			sidtable[sid]->order_total++;
			sidtable[sid]->order_last++;
		}
	}
	sidtable[sid]->seq = seq;
	sidtable[sid]->pkt_total++;
	sidtable[sid]->pkt_last++;
	if(rtp->rh_opts) {
		ro = (struct rtpopthdr *) (rtp+1);
		while(1) {
			switch (ro->roh_type) {
			      case RTPOPT_CSRC:
				src = (struct rtpsrchdr *) ro;
				in.s_addr = src->rsh_addr;
				printf("CSRC:  addr %s:%d\n",inet_ntoa(in),ntohs(src->rsh_uid));
				break;
			      case RTPOPT_SSRC:
				src = (struct rtpsrchdr *) ro;
				in.s_addr = src->rsh_addr;
				printf("SSRC:  addr %s:%d\n",inet_ntoa(in),ntohs(src->rsh_uid));
				break;
			      case RTPOPT_BOP:
				bop = (struct rtpbophdr *) ro;
				printf("BOP:  seq %d\n",ntohs(bop->rbh_seq));
				break;
			      case RTPOPT_CDESC:
				cdesc = (struct rtcpcdeschdr *) ro;
				printf("CDESC:  content %d rport %d cqual %d cdesc %d\n",
				       cdesc->rtch_content, ntohs(cdesc->rtch_rport),
				       cdesc->rtch_cqual, ntohl(cdesc->rtch_cdesc));
				break;
			      case RTPOPT_SDESC:
				sdesc = (struct rtcpsdeschdr *) ro;
				in.s_addr = sdesc->rtsh_addr;
/*				printf("SDESC:  uid %s:%d\n",inet_ntoa(in),ntohs(sdesc->rtsh_uid));
				printf("\t%s\n",(char *)ro + sizeof(*sdesc)); */
				if(sidtable[sid]->sdescp != NULL)
					free(sidtable[sid]->sdescp);
				sidtable[sid]->sdescp = (char *)malloc(ro->roh_optlen*4);
				bcopy(ro,sidtable[sid]->sdescp,ro->roh_optlen*4);
				break;
			      case RTPOPT_FDESC:
				printf("FDESC:  ??\n");
				break;
			      case RTPOPT_BYE:
				bye = (struct rtcpbyehdr *) ro;
				in.s_addr = bye->rtbh_addr;
				printf("BYE:  uid %s:%d\n",inet_ntoa(in),ntohs(bye->rtbh_uid));
				break;
			      case RTPOPT_QOS:
				printf("QOS:  ??\n");
				break;
			      case RTPOPT_RAD:
				printf("RAD:  ??\n");
				break;
			      default:
				printf("unknown option\n");
			}
			if (!ro->roh_fin)
				ro = (struct rtpopthdr *)((char *)ro + ro->roh_optlen*4);
			else break;
		}
	}
}

dispstats()
{
	struct rtcpsdeschdr *sdesc;
	struct in_addr in;
	int i;
	for (i=0; i < NSID_MAX; ++i)
		if (sidtable[i] != NULL) {
			if(sidtable[i]->sdescp != NULL) {
				sdesc = (struct rtcpsdeschdr *) sidtable[i]->sdescp;
				in.s_addr = sdesc->rtsh_addr;
				printf("uid %s:%d\t%s\n",inet_ntoa(in),ntohs(sdesc->rtsh_uid),
				       (char *)(sdesc + 1));
			}
			printf("total\t\tlost\t\torder\t\tdupl\n");
			printf("%d/%d\t\t%d/%d\t\t%d/%d\t\t%d/%d\n",
			       sidtable[i]->pkt_total,
			       sidtable[i]->pkt_last,
			       sidtable[i]->lost_total,
			       sidtable[i]->lost_last,
			       sidtable[i]->order_total,
			       sidtable[i]->order_last,
			       sidtable[i]->dup_total,
			       sidtable[i]->dup_last);
			sidtable[i]->pkt_last = 0;
			sidtable[i]->lost_last = 0;
			sidtable[i]->order_last = 0;
			sidtable[i]->dup_last = 0;
		}
}

			
