#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <netinet/in.h>
#include <unistd.h>
#include <math.h>
#include <fcntl.h>
#include "cmChunk.h"
#include "cmCookies.h"

#define BUF_SIZE 300000
#define ONE_BILLION 1000000000.0

void
main (int argc, char **argv)
{
    int vidFileFD;
    tChunkFileState *cfState;
    int numFrames;
    double fps;
    int qfactor;
    int width;
    int height;
    MJPEGHeader md_header;
    MediaIndepHeader mi_header;
    int num_read;
    char buffer[BUF_SIZE];
    char *vidFileName;
    char *outputFileName;
    int counter, vidNameFlag;
    int size, width2, height2;
    int fot_allocated;
    int *rawFOT;
    double *rawFTT;
    int frame_counter;
    int cur_pos, new_pos;
    int i;
    double clength, clength_s, clength_ns;
    double curTime_s, curTime_ns;
    int sec, nsec;


    vidFileName = NULL;
    outputFileName = NULL;
    if (argc < 3) {
usage:
	fprintf(stderr,
	         "Usage: vid2mjpgc ?-fps value? ?-qfactor value? vidFile outputFile\n");
	exit(1);
    }

    fps = 30.0;
    qfactor = 150;

    /*
     * Parse arguments.
     */
    if (argc > 3) {
	counter = 1;
       
	vidNameFlag = 0;
	while (counter < argc) {
	    if (strcmp(argv[counter], "-fps") == 0) {
		counter++;
		if (counter == argc) {
		    fprintf(stderr, "Must specify value after -fps option.\n");
		    exit(1);
		}
		fps = atof(argv[counter]);
	    } else if (strcmp(argv[counter], "-qfactor") == 0) {
		counter++;
		if (counter == argc) {
		    fprintf(stderr, "Must specify value after -qfactor option.\n");
		    exit(1);
		}
		qfactor = atoi(argv[counter]);	
	    } else if (!vidNameFlag) {
		vidFileName = argv[counter];
		vidNameFlag = 1;
	    } else {
		outputFileName = argv[counter];
	    }
	    counter++;
	}	
    } else {
	vidFileName = argv[1];
	outputFileName = argv[2];
    }

    /*
     * Open vid file.
     */
    if (vidFileName != NULL) {
	vidFileFD = open(vidFileName, O_RDONLY);
    } else {
	goto usage;
    }
    if (vidFileFD < 0) {
	fprintf(stderr, "Couldn't open vid file: %s\n", vidFileName);
	exit(3);
    }

    /*
     * Open output file as chunk file.
     */
    if (outputFileName != NULL) {
	cfState = Chunk_OpenFile(outputFileName, "cw");
    } else {
	goto usage;
    }
    if (cfState == NULL) {
      fprintf(stderr, "Couldn't create output chunk file: %s\n", vidFileName);
    }

    /* 
     * Construct internal representation of vid file.
     * This is basically ripped out of ReadAsVid in the mjpegFile object.
     */
    lseek(vidFileFD, 0, SEEK_SET);

    read(vidFileFD, &width, 4);
    width = ntohl(width);

    read(vidFileFD, &height, 4);
    height = ntohl(height);

    read(vidFileFD, &size, 4);
    size = ntohl(size);

    if ((width <= 0) || (height <= 0) || (size <= 0)) {
      fprintf(stderr, "Frame header in vid file made no sense.\n");
      exit(2);
    }

    lseek(vidFileFD, 20+size, SEEK_SET);

    read(vidFileFD, &width2, 4);
    width2 = ntohl(width2);

    read(vidFileFD, &height2, 4);
    height2 = ntohl(height2);

    if ((width != width2) || (height != height2)) {
      fprintf(stderr, "First two frames in vid file did not agree.\n");
      exit(3);
    }

    /*
     * Seems to be a .vid file. No real guarantee, but if width and height
     * ever do not agree, abort.
     */
    lseek(vidFileFD, 0, SEEK_SET);

    /* 
     * Create frame offset table. 
     */
    fot_allocated = 128;
    rawFOT = (int *) malloc(fot_allocated*sizeof(int)*2);

    frame_counter = 0;
    num_read = 0;

    num_read += read(vidFileFD, &width2, 4);
    num_read += read(vidFileFD, &height2, 4);
    num_read += read(vidFileFD, &size, 4);

    width2 = ntohl(width2);
    height2 = ntohl(height2);
    size = ntohl(size);
    
    while (num_read == 12) {
	if ((width2 != width) || (height2 != height)) {
	    goto error;
	}

	if (size < 0) {
	    goto error;
	}

	cur_pos = lseek(vidFileFD, 0, SEEK_CUR);
	lseek(vidFileFD, 8+size, SEEK_CUR);
	new_pos = lseek(vidFileFD, 0, SEEK_CUR);
	
	if (new_pos - cur_pos != 8+size) {
error:
	    fprintf(stderr, "Problem constructing tables. Aborting.\n");
	    exit(4);
	}
	  
	if (frame_counter >= fot_allocated) {
	    fot_allocated += 128;
	    rawFOT = (int *) realloc(rawFOT, fot_allocated*2*sizeof(int));
	}

	rawFOT[frame_counter*2] = cur_pos-12;
	rawFOT[(frame_counter*2)+1] = size+20;
	frame_counter++;

	num_read = 0;
	num_read += read(vidFileFD, &width2, 4);
	num_read += read(vidFileFD, &height2, 4);
	num_read += read(vidFileFD, &size, 4);

	width2 = ntohl(width2);
	height2 = ntohl(height2);
	size = ntohl(size);
    }

    if (fot_allocated > frame_counter) {
	rawFOT = (int *) realloc(rawFOT, frame_counter*2*sizeof(int));
    }

    /*
     * Now create FTT.
     */
    rawFTT = (double *) malloc((frame_counter+1)*sizeof(double));
    for (i=0; i<=frame_counter; i++) {
	rawFTT[i] = (((double) i) * (1.0 / fps));
    }

    numFrames = frame_counter;

    /*
     * Write media independent header to chunkfile.
     */
    mi_header.media = htonl(VIDEO_MEDIA_CODE);
    mi_header.format = htonl(MJPEG_FORMAT_CODE);
    mi_header.device = htonl(PARALLAX_DEV_CODE);
    mi_header.version = htonl(3);
    mi_header.num_frames = htonl(numFrames);
    clength = rawFTT[numFrames];
    clength_s = floor(clength);
    clength_ns = ((clength - floor(clength)) * ONE_BILLION);
    mi_header.length_s = htonl((int) clength_s);
    mi_header.length_ns = htonl((int) clength_ns);

    if (Chunk_WriteHeader(cfState, CM_MEDIA_INDEP_CF, "") != 0) {
      fprintf(stderr, "Error writing media indep. header\n");
      exit(4);
    }

    write(cfState->fd, &mi_header, (sizeof(MediaIndepHeader)));

    /*
     * Write media dependent header to chunkfile.
     */
    md_header.width = htonl(width);
    md_header.height = htonl(height);
    md_header.device = htonl(PARALLAX_DEV_CODE);
    md_header.subformat = htonl(VID_FRAME_CODE);
    md_header.qfactor = htonl(qfactor);

    if (Chunk_WriteHeader(cfState, CM_MJPEG_CF, "") != 0) {
	fprintf(stderr, "Error writing mjpeg chunk header.\n");
	exit(5);
    }

    write(cfState->fd, &md_header, (sizeof(MJPEGHeader)));

    /*
     * Write the video data.
     */
    if (Chunk_WriteHeader(cfState, CM_DATA_SEGMENT, "") != 0) {
	fprintf(stderr, "Error writing data chunk header.\n");
	exit(6);
    }

    lseek(vidFileFD, 0, SEEK_SET);
    num_read = read(vidFileFD, buffer, BUF_SIZE); 
    while (num_read != 0) {
	write(cfState->fd, buffer, num_read);
	num_read = read(vidFileFD, buffer, BUF_SIZE); 
    }

    /*
     * Write frame offset table.
     */
    if (Chunk_WriteHeader(cfState, CM_FOT, "") != 0) {
	fprintf(stderr, "Error writing frame offset table chunk header.\n");
	exit(7);
    }

    for (i=0; i<numFrames; i++) {
	int offset, length;

	offset = htonl(rawFOT[i*2]);
	length = htonl(rawFOT[(i*2)+1]);
	
	write(cfState->fd, &offset, 4);
	write(cfState->fd, &length, 4);
    }

    /*
     * Write frame time table.
     */
    if (Chunk_WriteHeader(cfState, CM_FTT, "") != 0) {
	fprintf(stderr, "Error writing frame time table chunk header.\n");
	exit(9);
    }

    for (i=0; i<=numFrames; i++) {
	curTime_s = floor(rawFTT[i]);
	curTime_ns = ((rawFTT[i] - floor(rawFTT[i]))*ONE_BILLION);
	
	sec = (int) curTime_s;
	nsec = (int) curTime_ns;

	fprintf(stderr, "Entry %d : %d sec + %d ns\n", i, sec, nsec);

	sec = htonl(sec);
	nsec = htonl(nsec);
	
	write(cfState->fd, &sec, 4);
	write(cfState->fd, &nsec, 4);
    }

    Chunk_CloseFile(cfState);   
}
