/*
 * Copyright (c) 1996 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
 * OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE
 * UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


/*--------------------------------------------------------------*\
 *	vwGeneral.h
 * author:	Eric14@cs.berkeley.edu
 * date:	10-22-94
 *
 * purpose:  Common defines
 *
\*--------------------------------------------------------------*/

#if !defined( _VWGENERAL_H_ )
    #define _VWGENERAL_H_

#include <stdio.h>

#ifdef NO_STDLIB_H
#   include "compat/stdlib.h"
#else
#   include <stdlib.h>
#endif

#include <stdarg.h>

#ifdef NO_STRING_H
#include "compat/string.h"
#else
#include <string.h>
#endif

#include <tcl.h>


/* GNU(x)
 *     Conditionally handle GNU C/C++ extension <x>.
 *     Evaluate x if we're using GNU C/C++, otherwise treat it as white space.
 */
#ifdef __GNUC__
#define GNU(x) x
#else
#define GNU(x)
#endif


/* PRINTF_ARGS(x, y)
 *     Indicate that the associated function uses printf-style arguments
 *     that should be statically checked. <x> indicates the argument number
 *     that contains the format string; <y> indicates the first of the
 *     source arguments (matches "..." in the prototype). This exploits
 *     a GNU C extension and thus uses the GNU(x) macro.
 */
#define PRINTF_ARGS(x, y) \
  GNU(__attribute__ ((format (printf, x, y))))


#ifndef TRUE
#   define TRUE 1
#endif

#ifndef FALSE
#   define FALSE 0
#endif

#define _SCREAM_S	sprintf(gGen_ErrorStr,
#define _DIAG_E     ); Gen_DiagOut();
#define _SCREAM_E   ); Gen_ScreamOut();
#define _DIE_E	); Gen_DieOut(); ASSERT(0);

#define DIAG(s)	{_SCREAM_S s _DIAG_E}
#define SCREAM(s)	{_SCREAM_S s _SCREAM_E}
#define SCREAMIF(v,s) {if (v) {_SCREAM_S s _SCREAM_E}}
#define DIE(s)	{_SCREAM_S s _DIE_E}
#define DIEIF(v,s)	{if (v) {_SCREAM_S s _DIE_E}}
#define C_ ,

typedef Tcl_Interp cTI;

#define ASSERT(_EX) \
	((_EX) ? (void)0 : Gen_Assert(#_EX, __FILE__, __LINE__))


#define kGen_BadTime      (-1.666e20)


void	Gen_Assert(char* exp, char* file, int line);

EXTERN void	Gen_ScreamOut (void);
EXTERN void	Gen_DieOut (void);
EXTERN void	Gen_DiagOut (void);
extern char gGen_ErrorStr[];


#define Gen_CkError(chk,mess) \
    {   if ( chk ) return Gen_Result(ti, "#1### " mess);}
#define Gen_GetInt(varP,str) \
    Gen_CkError(Tcl_GetInt(ti,str,varP),"Bad int arg '%s'" C_ str);
#define Gen_GetDouble(varP,str) \
    Gen_CkError(Tcl_GetDouble(ti,(str),(varP)),"Bad float arg '%s'" C_ str);
#define Gen_GetBoolean(varP,str) \
    Gen_CkError(Tcl_GetBoolean(ti,str,varP), "Bad boolean arg '%s'" C_ str);

#define Gen_ForgetPtr(o) \
    if (o){free(o); (o) = NULL;}

int	Gen_Result(cTI* ti, char* format, ...) PRINTF_ARGS(2,3);
int	Gen_Format (Tcl_DString* dstr, char* format, ...) PRINTF_ARGS(2,3);


int	Gen_VAResult(cTI* ti, char* format, va_list* ap);
int	Gen_VAFormat(cTI* ti, Tcl_DString* dstr, char* format, va_list* ap);

int	Gen_HashSet (Tcl_HashTable* tab, char* key, void* val);
int	Gen_HashCopy (Tcl_HashTable* destT, Tcl_HashTable* srcT);
char*	Gen_HashGet (Tcl_HashTable* tab, char* key);

ClientData	Gen_GetCommandData(cTI* ti, char* cmdName);


#endif /* !defined( _VWGENERAL_H_ ) */
